@@ -2,6 +2,7 @@
 #  include "../src/config.h"
 #endif
 
+#include <inttypes.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -11,6 +12,8 @@
 #include <math.h>
 
 #include <cdi.h>
+#include <cdi_uuid.h>
+
 int      vlistInqVarMissvalUsed(int vlistID, int varID);
 #ifndef DBL_IS_NAN
 #if  defined  (HAVE_DECL_ISNAN)
@@ -32,17 +35,24 @@ int      vlistInqVarMissvalUsed(int vlistID, int varID);
 #  define IS_EQUAL(x,y)     (!IS_NOT_EQUAL(x,y))
 #endif
 
-
-static inline int
-cdiUUIDIsNull(const unsigned char uuid[CDI_UUID_SIZE])
-{
-  static unsigned char uuid_nil[CDI_UUID_SIZE];
-  return !memcmp(uuid, uuid_nil, CDI_UUID_SIZE);
-}
+#ifndef HOST_ENDIANNESS
+#ifdef __cplusplus
+static const uint32_t HOST_ENDIANNESS_temp[1] = { UINT32_C(0x00030201) };
+#define HOST_ENDIANNESS (((const unsigned char *)HOST_ENDIANNESS_temp)[0])
+#else
+#define HOST_ENDIANNESS (((const unsigned char *)&(const uint32_t[1]){UINT32_C(0x00030201)})[0])
+#endif
+#endif
 
 #include "printinfo.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
 void cdiDefTableID(int tableID);
+#if defined (__cplusplus)
+}
+#endif
 
 int getopt(int argc, char *const argv[], const char *optstring);
 
@@ -50,21 +60,23 @@ extern char *optarg;
 extern int optind, opterr, optopt;
 
 
-char *Progname;
+static char *Progname;
 
-int DefaultFileType  = CDI_UNDEFID;
-int DefaultDataType  = CDI_UNDEFID;
-int DefaultByteorder = CDI_UNDEFID;
+static int DefaultFileType  = CDI_UNDEFID;
+static int DefaultDataType  = CDI_UNDEFID;
+static int DefaultByteorder = CDI_UNDEFID;
 
-int comptype  = COMPRESS_NONE;  // Compression type
-int complevel = 0;              // Compression level
+static int comptype  = COMPRESS_NONE;  // Compression type
+static int complevel = 0;              // Compression level
 
+enum datamode {SP_MODE, DP_MODE};
+static int datamode = DP_MODE;
 
 static
 void version(void)
 {
   int   filetypes[] = {FILETYPE_SRV, FILETYPE_EXT, FILETYPE_IEG, FILETYPE_GRB, FILETYPE_GRB2, FILETYPE_NC, FILETYPE_NC2, FILETYPE_NC4, FILETYPE_NC4C};
-  char *typenames[] = {        "srv",        "ext",        "ieg",        "grb",        "grb2",        "nc",        "nc2",        "nc4",        "nc4c"};
+   const char *typenames[] = {        "srv",        "ext",        "ieg",        "grb",        "grb2",        "nc",        "nc2",        "nc4",        "nc4c"};
 
   fprintf(stderr, "CDI version 1.8\n");
 #if defined (COMPILER)
@@ -141,7 +153,7 @@ void version(void)
 static
 void usage(void)
 {
-  char *name;
+  const char *name;
   int id;
 
   fprintf(stderr, "usage : %s  [Option]  [ifile]  [ofile]\n", Progname);
@@ -268,9 +280,7 @@ void printInfo(int vdate, int vtime, char *varname, double level,
     fprintf(stdout, "Found %d of %d missing values!\n", imiss, nmiss);
 }
 
-#define MAXCHARS 82
-
-const char * tunit2str(int tunits)
+static const char *tunit2str(int tunits)
 {
   if      ( tunits == TUNIT_YEAR )       return ("years");
   else if ( tunits == TUNIT_MONTH )      return ("months");
@@ -287,7 +297,7 @@ const char * tunit2str(int tunits)
 }
 
 
-const char* calendar2str(int calendar)
+static const char *calendar2str(int calendar)
 {
   if      ( calendar == CALENDAR_STANDARD )  return ("standard");
   else if ( calendar == CALENDAR_PROLEPTIC ) return ("proleptic_gregorian");
@@ -298,14 +308,15 @@ const char* calendar2str(int calendar)
 }
 
 static
-void limit_string_length(char* string)
+void limit_string_length(char* string, size_t maxlen)
 {
+  string[maxlen-1] = 0;
   size_t len = strlen(string);
 
   if ( len > 10 )
     {
       for ( size_t i = 3; i < len; ++i )
-	if ( string[i] == ' ' )
+	if ( string[i] == ' ' || string[i] == ',' || (i>10 && string[i] == '.') )
 	  {
 	    string[i] = 0;
 	    break;
@@ -320,7 +331,7 @@ void printShortinfo(int streamID, int vlistID, int vardis)
   int gridsize = 0;
   int gridID, zaxisID, param;
   int vdate, vtime;
-  int nvars, ntsteps;
+  int ntsteps;
   int levelsize;
   int tsteptype, taxisID;
   char tmpname[CDI_MAX_NAME];
@@ -335,14 +346,19 @@ void printShortinfo(int streamID, int vlistID, int vardis)
       fprintf(stdout, " : ");
       printFiletype(streamID, vlistID);
 
-      if ( vardis )
-	fprintf(stdout,
-		"   Var : Institut Source   Ttype    Levels Num  Gridsize Num Dtype : Parameter name\n");
-      else
-	fprintf(stdout,
-		"   Var : Institut Source   Ttype    Levels Num  Gridsize Num Dtype : Parameter ID\n");
+      //vlistPrint(vlistID);
+      int nvars = vlistNvars(vlistID);
+      int nsubtypes = vlistNsubtypes(vlistID);
 
-      nvars = vlistNvars(vlistID);
+      if ( nsubtypes > 0 )
+        fprintf(stdout, "   Var : Institut Source   Ttype    Subtypes Num  Levels Num  Gridsize Num Dtype : ");
+      else
+        fprintf(stdout, "   Var : Institut Source   Ttype    Levels Num  Gridsize Num Dtype : ");
+
+      if ( vardis )
+	fprintf(stdout, "Parameter name\n");
+      else
+	fprintf(stdout, "Parameter ID\n");
 
       for ( varID = 0; varID < nvars; varID++ )
 	{
@@ -355,15 +371,15 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 	  /* institute info */
 	  instptr = institutInqNamePtr(vlistInqVarInstitut(vlistID, varID));
 	  strcpy(tmpname, "unknown");
-	  if ( instptr ) strcpy(tmpname, instptr);
-	  limit_string_length(tmpname);
+	  if ( instptr ) strncpy(tmpname, instptr, CDI_MAX_NAME);
+	  limit_string_length(tmpname, CDI_MAX_NAME);
 	  fprintf(stdout, "%-8s ", tmpname);
 
 	  /* source info */
 	  modelptr = modelInqNamePtr(vlistInqVarModel(vlistID, varID));
 	  strcpy(tmpname, "unknown");
-	  if ( modelptr ) strcpy(tmpname, modelptr);
-	  limit_string_length(tmpname);
+	  if ( modelptr ) strncpy(tmpname, modelptr, CDI_MAX_NAME);
+	  limit_string_length(tmpname, CDI_MAX_NAME);
 	  fprintf(stdout, "%-8s ", tmpname);
 
 	  /* tsteptype */
@@ -380,6 +396,14 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 	  else if ( tsteptype == TSTEP_DIFF     ) fprintf(stdout, "%-8s ", "diff");
 	  else                                    fprintf(stdout, "%-8s ", "unknown");
 
+          if ( nsubtypes > 0 )
+            {
+              int subtypeID = vlistInqVarSubtype(vlistID, varID);
+              int subtypesize = subtypeInqSize(subtypeID);
+              fprintf(stdout, " %6d  ", subtypesize);
+              fprintf(stdout, "%3d ", vlistSubtypeIndex(vlistID, subtypeID) + 1);
+            }
+
 	  /* layer info */
 	  levelsize = zaxisInqSize(zaxisID);
 	  fprintf(stdout, "%6d ", levelsize);
@@ -439,6 +463,14 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 
       printZaxisInfo(vlistID);
 
+      if ( nsubtypes > 0 )
+        {
+          fprintf(stdout, "   Subtypes");
+          fprintf(stdout, " :\n");
+
+          printSubtypeInfo(vlistID);
+        }
+
       taxisID = vlistInqTaxis(vlistID);
       ntsteps = vlistNtsteps(vlistID);
 
@@ -486,14 +518,10 @@ void printShortinfo(int streamID, int vlistID, int vardis)
 }
 
 
-#undef  IsBigendian
-#define IsBigendian()  ( u_byteorder.c[sizeof(long) - 1] )
-
 
 static
 void setDefaultDataType(char *datatypestr)
 {
-  static union {unsigned long l; unsigned char c[sizeof(long)];} u_byteorder = {1};
   int nbits = -1;
   enum {D_UINT, D_INT, D_FLT, D_CPX};
   int dtype = -1;
@@ -596,12 +624,12 @@ void setDefaultDataType(char *datatypestr)
     {
       if ( *datatypestr == 'l' || *datatypestr == 'L' )
 	{
-	  if ( IsBigendian() ) DefaultByteorder = CDI_LITTLEENDIAN;
+	  if ( HOST_ENDIANNESS == CDI_BIGENDIAN ) DefaultByteorder = CDI_LITTLEENDIAN;
 	  datatypestr++;
 	}
       else if ( *datatypestr == 'b' || *datatypestr == 'B' )
 	{
-	  if ( ! IsBigendian() ) DefaultByteorder = CDI_BIGENDIAN;
+	  if ( HOST_ENDIANNESS == CDI_LITTLEENDIAN ) DefaultByteorder = CDI_BIGENDIAN;
 	  datatypestr++;
 	}
       else
@@ -731,7 +759,7 @@ int main(int argc, char *argv[])
   if (Progname == 0) Progname = argv[0];
   else               Progname++;
 
-  while ( (c = getopt(argc, argv, "b:f:t:w:z:cdhlMmqRrsvVZ")) != EOF )
+  while ( (c = getopt(argc, argv, "b:f:t:w:z:cdhlMmqRrsvVxZ")) != EOF )
     {
       switch (c)
 	{
@@ -780,6 +808,9 @@ int main(int argc, char *argv[])
 	case 'w':
 	  wTable = optarg;
 	  break;
+	case 'x':
+	  datamode = SP_MODE;
+	  break;
 	case 'z':
 	  defineCompress(optarg);
 	  break;
@@ -843,8 +874,8 @@ int main(int argc, char *argv[])
 	  vlistPrint(vlistID1);
 	  ngrids = vlistNgrids(vlistID1);
 	  nzaxis = vlistNzaxis(vlistID1);
-	  for ( gridID = 0; gridID < ngrids; gridID++ ) gridPrint(gridID, 1);
-	  for ( zaxisID = 0; zaxisID < nzaxis; zaxisID++ ) zaxisPrint(zaxisID);
+	  for ( gridID = 0; gridID < ngrids; gridID++ ) gridPrint(gridID, gridID, 1);
+	  for ( zaxisID = 0; zaxisID < nzaxis; zaxisID++ ) zaxisPrint(zaxisID, zaxisID);
 	}
 
       nvars   = vlistNvars(vlistID1);
@@ -907,8 +938,7 @@ int main(int argc, char *argv[])
 	nts = cdiInqTimeSize(streamID1);
       */
       if (Debug)
-	printf("nts = %d\n"
-               "streamID1 = %d, streamID2 = %d\n",
+	printf("nts = %d streamID1 = %d, streamID2 = %d\n",
                nts, streamID1, streamID2);
 
       if ( Shortinfo )
