@@ -1,17 +1,27 @@
+// This file is used in CDI and CDO !!!
+
 #define DATE_FORMAT "%5.4d-%2.2d-%2.2d"
 #define TIME_FORMAT "%2.2d:%2.2d:%2.2d"
 
-void uuid2str(const unsigned char uuid[CDI_UUID_SIZE], char *uuidstr);
+void datetime2str(int date, int time, char *datetimestr, int maxlen)
+{
+  int year, month, day;
+  cdiDecodeDate(date, &year, &month, &day);
+  int hour, minute, second;
+  cdiDecodeTime(time, &hour, &minute, &second);
+
+  int len = sprintf(datetimestr, DATE_FORMAT "T" TIME_FORMAT, year, month, day, hour, minute, second);
+  if ( len > ( maxlen-1) )
+    fprintf(stderr, "Internal problem (%s): sizeof input string is too small!\n", __func__);
+}
+
 
 void date2str(int date, char *datestr, int maxlen)
 {
   int year, month, day;
-  int len;
-
   cdiDecodeDate(date, &year, &month, &day);
 
-  len = sprintf(datestr, DATE_FORMAT, year, month, day);
-
+  int len = sprintf(datestr, DATE_FORMAT, year, month, day);
   if ( len > ( maxlen-1) )
     fprintf(stderr, "Internal problem (%s): sizeof input string is too small!\n", __func__);
 }
@@ -20,12 +30,9 @@ void date2str(int date, char *datestr, int maxlen)
 void time2str(int time, char *timestr, int maxlen)
 {
   int hour, minute, second;
-  int len;
-
   cdiDecodeTime(time, &hour, &minute, &second);
 
-  len = sprintf(timestr, TIME_FORMAT, hour, minute, second);
-
+  int len = sprintf(timestr, TIME_FORMAT, hour, minute, second);
   if ( len > ( maxlen-1) )
     fprintf(stderr, "Internal problem (%s): sizeof input string is too small!\n", __func__);
 }
@@ -33,9 +40,7 @@ void time2str(int time, char *timestr, int maxlen)
 
 void printFiletype(int streamID, int vlistID)
 {
-  int filetype;
-
-  filetype = streamInqFiletype(streamID);
+  int filetype = streamInqFiletype(streamID);
 
   switch ( filetype )
     {
@@ -108,12 +113,9 @@ void printFiletype(int streamID, int vlistID)
 
   if ( filetype == FILETYPE_GRB2 )
     {
-      int nvars, varID;
       int comptype;
-
-      nvars = vlistNvars(vlistID);
-
-      for ( varID = 0; varID < nvars; varID++ )
+      int nvars = vlistNvars(vlistID);
+      for ( int varID = 0; varID < nvars; varID++ )
 	{
 	  comptype = vlistInqVarCompType(vlistID, varID);
 	  if ( comptype )
@@ -222,7 +224,7 @@ void printGridInfo(int vlistID)
 	      latpole = gridInqYpole(gridID);
 	      angle   = gridInqAngle(gridID);
 	      fprintf(stdout, "%33s : lon=%g  lat=%g", "northpole", lonpole, latpole);
-	      if ( angle > 0 ) fprintf(stdout, "  angle=%g", angle);
+	      if ( IS_NOT_EQUAL(angle, 0) ) fprintf(stdout, "  angle=%g", angle);
 	      fprintf(stdout, "\n");
 	    }
 
@@ -371,7 +373,7 @@ void printGridInfo(int vlistID)
       if ( !cdiUUIDIsNull(uuidOfHGrid) )
         {
           char uuidOfHGridStr[37];
-          uuid2str(uuidOfHGrid, uuidOfHGridStr);
+          cdiUUID2Str(uuidOfHGrid, uuidOfHGridStr);
           if ( uuidOfHGridStr[0] != 0  && strlen(uuidOfHGridStr) == 36 )
             {
 	      fprintf(stdout, "%33s : %s\n", "uuid", uuidOfHGridStr);
@@ -383,14 +385,13 @@ void printGridInfo(int vlistID)
 static
 void printZaxisInfo(int vlistID)
 {
-  int nzaxis, index;
   int zaxisID, zaxistype, levelsize, levelID;
   int ltype;
   double *levels = NULL;
   char zaxisname[CDI_MAX_NAME], zname[CDI_MAX_NAME], zunits[CDI_MAX_NAME];
 
-  nzaxis = vlistNzaxis(vlistID);
-  for ( index = 0; index < nzaxis; index++)
+  int nzaxis = vlistNzaxis(vlistID);
+  for ( int index = 0; index < nzaxis; index++)
     {
       double zfirst = 0, zlast = 0, zinc = 0;
       zaxisID   = vlistZaxis(vlistID, index);
@@ -459,6 +460,21 @@ void printZaxisInfo(int vlistID)
           fprintf(stdout, "\n");
         }
 
+      if ( zaxistype == ZAXIS_HYBRID )
+        {
+          char psname[CDI_MAX_NAME];
+          psname[0] = 0;
+          zaxisInqPsName(zaxisID, psname);
+          int vctsize = zaxisInqVctSize(zaxisID);
+          if ( vctsize || psname[0] )
+            {
+	      fprintf(stdout, "%33s :", "available");
+              if ( vctsize   ) fprintf(stdout, " vct");
+              if ( psname[0] ) fprintf(stdout, "  ps: %s", psname);
+              fprintf(stdout, "\n");
+            }
+        }
+
       if ( zaxistype == ZAXIS_REFERENCE )
         {
           int number   = zaxisInqNumber(zaxisID);
@@ -474,7 +490,7 @@ void printZaxisInfo(int vlistID)
           if ( !cdiUUIDIsNull(uuidOfVGrid) )
             {
               char uuidOfVGridStr[37];
-              uuid2str(uuidOfVGrid, uuidOfVGridStr);
+              cdiUUID2Str(uuidOfVGrid, uuidOfVGridStr);
               if ( uuidOfVGridStr[0] != 0  && strlen(uuidOfVGridStr) == 36 )
                 {
                   fprintf(stdout, "%33s : ", "uuid");
@@ -485,6 +501,20 @@ void printZaxisInfo(int vlistID)
     }
 }
 
+static
+void printSubtypeInfo(int vlistID)
+{
+  int nsubtypes = vlistNsubtypes(vlistID);
+  for ( int index = 0; index < nsubtypes; index++)
+    {
+      int subtypeID = vlistSubtype(vlistID, index);
+      int subtypesize = subtypeInqSize(subtypeID);
+      // subtypePrint(subtypeID);
+      fprintf(stdout, "  %4d : %-24s :", vlistSubtypeIndex(vlistID, subtypeID)+1, "tiles");
+      fprintf(stdout, " ntiles=%d", subtypesize);
+      fprintf(stdout, "\n");
+    }
+}
 
 static
 int printDateTime(int ntimeout, int vdate, int vtime)
@@ -511,7 +541,6 @@ int printDateTime(int ntimeout, int vdate, int vtime)
 static
 int printDot(int ndotout, int *nfact, int *ncout)
 {
-
   //printf("ncout %d %d %d\n",*ncout, (*ncout)%(*nfact), *nfact);
   if ( (*ncout)%(*nfact) == 0 )
     {
@@ -530,7 +559,7 @@ int printDot(int ndotout, int *nfact, int *ncout)
 
   (*ncout)++;
 
-  return (ndotout);
+  return ndotout;
 }
 
 static
@@ -592,10 +621,13 @@ void printTimesteps(int streamID, int taxisID, int verbose)
       fprintf(stdout, "\n");
 
       ntimeout = 0;
-      int toff = tsID%4;
-      if ( toff > 0 ) toff = 4 - toff;
-      for ( int i = 0; i < toff; ++i ) next_vdatetime = next_vdatetime->next;
-
+      int toff = 0;
+      if ( tsID > 2*NUM_TIMESTEP )
+        {
+          toff = tsID%4;
+          if ( toff > 0 ) toff = 4 - toff;
+          for ( int i = 0; i < toff; ++i ) next_vdatetime = next_vdatetime->next;
+        }
       for ( int i = toff; i < nvdatetime; ++i )
 	{
 	  vdate = next_vdatetime->vdate;
