@@ -9,11 +9,12 @@
 
 #include <stdio.h>
 #include <sys/types.h>
-/*
-#if defined(__cplusplus)
+
+#ifdef __cplusplus
 extern "C" {
 #endif
-*/
+
+
 #define  CDI_MAX_NAME           256   /* max length of a name                 */
 
 #define  CDI_UNDEFID             -1
@@ -23,6 +24,7 @@ extern "C" {
 
 #define  CDI_BIGENDIAN            0   /* Byte order BIGENDIAN                 */
 #define  CDI_LITTLEENDIAN         1   /* Byte order LITTLEENDIAN              */
+#define  CDI_PDPENDIAN            2
 
 #define  CDI_REAL                 1   /* Real numbers                         */
 #define  CDI_COMP                 2   /* Complex numbers                      */
@@ -31,12 +33,13 @@ extern "C" {
 /* Error identifier */
 
 #define	 CDI_NOERR        	  0   /* No Error                             */
+#define  CDI_EEOF                -1   /* The end of file was encountered      */
 #define  CDI_ESYSTEM            -10   /* Operating system error               */
 #define  CDI_EINVAL             -20   /* Invalid argument                     */
 #define  CDI_EUFTYPE            -21   /* Unsupported file type                */
 #define  CDI_ELIBNAVAIL         -22   /* xxx library not available            */
 #define  CDI_EUFSTRUCT          -23   /* Unsupported file structure           */
-#define  CDI_EUNC4              -24   /* Unsupported netCDF4 structure        */
+#define  CDI_EUNC4              -24   /* Unsupported NetCDF4 structure        */
 #define  CDI_ELIMIT             -99   /* Internal limits exceeded             */
 
 /* File types */
@@ -44,10 +47,10 @@ extern "C" {
 #define  FILETYPE_UNDEF          -1   /* Unknown/not yet defined file type */
 #define  FILETYPE_GRB             1   /* File type GRIB                       */
 #define  FILETYPE_GRB2            2   /* File type GRIB version 2             */
-#define  FILETYPE_NC              3   /* File type netCDF                     */
-#define  FILETYPE_NC2             4   /* File type netCDF version 2 (64-bit)  */
-#define  FILETYPE_NC4             5   /* File type netCDF version 4           */
-#define  FILETYPE_NC4C            6   /* File type netCDF version 4 (classic) */
+#define  FILETYPE_NC              3   /* File type NetCDF                     */
+#define  FILETYPE_NC2             4   /* File type NetCDF version 2 (64-bit)  */
+#define  FILETYPE_NC4             5   /* File type NetCDF version 4           */
+#define  FILETYPE_NC4C            6   /* File type NetCDF version 4 (classic) */
 #define  FILETYPE_SRV             7   /* File type SERVICE                    */
 #define  FILETYPE_EXT             8   /* File type EXTRA                      */
 #define  FILETYPE_IEG             9   /* File type IEG                        */
@@ -113,6 +116,7 @@ extern "C" {
 #define  DATATYPE_TXT           253
 #define  DATATYPE_CPX           254
 #define  DATATYPE_UCHAR         255
+#define  DATATYPE_LONG          256
 
 /* Chunks */
 
@@ -145,7 +149,7 @@ extern "C" {
 #define  ZAXIS_HYBRID               2  /* Hybrid level                                          */
 #define  ZAXIS_HYBRID_HALF          3  /* Hybrid half level                                     */
 #define  ZAXIS_PRESSURE             4  /* Isobaric pressure level in Pascal                     */
-#define  ZAXIS_HEIGHT               5  /* Height above ground in meters                         */
+#define  ZAXIS_HEIGHT               5  /* Height above ground                                   */
 #define  ZAXIS_DEPTH_BELOW_SEA      6  /* Depth below sea level in meters                       */
 #define  ZAXIS_DEPTH_BELOW_LAND     7  /* Depth below land surface in centimeters               */
 #define  ZAXIS_ISENTROPIC           8  /* Isentropic                                            */
@@ -167,6 +171,26 @@ extern "C" {
 #define  ZAXIS_MIX_LAYER           24  /* Mixing Layer                                          */
 #define  ZAXIS_REFERENCE           25  /* zaxis reference number                                */
 
+/* SUBTYPE types */
+
+enum {
+  SUBTYPE_TILES                         = 0  /* Tiles variable                                  */
+};
+
+#define MAX_KV_PAIRS_MATCH 10
+
+/* Data structure defining a key-value search, possibly with multiple
+   key-value pairs in combination.
+
+   Currently, only multiple pairs combined by AND are supported.
+*/
+typedef struct  {
+  int nAND;                                   /* no. of key-value pairs that have to match */
+  int key_value_pairs[2][MAX_KV_PAIRS_MATCH]; /* key-value pairs */
+} subtype_query_t;
+
+
+
 /* TIME types */
 
 #define  TIME_CONSTANT              0  /* obsolate, use TSTEP_CONSTANT                          */
@@ -221,11 +245,20 @@ extern "C" {
 /* number of unsigned char needed to store UUID */
 #define  CDI_UUID_SIZE           16
 
+/* Structs that are used to return data to the user */
+
+typedef struct CdiParam { int discipline; int category; int number; } CdiParam;
+
+
+/* Opaque types */
+typedef struct CdiIterator CdiIterator;
+typedef struct CdiGribIterator CdiGribIterator;
+
 /* CDI control routines */
 
 void    cdiReset(void);
 
-char   *cdiStringError(int cdiErrno);
+const char *cdiStringError(int cdiErrno);
 
 void    cdiDebug(int debug);
 
@@ -238,8 +271,9 @@ void    cdiDefMissval(double missval);
 double  cdiInqMissval(void);
 void    cdiDefGlobal(const char *string, int val);
 
-int     namespaceNew();
+int     namespaceNew(void);
 void    namespaceSetActive(int namespaceID);
+int     namespaceGetActive(void);
 void    namespaceDelete(int namespaceID);
 
 
@@ -285,8 +319,6 @@ void    streamDefVlist(int streamID, int vlistID);
 
 /*      streamInqVlist: Get the Vlist of a stream */
 int     streamInqVlist(int streamID);
-/*      PIO: */
-int     streamInqVlistIDorig(int streamID);
 
 /*      streamInqFiletype: Get the filetype */
 int     streamInqFiletype(int streamID);
@@ -318,8 +350,9 @@ int     streamInqTimestep(int streamID, int tsID);
 /*      PIO: query currently set timestep id  */
 int     streamInqCurTimestepID(int streamID);
 
-char   *streamFilename(int streamID);
-char   *streamFilesuffix(int filetype);
+const char *streamFilename(int streamID);
+const char *streamFilesuffix(int filetype);
+
 off_t   streamNvals(int streamID);
 
 int     streamInqNvars ( int streamID );
@@ -327,36 +360,95 @@ int     streamInqNvars ( int streamID );
 /* STREAM var I/O routines */
 
 /*      streamWriteVar: Write a variable */
-void    streamWriteVar(int streamID, int varID, const double *data_vec, int nmiss);
-void    streamWriteVarF(int streamID, int varID, const float *data_vec, int nmiss);
+void    streamWriteVar(int streamID, int varID, const double data[], int nmiss);
+void    streamWriteVarF(int streamID, int varID, const float data[], int nmiss);
 
 /*      streamReadVar: Read a variable */
-void    streamReadVar(int streamID, int varID, double *data_vec, int *nmiss);
-void    streamReadVarF(int streamID, int varID, float *data_vec, int *nmiss);
+void    streamReadVar(int streamID, int varID, double data[], int *nmiss);
+void    streamReadVarF(int streamID, int varID, float data[], int *nmiss);
 
 /*      streamWriteVarSlice: Write a horizontal slice of a variable */
-void    streamWriteVarSlice(int streamID, int varID, int levelID, const double *data_vec, int nmiss);
-void    streamWriteVarSliceF(int streamID, int varID, int levelID, const float *data_vec, int nmiss);
+void    streamWriteVarSlice(int streamID, int varID, int levelID, const double data[], int nmiss);
+void    streamWriteVarSliceF(int streamID, int varID, int levelID, const float data[], int nmiss);
 
 /*      streamReadVarSlice: Read a horizontal slice of a variable */
-void    streamReadVarSlice(int streamID, int varID, int levelID, double *data_vec, int *nmiss);
-void    streamReadVarSliceF(int streamID, int varID, int levelID, float *data_vec, int *nmiss);
+void    streamReadVarSlice(int streamID, int varID, int levelID, double data[], int *nmiss);
+void    streamReadVarSliceF(int streamID, int varID, int levelID, float data[], int *nmiss);
 
-void    streamWriteVarChunk(int streamID, int varID, const int rect[][2],
-                            const double *data_vec, int nmiss);
+void    streamWriteVarChunk(int streamID, int varID, const int rect[3][2], const double data[], int nmiss);
 
 
 /* STREAM record I/O routines */
 
 void    streamDefRecord(int streamID, int  varID, int  levelID);
 void    streamInqRecord(int streamID, int *varID, int *levelID);
-void    streamWriteRecord(int streamID, const double *data_vec, int nmiss);
-void    streamWriteRecordF(int streamID, const float *data_vec, int nmiss);
-void    streamReadRecord(int streamID, double *data_vec, int *nmiss);
+void    streamWriteRecord(int streamID, const double data[], int nmiss);
+void    streamWriteRecordF(int streamID, const float data[], int nmiss);
+void    streamReadRecord(int streamID, double data[], int *nmiss);
+void    streamReadRecordF(int streamID, float data[], int *nmiss);
 void    streamCopyRecord(int streamIDdest, int streamIDsrc);
 
 void    streamInqGRIBinfo(int streamID, int *intnum, float *fltnum, off_t *bignum);
 
+
+/* File driven I/O (may yield better performance than using the streamXXX functions) */
+
+//Creation & Destruction
+CdiIterator *cdiIterator_new(const char *path);  //Requires a subsequent call to cdiIteratorNextField() to point the iterator at the first field.
+CdiIterator *cdiIterator_clone(CdiIterator *me);
+char *cdiIterator_serialize(CdiIterator *me);  //Returns a malloc'ed string.
+CdiIterator *cdiIterator_deserialize(const char *description);  //description is a string that was returned by cdiIteratorSerialize(). Returns a copy of the original iterator.
+void cdiIterator_print(CdiIterator *me, FILE *stream);
+void cdiIterator_delete(CdiIterator *me);
+
+//Advancing an iterator
+int cdiIterator_nextField(CdiIterator *me);      //Points the iterator at the next field, returns CDI_EEOF if there are no more fields in the file.
+
+//Introspecting metadata
+//All outXXX arguments to these functions may be NULL.
+char *cdiIterator_inqStartTime(CdiIterator *me);      //Returns the (start) time as an ISO-8601 coded string. The caller is responsible to Free() the returned string.
+char *cdiIterator_inqEndTime(CdiIterator *me);      //Returns the end time of an integration period as an ISO-8601 coded string, or NULL if there is no end time. The caller is responsible to Free() the returned string.
+char *cdiIterator_inqRTime(CdiIterator *me);      //Returns the reference date as an ISO-8601 coded string. The caller is responsible to Free() the returned string.
+char *cdiIterator_inqVTime(CdiIterator *me);      //Returns the validity date as an ISO-8601 coded string. The caller is responsible to Free() the returned string.
+int cdiIterator_inqLevelType(CdiIterator *me, int levelSelector, char **outName_optional, char **outLongName_optional, char **outStdName_optional, char **outUnit_optional);      //callers are responsible to Free() strings that they request
+int cdiIterator_inqLevel(CdiIterator *me, int levelSelector, double *outValue1_optional, double *outValue2_optional);       //outValue2 is only written to if the level is a hybrid level
+int cdiIterator_inqLevelUuid(CdiIterator *me, int *outVgridNumber_optional, int *outLevelCount_optional, unsigned char outUuid_optional[CDI_UUID_SIZE]);   //outUuid must point to a buffer of 16 bytes, returns an error code if no generalized zaxis is used.
+int cdiIterator_inqTile(CdiIterator *me, int *outTileIndex, int *outTileAttribute); //Returns CDI_EINVAL if there is no tile information connected to the current field, *outTileIndex and *outTileAttribute will be set to -1 in this case.
+int cdiIterator_inqTileCount(CdiIterator *me, int *outTileCount, int *outTileAttributeCount); //outTileAttributeCount is the count for the tile associated with the current field, a total attribute count cannot be inquired. Returns CDI_EINVAL if there is no tile information connected to the current field, *outTileCount and *outTileAttributeCount will be set to 0 in this case.
+CdiParam cdiIterator_inqParam(CdiIterator *me);
+void cdiIterator_inqParamParts(CdiIterator *me, int *outDiscipline, int *outCategory, int *outNumber);	//Some FORTRAN compilers produce wrong code for the cdiIterator_inqParam()-wrapper, rendering it unusable from FORTRAN. This function is the workaround.
+int cdiIterator_inqDatatype(CdiIterator *me);
+int cdiIterator_inqTsteptype(CdiIterator *me);
+char *cdiIterator_inqVariableName(CdiIterator *me);        //The caller is responsible to Free() the returned buffer.
+int cdiIterator_inqGridId(CdiIterator *me);         //The returned id is only valid until the next call to cdiIteratorNextField().
+
+//Reading data
+void cdiIterator_readField(CdiIterator *me, double data[], size_t *nmiss_optional);
+void cdiIterator_readFieldF(CdiIterator *me, float data[], size_t *nmiss_optional);
+//TODO[NH]: Add functions to read partial fields.
+
+
+//Direct access to grib fields
+CdiGribIterator *cdiGribIterator_clone(CdiIterator *me);  //Returns NULL if the associated file is not a GRIB file.
+void cdiGribIterator_delete(CdiGribIterator *me);
+
+//Callthroughs to GRIB-API
+int cdiGribIterator_getLong(CdiGribIterator *me, const char *key, long *value); //Same semantics as grib_get_long().
+int cdiGribIterator_getDouble(CdiGribIterator *me, const char *key, double *value); //Same semantics as grib_get_double().
+int cdiGribIterator_getLength(CdiGribIterator *me, const char *key, size_t *value);     //Same semantics as grib_get_length().
+int cdiGribIterator_getString(CdiGribIterator *me, const char *key, char *value, size_t *length);       //Same semantics as grib_get_string().
+int cdiGribIterator_getSize(CdiGribIterator *me, const char *key, size_t *value);     //Same semantics as grib_get_size().
+int cdiGribIterator_getLongArray(CdiGribIterator *me, const char *key, long *value, size_t *array_size);       //Same semantics as grib_get_long_array().
+int cdiGribIterator_getDoubleArray(CdiGribIterator *me, const char *key, double *value, size_t *array_size);       //Same semantics as grib_get_double_array().
+
+//Convenience functions for accessing GRIB-API keys
+int cdiGribIterator_inqEdition(CdiGribIterator *me);
+long cdiGribIterator_inqLongValue(CdiGribIterator *me, const char *key);   //Aborts on failure to fetch the given key.
+long cdiGribIterator_inqLongDefaultValue(CdiGribIterator *me, const char *key, long defaultValue); //Returns the default value if the given key is not present.
+double cdiGribIterator_inqDoubleValue(CdiGribIterator *me, const char *key);   //Aborts on failure to fetch the given key.
+double cdiGribIterator_inqDoubleDefaultValue(CdiGribIterator *me, const char *key, double defaultValue); //Returns the default value if the given key is not present.
+char *cdiGribIterator_inqStringValue(CdiGribIterator *me, const char *key);        //Returns a malloc'ed string.
+
 /* VLIST routines */
 
 /*      vlistCreate: Create a variable list */
@@ -396,6 +488,9 @@ int     vlistNgrids(int vlistID);
 /*      vlistNzaxis: Number of zaxis in a variable list */
 int     vlistNzaxis(int vlistID);
 
+/*      vlistNsubtypes: Number of subtypes in a variable list */
+int     vlistNsubtypes(int vlistID);
+
 void    vlistDefNtsteps(int vlistID, int nts);
 int     vlistNtsteps(int vlistID);
 int     vlistGridsizeMax(int vlistID);
@@ -408,6 +503,8 @@ int     vlistZaxisIndex(int vlistID, int zaxisID);
 void    vlistChangeZaxisIndex(int vlistID, int index, int zaxisID);
 void    vlistChangeZaxis(int vlistID, int zaxisID1, int zaxisID2);
 int     vlistNrecs(int vlistID);
+int     vlistSubtype(int vlistID, int index);
+int     vlistSubtypeIndex(int vlistID, int subtypeID);
 
 /*      vlistDefTaxis: Define the time axis of a variable list */
 void    vlistDefTaxis(int vlistID, int taxisID);
@@ -425,7 +522,10 @@ int     vlistInqModel(int vlistID);
 
 /* VLIST VAR routines */
 
-/*      vlistDefVar: Create a new Variable */
+/*      vlistDefVarTiles: Create a new tile-based variable */
+int     vlistDefVarTiles(int vlistID, int gridID, int zaxisID, int tsteptype, int tilesetID);
+
+/*      vlistDefVar: Create a new variable */
 int     vlistDefVar(int vlistID, int gridID, int zaxisID, int tsteptype);
 
 void    vlistChangeVarGrid(int vlistID, int varID, int gridID);
@@ -485,6 +585,9 @@ void    vlistDefVarName(int vlistID, int varID, const char *name);
 /*      vlistInqVarName: Get the name of a Variable */
 void    vlistInqVarName(int vlistID, int varID, char *name);
 
+/*      vlistCopyVarName: Safe and convenient version of vlistInqVarName */
+char   *vlistCopyVarName(int vlistId, int varId);
+
 /*      vlistDefVarStdname: Define the standard name of a Variable */
 void    vlistDefVarStdname(int vlistID, int varID, const char *stdname);
 
@@ -547,7 +650,7 @@ void    vlistDefVarEnsemble(int vlistID, int varID, int ensID, int ensCount, int
 int     vlistInqVarEnsemble(int vlistID, int varID, int *ensID, int *ensCount, int *forecast_type);
 
 /* cdiClearAdditionalKeys: Clear the list of additional GRIB keys. */
-void    cdiClearAdditionalKeys();
+void    cdiClearAdditionalKeys(void);
 /* cdiDefAdditionalKey: Register an additional GRIB key which is read when file is opened. */
 void    cdiDefAdditionalKey(const char *string);
 
@@ -563,6 +666,10 @@ double  vlistInqVarDblKey(int vlistID, int varID, const char *name);
 /* vlistInqVarIntKey: raw access to GRIB meta-data */
 int     vlistInqVarIntKey(int vlistID, int varID, const char *name);
 
+/* needed only for CDO operator after */
+const char *vlistInqVarNamePtr(int vlistID, int varID);
+const char *vlistInqVarLongnamePtr(int vlistID, int varID);
+const char *vlistInqVarUnitsPtr(int vlistID, int varID);
 
 /* VLIST attributes */
 
@@ -573,16 +680,16 @@ int     vlistInqAtt(int vlistID, int varID, int attrnum, char *name, int *typep,
 int     vlistDelAtt(int vlistID, int varID, const char *name);
 
 /*      vlistDefAttInt: Define an integer attribute */
-int     vlistDefAttInt(int vlistID, int varID, const char *name, int type, int len, const int *ip_vec);
+int     vlistDefAttInt(int vlistID, int varID, const char *name, int type, int len, const int ip[]);
 /*      vlistDefAttFlt: Define a floating point attribute */
-int     vlistDefAttFlt(int vlistID, int varID, const char *name, int type, int len, const double *dp_vec);
+int     vlistDefAttFlt(int vlistID, int varID, const char *name, int type, int len, const double dp[]);
 /*      vlistDefAttTxt: Define a text attribute */
 int     vlistDefAttTxt(int vlistID, int varID, const char *name, int len, const char *tp_cbuf);
 
 /*      vlistInqAttInt: Get the value(s) of an integer attribute */
-int     vlistInqAttInt(int vlistID, int varID, const char *name, int mlen, int *ip_vec);
+int     vlistInqAttInt(int vlistID, int varID, const char *name, int mlen, int ip[]);
 /*      vlistInqAttFlt: Get the value(s) of a floating point attribute */
-int     vlistInqAttFlt(int vlistID, int varID, const char *name, int mlen, double *dp_vec);
+int     vlistInqAttFlt(int vlistID, int varID, const char *name, int mlen, double dp[]);
 /*      vlistInqAttTxt: Get the value(s) of a text attribute */
 int     vlistInqAttTxt(int vlistID, int varID, const char *name, int mlen, char *tp_cbuf);
 
@@ -594,13 +701,13 @@ const char *gridNamePtr(int gridtype);
 
 void    gridCompress(int gridID);
 
-void    gridDefMaskGME(int gridID, const int *mask_vec);
-int     gridInqMaskGME(int gridID, int *mask_vec);
+void    gridDefMaskGME(int gridID, const int mask[]);
+int     gridInqMaskGME(int gridID, int mask[]);
 
-void    gridDefMask(int gridID, const int *mask_vec);
-int     gridInqMask(int gridID, int *mask_vec);
+void    gridDefMask(int gridID, const int mask[]);
+int     gridInqMask(int gridID, int mask[]);
 
-void    gridPrint(int gridID, int opt);
+void    gridPrint(int gridID, int index, int opt);
 
 /*      gridCreate: Create a horizontal Grid */
 int     gridCreate(int gridtype, int size);
@@ -636,16 +743,16 @@ void    gridDefNP(int gridID, int np);
 int     gridInqNP(int gridID);
 
 /*      gridDefXvals: Define the values of a X-axis */
-void    gridDefXvals(int gridID, const double *xvals_vec);
+void    gridDefXvals(int gridID, const double xvals[]);
 
 /*      gridInqXvals: Get all values of a X-axis */
-int     gridInqXvals(int gridID, double *xvals_vec);
+int     gridInqXvals(int gridID, double xvals[]);
 
 /*      gridDefYvals: Define the values of a Y-axis */
-void    gridDefYvals(int gridID, const double *yvals_vec);
+void    gridDefYvals(int gridID, const double yvals[]);
 
 /*      gridInqYvals: Get all values of a Y-axis */
-int     gridInqYvals(int gridID, double *yvals_vec);
+int     gridInqYvals(int gridID, double yvals[]);
 
 /*      gridDefXname: Define the name of a X-axis */
 void    gridDefXname(int gridID, const char *xname);
@@ -745,11 +852,18 @@ void    gridDefReference(int gridID, const char *reference);
 int     gridInqReference(int gridID, char *reference);
 
 /*      gridDefUUID: Define the UUID of an unstructured grid */
+#ifdef __cplusplus
+void    gridDefUUID(int gridID, const unsigned char *uuid);
+#else
 void    gridDefUUID(int gridID, const unsigned char uuid[CDI_UUID_SIZE]);
+#endif
 
 /*      gridInqUUID: Get the UUID of an unstructured grid */
+#ifdef __cplusplus
+void    gridInqUUID(int gridID, unsigned char *uuid);
+#else
 void    gridInqUUID(int gridID, unsigned char uuid[CDI_UUID_SIZE]);
-
+#endif
 
 /* Lambert Conformal Conic grid (GRIB version) */
 void gridDefLCC(int gridID, double originLon, double originLat, double lonParY, double lat1, double lat2, double xinc, double yinc, int projflag, int scanflag);
@@ -764,8 +878,8 @@ void gridDefLaea(int gridID, double earth_radius, double lon_0, double lat_0);
 void gridInqLaea(int gridID, double *earth_radius, double *lon_0, double *lat_0);
 
 
-void    gridDefArea(int gridID, const double *area_vec);
-void    gridInqArea(int gridID, double *area_vec);
+void    gridDefArea(int gridID, const double area[]);
+void    gridInqArea(int gridID, double area[]);
 int     gridHasArea(int gridID);
 
 /*      gridDefNvertex: Define the number of vertex of a Gridbox */
@@ -775,19 +889,19 @@ void    gridDefNvertex(int gridID, int nvertex);
 int     gridInqNvertex(int gridID);
 
 /*      gridDefXbounds: Define the bounds of a X-axis */
-void    gridDefXbounds(int gridID, const double *xbounds_vec);
+void    gridDefXbounds(int gridID, const double xbounds[]);
 
 /*      gridInqXbounds: Get the bounds of a X-axis */
-int     gridInqXbounds(int gridID, double *xbounds_vec);
+int     gridInqXbounds(int gridID, double xbounds[]);
 
 /*      gridDefYbounds: Define the bounds of a Y-axis */
-void    gridDefYbounds(int gridID, const double *ybounds_vec);
+void    gridDefYbounds(int gridID, const double ybounds[]);
 
 /*      gridInqYbounds: Get the bounds of a Y-axis */
-int     gridInqYbounds(int gridID, double *ybounds_vec);
+int     gridInqYbounds(int gridID, double ybounds[]);
 
-void    gridDefRowlon(int gridID, int nrowlon, const int rowlon_vec[]);
-void    gridInqRowlon(int gridID, int *rowlon_vec);
+void    gridDefRowlon(int gridID, int nrowlon, const int rowlon[]);
+void    gridInqRowlon(int gridID, int rowlon[]);
 void    gridChangeType(int gridID, int gridtype);
 
 void    gridDefComplexPacking(int gridID, int lpack);
@@ -814,13 +928,13 @@ int     zaxisDuplicate(int zaxisID);
 
 void    zaxisResize(int zaxisID, int size);
 
-void    zaxisPrint(int zaxisID);
+void    zaxisPrint(int zaxisID, int index);
 
 /*      zaxisDefLevels: Define the levels of a Z-axis */
-void    zaxisDefLevels(int zaxisID, const double *levels_vec);
+void    zaxisDefLevels(int zaxisID, const double levels[]);
 
 /*      zaxisInqLevels: Get all levels of a Z-axis */
-void    zaxisInqLevels(int zaxisID, double *levels_vec);
+void    zaxisInqLevels(int zaxisID, double levels[]);
 
 /*      zaxisDefLevel: Define one level of a Z-axis */
 void    zaxisDefLevel(int zaxisID, int levelID, double levels);
@@ -847,19 +961,19 @@ void    zaxisDefUUID(int zaxisID, const unsigned char uuid[CDI_UUID_SIZE]);
 void    zaxisInqUUID(int zaxisID, unsigned char uuid[CDI_UUID_SIZE]);
 
 /*      zaxisDefName: Define the name of a Z-axis */
-void    zaxisDefName(int zaxisID, const char *name);
+void    zaxisDefName(int zaxisID, const char *name_optional);
 
 /*      zaxisInqName: Get the name of a Z-axis */
 void    zaxisInqName(int zaxisID, char *name);
 
 /*      zaxisDefLongname: Define the longname of a Z-axis */
-void    zaxisDefLongname(int zaxisID, const char *longname);
+void    zaxisDefLongname(int zaxisID, const char *longname_optional);
 
 /*      zaxisInqLongname: Get the longname of a Z-axis */
 void    zaxisInqLongname(int zaxisID, char *longname);
 
 /*      zaxisDefUnits: Define the units of a Z-axis */
-void    zaxisDefUnits(int zaxisID, const char *units);
+void    zaxisDefUnits(int zaxisID, const char *units_optional);
 
 /*      zaxisInqUnits: Get the units of a Z-axis */
 void    zaxisInqUnits(int zaxisID, char *units);
@@ -867,28 +981,37 @@ void    zaxisInqUnits(int zaxisID, char *units);
 /*      zaxisInqStdname: Get the standard name of a Z-axis */
 void    zaxisInqStdname(int zaxisID, char *stdname);
 
+/*      zaxisDefPsName: Define the name of the surface pressure variable of a hybrid sigma pressure Z-axis */
+void    zaxisDefPsName(int zaxisID, const char *psname_optional);
+
+/*      zaxisInqPsName: Get the name of the surface pressure variable of a hybrid sigma pressure Z-axis */
+void    zaxisInqPsName(int zaxisID, char *psname);
+
 void    zaxisDefPrec(int zaxisID, int prec);
 int     zaxisInqPrec(int zaxisID);
 
 void    zaxisDefPositive(int zaxisID, int positive);
 int     zaxisInqPositive(int zaxisID);
 
+void    zaxisDefScalar(int zaxisID);
+int     zaxisInqScalar(int zaxisID);
+
 void    zaxisDefLtype(int zaxisID, int ltype);
 int     zaxisInqLtype(int zaxisID);
 
 const double *zaxisInqLevelsPtr(int zaxisID);
-void    zaxisDefVct(int zaxisID, int size, const double *vct_vec);
-void    zaxisInqVct(int zaxisID, double *vct_vec);
+void    zaxisDefVct(int zaxisID, int size, const double vct[]);
+void    zaxisInqVct(int zaxisID, double vct[]);
 int     zaxisInqVctSize(int zaxisID);
 const double *zaxisInqVctPtr(int zaxisID);
-void    zaxisDefLbounds(int zaxisID, const double *lbounds_vec);
-int     zaxisInqLbounds(int zaxisID, double *lbounds_vec);
+void    zaxisDefLbounds(int zaxisID, const double lbounds[]);
+int     zaxisInqLbounds(int zaxisID, double lbounds_optional[]);
 double  zaxisInqLbound(int zaxisID, int index);
-void    zaxisDefUbounds(int zaxisID, const double *ubounds_vec);
-int     zaxisInqUbounds(int zaxisID, double *ubounds_vec);
+void    zaxisDefUbounds(int zaxisID, const double ubounds[]);
+int     zaxisInqUbounds(int zaxisID, double ubounds_optional[]);
 double  zaxisInqUbound(int zaxisID, int index);
-void    zaxisDefWeights(int zaxisID, const double *weights_vec);
-int     zaxisInqWeights(int zaxisID, double *weights_vec);
+void    zaxisDefWeights(int zaxisID, const double weights[]);
+int     zaxisInqWeights(int zaxisID, double weights_optional[]);
 void    zaxisChangeType(int zaxisID, int zaxistype);
 
 /* TAXIS routines */
@@ -974,7 +1097,7 @@ int     taxisInqType(int taxisID);
 
 int     taxisInqNumavg(int taxisID);
 
-char   *tunitNamePtr(int tunitID);
+const char *tunitNamePtr(int tunitID);
 
 
 /* Institut routines */
@@ -985,25 +1108,29 @@ int     institutInqNumber(void);
 int     institutInqCenter(int instID);
 int     institutInqSubcenter(int instID);
 const char *institutInqNamePtr(int instID);
-char   *institutInqLongnamePtr(int instID);
+const char *institutInqLongnamePtr(int instID);
 
 /* Model routines */
 
 int     modelDef(int instID, int modelgribID, const char *name);
-int     modelInq(int instID, int modelgribID, char *name);
-int     modelInqInstitut(int modelID);
+int     modelInq(int instID, int modelgribID, const char *name);
+int     modelInqInstitut(int modelID) ;
 int     modelInqGribID(int modelID);
 const char *modelInqNamePtr(int modelID);
 
 /* Table routines */
 
+/*      tableWriteC: write table of parameters to file in C language format */
 void    tableWriteC(const char *filename, int tableID);
+/*      tableFWriteC: write table of parameters to FILE* in C language format */
 void    tableFWriteC(FILE *ptfp, int tableID);
+/*      tableWrite: write table of parameters to file in tabular format */
 void    tableWrite(const char *filename, int tableID);
+/*      tableRead: read table of parameters from file in tabular format */
 int     tableRead(const char *tablefile);
 int     tableDef(int modelID, int tablenum, const char *tablename);
 
-char   *tableInqNamePtr(int tableID);
+const char *tableInqNamePtr(int tableID);
 void    tableDefEntry(int tableID, int code, const char *name, const char *longname, const char *units);
 
 int     tableInq(int modelID, int tablenum, const char *tablename);
@@ -1019,6 +1146,7 @@ int     tableInqParName(int tableID, int code, char *name);
 int     tableInqParLongname(int tableID, int code, char *longname);
 int     tableInqParUnits(int tableID, int code, char *units);
 
+/* needed only for CDO operator after */
 const char *tableInqParNamePtr(int tableID, int parID);
 const char *tableInqParLongnamePtr(int tableID, int parID);
 const char *tableInqParUnitsPtr(int tableID, int parID);
@@ -1029,14 +1157,52 @@ void    streamDefHistory(int streamID, int size, const char *history);
 int     streamInqHistorySize(int streamID);
 void    streamInqHistoryString(int streamID, char *history);
 
+/* Subtype routines */
+
+/*      subtypeCreate: Create a variable subtype */
+int     subtypeCreate(int subtype);
+
+/*      Gives a textual summary of the variable subtype */
+void    subtypePrint(int subtypeID);
+
+/* Compares two subtype data structures. */
+int     subtypeCompare(int subtypeID1, int subtypeID2);
+
+/*      subtypeInqSize: Get the size of a subtype (e.g. no. of tiles). */
+int     subtypeInqSize(int subtypeID);
+
+/*      subtypeInqActiveIndex: Get the currently active index of a subtype (e.g. current tile index). */
+int     subtypeInqActiveIndex(int subtypeID);
+
+/*      subtypeDefActiveIndex: Set the currently active index of a subtype (e.g. current tile index). */
+void    subtypeDefActiveIndex(int subtypeID, int index);
+
+/*      Generate a "query object" out of a key-value pair. */
+subtype_query_t keyValuePair(const char *key, int value);
+
+/*       Generate an AND-combined "query object" out of two previous
+         query objects. */
+subtype_query_t matchAND(subtype_query_t q1, subtype_query_t q2);
+
+/*      subtypeInqSubEntry: Returns subtype entry ID for a given criterion. */
+int     subtypeInqSubEntry(int subtypeID, subtype_query_t criterion);
+
+/*      subtypeInqTile: Specialized version of subtypeInqSubEntry looking for tile/attribute pair. */
+int     subtypeInqTile(int subtypeID, int tileindex, int attribute);
+
+/*      subtypeInqAttribute: Inquire the value of a subtype attribute. Returns CDI_EINVAL if the attribute does not exist.*/
+int     subtypeInqAttribute(int subtypeID, int index, const char *key, int *outValue);
+
+/*      vlistInqVarSubtype: Return subtype ID for a given variable. */
+int     vlistInqVarSubtype(int vlistID, int varID);
 
 void gribapiLibraryVersion(int *major_version, int *minor_version, int *revision_version);
 
-/*
+
 #if defined (__cplusplus)
 }
 #endif
-*/
+
 #endif  /* CDI_H_ */
 /*
  * Local Variables:
