@@ -21,8 +21,6 @@
 #include "cgribex.h"
 #endif
 
-extern int cdiPioSerialOpenFileMap(int streamID);
-
 int cdiDefaultCalendar = CALENDAR_PROLEPTIC;
 
 int cdiDefaultInstID   = CDI_UNDEFID;
@@ -36,33 +34,34 @@ int cdiSplitLtype105   = CDI_UNDEFID;
 int cdiIgnoreAttCoordinates = FALSE;
 int cdiIgnoreValidRange     = FALSE;
 int cdiSkipRecords          = 0;
+int cdiConvention           = CDI_CONVENTION_ECHAM;
 int cdiInventoryMode        = 1;
+int CDI_Version_Info        = 1;
+int CDI_cmor_mode           = 0;
 size_t CDI_netcdf_hdr_pad   = 0UL;
+bool CDI_netcdf_lazy_grid_load = false;
 
 char *cdiPartabPath   = NULL;
 int   cdiPartabIntern = 1;
 
 double cdiDefaultMissval = -9.E33;
 
-const char Filetypes[][9] = {
+static const char Filetypes[][9] = {
   "UNKNOWN",
   "GRIB",
   "GRIB2",
-  "netCDF",
-  "netCDF2",
-  "netCDF4",
-  "netCDF4c",
+  "NetCDF",
+  "NetCDF2",
+  "NetCDF4",
+  "NetCDF4c",
   "SERVICE",
   "EXTRA",
   "IEG",
   "HDF5",
 };
 
-#undef  UNDEFID
-#define UNDEFID  CDI_UNDEFID
-
-
 int CDI_Debug   = 0;    /* If set to 1, debugging           */
+int CDI_Recopt = 0;
 
 int cdiGribApiDebug     = 0;
 int cdiDefaultLeveltype = -1;
@@ -71,7 +70,7 @@ int cdiSortName = 0;
 int cdiHaveMissval = 0;
 
 
-static long cdiGetenvInt(char *envName)
+static long cdiGetenvInt(const char *envName)
 {
   char *envString;
   long envValue = -1;
@@ -131,7 +130,7 @@ void cdiPrintVersion(void)
   fprintf(stderr, "GRIB_API library version : %s\n", gribapiLibraryVersionString());
 #endif
 #if  defined  (HAVE_LIBNETCDF)
-  fprintf(stderr, "  netCDF library version : %s\n", cdfLibraryVersion());
+  fprintf(stderr, "  NetCDF library version : %s\n", cdfLibraryVersion());
 #endif
 #if  defined  (HAVE_LIBHDF5)
   fprintf(stderr, "    HDF5 library version : %s\n", hdfLibraryVersion());
@@ -271,7 +270,7 @@ void cdiSetChunk(const char *chunkAlgo)
 void cdiInitialize(void)
 {
   static int Init_CDI = FALSE;
-  char *envString;
+  char *envstr;
   long value;
 
   if ( ! Init_CDI )
@@ -289,6 +288,9 @@ void cdiInitialize(void)
       value = cdiGetenvInt("CDI_GRIBAPI_DEBUG");
       if ( value >= 0 ) cdiGribApiDebug = (int) value;
 
+      value = cdiGetenvInt("CDI_RECOPT");
+      if ( value >= 0 ) CDI_Recopt = (int) value;
+
       value = cdiGetenvInt("CDI_REGULARGRID");
       if ( value >= 0 ) cdiDataUnreduced = (int) value;
 
@@ -304,38 +306,49 @@ void cdiInitialize(void)
       value = cdiGetenvInt("CDI_NETCDF_HDR_PAD");
       if ( value >= 0 ) CDI_netcdf_hdr_pad = (size_t) value;
 
-      envString = getenv("CDI_MISSVAL");
-      if ( envString ) cdiDefaultMissval = atof(envString);
+      envstr = getenv("CDI_MISSVAL");
+      if ( envstr ) cdiDefaultMissval = atof(envstr);
       /*
-      envString = getenv("NC_MISSING_VALUE");
-      if ( envString ) cdiNcMissingValue = atoi(envString);
+      envstr = getenv("NC_MISSING_VALUE");
+      if ( envstr ) cdiNcMissingValue = atoi(envstr);
       */
-      envString = getenv("NC_CHUNKSIZEHINT");
-      if ( envString ) cdiNcChunksizehint = atoi(envString);
+      envstr = getenv("NC_CHUNKSIZEHINT");
+      if ( envstr ) cdiNcChunksizehint = atoi(envstr);
 
-      envString = getenv("CDI_CHUNK_ALGO");
-      if ( envString ) cdiSetChunk(envString);
+      envstr = getenv("CDI_CHUNK_ALGO");
+      if ( envstr ) cdiSetChunk(envstr);
 
-      envString = getenv("SPLIT_LTYPE_105");
-      if ( envString ) cdiSplitLtype105 = atoi(envString);
+      envstr = getenv("SPLIT_LTYPE_105");
+      if ( envstr ) cdiSplitLtype105 = atoi(envstr);
 
-      envString = getenv("IGNORE_ATT_COORDINATES");
-      if ( envString ) cdiIgnoreAttCoordinates = atoi(envString);
+      envstr = getenv("IGNORE_ATT_COORDINATES");
+      if ( envstr ) cdiIgnoreAttCoordinates = atoi(envstr);
 
-      envString = getenv("IGNORE_VALID_RANGE");
-      if ( envString ) cdiIgnoreValidRange = atoi(envString);
+      envstr = getenv("IGNORE_VALID_RANGE");
+      if ( envstr ) cdiIgnoreValidRange = atoi(envstr);
 
-      envString = getenv("CDI_SKIP_RECORDS");
-      if ( envString )
+      envstr = getenv("CDI_SKIP_RECORDS");
+      if ( envstr )
 	{
-	  cdiSkipRecords = atoi(envString);
+	  cdiSkipRecords = atoi(envstr);
 	  cdiSkipRecords = cdiSkipRecords > 0 ? cdiSkipRecords : 0;
 	}
 
-      envString = getenv("CDI_INVENTORY_MODE");
-      if ( envString )
+      envstr = getenv("CDI_CONVENTION");
+      if ( envstr )
 	{
-	  if ( strncmp(envString, "time", 4) == 0 )
+	  if ( strcmp(envstr, "CF") == 0 || strcmp(envstr, "cf") == 0 )
+	    {
+	      cdiConvention = CDI_CONVENTION_CF;
+	      if ( CDI_Debug )
+		Message("CDI convention was set to CF!");
+	    }
+	}
+
+      envstr = getenv("CDI_INVENTORY_MODE");
+      if ( envstr )
+	{
+	  if ( strncmp(envstr, "time", 4) == 0 )
 	    {
 	      cdiInventoryMode = 2;
 	      if ( CDI_Debug )
@@ -343,34 +356,47 @@ void cdiInitialize(void)
 	    }
 	}
 
-      envString = getenv("CDI_CALENDAR");
-      if ( envString )
+      envstr = getenv("CDI_VERSION_INFO");
+      if ( envstr )
+        {
+          int ival = atoi(envstr);
+          if ( ival == 0 || ival == 1 )
+            {
+              CDI_Version_Info = ival;
+              if ( CDI_Debug )
+                Message("CDI_Version_Info = %s", envstr);
+            }
+        }
+
+
+      envstr = getenv("CDI_CALENDAR");
+      if ( envstr )
 	{
-	  if      ( strncmp(envString, "standard", 8) == 0 )
+	  if      ( strncmp(envstr, "standard", 8) == 0 )
 	    cdiDefaultCalendar = CALENDAR_STANDARD;
-	  else if ( strncmp(envString, "proleptic", 9) == 0 )
+	  else if ( strncmp(envstr, "proleptic", 9) == 0 )
 	    cdiDefaultCalendar = CALENDAR_PROLEPTIC;
-	  else if ( strncmp(envString, "360days", 7) == 0 )
+	  else if ( strncmp(envstr, "360days", 7) == 0 )
 	    cdiDefaultCalendar = CALENDAR_360DAYS;
-	  else if ( strncmp(envString, "365days", 7) == 0 )
+	  else if ( strncmp(envstr, "365days", 7) == 0 )
 	    cdiDefaultCalendar = CALENDAR_365DAYS;
-	  else if ( strncmp(envString, "366days", 7) == 0 )
+	  else if ( strncmp(envstr, "366days", 7) == 0 )
 	    cdiDefaultCalendar = CALENDAR_366DAYS;
-	  else if ( strncmp(envString, "none", 4) == 0 )
+	  else if ( strncmp(envstr, "none", 4) == 0 )
 	    cdiDefaultCalendar = CALENDAR_NONE;
 
 	  if ( CDI_Debug )
-	    Message("Default calendar set to %s!", envString);
+	    Message("Default calendar set to %s!", envstr);
 	}
 #if  defined  (HAVE_LIBCGRIBEX)
       gribSetCalendar(cdiDefaultCalendar);
 #endif
 
-      envString = getenv("PARTAB_INTERN");
-      if ( envString ) cdiPartabIntern = atoi(envString);
+      envstr = getenv("PARTAB_INTERN");
+      if ( envstr ) cdiPartabIntern = atoi(envstr);
 
-      envString = getenv("PARTAB_PATH");
-      if ( envString ) cdiPartabPath = strdup(envString);
+      envstr = getenv("PARTAB_PATH");
+      if ( envstr ) cdiPartabPath = strdup(envstr);
     }
 }
 
@@ -389,12 +415,6 @@ const char *strfiletype(int filetype)
 }
 
 
-int streamSize(void)
-{
-  return reshCountType ( &streamOps );
-}
-
-
 void cdiDefGlobal(const char *string, int val)
 {
   if      ( strcmp(string, "REGULARGRID")      == 0 ) cdiDataUnreduced = val;
@@ -402,7 +422,10 @@ void cdiDefGlobal(const char *string, int val)
   else if ( strcmp(string, "SORTNAME")         == 0 ) cdiSortName = val;
   else if ( strcmp(string, "HAVE_MISSVAL")     == 0 ) cdiHaveMissval = val;
   else if ( strcmp(string, "NC_CHUNKSIZEHINT") == 0 ) cdiNcChunksizehint = val;
+  else if ( strcmp(string, "CMOR_MODE")        == 0 ) CDI_cmor_mode = val;
   else if ( strcmp(string, "NETCDF_HDR_PAD")   == 0 ) CDI_netcdf_hdr_pad = (size_t) val;
+  else if ( strcmp(string, "NETCDF_LAZY_GRID_LOAD") == 0)
+    CDI_netcdf_lazy_grid_load = (bool)val;
   else Warning("Unsupported global key: %s", string);
 }
 
@@ -431,3 +454,4 @@ double cdiInqMissval(void)
  * require-trailing-newline: t
  * End:
  */
+
