@@ -10,15 +10,24 @@
 #if !defined (NAMESPACE_H)
 #include "namespace.h"
 #endif
+#include "error.h"
 
 int _ExitOnError   = 1;	/* If set to 1, exit on error       */
 int _Verbose = 1;	/* If set to 1, errors are reported */
 int _Debug   = 0;       /* If set to 1, debugging           */
 
+/* If we're not using GNU C, elide __attribute__ */
+#if ! defined __GNUC__ && ! defined __attribute__
+#  define  __attribute__(x)  /*NOTHING*/
+#endif
+
+void SysError_(const char *caller, const char *fmt, ...)
+  __attribute__((noreturn));
 
 void SysError_(const char *caller, const char *fmt, ...)
 {
   va_list args;
+  int saved_errno = errno;
 
   va_start(args, fmt);
 
@@ -29,8 +38,11 @@ void SysError_(const char *caller, const char *fmt, ...)
 
   va_end(args);
 
-  if ( errno )
-    perror("System error message ");
+  if ( saved_errno )
+    {
+      errno = saved_errno;
+      perror("System error message");
+    }
 
   exit(EXIT_FAILURE);
 }
@@ -85,6 +97,9 @@ cdiAbortC_serial(const char *caller, const char *filename,
   exit(EXIT_FAILURE);
 }
 
+typedef void (*cdiWarningFunc)(const char * caller, const char * fmt,
+                               va_list ap);
+
 void Warning_(const char *caller, const char *fmt, ...)
 {
   va_list args;
@@ -93,14 +108,21 @@ void Warning_(const char *caller, const char *fmt, ...)
 
   if ( _Verbose )
     {
-       fprintf(stderr, "Warning (%s) : ", caller);
-      vfprintf(stderr, fmt, args);
-       fprintf(stderr, "\n");
+      cdiWarningFunc cdiWarning_p
+        = (cdiWarningFunc)namespaceSwitchGet(NSSWITCH_WARNING).func;
+      cdiWarning_p(caller, fmt, args);
     }
 
   va_end(args);
 }
 
+void cdiWarning(const char *caller, const char *fmt, va_list ap)
+{
+  fprintf(stderr, "Warning (%s) : ", caller);
+  vfprintf(stderr, fmt, ap);
+  fputc('\n', stderr);
+}
+
 
 void Message_(const char *caller, const char *fmt, ...)
 {
