@@ -2,8 +2,6 @@
 #  include "config.h"
 #endif
 
-static int dummy;
-
 #ifdef HAVE_LIBGRIB_API
 
 #include "gribapi_utilities.h"
@@ -12,14 +10,14 @@ static int dummy;
 #include "dmemory.h"
 #include "error.h"
 #include "gribapi.h"
-#include "proprietarySystemWorkarounds.h"
+#include "grid.h"
 
 #include <assert.h>
 #include <time.h>
 
 #define FAIL_ON_GRIB_ERROR(function, gribHandle, key, ...) do\
 {\
-  int errorCode = function(gribHandle, key, __VA_ARGS__);\
+  int errorCode = (int)function(gribHandle, key, __VA_ARGS__);  \
   if(errorCode)\
     {\
       fprintf(stderr, "%s:%d: Error in function `%s`: `%s` returned error code %d for key \"%s\"", __FILE__, __LINE__, __func__, #function, errorCode, key);\
@@ -30,24 +28,77 @@ static int dummy;
 //A simple wrapper for grib_get_string() that returns a newly allocated string.
 char* gribCopyString(grib_handle* gribHandle, const char* key)
 {
+  char* result = NULL;
   size_t length;
-  if(grib_get_length(gribHandle, key, &length)) return NULL;
-  char* result = xmalloc(length);
-  if(!grib_get_string(gribHandle, key, result, &length)) return result;
-  free(result);
-  return NULL;
+#ifdef HAVE_GRIB_GET_LENGTH
+  if(!grib_get_length(gribHandle, key, &length))
+    {
+    char* result = (char *) Malloc(length);
+    if(!grib_get_string(gribHandle, key, result, &length))
+    result = (char *) Realloc(result, length);
+
+    else
+      {
+          Free(result);
+          result = NULL;
+      }
+    }
+#else
+  length = 1024;         /* there's an implementation limit
+                          * that makes strings longer than
+                          * this unlikely in grib_api versions
+                          * not providing grib_get_length */
+  int rc;
+  result = (char *) Malloc(length);
+  while ((rc = grib_get_string(gribHandle, key, result, &length))
+         == GRIB_BUFFER_TOO_SMALL || rc == GRIB_ARRAY_TOO_SMALL)
+    {
+      if (length <= 1024UL * 1024UL)
+        {
+          length *= 2;
+          result = Realloc(result, length);
+        }
+      else
+        break;
+    }
+  if (!rc)
+    result = Realloc(result, length);
+  else
+    {
+      Free(result);
+      result = NULL;
+    }
+#endif
+  return result;
 }
 
 //A simple wrapper for grib_get_string() for the usecase that the result is only compared to a given constant string.
 //Returns true if the key exists and the value is equal to the given string.
 bool gribCheckString(grib_handle* gribHandle, const char* key, const char* expectedValue)
 {
-  size_t expectedLength = strlen(expectedValue) + 1, length;
+  size_t expectedLength = strlen(expectedValue) + 1;
+#ifdef HAVE_GRIB_GET_LENGTH
+  size_t length;
   if(grib_get_length(gribHandle, key, &length)) return false;
   if(length != expectedLength) return false;
-  char value[length];
+  char *value = (char *) Malloc(length);
   if(grib_get_string(gribHandle, key, value, &length)) return false;
-  return !strcmp(value, expectedValue);
+  int rc = !strcmp(value, expectedValue);
+  Free(value);
+#else
+  char *value = gribCopyString(gribHandle, key);
+  int rc;
+  if (value)
+  {
+    rc = strlen(value) + 1 == expectedLength ?
+      !strcmp(value, expectedValue)
+      : false;
+  }
+  else
+    rc = false;
+  Free(value);
+#endif
+  return rc;
 }
 
 //A simple wrapper for grib_get_long() for the usecase that the result is only compared to a given constant value.
@@ -126,7 +177,7 @@ long gribEditionNumber(grib_handle* gh)
 static char* setUtc()
 {
   char* temp = getenv("TZ"), *result = NULL;
-  if(temp) result = myStrDup(temp);
+  if(temp) result = strdup(temp);
   setenv("TZ", "UTC", 1);
   return result;
 }
@@ -137,7 +188,7 @@ static void resetTz(char* savedTz)
   if(savedTz)
     {
       setenv("TZ", savedTz, 1);
-      free(savedTz);
+      Free(savedTz);
     }
   else
     {
@@ -163,13 +214,13 @@ static int addSecondsToDate(struct tm* me, long long amount)
   //It is irrelevant here whether days are zero or one based, the correction would have be undone again so that it is effectless.
   long long seconds = ((me->tm_mday*24ll + me->tm_hour)*60 + me->tm_min)*60 + me->tm_sec;    //The portion of the date that uses fixed increments.
   seconds += amount;
-  me->tm_mday = seconds/24/60/60;
-  seconds -= me->tm_mday*24*60*60;
-  me->tm_hour = seconds/60/60;
-  seconds -= me->tm_hour*60*60;
-  me->tm_min = seconds/60;
-  seconds -= me->tm_min*60;
-  me->tm_sec = seconds;
+  me->tm_mday = (int)(seconds / 24 / 60 / 60);
+  seconds -= (long long)me->tm_mday * 24 * 60 * 60;
+  me->tm_hour = (int)(seconds / 60 / 60);
+  seconds -= (long long)me->tm_hour * 60 * 60;
+  me->tm_min = (int)(seconds / 60);
+  seconds -= (long long)(me->tm_min * 60);
+  me->tm_sec = (int)seconds;
   return normalizeDays(me);
 }
 
@@ -177,9 +228,9 @@ static void addMonthsToDate(struct tm* me, long long amount)
 {
   long long months = me->tm_year*12ll + me->tm_mon;
   months += amount;
-  me->tm_year = months/12;
-  months -= me->tm_year*12;
-  me->tm_mon = months;
+  me->tm_year = (int)(months/12);
+  months -= (long long)me->tm_year*12;
+  me->tm_mon = (int)months;
 }
 
 //unit is a value according to code table 4.4 of the GRIB2 specification, returns non-zero on error
@@ -208,7 +259,10 @@ static int addToDate(struct tm* me, long long amount, long unit)
 
 static char* makeDateString(struct tm* me)
 {
-  return myAsprintf("%04d-%02d-%02dT%02d:%02d:%02d.000", me->tm_year + 1900, me->tm_mon + 1, me->tm_mday, me->tm_hour, me->tm_min, me->tm_sec);
+  char *result
+    = (char *) Malloc(       4+1+ 2+1+ 2+1+ 2+1+ 2+1+ 2+ 4+ 1);
+  sprintf(result, "%04d-%02d-%02dT%02d:%02d:%02d.000", me->tm_year + 1900, me->tm_mon + 1, me->tm_mday, me->tm_hour, me->tm_min, me->tm_sec);
+  return result;
 }
 
 //FIXME: This ignores any calendar definition that might be present.
@@ -221,7 +275,8 @@ static int getAvailabilityOfRelativeTimes(grib_handle* gh, bool* outHaveForecast
         *outHaveForecastTime = false, *outHaveTimeRange = false;
         return 0;
 
-      case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7: case 15: case 32: case 33: case 40: case 41: case 44: case 45: case 48: case 51: case 53: case 54: case 60: case 1000: case 1002: case 1100: case 40033:
+      //case 55 and case 40455 are the same: 55 is the proposed standard value, 40455 is the value in the local use range that is used by the dwd until the standard is updated.
+      case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7: case 15: case 32: case 33: case 40: case 41: case 44: case 45: case 48: case 51: case 53: case 54: case 55: case 60: case 1000: case 1002: case 1100: case 40033: case 40455:
         *outHaveForecastTime = true, *outHaveTimeRange = false;
         return 0;
 
@@ -234,41 +289,45 @@ static int getAvailabilityOfRelativeTimes(grib_handle* gh, bool* outHaveForecast
     }
 }
 
-char* gribMakeTimeString(grib_handle* gh, bool getEndTime)
+char* gribMakeTimeString(grib_handle* gh, CdiTimeType timeType)
 {
   //Get the parts of the reference date.
-  struct tm date = {
-      .tm_mon = gribGetLong(gh, "month") - 1,   //months are zero based in struct tm and one based in GRIB
-      .tm_mday = gribGetLong(gh, "day"),
-      .tm_hour = gribGetLong(gh, "hour"),
-      .tm_min = gribGetLong(gh, "minute")
-  };
+  struct tm date;
+  date.tm_mon = (int)gribGetLong(gh, "month") - 1;   //months are zero based in struct tm and one based in GRIB
+  date.tm_mday = (int)gribGetLong(gh, "day");
+  date.tm_hour = (int)gribGetLong(gh, "hour");
+  date.tm_min = (int)gribGetLong(gh, "minute");
+
   if(gribEditionNumber(gh) == 1)
     {
-      date.tm_year = gribGetLong(gh, "yearOfCentury");  //years are -1900 based both in struct tm and GRIB1
+      date.tm_year = (int)gribGetLong(gh, "yearOfCentury");  //years are -1900 based both in struct tm and GRIB1
     }
   else
     {
-      date.tm_year = gribGetLong(gh, "year") - 1900;   //years are -1900 based in struct tm and zero based in GRIB2
-      date.tm_sec = gribGetLong(gh, "second");
+      date.tm_year = (int)gribGetLong(gh, "year") - 1900;   //years are -1900 based in struct tm and zero based in GRIB2
+      date.tm_sec = (int)gribGetLong(gh, "second");
 
-      //Determine whether we have a forecast time and a time range.
-      bool haveForecastTime, haveTimeRange;
-      if(getAvailabilityOfRelativeTimes(gh, &haveForecastTime, &haveTimeRange)) return NULL;
-      if(getEndTime && !haveTimeRange) return NULL;     //tell the caller that the requested time does not exist
-
-      //If we have relative times, apply them to the date
-      if(haveForecastTime)
+      //If the start or end time are requested, we need to take the relative times into account.
+      if(timeType != kCdiTimeType_referenceTime)
         {
-          long offset = gribGetLongDefault(gh, "forecastTime", 0);  //if(stepUnits == indicatorOfUnitOfTimeRange) assert(startStep == forecastTime)
-          long offsetUnit = gribGetLongDefault(gh, "indicatorOfUnitOfTimeRange", 255);
-          if(addToDate(&date, offset, offsetUnit)) return NULL;
-          if(getEndTime)
+          //Determine whether we have a forecast time and a time range.
+          bool haveForecastTime, haveTimeRange;
+          if(getAvailabilityOfRelativeTimes(gh, &haveForecastTime, &haveTimeRange)) return NULL;
+          if(timeType == kCdiTimeType_endTime && !haveTimeRange) return NULL;     //tell the caller that the requested time does not exist
+
+          //If we have relative times, apply the relative times to the date
+          if(haveForecastTime)
             {
-              assert(haveTimeRange);
-              long range = gribGetLongDefault(gh, "lengthOfTimeRange", 0);       //if(stepUnits == indicatorOfUnitForTimeRange) assert(endStep == startStep + lengthOfTimeRange)
-              long rangeUnit = gribGetLongDefault(gh, "indicatorOfUnitForTimeRange", 255);
-              if(addToDate(&date, range, rangeUnit)) return NULL;
+              long offset = gribGetLongDefault(gh, "forecastTime", 0);  //if(stepUnits == indicatorOfUnitOfTimeRange) assert(startStep == forecastTime)
+              long offsetUnit = gribGetLongDefault(gh, "indicatorOfUnitOfTimeRange", 255);
+              if(addToDate(&date, offset, offsetUnit)) return NULL;
+              if(timeType == kCdiTimeType_endTime)
+                {
+                  assert(haveTimeRange);
+                  long range = gribGetLongDefault(gh, "lengthOfTimeRange", 0);       //if(stepUnits == indicatorOfUnitForTimeRange) assert(endStep == startStep + lengthOfTimeRange)
+                  long rangeUnit = gribGetLongDefault(gh, "indicatorOfUnitForTimeRange", 255);
+                  if(addToDate(&date, range, rangeUnit)) return NULL;
+                }
             }
         }
     }
@@ -399,7 +458,8 @@ void gribapiGetGrid(grib_handle *gh, grid_t *grid)
       ISEC4_NumValues = ISEC2_NumLon*ISEC2_NumLat;
     }
   */
-  memset(grid, 0, sizeof(grid_t));
+  grid_init(grid);
+  cdiGridTypeInit(grid, gridtype, 0);
 
   size_t datasize;
   FAIL_ON_GRIB_ERROR(grib_get_size, gh, "values", &datasize);
@@ -499,13 +559,13 @@ void gribapiGetGrid(grib_handle *gh, grid_t *grid)
         /* FIXME: assert(numberOfPoints <= INT_MAX && numberOfPoints >= INT_MIN) */
         grid->size   = (int)numberOfPoints;
 
-        grid->rowlon = (int *) malloc((size_t)nlat * sizeof (int));
-        pl          = (long *) malloc((size_t)nlat * sizeof (long));
+        grid->rowlon = (int *) Malloc((size_t)nlat * sizeof (int));
+        pl          = (long *) Malloc((size_t)nlat * sizeof (long));
         dummy       = (size_t)nlat;
         FAIL_ON_GRIB_ERROR(grib_get_long_array, gh, "pl", pl, &dummy);
         /* FIXME: assert(pl[i] >= INT_MIN && pl[i] <= INT_MIN) */
         for (int i = 0; i < nlat; ++i ) grid->rowlon[i] = (int)pl[i];
-        free(pl);
+        Free(pl);
 
         grid->ysize  = nlat;
         grid->xinc   = 0;
@@ -561,14 +621,14 @@ void gribapiGetGrid(grib_handle *gh, grid_t *grid)
         long lpar;
 
         FAIL_ON_GRIB_ERROR(grib_get_long, gh, "Nx", &lpar);
-        nlon = lpar;
+        nlon = (int)lpar;
         FAIL_ON_GRIB_ERROR(grib_get_long, gh, "Ny", &lpar);
-        nlat = lpar;
+        nlat = (int)lpar;
 
         if ( numberOfPoints != nlon*nlat )
           Error("numberOfPoints (%d) and gridSize (%d) differ!", (int)numberOfPoints, nlon*nlat);
 
-        grid->size  = numberOfPoints;
+        grid->size  = (int)numberOfPoints;
         grid->xsize = nlon;
         grid->ysize = nlat;
 
@@ -700,8 +760,9 @@ void gribapiGetGrid(grib_handle *gh, grid_t *grid)
       FAIL_ON_GRIB_ERROR(grib_get_double, gh, "longitudeOfSouthernPoleInDegrees", &grid->xpole);
       FAIL_ON_GRIB_ERROR(grib_get_double, gh, "angleOfRotation", &grid->angle);
       /* change from south to north pole */
-      grid->ypole = -grid->ypole;
+      if ( fabs(grid->ypole) > 0 ) grid->ypole = -grid->ypole;
       grid->xpole =  grid->xpole - 180;
+      if ( fabs(grid->angle) > 0 ) grid->angle = -grid->angle;
     }
 
   grid->xvals = NULL;
@@ -709,3 +770,12 @@ void gribapiGetGrid(grib_handle *gh, grid_t *grid)
   grid->type  = gridtype;
 }
 #endif
+/*
+ * Local Variables:
+ * c-file-style: "Java"
+ * c-basic-offset: 2
+ * indent-tabs-mode: nil
+ * show-trailing-whitespace: t
+ * require-trailing-newline: t
+ * End:
+ */
