@@ -5,34 +5,35 @@
 #  include "config.h"
 #endif
 
-#ifdef USE_MPI
+#include <stdbool.h>
 
 #include <mpi.h>
 #include <yaxt.h>
 
-typedef enum
-  {
-    FINALIZE,
-    RESOURCES,
-    WINCREATE,
-    WRITETS,
-    COLLBUFTX,
-    COLLBUFNMISS,
-  } command;
+enum collectorCommandTags {
+  FINALIZE,
+  RESOURCES,
+  STREAMOPEN,
+  STREAMCLOSE,
+  STREAMDEFVLIST,
+  WRITETS,
+  BLOCK_XFER,
+};
 
 #define MAXWINBUFFERSIZE ((size_t)2048 * 1024 * 1024)
 
 enum
 {
-  numRPCFuncs = 4,
-  STREAMOPEN = -1,
-  STREAMDEFVLIST = -2,
-  STREAMCLOSE = -3,
-  STREAMDEFTIMESTEP = -4,
+  numRPCFuncs = 1,
+  STREAMDEFTIMESTEP = -1,
   HEADERSIZEMARKER = -numRPCFuncs - 1,
   PARTDESCMARKER = -numRPCFuncs - 2,
 };
-enum { MAXDATAFILENAME = 256, MINFUNCID = -numRPCFuncs, MAXFUNCID = -1 };
+enum {
+  MINFUNCID = -numRPCFuncs,
+  MAXFUNCID = -1,
+  defVlistNInts = 2,
+};
 extern const char * const funcMap[numRPCFuncs];
 
 struct headerSize
@@ -47,18 +48,10 @@ struct dataRecord
 
 union funcArgs
 {
-  struct
-  {
-    int streamID, vlistID;
-  } streamChange;
   struct
   {
     int streamID, tsID;
   } streamNewTimestep;
-  struct
-  {
-    int fnamelen, filetype;
-  } newFile;
 };
 
 /* Describes offset and ID of serialized partition descriptor.
@@ -94,9 +87,24 @@ enum
   PIO_WIN_ALIGN = sizeof (double),
 };
 
-#endif
+struct clientBufSize
+{
+  size_t bufSize;
+  int numDataRecords, numRPCRecords;
+};
+
+struct collSpec
+{
+  int numClients;
+  int numServers;
+  bool sendRPCData;
+};
+
+struct clientBufSize
+computeClientStreamBufSize(int streamID, const struct collSpec *collector);
 
 #endif
+
 /*
  * Local Variables:
  * c-file-style: "Java"
