@@ -8,15 +8,16 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <unistd.h>
+
 #include "pio_util.h"
 #include "cdi.h"
 #include "dmemory.h"
 
 static
-char commands[][13] = { "FINALIZE\0",
-                        "RESOURCES\0",
-                        "WINCREATE\0",
-                        "WRITETS\0"};
+char commands[][13] = { "FINALIZE",
+                        "RESOURCES",
+                        "WRITETS"};
 
 /*****************************************************************************/
 void
@@ -66,7 +67,7 @@ void pcdiXMPI(int iret, const char *filename, int line)
           "errorString: \"%s\"\n",
           rank, filename, line,
           errorString[0], errorString[1]);
-  MPI_Abort(MPI_COMM_WORLD, iret);
+  MPI_Abort(MPI_COMM_WORLD, 1);
 }
 
 /*****************************************************************************/
@@ -78,9 +79,9 @@ void pcdiXMPIStat ( int iret, const char *filename, int line, MPI_Status *status
 
   if ( iret == MPI_ERR_IN_STATUS )
     {
+      fprintf ( stderr, "------- checking error in request ----------\n" );
       switch ( status->MPI_ERROR )
         {
-          fprintf ( stderr, "------- checking error in request ----------\n" );
         case MPI_SUCCESS :
           fprintf ( stderr, "-------- mpi_success -----------\n" );
           break;
@@ -104,34 +105,6 @@ void pcdiXMPIStat ( int iret, const char *filename, int line, MPI_Status *status
 
 /****************************************************/
 
-void pcdiDebugComm ( const char *filename, const char *functionname, int line, MPI_Comm *comm )
-{
-  int rank = -1, size, len, rankGlob = -1;
-  char *name;
-
-  name = ( char * ) xmalloc ( MPI_MAX_OBJECT_NAME );
-  memset ( name, 0, ( MPI_MAX_OBJECT_NAME ) * sizeof ( char ));
-  MPI_Comm_get_name ( * comm, name, &len );
-  MPI_Comm_size ( * comm, &size );
-  {
-    int init_flag = 0, finished_flag = 0;
-    if (MPI_Initialized(&init_flag) == MPI_SUCCESS && init_flag
-        && MPI_Finalized(&finished_flag) == MPI_SUCCESS && !finished_flag)
-      {
-        MPI_Comm_rank(*comm, &rank);
-        MPI_Comm_rank(MPI_COMM_WORLD, &rankGlob);
-      }
-  }
-  fprintf ( stdout,
-            "pe%d in %s, %s, line %d: comm: name=%s, size=%d, rank=%d\n",
-            rankGlob, functionname, filename, line,
-            name, size, rank );
-  free ( name );
-
-}
-
-/****************************************************/
-
 void pcdiDebugMsg ( const char * cdiPioDebugString, const char *filename,
                     const char *functionname, int line, int tag, int source,
                     int nfinished )
@@ -159,7 +132,7 @@ void pcdiDebugMsg2 ( const char *filename, const char *functionname, int line,
 
 /****************************************************/
 
-void printArray ( const char * cdiPioDebugString, char * ps, const void * array, int n,
+void printArray ( const char *cdiPioDebugString, const char *ps, const void *array, int n,
                   int datatype, const char * funname, const char * filename, int line )
 {
   int i;
@@ -216,18 +189,46 @@ cdiPioQueryVarDims(int varShape[3], int vlistID, int varID)
     case GRID_CURVILINEAR:
     case GRID_UNSTRUCTURED:
       xabort("unimplemented grid type: %d", gridType);
-      break;
     }
   varShape[2] = zaxisInqSize(zaxisID);
   /* FIXME: other grids have different dimensionality */
   return (varShape[2] == 1)?2:3;
 }
 
+
+
+void
+deco1D_CCP(size_t nelems, const size_t *restrict weightPfxSums,
+           size_t nparts, size_t *restrict separators)
+{
+  separators[0] = 0;
+  separators[nparts] = nelems;
+  size_t i = 0, k = 1;
+  size_t weightTotal = weightPfxSums[nelems];
+  while (k < nparts)
+    {
+      size_t target = k * weightTotal / nparts;
+      do {
+        ++i;
+      } while (i < nelems && weightPfxSums[i] < target);
+      separators[k] = i;
+      ++k;
+      --i;
+    }
+  /* todo: implement h2 and dp+ algorithms from
+   * A. Pınar, C. Aykanat
+   * Fast optimal load balancing algorithms for 1D partitioning
+   */
+}
+
+
+
 /****************************************************/
 /*
  * Local Variables:
  * c-file-style: "Java"
  * c-basic-offset: 2
+ * coding: utf-8
  * indent-tabs-mode: nil
  * show-trailing-whitespace: t
  * require-trailing-newline: t
