@@ -5,26 +5,23 @@
 #include <stddef.h>
 #include <string.h>
 
-#include "dmemory.h"
-
 #include "cdi.h"
+#include "calendar.h"
+#include "dmemory.h"
 #include "error.h"
 #include "taxis.h"
 #include "cdi_cksum.h"
 #include "cdi_int.h"
-#include "calendar.h"
 #include "namespace.h"
 #include "serialize.h"
 #include "resource_handle.h"
 #include "resource_unpack.h"
 
-extern int cdiDefaultCalendar;
-
 static int DefaultTimeType = TAXIS_ABSOLUTE;
 static int DefaultTimeUnit = TUNIT_HOUR;
 
 
-char *Timeunits[] = {
+static const char *Timeunits[] = {
   "undefined",
   "seconds",
   "minutes",
@@ -58,7 +55,7 @@ const resOps taxisOps = {
 };
 
 #define container_of(ptr, type, member) \
-  ((type *)((unsigned char *)ptr - offsetof(type,member)))
+  ((type *)(void*)((unsigned char *)ptr - offsetof(type,member)))
 
 struct refcount_string
 {
@@ -70,7 +67,7 @@ static char *
 new_refcount_string(size_t len)
 {
   struct refcount_string *container
-    = xmalloc(sizeof (*container) + len + 1);
+    = (struct refcount_string *) Malloc(sizeof (*container) + len + 1);
   container->ref_count = 1;
   return container->string;
 }
@@ -83,7 +80,7 @@ delete_refcount_string(void *p)
       struct refcount_string *container
         = container_of(p, struct refcount_string, string);
       if (!--(container->ref_count))
-        free(container);
+        Free(container);
     }
 }
 
@@ -105,10 +102,10 @@ dup_refcount_string(char *p)
 static int  TAXIS_Debug = 0;   /* If set to 1, debugging */
 
 
-char *tunitNamePtr(int unitID)
+const char *tunitNamePtr(int unitID)
 {
-  char *name;
-  int size = sizeof(Timeunits)/sizeof(char *);
+  const char *name;
+  int size = sizeof(Timeunits)/sizeof(*Timeunits);
 
   if ( unitID > 0 && unitID < size )
     name = Timeunits[unitID];
@@ -179,7 +176,7 @@ void taxisDefaultValue(taxis_t* taxisptr)
 static taxis_t *
 taxisNewEntry(cdiResH resH)
 {
-  taxis_t *taxisptr = (taxis_t*) xmalloc(sizeof(taxis_t));
+  taxis_t *taxisptr = (taxis_t*) Malloc(sizeof(taxis_t));
 
   taxisDefaultValue(taxisptr);
   if (resH == CDI_UNDEFID)
@@ -217,13 +214,6 @@ void taxis_copy(taxis_t *taxisptr2, taxis_t *taxisptr1)
 }
 #endif
 
-static
-void taxis_check_ptr(const char *caller, taxis_t *taxisptr)
-{
-  if ( taxisptr == NULL )
-    Errorc("taxis undefined!");
-}
-
 /*
 @Function  taxisCreate
 @Title     Create a Time axis
@@ -276,12 +266,8 @@ int taxisCreate(int taxistype)
 
 void taxisDestroyKernel(taxis_t *taxisptr)
 {
-  taxis_check_ptr(__func__, taxisptr);
-  int id = taxisptr->self;
   delete_refcount_string(taxisptr->name);
   delete_refcount_string(taxisptr->longname);
-  if (id != CDI_UNDEFID)
-    reshRemove(id, &taxisOps);
 }
 
 /*
@@ -297,15 +283,16 @@ void taxisDestroyKernel(taxis_t *taxisptr)
 void taxisDestroy(int taxisID)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
+  reshRemove(taxisID, &taxisOps);
   taxisDestroyKernel(taxisptr);
-  free(taxisptr);
+  Free(taxisptr);
 }
 
 
 void taxisDestroyP( void * taxisptr )
 {
   taxisDestroyKernel((taxis_t *)taxisptr);
-  free(taxisptr);
+  Free(taxisptr);
 }
 
 
@@ -328,8 +315,6 @@ void taxisDefType(int taxisID, int type)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->type != type)
     {
       taxisptr->type = type;
@@ -355,8 +340,6 @@ void taxisDefVdate(int taxisID, int vdate)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->vdate != vdate)
     {
       taxisptr->vdate = vdate;
@@ -382,8 +365,6 @@ void taxisDefVtime(int taxisID, int vtime)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->vtime != vtime)
     {
       taxisptr->vtime = vtime;
@@ -409,8 +390,6 @@ void taxisDefRdate(int taxisID, int rdate)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->rdate != rdate)
     {
       taxisptr->rdate = rdate;
@@ -436,8 +415,6 @@ void taxisDefRtime(int taxisID, int rtime)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->rtime != rtime)
     {
       taxisptr->rtime = rtime;
@@ -463,8 +440,6 @@ void taxisDefFdate(int taxisID, int fdate)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->fdate != fdate)
     {
       taxisptr->fdate = fdate;
@@ -490,8 +465,6 @@ void taxisDefFtime(int taxisID, int ftime)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->ftime != ftime)
     {
       taxisptr->ftime = ftime;
@@ -519,8 +492,6 @@ void taxisDefCalendar(int taxisID, int calendar)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->calendar != calendar)
     {
       taxisptr->calendar = calendar;
@@ -533,8 +504,6 @@ void taxisDefTunit(int taxisID, int unit)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->unit != unit)
     {
       taxisptr->unit = unit;
@@ -547,8 +516,6 @@ void taxisDefForecastTunit(int taxisID, int unit)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->fc_unit != unit)
     {
       taxisptr->fc_unit = unit;
@@ -561,9 +528,7 @@ void taxisDefForecastPeriod(int taxisID, double fc_period)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
-  if (taxisptr->fc_period != fc_period)
+  if ( IS_NOT_EQUAL(taxisptr->fc_period, fc_period) )
     {
       taxisptr->fc_period = fc_period;
       reshSetStatus(taxisID, &taxisOps, RESH_DESYNC_IN_USE);
@@ -575,8 +540,6 @@ void taxisDefNumavg(int taxisID, int numavg)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->numavg != numavg)
     {
       taxisptr->numavg = numavg;
@@ -592,8 +555,6 @@ int taxisInqType(int taxisID)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
-  taxis_check_ptr(__func__, taxisptr);
-
   return (taxisptr->type);
 }
 
@@ -602,8 +563,6 @@ int taxisHasBounds(int taxisID)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
-  taxis_check_ptr(__func__, taxisptr);
-
   return (taxisptr->has_bounds);
 }
 
@@ -612,8 +571,6 @@ void taxisDeleteBounds(int taxisID)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->has_bounds != FALSE)
     {
       taxisptr->has_bounds = FALSE;
@@ -627,9 +584,6 @@ void taxisCopyTimestep(int taxisID2, int taxisID1)
   taxis_t *taxisptr1 = (taxis_t *)reshGetVal(taxisID1, &taxisOps),
     *taxisptr2 = (taxis_t *)reshGetVal(taxisID2, &taxisOps);
 
-  taxis_check_ptr(__func__, taxisptr1);
-  taxis_check_ptr(__func__, taxisptr2);
-
   reshLock();
 
   taxisptr2->rdate = taxisptr1->rdate;
@@ -676,8 +630,6 @@ int taxisInqVdate(int taxisID)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
-  taxis_check_ptr(__func__, taxisptr);
-
   return (taxisptr->vdate);
 }
 
@@ -686,8 +638,6 @@ void taxisInqVdateBounds(int taxisID, int *vdate_lb, int *vdate_ub)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
-  taxis_check_ptr(__func__, taxisptr);
-
   *vdate_lb = taxisptr->vdate_lb;
   *vdate_ub = taxisptr->vdate_ub;
 }
@@ -697,8 +647,6 @@ void taxisDefVdateBounds(int taxisID, int vdate_lb, int vdate_ub)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->vdate_lb != vdate_lb
       || taxisptr->vdate_ub != vdate_ub
       || taxisptr->has_bounds != TRUE)
@@ -730,8 +678,6 @@ int taxisInqVtime(int taxisID)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
-  taxis_check_ptr(__func__, taxisptr);
-
   return (taxisptr->vtime);
 }
 
@@ -740,8 +686,6 @@ void taxisInqVtimeBounds(int taxisID, int *vtime_lb, int *vtime_ub)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
-  taxis_check_ptr(__func__, taxisptr);
-
   *vtime_lb = taxisptr->vtime_lb;
   *vtime_ub = taxisptr->vtime_ub;
 }
@@ -751,8 +695,6 @@ void taxisDefVtimeBounds(int taxisID, int vtime_lb, int vtime_ub)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   if (taxisptr->vtime_lb != vtime_lb
       || taxisptr->vtime_ub != vtime_ub
       || taxisptr->has_bounds != TRUE)
@@ -782,9 +724,7 @@ The function @func{taxisInqRdate} returns the reference date of a Time axis.
 */
 int taxisInqRdate(int taxisID)
 {
-  taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
-
-  taxis_check_ptr(__func__, taxisptr);
+  taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
   if ( taxisptr->rdate == -1 )
     {
@@ -814,9 +754,7 @@ The function @func{taxisInqRtime} returns the reference time of a Time axis.
 */
 int taxisInqRtime(int taxisID)
 {
-  taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
-
-  taxis_check_ptr(__func__, taxisptr);
+  taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
   if ( taxisptr->rdate == -1 )
     {
@@ -846,9 +784,7 @@ The function @func{taxisInqFdate} returns the forecast reference date of a Time
 */
 int taxisInqFdate(int taxisID)
 {
-  taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
-
-  taxis_check_ptr(__func__, taxisptr);
+  taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
   if ( taxisptr->fdate == -1 )
     {
@@ -877,9 +813,7 @@ The function @func{taxisInqFtime} returns the forecast reference time of a Time
 */
 int taxisInqFtime(int taxisID)
 {
-  taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
-
-  taxis_check_ptr(__func__, taxisptr);
+  taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
   if ( taxisptr->fdate == -1 )
     {
@@ -913,8 +847,6 @@ int taxisInqCalendar(int taxisID)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   return (taxisptr->calendar);
 }
 
@@ -923,8 +855,6 @@ int taxisInqTunit(int taxisID)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   return (taxisptr->unit);
 }
 
@@ -933,8 +863,6 @@ int taxisInqForecastTunit(int taxisID)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   return (taxisptr->fc_unit);
 }
 
@@ -943,8 +871,6 @@ double taxisInqForecastPeriod(int taxisID)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   return (taxisptr->fc_period);
 }
 
@@ -953,8 +879,6 @@ int taxisInqNumavg(int taxisID)
 {
   taxis_t *taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
 
-  taxis_check_ptr(__func__, taxisptr);
-
   return (taxisptr->numavg);
 }
 
@@ -963,8 +887,6 @@ taxis_t *taxisPtr(int taxisID)
 {
   taxis_t *taxisptr = (taxis_t *)reshGetVal(taxisID, &taxisOps);
 
-  taxis_check_ptr(__func__, taxisptr);
-
   return (taxisptr);
 }
 
@@ -973,7 +895,6 @@ ptaxisDefName(taxis_t *taxisptr, const char *name)
 {
   if (name)
     {
-      taxis_check_ptr(__func__, taxisptr);
       size_t len = strlen(name);
       delete_refcount_string(taxisptr->name);
       char *taxisname = taxisptr->name = new_refcount_string(len);
@@ -986,7 +907,6 @@ ptaxisDefLongname(taxis_t *taxisptr, const char *longname)
 {
   if (longname)
     {
-      taxis_check_ptr(__func__, taxisptr);
       size_t len = strlen(longname);
       delete_refcount_string(taxisptr->longname);
       char *taxislongname = taxisptr->longname = new_refcount_string(len);
@@ -994,10 +914,15 @@ ptaxisDefLongname(taxis_t *taxisptr, const char *longname)
     }
 }
 
-void cdiDecodeTimevalue(int timeunit, double timevalue, int *days, int *secs)
+
+static void
+cdiDecodeTimevalue(int timeunit, double timevalue, int *days, int *secs)
 {
   static int lwarn = TRUE;
 
+  *days = 0;
+  *secs = 0;
+
   if ( timeunit == TUNIT_MINUTE )
     {
       timevalue *= 60;
@@ -1012,7 +937,8 @@ void cdiDecodeTimevalue(int timeunit, double timevalue, int *days, int *secs)
   if ( timeunit == TUNIT_SECOND )
     {
       *days = (int) (timevalue/86400);
-      *secs = (int) (timevalue - *days*86400.);
+      double seconds = timevalue - *days*86400.;
+      *secs = (int)lround(seconds);
       if ( *secs < 0 ) { *days -= 1; *secs += 86400; };
       /*
       {
@@ -1025,7 +951,8 @@ void cdiDecodeTimevalue(int timeunit, double timevalue, int *days, int *secs)
   else if ( timeunit == TUNIT_DAY )
     {
       *days = (int) timevalue;
-      *secs = (int) ((timevalue - *days)*86400 + 0.5);
+      double seconds = (timevalue - *days)*86400;
+      *secs = (int)lround(seconds);
       if ( *secs < 0 ) { *days -= 1; *secs += 86400; };
       /*
       {
@@ -1081,30 +1008,26 @@ void cdiEncodeTimevalue(int days, int secs, int timeunit, double *timevalue)
     }
 }
 
-int days_per_month(int calendar, int year, int month);
 
 void timeval2vtime(double timevalue, taxis_t *taxis, int *vdate, int *vtime)
 {
+  int rdate = taxis->rdate;
+  int rtime = taxis->rtime;
+
+  if ( DBL_IS_EQUAL(timevalue, 0.) )
+    {
+      *vdate = rdate;
+      *vtime = rtime;
+      return;
+    }
+
   int year, month, day, hour, minute, second;
-  int rdate, rtime;
-  int timeunit;
-  int calendar;
-  int julday, secofday, days, secs;
-
-  *vdate = 0;
-  *vtime = 0;
-
-  timeunit = (*taxis).unit;
-  calendar = (*taxis).calendar;
-
-  rdate  = (*taxis).rdate;
-  rtime  = (*taxis).rtime;
-
-  if ( rdate == 0 && rtime == 0 && DBL_IS_EQUAL(timevalue, 0.) ) return;
-
   cdiDecodeDate(rdate, &year, &month, &day);
   cdiDecodeTime(rtime, &hour, &minute, &second);
 
+  int timeunit = taxis->unit;
+  int calendar = taxis->calendar;
+
   if ( timeunit == TUNIT_MONTH && calendar == CALENDAR_360DAYS )
     {
       timeunit = TUNIT_DAY;
@@ -1113,24 +1036,22 @@ void timeval2vtime(double timevalue, taxis_t *taxis, int *vdate, int *vtime)
 
   if ( timeunit == TUNIT_MONTH || timeunit == TUNIT_YEAR )
     {
-      int nmon, dpm;
-      double fmon;
-
       if ( timeunit == TUNIT_YEAR ) timevalue *= 12;
 
-      nmon = (int) timevalue;
-      fmon = timevalue - nmon;
+      int nmon = (int) timevalue;
+      double fmon = timevalue - nmon;
 
       month += nmon;
 
       while ( month > 12 ) { month -= 12; year++; }
       while ( month <  1 ) { month += 12; year--; }
 
-      dpm = days_per_month(calendar, year, month);
+      int dpm = days_per_month(calendar, year, month);
       timeunit = TUNIT_DAY;
       timevalue = fmon*dpm;
     }
 
+  int julday, secofday, days, secs;
   encode_caldaysec(calendar, year, month, day, hour, minute, second, &julday, &secofday);
 
   cdiDecodeTimevalue(timeunit, timevalue, &days, &secs);
@@ -1223,7 +1144,7 @@ double vtime2timeval(int vdate, int vtime, taxis_t *taxis)
 }
 
 
-void conv_timeval(double timevalue, int *rvdate, int *rvtime)
+static void conv_timeval(double timevalue, int *rvdate, int *rvtime)
 {
   int vdate = 0, vtime = 0;
   int hour, minute, second;
@@ -1245,7 +1166,8 @@ void conv_timeval(double timevalue, int *rvdate, int *rvtime)
 }
 
 
-void splitTimevalue(double timevalue, int timeunit, int *date, int *time)
+static void
+splitTimevalue(double timevalue, int timeunit, int *date, int *time)
 {
   int vdate = 0, vtime = 0;
   int hour, minute, second;
@@ -1268,7 +1190,7 @@ void splitTimevalue(double timevalue, int timeunit, int *date, int *time)
     }
   else if ( timeunit == TUNIT_MONTH )
     {
-      vdate = (int) timevalue*100;
+      vdate = (int) timevalue*100 - ((vdate < 0) * 2 - 1);
       vtime = 0;
     }
   else if ( timeunit == TUNIT_YEAR )
@@ -1285,6 +1207,7 @@ void splitTimevalue(double timevalue, int timeunit, int *date, int *time)
 	}
 
       vdate = (int) timevalue*10000;
+      vdate += 101;
       vtime = 0;
     }
   else
@@ -1412,11 +1335,13 @@ double cdiEncodeTimeval(int date, int time, taxis_t *taxis)
 	{
 	  int year, month, day;
 	  cdiDecodeDate(date, &year, &month, &day);
-	  if ( day == 0 )
-	    timevalue = date/100;
-	  else
-	    timevalue = date/100 + 0.5;
-	}
+          timevalue = date/100;
+	  if ( day != 0 )
+            {
+              if ( date < 0 ) timevalue -= 0.5;
+              else            timevalue += 0.5;
+            }
+        }
       else
 	{
 	  int hour, minute, second;
@@ -1482,55 +1407,52 @@ taxisPrintKernel(taxis_t * taxisptr, FILE * fp)
   int vdate_lb, vdate_ub;
   int vtime_lb, vtime_ub;
 
-  taxis_check_ptr ( __func__, taxisptr );
-
   taxisInqVdateBounds ( taxisptr->self, &vdate_lb, &vdate_ub);
   taxisInqVtimeBounds ( taxisptr->self, &vtime_lb, &vtime_ub);
 
-  fprintf ( fp, "#\n");
-  fprintf ( fp, "# taxisID %d\n", taxisptr->self);
-  fprintf ( fp, "#\n");
-  fprintf ( fp, "self        = %d\n", taxisptr->self );
-  fprintf ( fp, "used        = %d\n", taxisptr->used );
-  fprintf ( fp, "type        = %d\n", taxisptr->type );
-  fprintf ( fp, "vdate       = %d\n", taxisptr->vdate );
-  fprintf ( fp, "vtime       = %d\n", taxisptr->vtime );
-  fprintf ( fp, "rdate       = %d\n", taxisptr->rdate );
-  fprintf ( fp, "rtime       = %d\n", taxisptr->rtime );
-  fprintf ( fp, "fdate       = %d\n", taxisptr->fdate );
-  fprintf ( fp, "ftime       = %d\n", taxisptr->ftime );
-  fprintf ( fp, "calendar    = %d\n", taxisptr->calendar );
-  fprintf ( fp, "unit        = %d\n", taxisptr->unit );
-  fprintf ( fp, "numavg      = %d\n", taxisptr->numavg );
-  fprintf ( fp, "climatology = %d\n", taxisptr->climatology );
-  fprintf ( fp, "has_bounds  = %d\n", taxisptr->has_bounds );
-  fprintf ( fp, "vdate_lb    = %d\n", vdate_lb );
-  fprintf ( fp, "vtime_lb    = %d\n", vtime_lb );
-  fprintf ( fp, "vdate_ub    = %d\n", vdate_ub );
-  fprintf ( fp, "vtime_ub    = %d\n", vtime_ub );
-  fprintf ( fp, "fc_unit     = %d\n", taxisptr->fc_unit );
-  fprintf ( fp, "fc_period   = %g\n", taxisptr->fc_period );
-  fprintf ( fp, "\n");
-}
-
-void taxisPrint ( int taxisID )
-{
-  taxis_t * taxisptr;
-
-  taxisptr = ( taxis_t * ) reshGetVal ( taxisID, &taxisOps );
-  taxisPrintKernel ( taxisptr, stdout );
+  fprintf(fp,
+          "#\n"
+          "# taxisID %d\n"
+          "#\n"
+          "self        = %d\n"
+          "used        = %d\n"
+          "type        = %d\n"
+          "vdate       = %d\n"
+          "vtime       = %d\n"
+          "rdate       = %d\n"
+          "rtime       = %d\n"
+          "fdate       = %d\n"
+          "ftime       = %d\n"
+          "calendar    = %d\n"
+          "unit        = %d\n"
+          "numavg      = %d\n"
+          "climatology = %d\n"
+          "has_bounds  = %d\n"
+          "vdate_lb    = %d\n"
+          "vtime_lb    = %d\n"
+          "vdate_ub    = %d\n"
+          "vtime_ub    = %d\n"
+          "fc_unit     = %d\n"
+          "fc_period   = %g\n"
+          "\n", taxisptr->self, taxisptr->self,
+          taxisptr->used, taxisptr->type,
+          taxisptr->vdate, taxisptr->vtime,
+          taxisptr->rdate, taxisptr->rtime,
+          taxisptr->fdate, taxisptr->ftime,
+          taxisptr->calendar, taxisptr->unit,
+          taxisptr->numavg, taxisptr->climatology,
+          taxisptr->has_bounds,
+          vdate_lb, vtime_lb, vdate_ub, vtime_ub,
+          taxisptr->fc_unit, taxisptr->fc_period );
 }
 
 static int
 taxisCompareP(void *taxisptr1, void *taxisptr2)
 {
-  taxis_t * t1, * t2;
+  const taxis_t *t1 = ( const taxis_t * ) taxisptr1,
+    *t2 = ( const taxis_t * ) taxisptr2;
 
-  t1 = ( taxis_t * ) taxisptr1;
-  t2 = ( taxis_t * ) taxisptr2;
-
-  xassert ( t1 );
-  xassert ( t2 );
+  xassert ( t1 && t2 );
 
   return ! ( t1->used        == t2->used        &&
 	     t1->type        == t2->type        &&
@@ -1599,7 +1521,7 @@ taxisUnpack(char * unpackBuffer, int unpackBufferSize, int * unpackBufferPos,
 
   xassert(!force_id || targetID == taxisP->self);
 
-  taxisP->used        = intBuffer[idx++];
+  taxisP->used        = (short)intBuffer[idx++];
   taxisP->type        = intBuffer[idx++];
   taxisP->vdate       = intBuffer[idx++];
   taxisP->vtime       = intBuffer[idx++];
@@ -1612,7 +1534,7 @@ taxisUnpack(char * unpackBuffer, int unpackBufferSize, int * unpackBufferPos,
   taxisP->fc_unit     = intBuffer[idx++];
   taxisP->numavg      = intBuffer[idx++];
   taxisP->climatology = intBuffer[idx++];
-  taxisP->has_bounds  = intBuffer[idx++];
+  taxisP->has_bounds  = (short)intBuffer[idx++];
   taxisP->vdate_lb    = intBuffer[idx++];
   taxisP->vtime_lb    = intBuffer[idx++];
   taxisP->vdate_ub    = intBuffer[idx++];
@@ -1638,6 +1560,9 @@ taxisUnpack(char * unpackBuffer, int unpackBufferSize, int * unpackBufferPos,
       taxisP->longname = longname;
     }
 
+  reshSetStatus(taxisP->self, &taxisOps,
+                reshGetStatus(taxisP->self, &taxisOps) & ~RESH_SYNC_BIT);
+
   return taxisP->self;
 }
 
@@ -1670,8 +1595,8 @@ taxisPack(void * voidP, void * packBuffer, int packBufferSize, int * packBufferP
   intBuffer[idx++] = taxisP->vtime_lb;
   intBuffer[idx++] = taxisP->vdate_ub;
   intBuffer[idx++] = taxisP->vtime_ub;
-  intBuffer[idx++] = taxisP->name ? strlen(taxisP->name) : 0;
-  intBuffer[idx++] = taxisP->longname ? strlen(taxisP->longname) : 0;
+  intBuffer[idx++] = taxisP->name ? (int)strlen(taxisP->name) : 0;
+  intBuffer[idx++] = taxisP->longname ? (int)strlen(taxisP->longname) : 0;
 
   serializePack(intBuffer, taxisNint, DATATYPE_INT,
                 packBuffer, packBufferSize, packBufferPos, context);
