@@ -15,11 +15,12 @@
 #include "serialize.h"
 #include "error.h"
 
-extern
-#if !defined(__cplusplus)
-const
+#if  defined  (HAVE_LIBGRIB_API)
+#  include "file.h"
+
+#  include <grib_api.h>
 #endif
-resOps vlist_ops;
+
 
 static
 void vlistvarInitEntry(int vlistID, int varID)
@@ -40,6 +41,7 @@ void vlistvarInitEntry(int vlistID, int varID)
   vlistptr->vars[varID].xyz           = 321;
   vlistptr->vars[varID].gridID        = CDI_UNDEFID;
   vlistptr->vars[varID].zaxisID       = CDI_UNDEFID;
+  vlistptr->vars[varID].subtypeID     = CDI_UNDEFID;
   vlistptr->vars[varID].instID        = CDI_UNDEFID;
   vlistptr->vars[varID].modelID       = CDI_UNDEFID;
   vlistptr->vars[varID].tableID       = CDI_UNDEFID;
@@ -62,26 +64,13 @@ void vlistvarInitEntry(int vlistID, int varID)
   vlistptr->vars[varID].validrange[1] = VALIDMISS;
   vlistptr->vars[varID].ensdata       = NULL;
   vlistptr->vars[varID].iorank        = CDI_UNDEFID;
-
-#if  defined  (HAVE_LIBGRIB_API)
-  /* ---------------------------------- */
-  /* Local change: 2013-01-28, FP (DWD) */
-  /* ---------------------------------- */
-
-  vlistptr->vars[varID].opt_grib_dbl_nentries = 0;
-  vlistptr->vars[varID].opt_grib_int_nentries = 0;
-  int i;
-  for (i=0; i<MAX_OPT_GRIB_ENTRIES; i++) {
-    vlistptr->vars[varID].opt_grib_int_val[i] =   0;
-    vlistptr->vars[varID].opt_grib_dbl_val[i] = 0.0;
-    vlistptr->vars[varID].opt_grib_int_update[i] = FALSE;
-    vlistptr->vars[varID].opt_grib_dbl_update[i] = FALSE;
-    vlistptr->vars[varID].opt_grib_int_keyword[i] = NULL;
-    vlistptr->vars[varID].opt_grib_dbl_keyword[i] = NULL;
-  } // for
-#endif
+  vlistptr->vars[varID].opt_grib_kvpair_size = 0;
+  vlistptr->vars[varID].opt_grib_kvpair      = NULL;
+  vlistptr->vars[varID].opt_grib_nentries    = 0;
 }
 
+
+
 static
 int vlistvarNewEntry(int vlistID)
 {
@@ -99,7 +88,7 @@ int vlistvarNewEntry(int vlistID)
   if ( ! vlistvarSize )
     {
       vlistvarSize = 2;
-      vlistvar = (var_t *)xmalloc((size_t)vlistvarSize * sizeof (var_t));
+      vlistvar = (var_t *) Malloc((size_t)vlistvarSize * sizeof (var_t));
       for (int i = 0; i < vlistvarSize; i++ )
 	vlistvar[i].isUsed = FALSE;
     }
@@ -113,14 +102,9 @@ int vlistvarNewEntry(int vlistID)
   */
   if ( varID == vlistvarSize )
     {
-      int i;
-
-      vlistvarSize = 2*vlistvarSize;
-      vlistvar = (var_t *)xrealloc(vlistvar,
-                                   (size_t)vlistvarSize * sizeof(var_t));
-      varID = vlistvarSize/2;
-
-      for ( i = varID; i < vlistvarSize; i++ )
+      vlistvar = (var_t *) Realloc(vlistvar,
+                                   (size_t)(vlistvarSize *= 2) * sizeof(var_t));
+      for ( int i = varID; i < vlistvarSize; i++ )
 	vlistvar[i].isUsed = FALSE;
     }
 
@@ -134,7 +118,6 @@ int vlistvarNewEntry(int vlistID)
   return (varID);
 }
 
-static
 void vlistCheckVarID(const char *caller, int vlistID, int varID)
 {
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
@@ -149,6 +132,36 @@ void vlistCheckVarID(const char *caller, int vlistID, int varID)
     Errorc("varID %d undefined!", varID);
 }
 
+
+int vlistDefVarTiles(int vlistID, int gridID, int zaxisID, int tsteptype, int tilesetID)
+{
+  vlist_t *vlistptr = vlist_to_pointer(vlistID);
+  if ( CDI_Debug )
+    Message("gridID = %d  zaxisID = %d  tsteptype = %d", gridID, zaxisID, tsteptype);
+
+  int varID = vlistvarNewEntry(vlistID);
+
+  vlistptr->nvars++;
+  vlistptr->vars[varID].gridID    = gridID;
+  vlistptr->vars[varID].zaxisID   = zaxisID;
+  vlistptr->vars[varID].tsteptype = tsteptype;
+  vlistptr->vars[varID].subtypeID = tilesetID;
+
+  if ( tsteptype < 0 )
+    {
+      Message("Unexpected tstep type %d, set to TSTEP_INSTANT!", tsteptype);
+      vlistptr->vars[varID].tsteptype = TSTEP_INSTANT;
+    }
+
+  vlistAdd2GridIDs(vlistptr, gridID);
+  vlistAdd2ZaxisIDs(vlistptr, zaxisID);
+  vlistAdd2SubtypeIDs(vlistptr, tilesetID);
+
+  vlistptr->vars[varID].param = cdiEncodeParam(-(varID + 1), 255, 255);
+  reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
+  return (varID);
+}
+
 /*
 @Function  vlistDefVar
 @Title     Define a Variable
@@ -188,30 +201,8 @@ vlistDestroy(vlistID);
 */
 int vlistDefVar(int vlistID, int gridID, int zaxisID, int tsteptype)
 {
-  vlist_t *vlistptr = vlist_to_pointer(vlistID);
-  if ( CDI_Debug )
-    Message("gridID = %d  zaxisID = %d  tsteptype = %d", gridID, zaxisID, tsteptype);
-
-  int varID = vlistvarNewEntry(vlistID);
-
-  vlistptr->nvars++;
-
-  vlistptr->vars[varID].gridID  = gridID;
-  vlistptr->vars[varID].zaxisID = zaxisID;
-  vlistptr->vars[varID].tsteptype = tsteptype;
-
-  if ( tsteptype < 0 )
-    {
-      Message("Unexpected tstep type %d, set to TSTEP_INSTANT!", tsteptype);
-      vlistptr->vars[varID].tsteptype = TSTEP_INSTANT;
-    }
-
-  vlistAdd2GridIDs(vlistptr, gridID);
-  vlistAdd2ZaxisIDs(vlistptr, zaxisID);
-
-  vlistptr->vars[varID].param = cdiEncodeParam(-(varID + 1), 255, 255);
-  reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
-  return (varID);
+  /* call "vlistDefVarTiles" with a trivial tile index: */
+  return vlistDefVarTiles(vlistID, gridID, zaxisID, tsteptype, CDI_UNDEFID);
 }
 
 void
@@ -223,7 +214,7 @@ cdiVlistCreateVarLevInfo(vlist_t *vlistptr, int varID)
   size_t nlevs = (size_t)zaxisInqSize(zaxisID);
 
   vlistptr->vars[varID].levinfo
-    = (levinfo_t*)xmalloc((size_t)nlevs * sizeof(levinfo_t));
+    = (levinfo_t*) Malloc((size_t)nlevs * sizeof(levinfo_t));
 
   for (size_t levID = 0; levID < nlevs; levID++ )
     vlistptr->vars[varID].levinfo[levID] = DEFAULT_LEVINFO((int)levID);
@@ -253,7 +244,7 @@ void vlistDefVarParam(int vlistID, int varID, int param)
   if (vlistptr->vars[varID].param != param)
     {
       vlistptr->vars[varID].param = param;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -285,7 +276,7 @@ void vlistDefVarCode(int vlistID, int varID, int code)
   if (vlistptr->vars[varID].param != newParam)
     {
       vlistptr->vars[varID].param = newParam;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -355,6 +346,28 @@ int vlistInqVarZaxis(int vlistID, int varID)
   return (vlistptr->vars[varID].zaxisID);
 }
 
+
+/*
+@Function  vlistInqVarSubtype
+@Title     Get the Subtype ID of a Variable
+
+@Description
+The function @func{vlistInqVarSubtype} returns the subtype ID of a variable.
+
+@Result
+@func{vlistInqVarSubtype} returns the subtype ID of the variable.
+
+@EndFunction
+*/
+int vlistInqVarSubtype(int vlistID, int varID)
+{
+  vlist_t *vlistptr = vlist_to_pointer(vlistID);
+
+  vlistCheckVarID(__func__, vlistID, varID);
+  return (vlistptr->vars[varID].subtypeID);
+}
+
+
 /*
 @Function  vlistInqVarParam
 @Title     Get the parameter number of a Variable
@@ -407,7 +420,8 @@ int vlistInqVarCode(int vlistID, int varID)
   int param = vlistptr->vars[varID].param;
   int pdis, pcat, pnum;
   cdiDecodeParam(param, &pnum, &pcat, &pdis);
-  int code = pdis == 255 ? pnum : -varID-1;
+  int code = pnum;
+  if ( pdis != 255 ) code = -varID-1; // GRIB2 Parameter
 
   if ( code < 0 && vlistptr->vars[varID].tableID != -1 && vlistptr->vars[varID].name != NULL )
     {
@@ -502,11 +516,62 @@ void vlistInqVarName(int vlistID, int varID, char *name)
 	}
     }
   else
-    strcpy(name, vlistptr->vars[varID].name);
+    strcpy(name, vlistptr->vars[varID].name);   //FIXME: This may overrun the provided buffer.
 
   return;
 }
 
+/*
+@Function vlistCopyVarName
+@Tatle    Get the name of a Variable in a safe way
+
+@Prototype char* vlistCopyVarName(int vlistId, int varId)
+@Parameter
+    @Item  vlistID  Variable list ID, from a previous call to @fref{vlistCreate} or @fref{streamInqVlist}.
+    @Item  varID    Variable identifier.
+
+@Return A pointer to a malloc'ed string. Must be cleaned up with Free().
+
+@Description
+This is the buffer overflow immune version of vlistInqVarName().
+The memory for the returned string is allocated to fit the string via Malloc().
+
+@EndFunction
+*/
+char* vlistCopyVarName(int vlistId, int varId)
+{
+  vlist_t* vlistptr = vlist_to_pointer(vlistId);
+  vlistCheckVarID(__func__, vlistId, varId);
+
+  //If a name is set in the variable description, use that.
+  const char* name = vlistptr->vars[varId].name;
+  if(name) return strdup(name);
+
+  //Otherwise we check if we should use the table of parameter descriptions.
+  int param = vlistptr->vars[varId].param;
+  int discipline, category, number;
+  cdiDecodeParam(param, &number, &category, &discipline);
+  char *result = NULL;
+  if(discipline == 255)
+    {
+      int tableId = vlistptr->vars[varId].tableID;
+      if(( name = tableInqParNamePtr(tableId, number) ))
+        result = strdup(name);
+      {
+        //No luck, fall back to outputting a name of the format "var<num>".
+        result = (char *) Malloc(3 + 3 * sizeof (int) * CHAR_BIT / 8 + 2);
+        sprintf(result, "var%d", number);
+      }
+    }
+  else
+    {
+      result = (char *) Malloc(5 + 2 + 3 * (3 * sizeof (int) * CHAR_BIT + 1) + 1);
+      sprintf(result, "param%d.%d.%d", number, category, discipline);
+    }
+  //Finally, we fall back to outputting a name of the format "param<num>.<cat>.<dis>".
+  return result;
+}
+
 /*
 @Function  vlistInqVarLongname
 @Title     Get the longname of a Variable
@@ -572,7 +637,7 @@ The function @func{vlistInqVarStdname} returns the standard name of a variable i
 otherwise the result is an empty string.
 
 @Result
-@func{vlistInqVarName} returns the standard name of the variable to the parameter stdname.
+@func{vlistInqVarStdname} returns the standard name of the variable to the parameter stdname.
 
 @EndFunction
 */
@@ -756,7 +821,7 @@ void vlistDefVarDatatype(int vlistID, int varID, int datatype)
           case DATATYPE_INT32:  vlistptr->vars[varID].missval = -INT_MAX;   break;
           case DATATYPE_UINT32: vlistptr->vars[varID].missval =  UINT_MAX;  break;
           }
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -767,7 +832,7 @@ void vlistDefVarInstitut(int vlistID, int varID, int instID)
   if (vlistptr->vars[varID].instID != instID)
     {
       vlistptr->vars[varID].instID = instID;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -785,7 +850,7 @@ void vlistDefVarModel(int vlistID, int varID, int modelID)
   if (vlistptr->vars[varID].modelID != modelID)
     {
       vlistptr->vars[varID].modelID = modelID;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -811,7 +876,7 @@ void vlistDefVarTable(int vlistID, int varID, int tableID)
       int pnum, pcat, pdis;
       cdiDecodeParam(param, &pnum, &pcat, &pdis);
       vlistptr->vars[varID].param = cdiEncodeParam(pnum, tablenum, pdis);
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -847,12 +912,12 @@ void vlistDefVarName(int vlistID, int varID, const char *name)
     {
       if ( vlistptr->vars[varID].name )
 	{
-	  free(vlistptr->vars[varID].name);
+	  Free(vlistptr->vars[varID].name);
 	  vlistptr->vars[varID].name = NULL;
 	}
 
       vlistptr->vars[varID].name = strdupx(name);
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -881,12 +946,12 @@ void vlistDefVarLongname(int vlistID, int varID, const char *longname)
     {
       if ( vlistptr->vars[varID].longname )
 	{
-	  free(vlistptr->vars[varID].longname);
+	  Free(vlistptr->vars[varID].longname);
 	  vlistptr->vars[varID].longname = 0;
 	}
 
       vlistptr->vars[varID].longname = strdupx(longname);
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -915,12 +980,12 @@ void vlistDefVarStdname(int vlistID, int varID, const char *stdname)
     {
       if ( vlistptr->vars[varID].stdname )
 	{
-	  free(vlistptr->vars[varID].stdname);
+	  Free(vlistptr->vars[varID].stdname);
 	  vlistptr->vars[varID].stdname = 0;
 	}
 
       vlistptr->vars[varID].stdname = strdupx(stdname);
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -949,12 +1014,12 @@ void vlistDefVarUnits(int vlistID, int varID, const char *units)
     {
       if ( vlistptr->vars[varID].units )
 	{
-	  free(vlistptr->vars[varID].units);
+	  Free(vlistptr->vars[varID].units);
 	  vlistptr->vars[varID].units = 0;
 	}
 
       vlistptr->vars[varID].units = strdupx(units);
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1034,12 +1099,12 @@ void vlistDefVarExtra(int vlistID, int varID, const char *extra)
     {
       if ( vlistptr->vars[varID].extra )
 	{
-	  free(vlistptr->vars[varID].extra);
+	  Free(vlistptr->vars[varID].extra);
 	  vlistptr->vars[varID].extra = NULL;
 	}
 
       vlistptr->vars[varID].extra = strdupx(extra);
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1104,7 +1169,7 @@ void vlistDefVarValidrange(int vlistID, int varID, const double *validrange)
   vlistptr->vars[varID].validrange[0] = validrange[0];
   vlistptr->vars[varID].validrange[1] = validrange[1];
   vlistptr->vars[varID].lvalidrange = TRUE;
-  reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+  reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
 }
 
 
@@ -1137,7 +1202,7 @@ void vlistDefVarScalefactor(int vlistID, int varID, double scalefactor)
   if ( IS_NOT_EQUAL(vlistptr->vars[varID].scalefactor, scalefactor) )
     {
       vlistptr->vars[varID].scalefactor = scalefactor;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1151,7 +1216,7 @@ void vlistDefVarAddoffset(int vlistID, int varID, double addoffset)
   if ( IS_NOT_EQUAL(vlistptr->vars[varID].addoffset, addoffset))
     {
       vlistptr->vars[varID].addoffset = addoffset;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1162,7 +1227,7 @@ void vlistDefVarTsteptype(int vlistID, int varID, int tsteptype)
   if (vlistptr->vars[varID].tsteptype != tsteptype)
     {
       vlistptr->vars[varID].tsteptype = tsteptype;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1180,7 +1245,7 @@ void vlistDefVarTimave(int vlistID, int varID, int timave)
   if (vlistptr->vars[varID].timave != timave)
     {
       vlistptr->vars[varID].timave = timave;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1198,7 +1263,7 @@ void vlistDefVarTimaccu(int vlistID, int varID, int timaccu)
   if (vlistptr->vars[varID].timaccu != timaccu)
     {
       vlistptr->vars[varID].timaccu = timaccu;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1216,7 +1281,7 @@ void vlistDefVarTypeOfGeneratingProcess(int vlistID, int varID, int typeOfGenera
   if (vlistptr->vars[varID].typeOfGeneratingProcess != typeOfGeneratingProcess)
     {
       vlistptr->vars[varID].typeOfGeneratingProcess = typeOfGeneratingProcess;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1235,7 +1300,7 @@ void vlistDefVarProductDefinitionTemplate(int vlistID, int varID, int productDef
   if (vlistptr->vars[varID].productDefinitionTemplate != productDefinitionTemplate)
     {
       vlistptr->vars[varID].productDefinitionTemplate = productDefinitionTemplate;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1253,9 +1318,9 @@ void vlistDestroyVarName(int vlistID, int varID)
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
   if ( vlistptr->vars[varID].name )
     {
-      free(vlistptr->vars[varID].name);
+      Free(vlistptr->vars[varID].name);
       vlistptr->vars[varID].name = NULL;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1266,9 +1331,9 @@ void vlistDestroyVarLongname(int vlistID, int varID)
 
   if ( vlistptr->vars[varID].longname )
     {
-      free(vlistptr->vars[varID].longname);
+      Free(vlistptr->vars[varID].longname);
       vlistptr->vars[varID].longname = NULL;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1279,9 +1344,9 @@ void vlistDestroyVarStdname(int vlistID, int varID)
 
   if ( vlistptr->vars[varID].stdname )
     {
-      free(vlistptr->vars[varID].stdname);
+      Free(vlistptr->vars[varID].stdname);
       vlistptr->vars[varID].stdname = NULL;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1292,9 +1357,9 @@ void vlistDestroyVarUnits(int vlistID, int varID)
 
   if ( vlistptr->vars[varID].units )
     {
-      free(vlistptr->vars[varID].units);
+      Free(vlistptr->vars[varID].units);
       vlistptr->vars[varID].units = NULL;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1332,7 +1397,7 @@ void vlistDefFlag(int vlistID, int varID, int levID, int flag)
         }
     }
 
-  reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+  reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
 }
 
 
@@ -1430,7 +1495,7 @@ void vlistDefIndex(int vlistID, int varID, int levelID, int index)
   else
     return;
   vlistptr->vars[varID].levinfo[levelID].index = index;
-  reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+  reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
 }
 
 
@@ -1478,7 +1543,7 @@ void vlistChangeVarZaxis(int vlistID, int varID, int zaxisID)
     vlistAdd2ZaxisIDs(vlistptr, zaxisID);
 
   vlistptr->vars[varID].zaxisID = zaxisID;
-  reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+  reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
 }
 
 
@@ -1504,7 +1569,7 @@ void vlistChangeVarGrid(int vlistID, int varID, int gridID)
     vlistAdd2GridIDs(vlistptr, gridID);
 
   vlistptr->vars[varID].gridID = gridID;
-  reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+  reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
 }
 
 
@@ -1517,7 +1582,7 @@ void vlistDefVarCompType(int vlistID, int varID, int comptype)
   if (vlistptr->vars[varID].comptype != comptype)
     {
       vlistptr->vars[varID].comptype = comptype;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1541,7 +1606,7 @@ void vlistDefVarCompLevel(int vlistID, int varID, int complevel)
   if (vlistptr->vars[varID].complevel != complevel)
     {
       vlistptr->vars[varID].complevel = complevel;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1565,7 +1630,7 @@ void  vlistDefVarChunkType(int vlistID, int varID, int chunktype)
   if (vlistptr->vars[varID].chunktype != chunktype)
     {
       vlistptr->vars[varID].chunktype = chunktype;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1631,7 +1696,7 @@ void  vlistDefVarXYZ(int vlistID, int varID, int xyz)
   assert(xyz == 123 || xyz == 312 || xyz == 231 || xyz == 321 || xyz == 132 || xyz == 213);
 
   vlistptr->vars[varID].xyz = xyz;
-  reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+  reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
 }
 
 
@@ -1665,12 +1730,12 @@ void vlistDefVarEnsemble(int vlistID, int varID, int ensID, int ensCount, int fo
 
   if ( vlistptr->vars[varID].ensdata == NULL )
     vlistptr->vars[varID].ensdata
-      = (ensinfo_t *)xmalloc( sizeof( ensinfo_t ) );
+      = (ensinfo_t *) Malloc( sizeof( ensinfo_t ) );
 
   vlistptr->vars[varID].ensdata->ens_index          = ensID;
   vlistptr->vars[varID].ensdata->ens_count          = ensCount;
   vlistptr->vars[varID].ensdata->forecast_init_type = forecast_type;
-  reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+  reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
 }
 
 
@@ -1693,43 +1758,68 @@ int vlistInqVarEnsemble( int vlistID, int varID, int *ensID, int *ensCount, int
   return (status);
 }
 
-/* ---------------------------------- */
-/* Local change: 2013-01-28, FP (DWD) */
-/* ---------------------------------- */
+
 
 /* vlistDefVarIntKey: Set an arbitrary keyword/integer value pair for GRIB API */
 void vlistDefVarIntKey(int vlistID, int varID, const char *name, int value)
 {
 #if  defined  (HAVE_LIBGRIB_API)
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
+  if (vlistptr == NULL)  Error("Internal error!");
   int idx;
 
-  if ( !vlistptr->locked )
-    Error("User defined vlist object (vlistID=%d) isn't allowed!\n"
-          "Need a CDI internal vlist object from streamInqVlist(streamID).", vlistID);
+  if ( vlistptr->immutable )
+    Error("vlistDefVarIntKey() was called on an immutable vlist object (vlistID = %d)\n"
+          "Either call vlistDefVarIntKey() before passing the vlist object to streamDefVlist(),\n"
+          "or use the stream-internal vlist by calling streamInqVlist().", vlistID);
 
-  for ( idx=0; idx<vlistptr->vars[varID].opt_grib_int_nentries; idx++)
-    if ( strcmp(name, vlistptr->vars[varID].opt_grib_int_keyword[idx]) == 0 ) break;
+  for ( idx=0; idx<vlistptr->vars[varID].opt_grib_nentries; idx++)
+    if ( (strcmp(name, vlistptr->vars[varID].opt_grib_kvpair[idx].keyword) == 0 ) &&
+         (vlistptr->vars[varID].opt_grib_kvpair[idx].data_type == t_int) )  break;
 
-  if ( idx < vlistptr->vars[varID].opt_grib_int_nentries )
+  if ( idx < vlistptr->vars[varID].opt_grib_nentries )
     {
-      vlistptr->vars[varID].opt_grib_int_val[idx] = value;
-      vlistptr->vars[varID].opt_grib_int_update[idx] = TRUE;
+      vlistptr->vars[varID].opt_grib_kvpair[idx].int_val = value;
+      vlistptr->vars[varID].opt_grib_kvpair[idx].update  = TRUE;
     }
   else
     {
-      idx = vlistptr->vars[varID].opt_grib_int_nentries;
-      vlistptr->vars[varID].opt_grib_int_nentries++;
-      if ( idx >= MAX_OPT_GRIB_ENTRIES ) Error("Too many optional keyword/integer value pairs!");
-      vlistptr->vars[varID].opt_grib_int_val[idx] = value;
-      vlistptr->vars[varID].opt_grib_int_update[idx] = TRUE;
+      resize_opt_grib_entries(&vlistptr->vars[varID], vlistptr->vars[varID].opt_grib_nentries+1);
+      vlistptr->vars[varID].opt_grib_nentries += 1;
+      idx = vlistptr->vars[varID].opt_grib_nentries -1;
+      vlistptr->vars[varID].opt_grib_kvpair[idx].data_type   = t_int;
+      vlistptr->vars[varID].opt_grib_kvpair[idx].int_val     = value;
+      vlistptr->vars[varID].opt_grib_kvpair[idx].update      = TRUE;
       if ( name )
-        vlistptr->vars[varID].opt_grib_int_keyword[idx] = strdupx(name);
+        vlistptr->vars[varID].opt_grib_kvpair[idx].keyword = strdupx(name);
       else
         Error("Internal error, name undefined!");
     }
 
-  reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+  if ( CDI_Debug )
+    {
+      Message("define additional GRIB2 key \"%s\" (integer): %d", name, value);
+      Message("total list of registered, additional GRIB2 keys (total: %d):",
+              vlistptr->vars[varID].opt_grib_nentries);
+      for ( idx=0; idx<vlistptr->vars[varID].opt_grib_nentries; idx++)
+        if (vlistptr->vars[varID].opt_grib_kvpair[idx].data_type == t_int)
+          Message("%s -> integer %d",
+                  vlistptr->vars[varID].opt_grib_kvpair[idx].keyword,
+                  vlistptr->vars[varID].opt_grib_kvpair[idx].int_val);
+        else if (vlistptr->vars[varID].opt_grib_kvpair[idx].data_type == t_double)
+          Message("%s -> double %d",
+                  vlistptr->vars[varID].opt_grib_kvpair[idx].keyword,
+                  vlistptr->vars[varID].opt_grib_kvpair[idx].dbl_val);
+        else
+          Message("%s -> unknown", vlistptr->vars[varID].opt_grib_kvpair[idx].keyword);
+    }
+
+  reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
+#else
+  (void)vlistID;
+  (void)varID;
+  (void)name;
+  (void)value;
 #endif
 }
 
@@ -1738,48 +1828,70 @@ void vlistDefVarDblKey(int vlistID, int varID, const char *name, double value)
 {
 #if  defined  (HAVE_LIBGRIB_API)
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
+  if (vlistptr == NULL)  Error("Internal error!");
   int idx;
 
-  if ( !vlistptr->locked )
-    Error("User defined vlist object (vlistID=%d) isn't allowed!\n"
-          "Need a CDI internal vlist object from streamInqVlist(streamID).", vlistID);
+  if ( vlistptr->immutable )
+    Error("vlistDefVarDblKey() was called on an immutable vlist object (vlistID = %d)\n"
+          "Either call vlistDefVarIntKey() before passing the vlist object to streamDefVlist(),\n"
+          "or use the stream-internal vlist by calling streamInqVlist().", vlistID);
 
-  for ( idx=0; idx<vlistptr->vars[varID].opt_grib_dbl_nentries; idx++)
-    if ( strcmp(name, vlistptr->vars[varID].opt_grib_dbl_keyword[idx]) == 0 ) break;
+  for ( idx=0; idx<vlistptr->vars[varID].opt_grib_nentries; idx++)
+    if ( (strcmp(name, vlistptr->vars[varID].opt_grib_kvpair[idx].keyword) == 0 ) &&
+         (vlistptr->vars[varID].opt_grib_kvpair[idx].data_type == t_double) )  break;
 
-  if ( idx < vlistptr->vars[varID].opt_grib_dbl_nentries )
+  if ( idx < vlistptr->vars[varID].opt_grib_nentries )
     {
-      vlistptr->vars[varID].opt_grib_dbl_val[idx] = value;
-      vlistptr->vars[varID].opt_grib_dbl_update[idx] = TRUE;
+      vlistptr->vars[varID].opt_grib_kvpair[idx].dbl_val = value;
+      vlistptr->vars[varID].opt_grib_kvpair[idx].update  = TRUE;
     }
   else
     {
-      idx = vlistptr->vars[varID].opt_grib_dbl_nentries;
-      vlistptr->vars[varID].opt_grib_dbl_nentries++;
-      if ( idx >= MAX_OPT_GRIB_ENTRIES ) Error("Too many optional keyword/double value pairs!");
-      vlistptr->vars[varID].opt_grib_dbl_val[idx] = value;
-      vlistptr->vars[varID].opt_grib_dbl_update[idx] = TRUE;
+      resize_opt_grib_entries(&vlistptr->vars[varID], vlistptr->vars[varID].opt_grib_nentries+1);
+      vlistptr->vars[varID].opt_grib_nentries += 1;
+      idx = vlistptr->vars[varID].opt_grib_nentries - 1;
+      vlistptr->vars[varID].opt_grib_kvpair[idx].data_type = t_double;
+      vlistptr->vars[varID].opt_grib_kvpair[idx].dbl_val   = value;
+      vlistptr->vars[varID].opt_grib_kvpair[idx].update    = TRUE;
       if ( name )
-        vlistptr->vars[varID].opt_grib_dbl_keyword[idx] = strdupx(name);
+        vlistptr->vars[varID].opt_grib_kvpair[idx].keyword = strdupx(name);
       else
         Error("Internal error, name undefined!");
     }
 
-  reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+  if ( CDI_Debug )
+    {
+      Message("define additional GRIB2 key \"%s\" (double): %d", name, value);
+      Message("total list of registered, additional GRIB2 keys (total: %d):",
+              vlistptr->vars[varID].opt_grib_nentries);
+      for ( idx=0; idx<vlistptr->vars[varID].opt_grib_nentries; idx++)
+        if (vlistptr->vars[varID].opt_grib_kvpair[idx].data_type == t_int)
+          Message("%s -> integer %d",
+                  vlistptr->vars[varID].opt_grib_kvpair[idx].keyword,
+                  vlistptr->vars[varID].opt_grib_kvpair[idx].int_val);
+        else if (vlistptr->vars[varID].opt_grib_kvpair[idx].data_type == t_double)
+          Message("%s -> double %d",
+                  vlistptr->vars[varID].opt_grib_kvpair[idx].keyword,
+                  vlistptr->vars[varID].opt_grib_kvpair[idx].dbl_val);
+        else
+          Message("%s -> unknown", vlistptr->vars[varID].opt_grib_kvpair[idx].keyword);
+    }
+
+  reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
+#else
+  (void)vlistID;
+  (void)varID;
+  (void)name;
+  (void)value;
 #endif
 }
 
-#if  defined  (HAVE_LIBGRIB_API)
-#  include "file.h"
-#  include "grib_api.h"
-#endif
-
 
 /* cdiClearAdditionalKeys: Clears the list of additional GRIB keys. */
 void cdiClearAdditionalKeys()
 {
 #if  defined  (HAVE_LIBGRIB_API)
-  for (int i=0; i<cdiNAdditionalGRIBKeys; i++)  free(cdiAdditionalGRIBKeys[i]);
+  for (int i=0; i<cdiNAdditionalGRIBKeys; i++)  Free(cdiAdditionalGRIBKeys[i]);
   cdiNAdditionalGRIBKeys = 0;
 #endif
 }
@@ -1795,6 +1907,8 @@ void cdiDefAdditionalKey(const char *name)
     cdiAdditionalGRIBKeys[idx] = strdupx(name);
   else
     Error("Internal error!");
+#else
+  (void)name;
 #endif
 }
 
@@ -1805,17 +1919,15 @@ int vlistHasVarKey(int vlistID, int varID, const char* name)
   /* check if the GRIB key was previously read and is stored */
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
 
-  for (int i=0; i<vlistptr->vars[varID].opt_grib_dbl_nentries; i++)
+  for (int i=0; i<vlistptr->vars[varID].opt_grib_nentries; i++)
     {
-      if ( strcmp(name, vlistptr->vars[varID].opt_grib_dbl_keyword[i]) == 0 )
-	return 1;
-    }
-
-  for (int i=0; i<vlistptr->vars[varID].opt_grib_int_nentries; i++)
-    {
-      if ( strcmp(name, vlistptr->vars[varID].opt_grib_int_keyword[i]) == 0 )
+      if ( strcmp(name, vlistptr->vars[varID].opt_grib_kvpair[i].keyword) == 0 )
 	return 1;
     }
+#else
+  (void)vlistID;
+  (void)varID;
+  (void)name;
 #endif
   return 0;
 }
@@ -1828,9 +1940,18 @@ double vlistInqVarDblKey(int vlistID, int varID, const char* name)
   /* check if the GRIB key was previously read and is stored in "opt_grib_dbl_val" */
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
 
-  for (int i=0; i<vlistptr->vars[varID].opt_grib_dbl_nentries; i++)
-    if ( strcmp(name, vlistptr->vars[varID].opt_grib_dbl_keyword[i]) == 0 )
-      return vlistptr->vars[varID].opt_grib_dbl_val[i];
+  for (int i=0; i<vlistptr->vars[varID].opt_grib_nentries; i++)
+    {
+      int isub = subtypeInqActiveIndex(vlistptr->vars[varID].subtypeID);
+      if ( (strcmp(name, vlistptr->vars[varID].opt_grib_kvpair[i].keyword) == 0 ) &&
+           (vlistptr->vars[varID].opt_grib_kvpair[i].data_type == t_double)       &&
+           (vlistptr->vars[varID].opt_grib_kvpair[i].subtype_index == isub) )
+        return vlistptr->vars[varID].opt_grib_kvpair[i].dbl_val;
+    }
+#else
+  (void)vlistID;
+  (void)varID;
+  (void)name;
 #endif
   return value;
 }
@@ -1844,9 +1965,19 @@ int vlistInqVarIntKey(int vlistID, int varID, const char* name)
   /* check if the GRIB key was previously read and is stored in "opt_grib_int_val" */
   vlist_t *vlistptr = vlist_to_pointer(vlistID);
 
-  for (int i=0; i<vlistptr->vars[varID].opt_grib_int_nentries; i++)
-    if ( strcmp(name, vlistptr->vars[varID].opt_grib_int_keyword[i]) == 0 )
-      return vlistptr->vars[varID].opt_grib_int_val[i];
+  for (int i=0; i<vlistptr->vars[varID].opt_grib_nentries; i++)
+    {
+      int isub = subtypeInqActiveIndex(vlistptr->vars[varID].subtypeID);
+      if ( (strcmp(name, vlistptr->vars[varID].opt_grib_kvpair[i].keyword) == 0 ) &&
+           (vlistptr->vars[varID].opt_grib_kvpair[i].data_type == t_int)          &&
+           (vlistptr->vars[varID].opt_grib_kvpair[i].subtype_index == isub) )
+        return vlistptr->vars[varID].opt_grib_kvpair[i].int_val;
+    }
+
+#else
+  (void)vlistID;
+  (void)varID;
+  (void)name;
 #endif
   return (int) value;
 }
@@ -1861,7 +1992,7 @@ void vlistDefVarIOrank(int vlistID, int varID, int iorank)
   if (vlistptr->vars[varID].iorank != iorank)
     {
       vlistptr->vars[varID].iorank = iorank;
-      reshSetStatus(vlistID, &vlist_ops, RESH_DESYNC_IN_USE);
+      reshSetStatus(vlistID, &vlistOps, RESH_DESYNC_IN_USE);
     }
 }
 
@@ -1897,6 +2028,7 @@ int vlistVarCompare(vlist_t *a, int varIDA, vlist_t *b, int varIDB)
     | FCMPFLT(validrange[0]) | FCMPFLT(validrange[1]);
 #undef FCMP
 #undef FCMPFLT
+#undef FCMPSTR
 #undef FCMP2
   if ((diff |= ((pva->levinfo == NULL) ^ (pvb->levinfo == NULL))))
     return 1;
@@ -1924,7 +2056,7 @@ int vlistVarCompare(vlist_t *a, int varIDA, vlist_t *b, int varIDB)
 
 
 enum {
-  vlistvar_nints = 20,
+  vlistvar_nints = 21,
   vlistvar_ndbls = 3,
 };
 
@@ -1941,6 +2073,8 @@ int vlistVarGetPackSize(vlist_t *p, int varID, void *context)
     varsize += serializeGetSize((int)strlen(var->stdname), DATATYPE_TXT, context);
   if (var->units)
     varsize += serializeGetSize((int)strlen(var->units), DATATYPE_TXT, context);
+  if (var->extra)
+    varsize += serializeGetSize((int)strlen(var->extra), DATATYPE_TXT, context);
   varsize += serializeGetSize(4 * zaxisInqSize(var->zaxisID),
                               DATATYPE_INT, context);
   varsize += vlistAttsGetSize(p, varID, context);
@@ -1952,7 +2086,8 @@ void vlistVarPack(vlist_t *p, int varID, char * buf, int size, int *position,
 {
   double dtempbuf[vlistvar_ndbls];
   var_t *var = p->vars + varID;
-  int tempbuf[vlistvar_nints], namesz, longnamesz, stdnamesz, unitssz;
+  int tempbuf[vlistvar_nints], namesz, longnamesz, stdnamesz, unitssz,
+    extralen;
 
   tempbuf[0] = var->flag;
   tempbuf[1] = var->gridID;
@@ -1975,6 +2110,7 @@ void vlistVarPack(vlist_t *p, int varID, char * buf, int size, int *position,
   int nlevs = var->levinfo ? zaxisInqSize(var->zaxisID) : 0;
   tempbuf[18] = nlevs;
   tempbuf[19] = var->iorank;
+  tempbuf[20] = extralen = var->extra?(int)strlen(var->extra):0;
   dtempbuf[0] = var->missval;
   dtempbuf[1] = var->scalefactor;
   dtempbuf[2] = var->addoffset;
@@ -1993,6 +2129,9 @@ void vlistVarPack(vlist_t *p, int varID, char * buf, int size, int *position,
   if (unitssz)
     serializePack(var->units, unitssz, DATATYPE_TXT,
                   buf, size, position, context);
+  if (extralen)
+    serializePack(var->extra, extralen, DATATYPE_TXT,
+                  buf, size, position, context);
   if (nlevs)
     {
       int levbuf[nlevs][4];
@@ -2028,14 +2167,19 @@ void vlistVarUnpack(int vlistID, char * buf, int size, int *position,
   serializeUnpack(buf, size, position,
                   dtempbuf, vlistvar_ndbls, DATATYPE_FLT64, context);
 
+  /* ------------------------------------------- */
+  /* NOTE: Tile sets  currently not supported!!! */
+  /* ------------------------------------------- */
+
   int newvar = vlistDefVar ( vlistID,
 			 namespaceAdaptKey ( tempbuf[1], originNamespace ),
 			 namespaceAdaptKey ( tempbuf[2], originNamespace ),
 			 tempbuf[3]);
-  if (tempbuf[4] || tempbuf[5] || tempbuf[6] || tempbuf[7])
-    varname = (char *)xmalloc((size_t)imax(imax(imax(tempbuf[4],tempbuf[5]),
-                                                tempbuf[6]),
-                                           tempbuf[7]) + 1);
+  if (tempbuf[4] || tempbuf[5] || tempbuf[6] || tempbuf[7] || tempbuf[20])
+    varname = (char *)Malloc((size_t)imax(imax(imax(imax(tempbuf[4],
+                                                         tempbuf[5]),
+                                                    tempbuf[6]),
+                                               tempbuf[7]), tempbuf[20]) + 1);
   if (tempbuf[4])
   {
     serializeUnpack(buf, size, position,
@@ -2064,7 +2208,14 @@ void vlistVarUnpack(int vlistID, char * buf, int size, int *position,
     varname[tempbuf[7]] = '\0';
     vlistDefVarUnits(vlistID, newvar, varname);
   }
-  free(varname);
+  if (tempbuf[20])
+    {
+      serializeUnpack(buf, size, position,
+                      varname, tempbuf[20], DATATYPE_TXT, context);
+      varname[tempbuf[20]] = '\0';
+      vlistDefVarExtra(vlistID, newvar, varname);
+    }
+  Free(varname);
   vlistDefVarDatatype(vlistID, newvar, tempbuf[8]);
   vlistDefVarInstitut ( vlistID, newvar,
 			namespaceAdaptKey ( tempbuf[10], originNamespace ));
