@@ -11,11 +11,13 @@
 #include "cdi.h"
 #include "cdi_cksum.h"
 #include "cdi_int.h"
+#include "cdi_uuid.h"
 #include "resource_handle.h"
 #include "resource_unpack.h"
 #include "varscan.h"
 #include "namespace.h"
 #include "serialize.h"
+#include "zaxis.h"
 
 #define  LevelUp    1
 #define  LevelDown  2
@@ -23,10 +25,10 @@
 
 static const struct {
   unsigned char positive;   // 1: up;  2: down
-  char *name;
-  char *longname;
-  char *stdname;
-  char *units;
+  const char *name;
+  const char *longname;
+  const char *stdname;
+  const char *units;
 }
 ZaxistypeEntry[] = {
   { /*  0 */ 0, "sfc",               "surface",                "",               ""},
@@ -54,7 +56,7 @@ ZaxistypeEntry[] = {
   { /* 22 */ 0, "sedimentbottomta",  "sediment_bottom_ta",     "",               ""},
   { /* 23 */ 0, "sedimentbottomtw",  "sediment_bottom_tw",     "",               ""},
   { /* 24 */ 0, "mixlayer",          "mix_layer",              "",               ""},
-  { /* 25 */ 0, "height",            "generalized height",     "height",         ""},
+  { /* 25 */ 0, "height",            "generalized_height",     "height",         ""},
 };
 
 enum {
@@ -68,12 +70,14 @@ typedef struct {
   char     longname[CDI_MAX_NAME];
   char     stdname[CDI_MAX_NAME];
   char     units[CDI_MAX_NAME];
+  char     psname[CDI_MAX_NAME];
   double  *vals;
   double  *lbounds;
   double  *ubounds;
   double  *weights;
   int      self;
   int      prec;
+  int      scalar;
   int      type;
   int      ltype;    /* GRIB level type */
   int      ltype2;
@@ -87,7 +91,7 @@ typedef struct {
 }
 zaxis_t;
 
-static int zaxisCompareP(zaxis_t *z1, zaxis_t *z2);
+static int    zaxisCompareP    (zaxis_t *z1, zaxis_t *z2);
 static void   zaxisDestroyP    ( void * zaxisptr );
 static void   zaxisPrintP      ( void * zaxisptr, FILE * fp );
 static int    zaxisGetPackSize ( void * zaxisptr, void *context);
@@ -103,8 +107,33 @@ static const resOps zaxisOps = {
   zaxisTxCode
 };
 
+const resOps *getZaxisOps(void)
+{
+  return &zaxisOps;
+}
+
 static int  ZAXIS_Debug = 0;   /* If set to 1, debugging */
 
+void zaxisGetTypeDescription(int zaxisType, int* outPositive, const char** outName, const char** outLongName, const char** outStdName, const char** outUnit)
+{
+  if(zaxisType < 0 || zaxisType >= CDI_NumZaxistype)
+    {
+      if(outPositive) *outPositive = 0;
+      if(outName) *outName = NULL;
+      if(outLongName) *outLongName = NULL;
+      if(outStdName) *outStdName = NULL;
+      if(outUnit) *outUnit = NULL;
+    }
+  else
+    {
+      if(outPositive) *outPositive = ZaxistypeEntry[zaxisType].positive;
+      if(outName) *outName = ZaxistypeEntry[zaxisType].name;
+      if(outLongName) *outLongName = ZaxistypeEntry[zaxisType].longname;
+      if(outStdName) *outStdName = ZaxistypeEntry[zaxisType].stdname;
+      if(outUnit) *outUnit = ZaxistypeEntry[zaxisType].units;
+    }
+}
+
 static
 void zaxisDefaultValue(zaxis_t *zaxisptr)
 {
@@ -113,6 +142,7 @@ void zaxisDefaultValue(zaxis_t *zaxisptr)
   zaxisptr->longname[0] = 0;
   zaxisptr->stdname[0]  = 0;
   zaxisptr->units[0]    = 0;
+  zaxisptr->psname[0]   = 0;
   zaxisptr->vals        = NULL;
   zaxisptr->ubounds     = NULL;
   zaxisptr->lbounds     = NULL;
@@ -121,6 +151,7 @@ void zaxisDefaultValue(zaxis_t *zaxisptr)
   zaxisptr->ltype       = 0;
   zaxisptr->ltype2      = -1;
   zaxisptr->positive    = 0;
+  zaxisptr->scalar      = 0;
   zaxisptr->direction   = 0;
   zaxisptr->prec        = 0;
   zaxisptr->size        = 0;
@@ -135,7 +166,7 @@ void zaxisDefaultValue(zaxis_t *zaxisptr)
 static
 zaxis_t *zaxisNewEntry(int id)
 {
-  zaxis_t *zaxisptr = (zaxis_t *)xmalloc(sizeof(zaxis_t));
+  zaxis_t *zaxisptr = (zaxis_t *) Malloc(sizeof(zaxis_t));
 
   zaxisDefaultValue ( zaxisptr );
 
@@ -150,6 +181,13 @@ zaxis_t *zaxisNewEntry(int id)
   return (zaxisptr);
 }
 
+static inline zaxis_t *
+zaxisID2Ptr(int id)
+{
+  return (zaxis_t *)reshGetVal(id, &zaxisOps);
+}
+
+
 static
 void zaxisInit(void)
 {
@@ -172,19 +210,11 @@ void zaxis_copy(zaxis_t *zaxisptr2, zaxis_t *zaxisptr1)
   zaxisptr2->self = zaxisID2;
 }
 
-static
-void zaxisCheckPtr(const char *caller, int zaxisID, zaxis_t *zaxisptr)
+unsigned cdiZaxisCount(void)
 {
-  if ( zaxisptr == NULL )
-    Errorc("zaxis %d undefined!", zaxisID);
+  return reshCountType(&zaxisOps);
 }
 
-#define  zaxis_check_ptr(zaxisID, zaxisptr)  zaxisCheckPtr(__func__, zaxisID, zaxisptr)
-
-int zaxisSize(void)
-{
-  return reshCountType ( &zaxisOps );
-}
 
 static int
 zaxisCreate_(int zaxistype, int size, int id)
@@ -209,7 +239,7 @@ zaxisCreate_(int zaxistype, int size, int id)
   zaxisptr->positive = ZaxistypeEntry[zaxistype].positive;
 
   double *vals = zaxisptr->vals
-    = (double *)xmalloc((size_t)size * sizeof(double));
+    = (double *) Malloc((size_t)size * sizeof(double));
 
   for ( int ilev = 0; ilev < size; ilev++ )
     vals[ilev] = 0.0;
@@ -270,19 +300,17 @@ int zaxisCreate(int zaxistype, int size)
 
 static void zaxisDestroyKernel( zaxis_t * zaxisptr )
 {
-  int id;
-
   xassert ( zaxisptr );
 
-  id = zaxisptr->self;
+  int id = zaxisptr->self;
 
-  if ( zaxisptr->vals )    free ( zaxisptr->vals );
-  if ( zaxisptr->lbounds ) free ( zaxisptr->lbounds );
-  if ( zaxisptr->ubounds ) free ( zaxisptr->ubounds );
-  if ( zaxisptr->weights ) free ( zaxisptr->weights );
-  if ( zaxisptr->vct )     free ( zaxisptr->vct );
+  if ( zaxisptr->vals )    Free( zaxisptr->vals );
+  if ( zaxisptr->lbounds ) Free( zaxisptr->lbounds );
+  if ( zaxisptr->ubounds ) Free( zaxisptr->ubounds );
+  if ( zaxisptr->weights ) Free( zaxisptr->weights );
+  if ( zaxisptr->vct )     Free( zaxisptr->vct );
 
-  free ( zaxisptr );
+  Free( zaxisptr );
 
   reshRemove ( id, &zaxisOps );
 }
@@ -299,7 +327,7 @@ static void zaxisDestroyKernel( zaxis_t * zaxisptr )
 */
 void zaxisDestroy(int zaxisID)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   zaxisDestroyKernel ( zaxisptr );
 }
@@ -312,15 +340,11 @@ void zaxisDestroyP ( void * zaxisptr )
 }
 
 
-char *zaxisNamePtr(int zaxistype)
+const char *zaxisNamePtr(int zaxistype)
 {
-  char *name;
-
-  if ( zaxistype >= 0 && zaxistype < CDI_NumZaxistype )
-    name = ZaxistypeEntry[zaxistype].longname;
-  else
-    name = ZaxistypeEntry[ZAXIS_GENERIC].longname;
-
+  const char *name = (zaxistype >= 0 && zaxistype < CDI_NumZaxistype)
+    ? ZaxistypeEntry[zaxistype].longname
+    : ZaxistypeEntry[ZAXIS_GENERIC].longname;
   return (name);
 }
 
@@ -346,9 +370,7 @@ The function @func{zaxisDefName} defines the name of a Z-axis.
 */
 void zaxisDefName(int zaxisID, const char *name)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   if ( name )
     {
@@ -374,9 +396,7 @@ The function @func{zaxisDefLongname} defines the longname of a Z-axis.
 */
 void zaxisDefLongname(int zaxisID, const char *longname)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   if ( longname )
     {
@@ -402,9 +422,7 @@ The function @func{zaxisDefUnits} defines the units of a Z-axis.
 */
 void zaxisDefUnits(int zaxisID, const char *units)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   if ( units )
     {
@@ -414,6 +432,19 @@ void zaxisDefUnits(int zaxisID, const char *units)
     }
 }
 
+
+void zaxisDefPsName(int zaxisID, const char *psname)
+{
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
+
+  if ( psname )
+    {
+      strncpy(zaxisptr->psname, psname, CDI_MAX_NAME - 1);
+      zaxisptr->name[CDI_MAX_NAME - 1] = '\0';
+      reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
+    }
+}
+
 /*
 @Function  zaxisInqName
 @Title     Get the name of a Z-axis
@@ -435,10 +466,7 @@ The function @func{zaxisInqName} returns the name of a Z-axis.
 */
 void zaxisInqName(int zaxisID, char *name)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   strcpy(name, zaxisptr->name);
 }
 
@@ -463,10 +491,7 @@ The function @func{zaxisInqLongname} returns the longname of a Z-axis.
 */
 void zaxisInqLongname(int zaxisID, char *longname)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   strcpy(longname, zaxisptr->longname);
 }
 
@@ -491,29 +516,28 @@ The function @func{zaxisInqUnits} returns the units of a Z-axis.
 */
 void zaxisInqUnits(int zaxisID, char *units)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   strcpy(units, zaxisptr->units);
 }
 
 
 void zaxisInqStdname(int zaxisID, char *stdname)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   strcpy(stdname, zaxisptr->stdname);
 }
 
 
+void zaxisInqPsName(int zaxisID, char *psname)
+{
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
+  strcpy(psname, zaxisptr->psname);
+}
+
+
 void zaxisDefPrec(int zaxisID, int prec)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   if (zaxisptr->prec != prec)
     {
@@ -525,19 +549,14 @@ void zaxisDefPrec(int zaxisID, int prec)
 
 int zaxisInqPrec(int zaxisID)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   return (zaxisptr->prec);
 }
 
 
 void zaxisDefPositive(int zaxisID, int positive)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   if (zaxisptr->positive != positive)
     {
@@ -549,19 +568,29 @@ void zaxisDefPositive(int zaxisID, int positive)
 
 int zaxisInqPositive(int zaxisID)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
+  return zaxisptr->positive;
+}
 
-  zaxis_check_ptr(zaxisID, zaxisptr);
 
-  return (zaxisptr->positive);
+void zaxisDefScalar(int zaxisID)
+{
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
+
+  zaxisptr->scalar = 1;
+  reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
+}
+
+int zaxisInqScalar(int zaxisID)
+{
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
+  return zaxisptr->scalar;
 }
 
 
 void zaxisDefLtype(int zaxisID, int ltype)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   if (zaxisptr->ltype != ltype)
     {
@@ -573,19 +602,14 @@ void zaxisDefLtype(int zaxisID, int ltype)
 
 int zaxisInqLtype(int zaxisID)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
-  return (zaxisptr->ltype);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
+  return zaxisptr->ltype;
 }
 
 
 void zaxisDefLtype2(int zaxisID, int ltype2)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   if (zaxisptr->ltype2 != ltype2)
     {
@@ -597,11 +621,8 @@ void zaxisDefLtype2(int zaxisID, int ltype2)
 
 int zaxisInqLtype2(int zaxisID)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
-  return (zaxisptr->ltype2);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
+  return zaxisptr->ltype2;
 }
 
 /*
@@ -620,9 +641,7 @@ The function @func{zaxisDefLevels} defines the levels of a Z-axis.
 */
 void zaxisDefLevels(int zaxisID, const double *levels)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   int size = zaxisptr->size;
 
@@ -650,10 +669,7 @@ The function @func{zaxisDefLevel} defines one level of a Z-axis.
 */
 void zaxisDefLevel(int zaxisID, int levelID, double level)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   if ( levelID >= 0 && levelID < zaxisptr->size )
     zaxisptr->vals[levelID] = level;
   reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
@@ -662,10 +678,7 @@ void zaxisDefLevel(int zaxisID, int levelID, double level)
 
 void zaxisDefNlevRef(int zaxisID, const int nhlev)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   if (zaxisptr->nhlev != nhlev)
     {
       zaxisptr->nhlev = nhlev;
@@ -676,11 +689,8 @@ void zaxisDefNlevRef(int zaxisID, const int nhlev)
 
 int zaxisInqNlevRef(int zaxisID)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
-  return (zaxisptr->nhlev);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
+  return zaxisptr->nhlev;
 }
 
 /*
@@ -699,10 +709,7 @@ The function @func{zaxisDefNumber} defines the reference number for a generalize
 */
 void zaxisDefNumber(int zaxisID, const int number)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   if (zaxisptr->number != number)
     {
       zaxisptr->number = number;
@@ -727,11 +734,8 @@ The function @func{zaxisInqNumber} returns the reference number to a generalized
 */
 int zaxisInqNumber(int zaxisID)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
-  return (zaxisptr->number);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
+  return zaxisptr->number;
 }
 
 /*
@@ -750,10 +754,7 @@ The function @func{zaxisDefUUID} defines the UUID for a generalized  Z-axis.
 */
 void zaxisDefUUID(int zaxisID, const unsigned char uuid[CDI_UUID_SIZE])
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   memcpy(zaxisptr->uuid, uuid, CDI_UUID_SIZE);
   reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
 }
@@ -765,6 +766,7 @@ void zaxisDefUUID(int zaxisID, const unsigned char uuid[CDI_UUID_SIZE])
 @Prototype void zaxisInqUUID(int zaxisID, char *uuid)
 @Parameter
     @Item  zaxisID  Z-axis ID, from a previous call to @fref{zaxisCreate} or @fref{vlistInqVarZaxis}.
+    @Item uuid A user supplied buffer of at least 16 bytes.
 
 @Description
 The function @func{zaxisInqUUID} returns the UUID to a generalized Z-axis.
@@ -775,10 +777,7 @@ The function @func{zaxisInqUUID} returns the UUID to a generalized Z-axis.
 */
 void zaxisInqUUID(int zaxisID, unsigned char uuid[CDI_UUID_SIZE])
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   memcpy(uuid, zaxisptr->uuid, CDI_UUID_SIZE);
 }
 
@@ -801,53 +800,41 @@ The function @func{zaxisInqLevel} returns one level of a Z-axis.
 double zaxisInqLevel(int zaxisID, int levelID)
 {
   double level = 0;
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   if ( levelID >= 0 && levelID < zaxisptr->size )
     level = zaxisptr->vals[levelID];
 
-  return (level);
+  return level;
 }
 
 double zaxisInqLbound(int zaxisID, int index)
 {
   double level = 0;
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
-  if ( zaxisptr->lbounds )
-    if ( index >= 0 && index < zaxisptr->size )
+  if ( zaxisptr->lbounds && ( index >= 0 && index < zaxisptr->size ) )
       level = zaxisptr->lbounds[index];
 
-  return (level);
+  return level;
 }
 
 
 double zaxisInqUbound(int zaxisID, int index)
 {
   double level = 0;
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
-  if ( zaxisptr->ubounds )
-    if ( index >= 0 && index < zaxisptr->size )
-      level = zaxisptr->ubounds[index];
-
-  return (level);
+  if ( zaxisptr->ubounds && ( index >= 0 && index < zaxisptr->size ) )
+    level = zaxisptr->ubounds[index];
+  return level;
 }
 
 
 const double *zaxisInqLevelsPtr(int zaxisID)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
-  return ( zaxisptr->vals );
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
+  return zaxisptr->vals;
 }
 
 /*
@@ -869,10 +856,7 @@ The function @func{zaxisInqLevels} returns all levels of a Z-axis.
 */
 void zaxisInqLevels(int zaxisID, double *levels)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   int size = zaxisptr->size;
   for (int i = 0; i < size; i++ )
     levels[i] =  zaxisptr->vals[i];
@@ -882,9 +866,7 @@ void zaxisInqLevels(int zaxisID, double *levels)
 int zaxisInqLbounds(int zaxisID, double *lbounds)
 {
   int size = 0;
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   if ( zaxisptr->lbounds )
     {
@@ -902,9 +884,7 @@ int zaxisInqLbounds(int zaxisID, double *lbounds)
 int zaxisInqUbounds(int zaxisID, double *ubounds)
 {
   int size = 0;
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   if ( zaxisptr->ubounds )
     {
@@ -922,9 +902,7 @@ int zaxisInqUbounds(int zaxisID, double *ubounds)
 int zaxisInqWeights(int zaxisID, double *weights)
 {
   int size = 0;
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   if ( zaxisptr->weights )
     {
@@ -942,9 +920,7 @@ int zaxisInqWeights(int zaxisID, double *weights)
 int zaxisInqLevelID(int zaxisID, double level)
 {
   int levelID = CDI_UNDEFID;
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   int size = zaxisptr->size;
   for ( int i = 0; i < size; i++ )
@@ -954,7 +930,7 @@ int zaxisInqLevelID(int zaxisID, double level)
         break;
       }
 
-  return (levelID);
+  return levelID;
 }
 
 /*
@@ -984,10 +960,7 @@ The valid CDI Z-axis types are @func{ZAXIS_GENERIC}, @func{ZAXIS_SURFACE},
 */
 int zaxisInqType(int zaxisID)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   return (zaxisptr->type);
 }
 
@@ -1009,21 +982,14 @@ The function @func{zaxisInqSize} returns the size of a Z-axis.
 */
 int zaxisInqSize(int zaxisID)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
-  int size = zaxisptr->size;
-
-  return (size);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
+  return (zaxisptr->size);
 }
 
 
 void cdiCheckZaxis(int zaxisID)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   if ( zaxisInqType(zaxisID) == ZAXIS_GENERIC )
     {
@@ -1059,58 +1025,43 @@ void cdiCheckZaxis(int zaxisID)
 
 void zaxisDefVct(int zaxisID, int size, const double *vct)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
-  if ( zaxisptr->vct == 0 )
+  if ( zaxisptr->vct == 0 || zaxisptr->vctsize != size )
     {
       zaxisptr->vctsize = size;
-      zaxisptr->vct = (double *)xmalloc((size_t)size * sizeof (double));
-      memcpy(zaxisptr->vct, vct, (size_t)size * sizeof (double));
-      reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
+      zaxisptr->vct = (double *) Realloc(zaxisptr->vct, (size_t)size*sizeof(double));
     }
-  else
-    if ( zaxisptr->vctsize != size )
-      Warning("VCT was already defined");
+
+  memcpy(zaxisptr->vct, vct, (size_t)size*sizeof(double));
+  reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
 }
 
 
 void zaxisInqVct(int zaxisID, double *vct)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   memcpy(vct, zaxisptr->vct, (size_t)zaxisptr->vctsize * sizeof (double));
 }
 
 
 int zaxisInqVctSize(int zaxisID)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   return (zaxisptr->vctsize);
 }
 
 
 const double *zaxisInqVctPtr(int zaxisID)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   return (zaxisptr->vct);
 }
 
 
 void zaxisDefLbounds(int zaxisID, const double *lbounds)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   size_t size = (size_t)zaxisptr->size;
 
@@ -1119,7 +1070,7 @@ void zaxisDefLbounds(int zaxisID, const double *lbounds)
       Warning("Lower bounds already defined for zaxisID = %d", zaxisID);
 
   if ( zaxisptr->lbounds == NULL )
-    zaxisptr->lbounds = (double *)xmalloc(size*sizeof(double));
+    zaxisptr->lbounds = (double *) Malloc(size*sizeof(double));
 
   memcpy(zaxisptr->lbounds, lbounds, size*sizeof(double));
   reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
@@ -1128,9 +1079,7 @@ void zaxisDefLbounds(int zaxisID, const double *lbounds)
 
 void zaxisDefUbounds(int zaxisID, const double *ubounds)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   size_t size = (size_t)zaxisptr->size;
 
@@ -1139,7 +1088,7 @@ void zaxisDefUbounds(int zaxisID, const double *ubounds)
       Warning("Upper bounds already defined for zaxisID = %d", zaxisID);
 
   if ( zaxisptr->ubounds == NULL )
-    zaxisptr->ubounds = (double *)xmalloc(size*sizeof(double));
+    zaxisptr->ubounds = (double *) Malloc(size*sizeof(double));
 
   memcpy(zaxisptr->ubounds, ubounds, size*sizeof(double));
   reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
@@ -1148,9 +1097,7 @@ void zaxisDefUbounds(int zaxisID, const double *ubounds)
 
 void zaxisDefWeights(int zaxisID, const double *weights)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   size_t size = (size_t)zaxisptr->size;
 
@@ -1159,7 +1106,7 @@ void zaxisDefWeights(int zaxisID, const double *weights)
       Warning("Weights already defined for zaxisID = %d", zaxisID);
 
   if ( zaxisptr->weights == NULL )
-    zaxisptr->weights = (double *)xmalloc(size*sizeof(double));
+    zaxisptr->weights = (double *) Malloc(size*sizeof(double));
 
   memcpy(zaxisptr->weights, weights, size*sizeof(double));
   reshSetStatus(zaxisID, &zaxisOps, RESH_DESYNC_IN_USE);
@@ -1168,40 +1115,34 @@ void zaxisDefWeights(int zaxisID, const double *weights)
 
 void zaxisChangeType(int zaxisID, int zaxistype)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
-
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
   zaxisptr->type = zaxistype;
 }
 
 
 void zaxisResize(int zaxisID, int size)
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   xassert(size >= 0);
-  zaxis_check_ptr(zaxisID, zaxisptr);
 
   zaxisptr->size = size;
 
   if ( zaxisptr->vals )
-    zaxisptr->vals = (double *)xrealloc(zaxisptr->vals, (size_t)size * sizeof(double));
+    zaxisptr->vals = (double *) Realloc(zaxisptr->vals, (size_t)size * sizeof(double));
 }
 
 
 int zaxisDuplicate(int zaxisID)
 {
   int zaxisIDnew;
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
-
-  zaxis_check_ptr(zaxisID, zaxisptr);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
   int zaxistype = zaxisInqType(zaxisID);
   int zaxissize = zaxisInqSize(zaxisID);
 
   zaxisIDnew = zaxisCreate(zaxistype, zaxissize);
-  zaxis_t *zaxisptrnew = reshGetVal(zaxisIDnew, &zaxisOps);
+  zaxis_t *zaxisptrnew = zaxisID2Ptr(zaxisIDnew);
 
   zaxis_copy(zaxisptrnew, zaxisptr);
 
@@ -1213,7 +1154,7 @@ int zaxisDuplicate(int zaxisID)
     {
       size_t size = (size_t)zaxissize;
 
-      zaxisptrnew->vals = (double *)xmalloc(size * sizeof (double));
+      zaxisptrnew->vals = (double *) Malloc(size * sizeof (double));
       memcpy(zaxisptrnew->vals, zaxisptr->vals, size * sizeof (double));
     }
 
@@ -1221,7 +1162,7 @@ int zaxisDuplicate(int zaxisID)
     {
       size_t size = (size_t)zaxissize;
 
-      zaxisptrnew->lbounds = (double *)xmalloc(size * sizeof (double));
+      zaxisptrnew->lbounds = (double *) Malloc(size * sizeof (double));
       memcpy(zaxisptrnew->lbounds, zaxisptr->lbounds, size * sizeof(double));
     }
 
@@ -1229,7 +1170,7 @@ int zaxisDuplicate(int zaxisID)
     {
       size_t size = (size_t)zaxissize;
 
-      zaxisptrnew->ubounds = (double *)xmalloc(size * sizeof (double));
+      zaxisptrnew->ubounds = (double *) Malloc(size * sizeof (double));
       memcpy(zaxisptrnew->ubounds, zaxisptr->ubounds, size * sizeof (double));
     }
 
@@ -1240,7 +1181,7 @@ int zaxisDuplicate(int zaxisID)
       if ( size )
         {
           zaxisptrnew->vctsize = (int)size;
-          zaxisptrnew->vct = (double *)xmalloc(size * sizeof (double));
+          zaxisptrnew->vct = (double *) Malloc(size * sizeof (double));
           memcpy(zaxisptrnew->vct, zaxisptr->vct, size * sizeof (double));
         }
     }
@@ -1249,25 +1190,22 @@ int zaxisDuplicate(int zaxisID)
 }
 
 
-void zaxisPrintKernel ( zaxis_t * zaxisptr, FILE * fp )
+static void zaxisPrintKernel ( zaxis_t * zaxisptr, int index, FILE * fp )
 {
-  int zaxisID;
-  int type;
   unsigned char uuid[CDI_UUID_SIZE];
-  int nlevels, levelID;
-  int nbyte0, nbyte;
-  double level;
+  int levelID;
+  int nbyte;
 
   xassert ( zaxisptr );
 
-  zaxisID = zaxisptr->self;
+  int zaxisID = zaxisptr->self;
 
-  type    = zaxisptr->type;
-  nlevels = zaxisptr->size;
+  int type    = zaxisptr->type;
+  int nlevels = zaxisptr->size;
 
-  nbyte0 = 0;
+  int nbyte0 = 0;
   fprintf(fp, "#\n");
-  fprintf(fp, "# zaxisID %d\n", zaxisID);
+  fprintf(fp, "# zaxisID %d\n", index);
   fprintf(fp, "#\n");
   fprintf(fp, "zaxistype = %s\n", zaxisNamePtr(type));
   fprintf(fp, "size      = %d\n", nlevels);
@@ -1285,16 +1223,14 @@ void zaxisPrintKernel ( zaxis_t * zaxisptr, FILE * fp )
 	  fprintf(fp, "%*s", nbyte0, "");
 	  nbyte = nbyte0;
 	}
-      level = zaxisInqLevel(zaxisID, levelID);
-      nbyte += fprintf(fp, "%.9g ", level);
+      nbyte += fprintf(fp, "%.9g ", zaxisptr->vals[levelID]);
     }
   fprintf(fp, "\n");
 
   if ( zaxisptr->lbounds && zaxisptr->ubounds )
     {
-      double level1, level2;
+      nbyte0 = fprintf(fp, "lbounds   = ");
       nbyte = nbyte0;
-      nbyte0 = fprintf(fp, "bounds    = ");
       for ( levelID = 0; levelID < nlevels; levelID++ )
 	{
 	  if ( nbyte > 80 )
@@ -1303,27 +1239,35 @@ void zaxisPrintKernel ( zaxis_t * zaxisptr, FILE * fp )
 	      fprintf(fp, "%*s", nbyte0, "");
 	      nbyte = nbyte0;
 	    }
-	  level1 = zaxisInqLbound(zaxisID, levelID);
-	  level2 = zaxisInqUbound(zaxisID, levelID);
-	  nbyte += fprintf(fp, "%.9g-%.9g ", level1, level2);
+	  nbyte += fprintf(fp, "%.9g ", zaxisptr->lbounds[levelID]);
+	}
+      fprintf(fp, "\n");
+
+      nbyte0 = fprintf(fp, "ubounds   = ");
+      nbyte = nbyte0;
+      for ( levelID = 0; levelID < nlevels; levelID++ )
+	{
+	  if ( nbyte > 80 )
+	    {
+	      fprintf(fp, "\n");
+	      fprintf(fp, "%*s", nbyte0, "");
+	      nbyte = nbyte0;
+	    }
+	  nbyte += fprintf(fp, "%.9g ", zaxisptr->ubounds[levelID]);
 	}
       fprintf(fp, "\n");
     }
 
   if ( type == ZAXIS_HYBRID || type == ZAXIS_HYBRID_HALF )
     {
-      int i;
-      int vctsize;
-      const double *vct;
-
-      vctsize = zaxisptr->vctsize;
-      vct     = zaxisptr->vct;
+      int vctsize = zaxisptr->vctsize;
+      const double *vct = zaxisptr->vct;
       fprintf(fp, "vctsize   = %d\n", vctsize);
       if ( vctsize )
         {
           nbyte0 = fprintf(fp, "vct       = ");
           nbyte = nbyte0;
-          for ( i = 0; i < vctsize; i++ )
+          for ( int i = 0; i < vctsize; i++ )
             {
               if ( nbyte > 70 || i == vctsize/2 )
                 {
@@ -1352,21 +1296,23 @@ void zaxisPrintKernel ( zaxis_t * zaxisptr, FILE * fp )
 
   if ( type == ZAXIS_REFERENCE )
     {
-      const unsigned char *d;
       zaxisInqUUID(zaxisID, uuid);
-      d = uuid;
-      fprintf(fp, "uuid      = %02x%02x%02x%02x-%02x%02x-%02x%02x-%02x%02x-%02x%02x%02x%02x%02x%02x\n",
-              d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7],
-              d[8], d[9], d[10], d[11], d[12], d[13], d[14], d[15]);
+      if ( *uuid )
+        {
+          const unsigned char *d = uuid;
+          fprintf(fp, "uuid      = %02x%02x%02x%02x-%02x%02x-%02x%02x-%02x%02x-%02x%02x%02x%02x%02x%02x\n",
+                  d[0], d[1], d[2], d[3], d[4], d[5], d[6], d[7],
+                  d[8], d[9], d[10], d[11], d[12], d[13], d[14], d[15]);
+        }
     }
 }
 
 
-void zaxisPrint ( int zaxisID )
+void zaxisPrint ( int zaxisID, int index )
 {
-  zaxis_t *zaxisptr = reshGetVal(zaxisID, &zaxisOps);
+  zaxis_t *zaxisptr = zaxisID2Ptr(zaxisID);
 
-  zaxisPrintKernel ( zaxisptr, stdout );
+  zaxisPrintKernel ( zaxisptr, index, stdout );
 }
 
 
@@ -1377,7 +1323,7 @@ void zaxisPrintP ( void * voidptr, FILE * fp )
 
   xassert ( zaxisptr );
 
-  zaxisPrintKernel(zaxisptr, fp);
+  zaxisPrintKernel(zaxisptr, zaxisptr->self, fp);
 }
 
 
@@ -1472,7 +1418,6 @@ zaxisTxCode ( void )
 }
 
 enum { zaxisNint     = 8,
-       zaxisNstrings = 4,
        vals     = 1 << 0,
        lbounds  = 1 << 1,
        ubounds  = 1 << 2,
@@ -1481,6 +1426,8 @@ enum { zaxisNint     = 8,
        zaxisHasUUIDFlag = 1 << 5,
 };
 
+#define ZAXIS_STR_SERIALIZE { zaxisP->name, zaxisP->longname, \
+      zaxisP->stdname, zaxisP->units }
 
 static
 int zaxisGetMemberMask ( zaxis_t * zaxisP )
@@ -1529,9 +1476,14 @@ zaxisGetPackSize(void * voidP, void *context)
         + serializeGetSize(1, DATATYPE_UINT32, context);
     }
 
-  packBufferSize += serializeGetSize(zaxisNstrings * CDI_MAX_NAME, DATATYPE_TXT, context)
-    + serializeGetSize(1, DATATYPE_UINT32, context)
-    + serializeGetSize(1, DATATYPE_UCHAR, context);
+  {
+    const char *strTab[] = ZAXIS_STR_SERIALIZE;
+    size_t numStr = sizeof (strTab) / sizeof (strTab[0]);
+    packBufferSize
+      += serializeStrTabGetPackSize(strTab, (int)numStr, context);
+  }
+
+  packBufferSize += serializeGetSize(1, DATATYPE_UCHAR, context);
 
   if (!cdiUUIDIsNull(zaxisP->uuid))
     packBufferSize += serializeGetSize(CDI_UUID_SIZE, DATATYPE_UCHAR, context);
@@ -1547,7 +1499,6 @@ zaxisUnpack(char * unpackBuffer, int unpackBufferSize,
 {
   int intBuffer[zaxisNint], memberMask;
   uint32_t d;
-  char charBuffer[zaxisNstrings * CDI_MAX_NAME];
 
   serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
                   intBuffer, zaxisNint, DATATYPE_INT, context);
@@ -1575,7 +1526,7 @@ zaxisUnpack(char * unpackBuffer, int unpackBufferSize,
       int size = zaxisP->size;
       xassert(size >= 0);
 
-      zaxisP->vals = (double *)xmalloc((size_t)size * sizeof (double));
+      zaxisP->vals = (double *) Malloc((size_t)size * sizeof (double));
       serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
                       zaxisP->vals, size, DATATYPE_FLT64, context);
       serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
@@ -1588,7 +1539,7 @@ zaxisUnpack(char * unpackBuffer, int unpackBufferSize,
       int size = zaxisP->size;
       xassert(size >= 0);
 
-      zaxisP->lbounds = (double *)xmalloc((size_t)size * sizeof (double));
+      zaxisP->lbounds = (double *) Malloc((size_t)size * sizeof (double));
       serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
                       zaxisP->lbounds, size, DATATYPE_FLT64, context);
       serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
@@ -1601,7 +1552,7 @@ zaxisUnpack(char * unpackBuffer, int unpackBufferSize,
       int size = zaxisP->size;
       xassert(size >= 0);
 
-      zaxisP->ubounds = (double *)xmalloc((size_t)size * sizeof (double));
+      zaxisP->ubounds = (double *) Malloc((size_t)size * sizeof (double));
       serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
                       zaxisP->ubounds, size, DATATYPE_FLT64, context);
       serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
@@ -1614,7 +1565,7 @@ zaxisUnpack(char * unpackBuffer, int unpackBufferSize,
       int size = zaxisP->size;
       xassert(size >= 0);
 
-      zaxisP->weights = (double *)xmalloc((size_t)size * sizeof (double));
+      zaxisP->weights = (double *) Malloc((size_t)size * sizeof (double));
       serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
                       zaxisP->weights, size, DATATYPE_FLT64, context);
       serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
@@ -1627,7 +1578,7 @@ zaxisUnpack(char * unpackBuffer, int unpackBufferSize,
       int size = zaxisP->vctsize;
       xassert(size >= 0);
 
-      zaxisP->vct = (double *)xmalloc((size_t)size * sizeof (double));
+      zaxisP->vct = (double *) Malloc((size_t)size * sizeof (double));
       serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
                       zaxisP->vct, size, DATATYPE_FLT64, context);
       serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
@@ -1635,17 +1586,12 @@ zaxisUnpack(char * unpackBuffer, int unpackBufferSize,
       xassert(cdiCheckSum(DATATYPE_FLT64, size, zaxisP->vct) == d);
     }
 
-  serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
-                  charBuffer, zaxisNstrings * CDI_MAX_NAME, DATATYPE_TXT, context);
-  serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
-                  &d, 1, DATATYPE_UINT32, context);
-
-  xassert(d == cdiCheckSum(DATATYPE_TXT, zaxisNstrings * CDI_MAX_NAME, charBuffer));
-
-  memcpy ( zaxisP->name,     &charBuffer[CDI_MAX_NAME * 0], CDI_MAX_NAME );
-  memcpy ( zaxisP->longname, &charBuffer[CDI_MAX_NAME * 1], CDI_MAX_NAME );
-  memcpy ( zaxisP->stdname,  &charBuffer[CDI_MAX_NAME * 2], CDI_MAX_NAME );
-  memcpy ( zaxisP->units,    &charBuffer[CDI_MAX_NAME * 3], CDI_MAX_NAME );
+  {
+    char *strTab[] = ZAXIS_STR_SERIALIZE;
+    int numStr = sizeof (strTab) / sizeof (strTab[0]);
+    serializeStrTabUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
+                          strTab, numStr, context);
+  }
 
   serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
                   &zaxisP->positive, 1, DATATYPE_UCHAR, context);
@@ -1654,6 +1600,8 @@ zaxisUnpack(char * unpackBuffer, int unpackBufferSize,
     serializeUnpack(unpackBuffer, unpackBufferSize, unpackBufferPos,
                     zaxisP->uuid, CDI_UUID_SIZE, DATATYPE_UCHAR, context);
 
+  reshSetStatus(zaxisP->self, &zaxisOps,
+                reshGetStatus(zaxisP->self, &zaxisOps) & ~RESH_SYNC_BIT);
 }
 
 static void
@@ -1664,7 +1612,6 @@ zaxisPack(void * voidP, void * packBuffer, int packBufferSize,
   int intBuffer[zaxisNint];
   int memberMask;
   uint32_t d;
-  char charBuffer[zaxisNstrings * CDI_MAX_NAME];
 
   intBuffer[0]  = zaxisP->self;
   intBuffer[1]  = zaxisP->prec;
@@ -1735,16 +1682,12 @@ zaxisPack(void * voidP, void * packBuffer, int packBufferSize,
                     packBuffer, packBufferSize, packBufferPos, context);
     }
 
-  memcpy ( &charBuffer[CDI_MAX_NAME * 0], zaxisP->name,     CDI_MAX_NAME );
-  memcpy ( &charBuffer[CDI_MAX_NAME * 1], zaxisP->longname, CDI_MAX_NAME );
-  memcpy ( &charBuffer[CDI_MAX_NAME * 2], zaxisP->stdname,  CDI_MAX_NAME );
-  memcpy ( &charBuffer[CDI_MAX_NAME * 3], zaxisP->units,    CDI_MAX_NAME );
-
-  serializePack(charBuffer, zaxisNstrings * CDI_MAX_NAME, DATATYPE_TXT,
-                packBuffer, packBufferSize, packBufferPos, context);
-  d = cdiCheckSum(DATATYPE_TXT, zaxisNstrings * CDI_MAX_NAME, charBuffer);
-  serializePack(&d, 1, DATATYPE_UINT32,
-                packBuffer, packBufferSize, packBufferPos, context);
+  {
+    const char *strTab[] = ZAXIS_STR_SERIALIZE;
+    int numStr = sizeof (strTab) / sizeof (strTab[0]);
+    serializeStrTabPack(strTab, numStr,
+                        packBuffer, packBufferSize, packBufferPos, context);
+  }
 
   serializePack(&zaxisP->positive, 1, DATATYPE_UCHAR,
                 packBuffer, packBufferSize, packBufferPos, context);
@@ -1756,10 +1699,13 @@ zaxisPack(void * voidP, void * packBuffer, int packBufferSize,
 }
 
 
-void zaxisGetIndexList ( int nzaxis, int * zaxisResHs )
+void cdiZaxisGetIndexList(unsigned nzaxis, int *zaxisResHs)
 {
-  reshGetResHListOfType ( nzaxis, zaxisResHs, &zaxisOps );
+  reshGetResHListOfType(nzaxis, zaxisResHs, &zaxisOps);
 }
+
+#undef ZAXIS_STR_SERIALIZE
+
 /*
  * Local Variables:
  * c-file-style: "Java"
