@@ -42,18 +42,15 @@ static double starttime = 0.0;
 void afterInqHistory(int fileID);
 void afterDefHistory(int fileID, char *histstring);
 
-int   scan_par_obsolate(char *namelist, char *name, int def);
+int   scan_par_obsolate(char *namelist, const char *name, int def);
 void  scan_code(char *namelist, struct Variable *vars, int maxCodes, int *numCodes);
-int   scan_par(int verbose, char *namelist, char *name, int def);
+int   scan_par(int verbose, char *namelist, const char *name, int def);
 int   scan_time(int verbose, char *namelist, int *hours, int max_hours);
-void  scan_darray(char *namelist, char *name, double *values, int maxValues, int *numValues);
+void  scan_darray(char *namelist, const char *name, double *values, int maxValues, int *numValues);
 
 long  get_nfft(void);
 
-char   *zaxisNamePtr(int leveltype);
-char   *vlistInqVarNamePtr(int vlistID, int varID);
-char   *vlistInqVarLongnamePtr(int vlistID, int varID);
-char   *vlistInqVarUnitsPtr(int vlistID, int varID);
+char zaxistypename[CDI_MAX_NAME];
 
 typedef struct {
   int lana, nrecs;
@@ -147,7 +144,7 @@ void FreeMean(struct Variable *vars)
   for ( int code = 0; code < MaxCodes; code++ )
     if ( vars[code].mean )
       {
-	free(vars[code].mean);
+        Free(vars[code].mean);
 	vars[code].mean = NULL;
       }
 }
@@ -528,7 +525,7 @@ void after_setEndOfInterval(struct Control *globs, int nrecs)
       else if ( globs->OutputInterval == UNLIM_INTERVAL )
 	globs->EndOfInterval = FALSE;
       else
-	Error( "output interval %d not implemented!\n", globs->OutputInterval);
+	Error("output interval %d not implemented!", globs->OutputInterval);
     }
 }
 
@@ -655,11 +652,11 @@ void after_control(struct Control *globs, struct Variable *vars)
 	    {
 	      statusp = after_readTimestep(&rarg);
 	    }
-#if  defined  (HAVE_LIBPTHREAD)
+#if defined(HAVE_LIBPTHREAD)
 	  else
 	    {
 	      rval = pthread_create(&thrID, &attr, after_readTimestep, &rarg);
-	      if ( rval != 0 ) Error( "pthread_create failed!");
+	      if ( rval != 0 ) Error("pthread_create failed!");
 	    }
 #endif
 
@@ -668,22 +665,22 @@ void after_control(struct Control *globs, struct Variable *vars)
 	      if ( globs->Type >  0 ) after_legini_setup(globs, vars);
 	    }
 
-#if  defined  (HAVE_LIBPTHREAD)
+#if defined(HAVE_LIBPTHREAD)
 	  if ( ParallelRead )
 	    {
 	      pthread_join(thrID, &statusp);
 	      if ( *(int *)statusp < 0 )
-		Error( "after_readTimestep error! (status = %d)", *(int *)statusp);
+		Error("after_readTimestep error! (status = %d)", *(int *)statusp);
 	    }
 #endif
 	  tsFirst = FALSE;
 	}
-#if  defined  (HAVE_LIBPTHREAD)
+#if defined(HAVE_LIBPTHREAD)
       else
 	{
 	  pthread_join(thrID, &statusp);
 	  if ( *(int *)statusp < 0 )
-	    Error( "after_readTimestep error! (status = %d)", *(int *)statusp);
+	    Error("after_readTimestep error! (status = %d)", *(int *)statusp);
 	}
 #endif
       nrecs = *(int *)statusp;
@@ -697,7 +694,7 @@ void after_control(struct Control *globs, struct Variable *vars)
       if ( nrecs && ParallelRead )
 	{
 	  rval = pthread_create(&thrID, &attr, after_readTimestep, &rarg);
-	  if ( rval != 0 ) Error( "pthread_create failed!");
+	  if ( rval != 0 ) Error("pthread_create failed!");
 	}
 #endif
 
@@ -743,9 +740,8 @@ void after_setLevel(struct Control *globs)
   double level;
   int checkLevel = TRUE;
   int numplevelDefault;  /* default pressure level */
-  long plevelDefault[] = { 100000, 92500, 85000, 70000, 60000, 50000,  \
-			    40000, 30000, 25000, 20000, 15000, 10000,  \
-			     7000,  5000,  3000,  2000, 1000 };
+  long plevelDefault[] = { 100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000, 20000, 15000,
+                            10000,  7000,  5000,  3000,  2000, 1000 };
   int numhlevelDefault;  /* default height level */
   long hlevelDefault[] = {  0, 1000, 2000, 5000, 10000, 15000, 20000, 25000, 30000 };
 
@@ -955,9 +951,10 @@ void after_defineLevel(struct Control *globs, struct Variable *vars)
 	    }
 	}
       else
-	Error( "%s level data unsupported for TYPE %d",
-	      zaxisNamePtr(zaxisInqType(iVertID)), globs->Type);	    
-
+        {
+          zaxisName(zaxisInqType(iVertID), zaxistypename);
+          Error("%s level data unsupported for TYPE %d", zaxistypename, globs->Type);	    
+        }
       break;
     }
   case 30:
@@ -1092,7 +1089,7 @@ void after_setCodes(struct Control *globs, struct Variable *vars, int maxCodes,
 {
   int code;
   int table, modelID, tableID;
-  char *name, *longname;
+  const char *name, *longname;
   int varID;
 
   if ( globs->Verbose ) lprintf(stdout);
@@ -1173,6 +1170,7 @@ void after_checkNamelist(struct Control *globs)
     }
 }
 
+#if defined(AFTERBURNER)
 static
 void after_usage(void)
 {
@@ -1187,8 +1185,8 @@ void after_usage(void)
   /*  fprintf(stderr, "     option -h : help (this output)\n"); */
   /*  fprintf(stderr, "     option -p : parallel read on\n"); */
   fprintf(stderr, "  <InputFiles> : ECHAM or ECMWF Ana or ReAna files\n");
-  fprintf(stderr, "  <OutputFile> : GRIB, netCDF or SERVICE format file\n");
-  fprintf(stderr, "<VarianceFile> : GRIB, netCDF or SERVICE format file\n");
+  fprintf(stderr, "  <OutputFile> : GRIB, NetCDF or SERVICE format file\n");
+  fprintf(stderr, "<VarianceFile> : GRIB, NetCDF or SERVICE format file\n");
   fprintf(stderr, "  namelist is read from <stdin>\n");
   fprintf(stderr, "  output is written to <stdout>\n\n");
 
@@ -1201,6 +1199,7 @@ void after_usage(void)
 
   exit(1);
 }
+#endif
 
 static
 void after_parini(struct Control *globs, struct Variable *vars)
@@ -1211,7 +1210,7 @@ void after_parini(struct Control *globs, struct Variable *vars)
     {
 #if defined(CDO)
       fprintf(stderr, "Default namelist: \n");
-      fprintf(stderr, "  TYPE = 0, CODE = -1, LEVEL = -1, MULTI = 0, DAYIN = 30, MEAN = 0, TIMESEL = -1, UNITSEL = 0\n");
+      fprintf(stderr, "  TYPE=0, CODE=-1, LEVEL=-1, INTERVAL=0, MEAN=0, EXTRAPOLATE=0\n");
 #endif
       fprintf(stdout, "Enter namelist parameter:\n");
     }
@@ -1222,7 +1221,9 @@ void after_parini(struct Control *globs, struct Variable *vars)
       if ( length == 0L )
 	{
 	  fprintf(stderr,"\n stdin not connected\n");
+#if defined(AFTERBURNER)
 	  after_usage();
+#endif
 	}
       fseek(stdin, 0L, SEEK_SET);
     }
@@ -1273,7 +1274,7 @@ void after_parini(struct Control *globs, struct Variable *vars)
   int gribFormat = scan_par_obsolate(namelist, "grib",   0);
   int cdfFormat  = scan_par_obsolate(namelist, "netcdf", 0);
 
-  if ( gribFormat && cdfFormat ) Error( "GRIB or netCDF?");
+  if ( gribFormat && cdfFormat ) Error( "GRIB or NetCDF?");
 
   switch ( fileFormat )
     {
@@ -1429,7 +1430,7 @@ void after_precntl(struct Control *globs, struct Variable *vars)
     }
 
   if ( globs->Truncation == 0 && globs->Latitudes == 0 )
-    Error("Unsupported file structure!\n");
+    Error("Unsupported file structure!");
 
   if ( globs->Truncation == 0 )
     {
@@ -1467,14 +1468,19 @@ void after_precntl(struct Control *globs, struct Variable *vars)
 	      if ( leveltype == ZAXIS_HYBRID && globs->nvct == 0 )
 		{
 		  nhzaxis++;
-		  if ( numlevel != (zaxisInqVctSize(zaxisID)/2 - 1) )
+                  int nvct = zaxisInqVctSize(zaxisID);
+		  if ( numlevel != (nvct/2 - 1) )
 		    {
-		      if ( ! (numlevel == 191 && zaxisInqVctSize(zaxisID) == 0) )
+                      if ( nvct == 0 )
+                        {
+                          if ( numlevel != 191 )
+                            Warning("VCT missing for hybrid level data with %d levels!", numlevel);
+                        }
+                      else
 			{
-			  Warning( "Skip %d hybrid level data with %d levels!",
-				  (zaxisInqVctSize(zaxisID)/2 - 1), numlevel);
-			  continue;
+			  Warning("Skip %d hybrid level data with %d levels!", (nvct/2 - 1), numlevel);
 			}
+                      continue;
 		    }
 		}
 
@@ -1487,7 +1493,7 @@ void after_precntl(struct Control *globs, struct Variable *vars)
 
 	      iVertID = zaxisID;
 	      globs->NumLevelFound = numlevel;
-	      LevelFound = (double *) malloc(globs->NumLevelFound*sizeof(double));
+	      LevelFound = (double *) Malloc(globs->NumLevelFound*sizeof(double));
 	      for ( l = 0; l < globs->NumLevelFound; l++ )
 		LevelFound[l] = (int) zaxisInqLevel(zaxisID, l);
 
@@ -1501,7 +1507,7 @@ void after_precntl(struct Control *globs, struct Variable *vars)
 
 			  if ( globs->vct == NULL )
 			    {
-			      globs->vct = (double *) malloc(globs->nvct*sizeof(double));
+			      globs->vct = (double *) Malloc(globs->nvct*sizeof(double));
 			      memcpy(globs->vct, zaxisInqVctPtr(zaxisID), globs->nvct*sizeof(double));
 			    }
 			}
@@ -1511,20 +1517,19 @@ void after_precntl(struct Control *globs, struct Variable *vars)
 			    {
 			      fprintf(stderr," Using internal VCT for L191\n");
 			      globs->nvct = (191+1)*2;
-			      globs->vct = (double *) malloc(globs->nvct*sizeof(double));
+			      globs->vct = (double *) Malloc(globs->nvct*sizeof(double));
 			      memcpy(globs->vct, VCT_L191, globs->nvct*sizeof(double));
 			      zaxisDefVct(zaxisID, globs->nvct, globs->vct);
 			    }
 			  else
 			    {
-			      Error( "VCT not defined in inputfile!");
+			      Error("VCT not defined in inputfile!");
 			    }
 			}
 		    }
 
 		  if ( numlevel != (globs->nvct/2 - 1) )
-		    Error( "Number of hybrid levels %d does not match vct levels %d",
-			  numlevel, globs->nvct/2-1);
+		    Error("Number of hybrid levels %d does not match VCT levels %d", numlevel, globs->nvct/2-1);
 
 		  if ( globs->Debug )
 		    for ( i = 0; i < globs->nvct/2; i++ )
@@ -1608,9 +1613,9 @@ void after_precntl(struct Control *globs, struct Variable *vars)
     }
 
   if ( globs->Debug )
-    Message( "FieldDim = %d\n", FieldDim);
+    Message( "FieldDim = %d", FieldDim);
 
-  globs->Field = (double *) malloc(FieldDim*sizeof(double));
+  globs->Field = (double *) Malloc(FieldDim*sizeof(double));
 
   if ( globs->Debug )
     for ( code = 0; code < MaxCodes; code++ )
@@ -1634,7 +1639,7 @@ void after_postcntl(struct Control *globs, struct Variable *vars)
   int ovarID, ogridID, ozaxisID;
   int ovarID2;
   int ivarID, instID, modelID, tableID;
-  char *name, *longname, *units;
+  const char *name, *longname, *units;
   char histstring[99];
   int datatype;
 
@@ -1653,9 +1658,10 @@ void after_postcntl(struct Control *globs, struct Variable *vars)
 	{
 	  gridID = vars[code].igridID;
 	  zaxisID = vars[code].izaxisID;
+          zaxisName(zaxisInqType(zaxisID), zaxistypename);
 	  fprintf(stderr," Detected Code %3d  grid %-8s size %5d  level %2d %-8s\n",
 		  code, gridNamePtr(gridInqType(gridID)), gridInqSize(gridID),
-		  zaxisInqSize(zaxisID), zaxisNamePtr(zaxisInqType(zaxisID)));
+		  zaxisInqSize(zaxisID), zaxistypename);
 	}
 
 
@@ -1757,41 +1763,47 @@ void after_postcntl(struct Control *globs, struct Variable *vars)
 	{
 	  gridID  = vars[code].ogridID;
 	  zaxisID = vars[code].ozaxisID;
+          zaxisName(zaxisInqType(zaxisID), zaxistypename);
 	  fprintf(stderr," Selected Code %3d  grid %-8s size %5d  level %2d %-8s\n",
 		  code, gridNamePtr(gridInqType(gridID)), gridInqSize(gridID),
-		  zaxisInqSize(zaxisID), zaxisNamePtr(zaxisInqType(zaxisID)));
+		  zaxisInqSize(zaxisID), zaxistypename);
 	}
 }
 
-#if defined(AFTERBURNER)
 static
 void after_readVct(struct Control *globs, const char *vctfile)
 {
   char line[1024];
-  int i, n;
+  int i, n, nlines = 0;
   double va, vb;
 
   FILE *fp = fopen(vctfile, "r");
   if ( fp == NULL ) SysError( "Open failed on %s", vctfile);
 
-  while ( fgets(line, 1023, fp) ) globs->nvct++;
-
-  globs->nvct *= 2;
-  globs->vct = (double *) malloc(globs->nvct*sizeof(double));
-
-  rewind(fp);
-  for ( i = 0; i < globs->nvct/2; i++ )
+  while ( fgets(line, 1023, fp) )
     {
-      fgets(line, 1023, fp);
+      if ( line[0] == '#' || line[0] == '\0' ) continue;
+      nlines++;
+    }
+  
+  globs->nvct = nlines*2;
+  globs->vct = (double *) Malloc(globs->nvct*sizeof(double));
+ 
+  rewind(fp);
+
+  i = 0;
+  while ( fgets(line, 1023, fp) )
+    {
+      if ( line[0] == '#' || line[0] == '\0' ) continue;
       sscanf(line, "%d %lg %lg", &n, &va, &vb);
       globs->vct[i]               = va;
       globs->vct[i+globs->nvct/2] = vb;
+      i++;
     }
-  fprintf(stdout, "  Reading VCT for %d hybrid levels from file %s\n", globs->nvct/2-1, vctfile);
+  fprintf(stdout, "  Read VCT with %d hybrid levels from file %s\n", globs->nvct/2-1, vctfile);
 
   fclose(fp);
 }
-#endif
 
 #if defined(AFTERBURNER)
 static
@@ -2058,9 +2070,9 @@ void after_processing(struct Control *globs, struct Variable *vars)
 
   after_dimcalc(globs);
 
-  globs->rcoslat          = (double *) malloc(globs->Latitudes*sizeof(double));
-  globs->coslat           = (double *) malloc(globs->Latitudes*sizeof(double));
-  globs->DerivationFactor = (double *) malloc(globs->Latitudes*sizeof(double));
+  globs->rcoslat          = (double *) Malloc(globs->Latitudes*sizeof(double));
+  globs->coslat           = (double *) Malloc(globs->Latitudes*sizeof(double));
+  globs->DerivationFactor = (double *) Malloc(globs->Latitudes*sizeof(double));
 
   if ( globs->Type < 50 && globs->AnalysisData )
     {
@@ -2077,7 +2089,7 @@ void after_processing(struct Control *globs, struct Variable *vars)
 	Error("Can't write fourier coefficients to GRIB!");
       else if ( ofiletype == FILETYPE_NC || ofiletype == FILETYPE_NC2 ||
 		ofiletype == FILETYPE_NC4 )
-	Error("Can't write fourier coefficients to netCDF!");
+	Error("Can't write fourier coefficients to NetCDF!");
     }
 
   filename = strrchr(ifile,'/');
@@ -2124,8 +2136,8 @@ void after_processing(struct Control *globs, struct Variable *vars)
   /*  if ( vars[U_WIND].needed || vars[V_WIND].needed ) */
   if ( vars[U_WIND].comp || vars[V_WIND].comp )
     {
-      globs->dv2uv_f1 = (double *) malloc(globs->DimSP_half*sizeof(double));
-      globs->dv2uv_f2 = (double *) malloc(globs->DimSP_half*sizeof(double));
+      globs->dv2uv_f1 = (double *) Malloc(globs->DimSP_half*sizeof(double));
+      globs->dv2uv_f2 = (double *) Malloc(globs->DimSP_half*sizeof(double));
       geninx(globs->Truncation, globs->dv2uv_f1, globs->dv2uv_f2);
     }
 
@@ -2153,16 +2165,16 @@ void after_processing(struct Control *globs, struct Variable *vars)
 #endif
   streamClose(globs->istreamID);
 
-  if ( globs->rcoslat )          free(globs->rcoslat);
-  if ( globs->coslat )           free(globs->coslat);
-  if ( globs->DerivationFactor ) free(globs->DerivationFactor);
+  if ( globs->rcoslat )          Free(globs->rcoslat);
+  if ( globs->coslat )           Free(globs->coslat);
+  if ( globs->DerivationFactor ) Free(globs->DerivationFactor);
 
-  if ( globs->Field ) free(globs->Field);
+  if ( globs->Field ) Free(globs->Field);
 
-  if ( globs->poli ) free(globs->poli);
-  if ( globs->pold ) free(globs->pold);
-  if ( globs->pdev ) free(globs->pdev);
-  if ( globs->pol2 ) free(globs->pol2);  if ( globs->pol3 ) free(globs->pol3);
+  if ( globs->poli ) Free(globs->poli);
+  if ( globs->pold ) Free(globs->pold);
+  if ( globs->pdev ) Free(globs->pdev);
+  if ( globs->pol2 ) Free(globs->pol2);  if ( globs->pol3 ) Free(globs->pol3);
 }
 
 extern char *optarg;
@@ -2224,7 +2236,7 @@ int afterburner(int argc, char *argv[])
       fclose(fp);
     }
 
-  struct Control *globs = (struct Control *) malloc(sizeof(struct Control));
+  struct Control *globs = (struct Control *) Malloc(sizeof(struct Control));
   after_control_init(globs);
 
   globs->Verbose = 1;
@@ -2238,7 +2250,7 @@ int afterburner(int argc, char *argv[])
     switch (c)
       {
       case 'a': globs->AnalysisData = 1; break;
-      case 'b': Message( "option -b not longer needed!\n"); break;
+      case 'b': Message( "option -b not longer needed!"); break;
       case 'c': after_printCodes(); break;
       case 'd': globs->Debug = 1; break;
       case 'p': ParallelRead = TRUE; break;
@@ -2320,7 +2332,7 @@ int afterburner(int argc, char *argv[])
       if ( globs->Multi > 0 )
 	Error( "Namelist parameter MULTI works only with one inputfile");
 
-      ifiles = (char **) malloc(globs->Nfiles*sizeof(char*));
+      ifiles = (char **) Malloc(globs->Nfiles*sizeof(char*));
       for ( i = 0; i < globs->Nfiles; i++ )
 	ifiles[i] = argv[--fargcn];
     }
@@ -2336,7 +2348,7 @@ int afterburner(int argc, char *argv[])
       fprintf(stderr, "  Maximum ffts to run in parallel:  %ld\n", get_nfft());
     }
 
-  /* read option VCT */
+  /* read optional VCT */
   if ( Vctfile ) after_readVct(globs, Vctfile);
 
   /* --------------------- */
@@ -2351,7 +2363,7 @@ int afterburner(int argc, char *argv[])
 
   FreeMean(vars);
 
-  free(globs);
+  Free(globs);
 
   return(0);
 }
@@ -2364,11 +2376,17 @@ void *Afterburner(void *argument)
 
   lstdout = !cdoSilentMode;
 
-  struct Control *globs = (struct Control *) malloc(sizeof(struct Control));
+  struct Control *globs = (struct Control *) Malloc(sizeof(struct Control));
   after_control_init(globs);
 
   globs->Verbose = cdoVerbose;
 
+  if ( operatorArgc() == 1 )
+    {
+      const char *vctfile = operatorArgv()[0];
+      after_readVct(globs, vctfile);
+    }
+
   struct Variable vars[MaxCodes+5];
   for ( int code = 0; code < MaxCodes+5; code++ ) after_variable_init(&vars[code]);
 
@@ -2390,7 +2408,7 @@ void *Afterburner(void *argument)
       if ( globs->Multi > 0 )
 	Error( "Namelist parameter MULTI works only with one inputfile");
 
-      ifiles = (char **) malloc(globs->Nfiles*sizeof(char*));
+      ifiles = (char **) Malloc(globs->Nfiles*sizeof(char*));
       for ( int i = 0; i < globs->Nfiles; ++i )
 	ifiles[i] = cdoStreamName(--nfiles)->args;
       for ( int i = 0; i < globs->Nfiles; ++i ) printf("files %d %s\n", i+1, ifiles[i]);
@@ -2400,11 +2418,11 @@ void *Afterburner(void *argument)
 
   FreeMean(vars);
 
-  free(globs);
+  Free(globs);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
 #else
 int main(int argc, char *argv[])
