@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -35,11 +35,9 @@
 
 void *Arith(void *argument)
 {
-  int operatorID;
-  int operfunc;
   enum {FILL_NONE, FILL_TS, FILL_VAR, FILL_VARTS, FILL_FILE};
   int filltype = FILL_NONE;
-  int streamIDx1, streamIDx2, streamID1, streamID2, streamID3;
+  int nmiss;
   int gridsize;
   int nrecs, nrecs2, nvars = 0, nlev, recID;
   int nlevels2 = 1;
@@ -48,10 +46,7 @@ void *Arith(void *argument)
   int varID2, levelID2;
   int offset;
   int lfill1, lfill2;
-  int ntsteps1, ntsteps2;
-  int vlistIDx1, vlistIDx2, vlistID1, vlistID2, vlistID3;
-  int taxisIDx1, taxisID1, taxisID2, taxisID3;
-  field_t *fieldx1, *fieldx2, field1, field2;
+  int streamID3, vlistID3, taxisID3;
   int *varnmiss2 = NULL;
   int **varnmiss = NULL;
   double *vardata2 = NULL;
@@ -67,31 +62,34 @@ void *Arith(void *argument)
   cdoOperatorAdd("max",   func_max,   0, NULL);
   cdoOperatorAdd("atan2", func_atan2, 0, NULL);
 
-  operatorID = cdoOperatorID();
-  operfunc = cdoOperatorF1(operatorID);
+  int operatorID = cdoOperatorID();
+  int operfunc = cdoOperatorF1(operatorID);
+  operatorCheckArgc(0);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
-  streamID2 = streamOpenRead(cdoStreamName(1));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID2 = streamOpenRead(cdoStreamName(1));
 
-  streamIDx1 = streamID1;
-  streamIDx2 = streamID2;
-  fieldx1 = &field1;
-  fieldx2 = &field2;
+  int streamIDx1 = streamID1;
+  int streamIDx2 = streamID2;
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = streamInqVlist(streamID2);
-  vlistIDx1 = vlistID1;
-  vlistIDx2 = vlistID2;
+  field_t field1, field2;
+  field_t *fieldx1 = &field1;
+  field_t *fieldx2 = &field2;
+
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = streamInqVlist(streamID2);
+  int vlistIDx1 = vlistID1;
+  int vlistIDx2 = vlistID2;
 
   if ( cdoVerbose ) vlistPrint(vlistID1);
   if ( cdoVerbose ) vlistPrint(vlistID2);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = vlistInqTaxis(vlistID2);
-  taxisIDx1 = taxisID1;
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = vlistInqTaxis(vlistID2);
+  int taxisIDx1 = taxisID1;
 
-  ntsteps1 = vlistNtsteps(vlistID1);
-  ntsteps2 = vlistNtsteps(vlistID2);
+  int ntsteps1 = vlistNtsteps(vlistID1);
+  int ntsteps2 = vlistNtsteps(vlistID2);
   if ( ntsteps1 == 0 ) ntsteps1 = 1;
   if ( ntsteps2 == 0 ) ntsteps2 = 1;
 
@@ -150,12 +148,12 @@ void *Arith(void *argument)
 
   field_init(&field1);
   field_init(&field2);
-  field1.ptr = malloc(gridsize*sizeof(double));
-  field2.ptr = malloc(gridsize*sizeof(double));
+  field1.ptr = (double*) Malloc(gridsize*sizeof(double));
+  field2.ptr = (double*) Malloc(gridsize*sizeof(double));
   if ( filltype == FILL_VAR || filltype == FILL_VARTS )
     {
-      vardata2 = malloc(gridsize*nlevels2*sizeof(double));
-      varnmiss2 = malloc(nlevels2*sizeof(int));
+      vardata2 = (double*) Malloc(gridsize*nlevels2*sizeof(double));
+      varnmiss2 = (int*) Malloc(nlevels2*sizeof(int));
     }
 
   if ( cdoVerbose ) cdoPrint("Number of timesteps: file1 %d, file2 %d", ntsteps1, ntsteps2);
@@ -183,14 +181,14 @@ void *Arith(void *argument)
       if ( filltype == FILL_TS )
 	{
 	  nvars  = vlistNvars(vlistIDx2);
-	  vardata  = malloc(nvars*sizeof(double *));
-	  varnmiss = malloc(nvars*sizeof(int *));
+	  vardata  = (double **) Malloc(nvars*sizeof(double *));
+	  varnmiss = (int **) Malloc(nvars*sizeof(int *));
 	  for ( varID = 0; varID < nvars; varID++ )
 	    {
 	      gridsize = gridInqSize(vlistInqVarGrid(vlistIDx2, varID));
 	      nlev     = zaxisInqSize(vlistInqVarZaxis(vlistIDx2, varID));
-	      vardata[varID]  = malloc(nlev*gridsize*sizeof(double));
-	      varnmiss[varID] = malloc(nlev*sizeof(int));
+	      vardata[varID]  = (double*) Malloc(nlev*gridsize*sizeof(double));
+	      varnmiss[varID] = (int*) Malloc(nlev*sizeof(int));
 	    }
 	}
     }
@@ -253,8 +251,9 @@ void *Arith(void *argument)
       for ( recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamIDx1, &varID, &levelID);
-	  streamReadRecord(streamIDx1, fieldx1->ptr, &fieldx1->nmiss);
-
+	  streamReadRecord(streamIDx1, fieldx1->ptr, &nmiss);
+          fieldx1->nmiss = (size_t) nmiss;
+          
 	  if ( tsID == 0 || filltype == FILL_NONE || filltype == FILL_FILE || filltype == FILL_VARTS )
 	    {
 	      int lstatus = nlevels2 > 1 ? varID == 0 : recID == 0;
@@ -262,7 +261,8 @@ void *Arith(void *argument)
 	      if ( lstatus || (filltype != FILL_VAR && filltype != FILL_VARTS) )
 		{
 		  streamInqRecord(streamIDx2, &varID2, &levelID2);
-		  streamReadRecord(streamIDx2, fieldx2->ptr, &fieldx2->nmiss);
+		  streamReadRecord(streamIDx2, fieldx2->ptr, &nmiss);
+                  fieldx2->nmiss = (size_t) nmiss;
 		}
 
 	      if ( filltype == FILL_TS )
@@ -311,7 +311,7 @@ void *Arith(void *argument)
 	  farfun(&field1, field2, operfunc);
 
 	  streamDefRecord(streamID3, varID, levelID);
-	  streamWriteRecord(streamID3, field1.ptr, field1.nmiss);
+	  streamWriteRecord(streamID3, field1.ptr, (int)field1.nmiss);
 	}
 
       tsID++;
@@ -328,20 +328,20 @@ void *Arith(void *argument)
     {
       for ( varID = 0; varID < nvars; varID++ )
 	{
-	  free(vardata[varID]);
-	  free(varnmiss[varID]);
+	  Free(vardata[varID]);
+	  Free(varnmiss[varID]);
 	}
 
-      free(vardata);
-      free(varnmiss);
+      Free(vardata);
+      Free(varnmiss);
     }
 
-  if ( field1.ptr ) free(field1.ptr);
-  if ( field2.ptr ) free(field2.ptr);
-  if ( vardata2   ) free(vardata2);
-  if ( varnmiss2  ) free(varnmiss2);
+  if ( field1.ptr ) Free(field1.ptr);
+  if ( field2.ptr ) Free(field2.ptr);
+  if ( vardata2   ) Free(vardata2);
+  if ( varnmiss2  ) Free(varnmiss2);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
