@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -29,6 +29,7 @@
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
+#include "calendar.h"
 #include "pstream.h"
 
 
@@ -83,6 +84,7 @@ void *Arithdays(void *argument)
   int vdate, vtime;
   int year, month, day;
   int calendar;
+  int nmiss;
   double rconst;
   field_t field;
 
@@ -116,7 +118,7 @@ void *Arithdays(void *argument)
   gridsize = vlistGridsizeMax(vlistID1);
 
   field_init(&field);
-  field.ptr    = (double*) malloc(gridsize*sizeof(double));
+  field.ptr    = (double*) Malloc(gridsize*sizeof(double));
   field.weight = NULL;
 
   tsID = 0;
@@ -149,15 +151,16 @@ void *Arithdays(void *argument)
       for ( recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
-	  streamReadRecord(streamID1, field.ptr, &field.nmiss);
+	  streamReadRecord(streamID1, field.ptr, &nmiss);
 
+          field.nmiss   = (size_t)nmiss;
 	  field.grid    = vlistInqVarGrid(vlistID1, varID);
 	  field.missval = vlistInqVarMissval(vlistID1, varID);
 
 	  farcfun(&field, rconst, operfunc);
 
 	  streamDefRecord(streamID2, varID, levelID);
-	  streamWriteRecord(streamID2, field.ptr, field.nmiss);
+	  streamWriteRecord(streamID2, field.ptr, (int)field.nmiss);
 	}
       tsID++;
     }
@@ -165,9 +168,9 @@ void *Arithdays(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( field.ptr ) free(field.ptr);
+  if ( field.ptr ) Free(field.ptr);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
