@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -49,12 +49,16 @@ void *CDItest(void *argument)
 
   NCOPY = cdoOperatorAdd("ncopy",   0, 0, NULL);
 
+  UNUSED(NCOPY);
+
   //  if ( UNCHANGED_RECORD ) lcopy = TRUE;
 
   operatorID = cdoOperatorID();
 
+  UNUSED(operatorID);
+
   //  operatorInputArg("Number of copies");
-  if ( operatorArgc() == 1 ) max_copy = atoi(operatorArgv()[0]);
+  if ( operatorArgc() == 1 ) max_copy = parameter2int(operatorArgv()[0]);
 
   processStartTime(&s_utime, &s_stime);
 
@@ -75,7 +79,7 @@ void *CDItest(void *argument)
       streamDefVlist(streamID2, vlistID2);
 
       gridsize = vlistGridsizeMax(vlistID1);
-      array = malloc(gridsize*sizeof(double));
+      array = (double*) Malloc(gridsize*sizeof(double));
 
       tsID1 = 0;
       tsID2 = 0;
@@ -110,7 +114,7 @@ void *CDItest(void *argument)
       vlistDestroy(vlistID2);
       taxisDestroy(taxisID2);
 
-      if ( array ) free(array);
+      if ( array ) Free(array);
 
       n++;
 
@@ -130,5 +134,5 @@ void *CDItest(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
