@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,37 +30,43 @@
 
 void *Cat(void *argument)
 {
-  int streamID1, streamID2 = CDI_UNDEFID;
+  bool lconstvars = true;
   int nrecs;
-  int tsID1, tsID2 = 0, recID, varID, levelID;
-  int vlistID1, vlistID2 = CDI_UNDEFID;
-  int streamCnt, nfiles, indf;
-  int taxisID1, taxisID2 = CDI_UNDEFID;
-  int lcopy = FALSE;
-  int gridsize;
+  int tsID2 = 0, varID, levelID;
+  int streamID2 = CDI_UNDEFID;
+  int vlistID2 = CDI_UNDEFID;
+  int taxisID2 = CDI_UNDEFID;
   int nmiss;
-  int ntsteps, nvars;
+  double tw0 = 0, tw = 0;
   double *array = NULL;
 
   cdoInitialize(argument);
 
-  if ( UNCHANGED_RECORD ) lcopy = TRUE;
+  bool lcopy = false;
+  if ( UNCHANGED_RECORD ) lcopy = true;
 
-  streamCnt = cdoStreamCnt();
-  nfiles = streamCnt - 1;
+  int timer_cat = timer_new("cat");
+  if ( cdoTimer ) timer_start(timer_cat);
 
-  for ( indf = 0; indf < nfiles; indf++ )
+  int streamCnt = cdoStreamCnt();
+  int nfiles = streamCnt - 1;
+
+  for ( int indf = 0; indf < nfiles; ++indf )
     {
       if ( cdoVerbose ) cdoPrint("Process file: %s", cdoStreamName(indf)->args);
+      if ( cdoTimer ) tw0 = timer_val(timer_cat);
 
-      streamID1 = streamOpenRead(cdoStreamName(indf));
+      int streamID1 = streamOpenRead(cdoStreamName(indf));
 
-      vlistID1 = streamInqVlist(streamID1);
-      taxisID1 = vlistInqTaxis(vlistID1);
+      int vlistID1 = streamInqVlist(streamID1);
+      int taxisID1 = vlistInqTaxis(vlistID1);
 
       if ( indf == 0 )
 	{
-	  if ( fileExists(cdoStreamName(nfiles)->args) )
+	  int file_exists = fileExists(cdoStreamName(nfiles)->args);
+	  if ( cdoOverwriteMode ) file_exists = 0;
+
+	  if ( file_exists )
 	    {
 	      streamID2 = streamOpenAppend(cdoStreamName(nfiles));
 
@@ -83,8 +89,8 @@ void *Cat(void *argument)
 	      taxisID2 = taxisDuplicate(taxisID1);
 	      vlistDefTaxis(vlistID2, taxisID2);
 	  
-	      ntsteps = vlistNtsteps(vlistID1);
-	      nvars   = vlistNvars(vlistID1);
+	      int ntsteps = vlistNtsteps(vlistID1);
+	      int nvars   = vlistNvars(vlistID1);
 	      
 	      if ( ntsteps == 1 )
 		{
@@ -95,7 +101,8 @@ void *Cat(void *argument)
 		}
 
 	      if ( ntsteps == 0 && nfiles > 1 )
-		{		  
+		{
+                  lconstvars = false;
 		  for ( varID = 0; varID < nvars; ++varID )
 		    vlistDefVarTsteptype(vlistID2, varID, TSTEP_INSTANT);
 		}
@@ -105,8 +112,8 @@ void *Cat(void *argument)
 
 	  if ( ! lcopy )
 	    {
-	      gridsize = vlistGridsizeMax(vlistID1);
-	      array = malloc(gridsize*sizeof(double));
+	      size_t gridsize = (size_t) vlistGridsizeMax(vlistID1);
+	      array = (double*) Malloc(gridsize*sizeof(double));
 	    }
 	}
       else
@@ -114,16 +121,29 @@ void *Cat(void *argument)
 	  vlistCompare(vlistID1, vlistID2, CMP_ALL);
 	}
 
-      tsID1 = 0;
+      int ntsteps = vlistNtsteps(vlistID1);
+
+      int tsID1 = 0;
       while ( (nrecs = streamInqTimestep(streamID1, tsID1)) )
-	{
+	{          
+	  {
+	    double fstatus = indf+1.;
+	    if ( ntsteps > 1 ) fstatus = indf+(tsID1+1.)/ntsteps;
+	    if ( !cdoVerbose ) progressStatus(0, 1, fstatus/nfiles);
+	  }
+
 	  taxisCopyTimestep(taxisID2, taxisID1);
 
 	  streamDefTimestep(streamID2, tsID2);
 	       
-	  for ( recID = 0; recID < nrecs; recID++ )
+	  for ( int recID = 0; recID < nrecs; recID++ )
 	    {
 	      streamInqRecord(streamID1, &varID, &levelID);
+
+              if ( lconstvars && tsID2 > 0 && tsID1 == 0 )
+                if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT )
+                  continue;
+
 	      streamDefRecord(streamID2,  varID,  levelID);
 
 	      if ( lcopy )
@@ -136,17 +156,24 @@ void *Cat(void *argument)
 		  streamWriteRecord(streamID2, array, nmiss);
 		}
 	    }
+
 	  tsID1++;
 	  tsID2++;
 	}
+      
       streamClose(streamID1);
+
+      if ( cdoTimer ) tw = timer_val(timer_cat) - tw0;
+      if ( cdoTimer ) cdoPrint("Processed file: %s   %.2f seconds", cdoStreamName(indf)->args, tw);
     }
 
   streamClose(streamID2);
  
-  if ( array ) free(array);
+  if ( cdoTimer ) timer_stop(timer_cat);
+
+  if ( array ) Free(array);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
