@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,30 +30,27 @@
 
 void *Copy(void *argument)
 {
-  int SELALL, SZIP;
-  int operatorID;
-  int streamID1, streamID2 = CDI_UNDEFID;
+  bool lconstvars = true;
+  int streamID2 = CDI_UNDEFID;
+  int vlistID2 = CDI_UNDEFID;
+  int taxisID2 = CDI_UNDEFID;
   int nrecs;
-  int tsID1, tsID2, recID, varID, levelID;
-  int lcopy = FALSE;
-  int gridsize;
-  int vlistID1, vlistID2 = CDI_UNDEFID;
+  int recID, varID, levelID;
   int nmiss;
-  int streamCnt, nfiles, indf;
-  int taxisID1, taxisID2 = CDI_UNDEFID;
   int ntsteps, nvars;
   double *array = NULL;
   par_io_t parIO;
 
   cdoInitialize(argument);
 
-            cdoOperatorAdd("copy",   0, 0, NULL);
-  SELALL  = cdoOperatorAdd("selall", 0, 0, NULL);
-  SZIP    = cdoOperatorAdd("szip",   0, 0, NULL);
+                cdoOperatorAdd("copy",   0, 0, NULL);
+  int SELALL  = cdoOperatorAdd("selall", 0, 0, NULL);
+  int SZIP    = cdoOperatorAdd("szip",   0, 0, NULL);
 
-  if ( UNCHANGED_RECORD ) lcopy = TRUE;
+  bool lcopy = false;
+  if ( UNCHANGED_RECORD ) lcopy = true;
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
   if ( operatorID == SZIP )
     {
@@ -61,18 +58,18 @@ void *Copy(void *argument)
       cdoCompLevel = 0;
     }
 
-  streamCnt = cdoStreamCnt();
-  nfiles = streamCnt - 1;
+  int streamCnt = cdoStreamCnt();
+  int nfiles = streamCnt - 1;
 
-  tsID2 = 0;
-  for ( indf = 0; indf < nfiles; indf++ )
+  int tsID2 = 0;
+  for ( int indf = 0; indf < nfiles; indf++ )
     {
       if ( cdoVerbose ) cdoPrint("Process file: %s", cdoStreamName(indf)->args);
 
-      streamID1 = streamOpenRead(cdoStreamName(indf));
+      int streamID1 = streamOpenRead(cdoStreamName(indf));
 
-      vlistID1 = streamInqVlist(streamID1);
-      taxisID1 = vlistInqTaxis(vlistID1);
+      int vlistID1 = streamInqVlist(streamID1);
+      int taxisID1 = vlistInqTaxis(vlistID1);
 
       if ( indf == 0 )
 	{
@@ -95,18 +92,19 @@ void *Copy(void *argument)
 
 	  if ( ntsteps == 0 && nfiles > 1 )
 	    {	      
+              lconstvars = false;
 	      for ( varID = 0; varID < nvars; ++varID )
 		vlistDefVarTsteptype(vlistID2, varID, TSTEP_INSTANT);
 	    }
 
 	  streamDefVlist(streamID2, vlistID2);
 
-	  gridsize = vlistGridsizeMax(vlistID1);
-	  array = (double*) malloc(gridsize*sizeof(double));
+	  int gridsize = vlistGridsizeMax(vlistID1);
+	  array = (double*) Malloc(gridsize*sizeof(double));
 	  if ( cdoParIO )
 	    {
 	      fprintf(stderr, "Parallel reading enabled!\n");
-	      parIO.array = (double*) malloc(gridsize*sizeof(double));
+	      parIO.array = (double*) Malloc(gridsize*sizeof(double));
 	      parIO.array_size = gridsize;
 	    }
 	}
@@ -115,7 +113,7 @@ void *Copy(void *argument)
 	  vlistCompare(vlistID1, vlistID2, CMP_ALL);
 	}
 
-      tsID1 = 0;
+      int tsID1 = 0;
       while ( (nrecs = streamInqTimestep(streamID1, tsID1)) )
 	{
 	  taxisCopyTimestep(taxisID2, taxisID1);
@@ -127,6 +125,11 @@ void *Copy(void *argument)
 	      if ( lcopy && (operatorID == SELALL || operatorID == SZIP) )
 		{
 		  streamInqRecord(streamID1, &varID, &levelID);
+
+                  if ( lconstvars && tsID2 > 0 && tsID1 == 0 )
+                    if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT )
+                      continue;
+                  
 		  streamDefRecord(streamID2,  varID,  levelID);
 		  streamCopyRecord(streamID2, streamID1);
 		}
@@ -142,6 +145,11 @@ void *Copy(void *argument)
 		  else
 		    {
 		      streamInqRecord(streamID1, &varID, &levelID);
+
+                      if ( lconstvars && tsID2 > 0 && tsID1 == 0 )
+                        if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT )
+                          continue;
+
 		      streamReadRecord(streamID1, array, &nmiss);
 		    }
 		  /*
@@ -165,10 +173,10 @@ void *Copy(void *argument)
 
   streamClose(streamID2);
 
-  if ( array ) free(array);
+  if ( array ) Free(array);
   if ( vlistID2 != CDI_UNDEFID ) vlistDestroy(vlistID2);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
