@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,73 +30,63 @@
 
 void *Diff(void *argument)
 {
-  int DIFF, DIFFP, DIFFN, DIFFC;
-  int operatorID;
   int lhead = TRUE;
   int i;
-  int indg;
   int varID1, varID2, recID;
-  int gridsize;
   int ndiff;
   int code, param;
-  int gridID, zaxisID, vdate, vtime;
+  int gridID, zaxisID;
   int checkrel;
   int nrecs, nrecs2;
   int levelID;
-  int tsID;
   int dsgn, zero;
-  int streamID1, streamID2;
-  int vlistID1, vlistID2;
-  int taxisID;
   int nmiss1, nmiss2;
   int ndrec = 0, nd2rec = 0, ngrec = 0;
   char varname[CDI_MAX_NAME];
   char paramstr[32];
   char vdatestr[32], vtimestr[32];
   double absdiff;
-  double abslim = 0., abslim2 = 1.e-3, rellim = 0.5;
+  double abslim = 0., abslim2 = 1.e-3, rellim = 1.0;
   double absm, relm;
   double missval1, missval2;
-  double *array1, *array2;
 
   cdoInitialize(argument);
 
-  DIFF  = cdoOperatorAdd("diff",  0, 0, NULL);
-  DIFFP = cdoOperatorAdd("diffp", 0, 0, NULL);
-  DIFFN = cdoOperatorAdd("diffn", 0, 0, NULL);
-  DIFFC = cdoOperatorAdd("diffc", 0, 0, NULL);
+  int DIFF  = cdoOperatorAdd("diff",  0, 0, NULL);
+  int DIFFP = cdoOperatorAdd("diffp", 0, 0, NULL);
+  int DIFFN = cdoOperatorAdd("diffn", 0, 0, NULL);
+  int DIFFC = cdoOperatorAdd("diffc", 0, 0, NULL);
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
-  if ( operatorArgc() == 1 ) abslim = parameter2double(operatorArgv()[0]);
+  if ( operatorArgc() >= 1 ) abslim = parameter2double(operatorArgv()[0]);
   if ( abslim < -1.e33 || abslim > 1.e+33 ) cdoAbort("Abs. limit out of range\n");
+  if ( operatorArgc() == 2 ) rellim = parameter2double(operatorArgv()[1]);
+  if ( rellim < -1.e33 || rellim > 1.e+33 ) cdoAbort("Rel. limit out of range\n");
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
-  streamID2 = streamOpenRead(cdoStreamName(1));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID2 = streamOpenRead(cdoStreamName(1));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = streamInqVlist(streamID2);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = streamInqVlist(streamID2);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
 
-  gridsize = vlistGridsizeMax(vlistID1);
+  int gridsize = vlistGridsizeMax(vlistID1);
 
-  array1 = (double*) malloc(gridsize*sizeof(double));
-  array2 = (double*) malloc(gridsize*sizeof(double));
+  double *array1 = (double*) Malloc(gridsize*sizeof(double));
+  double *array2 = (double*) Malloc(gridsize*sizeof(double));
 
-  indg = 0;
-  tsID = 0;
-  taxisID = vlistInqTaxis(vlistID1);
+  int indg = 0;
+  int tsID = 0;
+  int taxisID = vlistInqTaxis(vlistID1);
   while ( TRUE )
     {
       nrecs = streamInqTimestep(streamID1, tsID);
       if ( nrecs > 0 )
 	{
-	  vdate = taxisInqVdate(taxisID);
-	  vtime = taxisInqVtime(taxisID);
-	  
-	  date2str(vdate, vdatestr, sizeof(vdatestr));
-	  time2str(vtime, vtimestr, sizeof(vtimestr));
+	  date2str(taxisInqVdate(taxisID), vdatestr, sizeof(vdatestr));
+	  time2str(taxisInqVtime(taxisID), vtimestr, sizeof(vtimestr));
 	}
 
       nrecs2 = streamInqTimestep(streamID2, tsID);
@@ -118,7 +108,8 @@ void *Diff(void *argument)
 	  missval1 = vlistInqVarMissval(vlistID1, varID1);
 	  missval2 = vlistInqVarMissval(vlistID2, varID2);
 
-	  checkrel = gridInqType(gridID) != GRID_SPECTRAL;
+	  //checkrel = gridInqType(gridID) != GRID_SPECTRAL;
+          checkrel = TRUE;
 
 	  cdiParamToString(param, paramstr, sizeof(paramstr));
 
@@ -163,7 +154,7 @@ void *Diff(void *argument)
 
 	  if ( ! cdoSilentMode || cdoVerbose )
 	    {
-	      if ( absm > abslim || (checkrel && relm > rellim) || cdoVerbose )
+	      if ( absm > abslim || (checkrel && relm >= rellim) || cdoVerbose )
 		{
 		  if ( lhead )
 		    {
@@ -224,8 +215,8 @@ void *Diff(void *argument)
 	    }
 
 	  ngrec++;
-	  if ( absm > abslim  || (checkrel && relm > rellim) ) ndrec++;
-	  if ( absm > abslim2 || (checkrel && relm > rellim) ) nd2rec++;
+	  if ( absm > abslim  || (checkrel && relm >= rellim) ) ndrec++;
+	  if ( absm > abslim2 || (checkrel && relm >= rellim) ) nd2rec++;
 	}
       tsID++;
     }
@@ -250,10 +241,10 @@ void *Diff(void *argument)
   streamClose(streamID1);
   streamClose(streamID2);
 
-  if ( array1 ) free(array1);
-  if ( array2 ) free(array2);
+  if ( array1 ) Free(array1);
+  if ( array2 ) Free(array2);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
