@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -38,13 +38,13 @@ void *Duplicate(void *argument)
   int nvars, nlevel;
   int ntsteps;
   int *vdate = NULL, *vtime = NULL;
-  int idup, ndup = 1;
+  int idup, ndup = 2;
   field_t ***vars = NULL;
 
   cdoInitialize(argument);
 
   if ( operatorArgc() > 1 ) cdoAbort("Too many arguments!");
-  else if ( operatorArgc() == 1 ) ndup = atoi(operatorArgv()[0]);
+  else if ( operatorArgc() == 1 ) ndup = parameter2int(operatorArgv()[0]);
 
   if ( cdoVerbose ) cdoPrint("ndup = %d\n", ndup);
 
@@ -86,9 +86,9 @@ void *Duplicate(void *argument)
       if ( tsID >= nalloc )
 	{
 	  nalloc += NALLOC_INC;
-	  vdate = realloc(vdate, nalloc*sizeof(int));
-	  vtime = realloc(vtime, nalloc*sizeof(int));
-	  vars  = realloc(vars, nalloc*sizeof(field_t **));
+	  vdate = (int*) Realloc(vdate, nalloc*sizeof(int));
+	  vtime = (int*) Realloc(vtime, nalloc*sizeof(int));
+	  vars  = (field_t ***) Realloc(vars, nalloc*sizeof(field_t **));
 	}
 
       vdate[tsID] = taxisInqVdate(taxisID1);
@@ -101,7 +101,7 @@ void *Duplicate(void *argument)
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  gridID   = vlistInqVarGrid(vlistID1, varID);
 	  gridsize = gridInqSize(gridID);
-	  vars[tsID][varID][levelID].ptr = malloc(gridsize*sizeof(double));
+	  vars[tsID][varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
 	  streamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
 	  vars[tsID][varID][levelID].nmiss = nmiss;
 	}
@@ -137,14 +137,14 @@ void *Duplicate(void *argument)
 
   for ( tsID = 0; tsID < nts; tsID++ ) field_free(vars[tsID], vlistID1);
 
-  if ( vars  ) free(vars);
-  if ( vdate ) free(vdate);
-  if ( vtime ) free(vtime);
+  if ( vars  ) Free(vars);
+  if ( vdate ) Free(vdate);
+  if ( vtime ) Free(vtime);
 
   streamClose(streamID2);
   streamClose(streamID1);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
