@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -54,7 +54,7 @@ void scale_eigvec_time(double *restrict out, int tsID, int nts, int npack, const
 		       double **covar, double **data, double missval, double sum_w)
 {
 #if defined(_OPENMP)
-#pragma omp parallel for shared(tsID, data, out)
+#pragma omp parallel for default(none) shared(npack, nts, tsID, pack, data, covar, out)
 #endif
   for ( int i = 0; i < npack; ++i )
     {
@@ -81,7 +81,7 @@ void scale_eigvec_time(double *restrict out, int tsID, int nts, int npack, const
 #endif
   for ( int i = 0; i < npack; ++i )
     {
-      // do not need to account for weights as eigenvectors are non-weighted                                   
+      // do not need to account for weights as eigenvectors are non-weighted                               
       sum += weight[pack[i]] * out[pack[i]] * out[pack[i]];
     }
 
@@ -221,7 +221,7 @@ void *EOFs(void * argument)
   int gridID1   = vlistInqVarGrid(vlistID1, 0);
   int gridsize  = vlistGridsizeMax(vlistID1);
   int nvars     = vlistNvars(vlistID1);
-  int nrecs     = vlistNrecs(vlistID1);
+  int nrecs;
 
   int ngrids = vlistNgrids(vlistID1);
   for ( int index = 1; index < ngrids; index++ )
@@ -230,7 +230,7 @@ void *EOFs(void * argument)
 	cdoAbort("Too many different grids!");
       }
 
-  double *weight = (double *) malloc(gridsize*sizeof(double));
+  double *weight = (double *) Malloc(gridsize*sizeof(double));
   for ( i = 0; i < gridsize; ++i ) weight[i] = 1.;
 
   if ( weight_mode == WEIGHT_ON )
@@ -323,18 +323,17 @@ void *EOFs(void * argument)
 
   /* allocation of temporary fields and output structures */
   int npack = -1;
-  int *pack            = (int *) malloc(gridsize*sizeof(int));
-  double *in           = (double *) malloc(gridsize*sizeof(double));
-  eofdata_t **eofdata  = (eofdata_t **) malloc(nvars*sizeof(eofdata_t*));
+  int *pack            = (int *) Malloc(gridsize*sizeof(int));
+  double *in           = (double *) Malloc(gridsize*sizeof(double));
+  eofdata_t **eofdata  = (eofdata_t **) Malloc(nvars*sizeof(eofdata_t*));
 
   for ( varID = 0; varID < nvars; ++varID )
     {
-      gridID1  = vlistInqVarGrid(vlistID1, varID);
       gridsize = vlistGridsizeMax(vlistID1);
       nlevs    = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
       missval  = vlistInqVarMissval(vlistID1, varID);
 
-      eofdata[varID] = (eofdata_t *) malloc(nlevs*sizeof(eofdata_t));
+      eofdata[varID] = (eofdata_t *) Malloc(nlevs*sizeof(eofdata_t));
 
       for ( levelID = 0; levelID < nlevs; ++levelID )
         {
@@ -346,7 +345,7 @@ void *EOFs(void * argument)
 	  eofdata[varID][levelID].data = NULL;
 
 	  if ( time_space )
-	    eofdata[varID][levelID].data = (double **) malloc(nts*sizeof(double *));
+	    eofdata[varID][levelID].data = (double **) Malloc(nts*sizeof(double *));
         }
     }
 
@@ -409,8 +408,8 @@ void *EOFs(void * argument)
 	      if ( !eofdata[varID][levelID].init )
 		{
 		  n = npack;
-		  double *covar_array = (double *) malloc(npack*npack*sizeof(double));
-		  covar = (double **) malloc(npack*sizeof(double *));
+		  double *covar_array = (double *) Malloc(npack*npack*sizeof(double));
+		  covar = (double **) Malloc(npack*sizeof(double *));
 		  for ( i = 0; i < npack; ++i ) covar[i] = covar_array + npack*i;
 		  for ( i = 0; i < npack; ++i )
 		    {
@@ -435,7 +434,7 @@ void *EOFs(void * argument)
 	    }
           else if ( time_space )
 	    {
-	      double *data = (double *) malloc(npack*sizeof(double));
+	      double *data = (double *) Malloc(npack*sizeof(double));
 	      eofdata[varID][levelID].data[tsID] = data;
 
 	      for ( ipack = 0; ipack < npack; ipack++ )
@@ -536,7 +535,7 @@ void *EOFs(void * argument)
 		    {
 		      if ( npack )
 			{
-			  eig_val = (double *) malloc(npack*sizeof(double));
+			  eig_val = (double *) Malloc(npack*sizeof(double));
 			  eofdata[varID][levelID].eig_val = eig_val;
 			}
 
@@ -567,11 +566,11 @@ void *EOFs(void * argument)
 		      if ( cdoVerbose )
 			cdoPrint("allocating covar with %i x %i elements | npack=%i", nts, nts, npack);
 
-		      covar_array = (double *) malloc(nts*nts*sizeof(double));
-		      covar = (double **) malloc(nts*sizeof(double *));
+		      covar_array = (double *) Malloc(nts*nts*sizeof(double));
+		      covar = (double **) Malloc(nts*sizeof(double *));
 		      for ( i = 0; i < nts; ++i ) covar[i] = covar_array + nts*i;
 
-		      eig_val = (double *) malloc(nts*sizeof(double));
+		      eig_val = (double *) Malloc(nts*sizeof(double));
 		      eofdata[varID][levelID].eig_val     = eig_val;
 		      eofdata[varID][levelID].covar_array = covar_array;
 		      eofdata[varID][levelID].covar       = covar;
@@ -647,24 +646,24 @@ void *EOFs(void * argument)
       
       for( levelID = 0; levelID < nlevs; levelID++ )
         { 	  
-	  if ( eofdata[varID][levelID].eig_val ) free(eofdata[varID][levelID].eig_val);
-	  if ( eofdata[varID][levelID].covar_array ) free(eofdata[varID][levelID].covar_array);
-	  if ( eofdata[varID][levelID].covar ) free(eofdata[varID][levelID].covar);
+	  if ( eofdata[varID][levelID].eig_val ) Free(eofdata[varID][levelID].eig_val);
+	  if ( eofdata[varID][levelID].covar_array ) Free(eofdata[varID][levelID].covar_array);
+	  if ( eofdata[varID][levelID].covar ) Free(eofdata[varID][levelID].covar);
 	  if ( time_space && eofdata[varID][levelID].data )
 	    {
 	      for ( tsID = 0; tsID < nts; tsID++ )
-		if ( eofdata[varID][levelID].data[tsID] ) free(eofdata[varID][levelID].data[tsID]);
-	      free(eofdata[varID][levelID].data);
+		if ( eofdata[varID][levelID].data[tsID] ) Free(eofdata[varID][levelID].data[tsID]);
+	      Free(eofdata[varID][levelID].data);
 	    }
 	}
       
-      free(eofdata[varID]);
+      Free(eofdata[varID]);
     }
 
-  free(eofdata);
-  free(in);
-  free(pack);
-  free(weight);
+  Free(eofdata);
+  Free(in);
+  Free(pack);
+  Free(weight);
 
   streamClose(streamID3);
   streamClose(streamID2);
@@ -680,5 +679,5 @@ void *EOFs(void * argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
