@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -189,8 +189,6 @@ void *FC(void *argument)
 	    }
 
 	  gridID2 = gridIDfc;
-	  nfc  = gridInqSize(gridID2);
-
 	  sptrans = sptrans_new(nlon, nlat, ntr, 0);
  	}
     }
@@ -230,7 +228,7 @@ void *FC(void *argument)
   // printf("nfc %d, ntr %d, nlat %d, nlon %d\n", nfc, ntr, nlat, nlon);
 
   nvars = vlistNvars(vlistID2);
-  vars  = (int*) malloc(nvars*sizeof(int));
+  vars  = (int*) Malloc(nvars*sizeof(int));
   for ( varID = 0; varID < nvars; varID++ )
     {
       if ( gridID1 == vlistInqVarGrid(vlistID1, varID) )
@@ -246,12 +244,12 @@ void *FC(void *argument)
   streamDefVlist(streamID2, vlistID2);
 
   gridsize = vlistGridsizeMax(vlistID1);
-  array1 = (double*) malloc(gridsize*sizeof(double));
+  array1 = (double*) Malloc(gridsize*sizeof(double));
 
   if ( gridID2 != -1 )
     {
       gridsize = gridInqSize(gridID2);
-      array2 = (double*) malloc(gridsize*sizeof(double));
+      array2 = (double*) Malloc(gridsize*sizeof(double));
     }
 
   tsID = 0;
@@ -303,13 +301,13 @@ void *FC(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( array2 ) free(array2);
-  if ( array1 ) free(array1);
-  if ( vars )   free(vars);
+  if ( array2 ) Free(array2);
+  if ( array1 ) Free(array1);
+  if ( vars )   Free(vars);
 
   sptrans_delete(sptrans);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
