@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -81,20 +81,46 @@ void printAtts(FILE *fp, int vlistID, int varID)
 }
 
 static
-void partab(FILE *fp, int vlistID, int option)
+void printHistory(FILE *fp, int streamID)
 {
+  int fileID = pstreamFileID(streamID);
+  size_t historysize = (size_t) streamInqHistorySize(fileID);
+  if ( historysize > 0 )
+    {
+      char *history = (char*) Malloc(historysize+1);
+      history[historysize] = 0;
+      streamInqHistoryString(fileID, history);
+      fprintf(fp, "  history=%s\n", history);
+    }
+}
+
+static
+void printSource(FILE *fp, int vlistID, int varID)
+{
+  /* institute info */
+  const char *instptr = institutInqLongnamePtr(vlistInqVarInstitut(vlistID, varID));
+  if ( instptr ) fprintf(fp, "  institution=%s\n", instptr);
+
+  /* source info */
+  const char *modelptr = modelInqNamePtr(vlistInqVarModel(vlistID, varID));
+  if ( modelptr ) fprintf(fp, "  source=%s\n", modelptr);
+}
+
+static
+void partab(FILE *fp, int streamID, int option)
+{
+  int vlistID = streamInqVlist(streamID);
   int varID, datatype = -1;
   int param;
   char pstr[32];
   char paramstr[32];
   char varname[CDI_MAX_NAME], varlongname[CDI_MAX_NAME], varstdname[CDI_MAX_NAME], varunits[CDI_MAX_NAME];
   int natts;
-  int nvars;
   int chunktype;
   int linebreak = 1;
   double missval;
       
-  nvars  = vlistNvars(vlistID);
+  int nvars = vlistNvars(vlistID);
   if ( option == 4 ) linebreak = 0;
 
   if ( option == 2 )
@@ -104,6 +130,8 @@ void partab(FILE *fp, int vlistID, int option)
 	{
 	  fprintf(fp, "&parameter\n");
 	  fprintf(fp, "  name=_GLOBAL_\n");
+          printHistory(fp, streamID);
+          printSource(fp, vlistID, 0);
 	  printAtts(fp, vlistID, CDI_GLOBAL);
 	  fprintf(fp, "/\n");
 	}
@@ -229,16 +257,16 @@ void filedes(int streamID)
       printf("  GRIB2 data\n");
       break;
     case FILETYPE_NC:
-      printf("  netCDF data\n");
+      printf("  NetCDF data\n");
       break;
     case FILETYPE_NC2:
-      printf("  netCDF2 data\n");
+      printf("  NetCDF2 data\n");
       break;
     case FILETYPE_NC4:
-      printf("  netCDF4 data\n");
+      printf("  NetCDF4 data\n");
       break;
     case FILETYPE_NC4C:
-      printf("  netCDF4 classic data\n");
+      printf("  NetCDF4 classic data\n");
       break;
     case FILETYPE_SRV:
       printf("  SERVICE data\n");
@@ -410,11 +438,10 @@ void *Filedes(void *argument)
   else if ( operatorID == PARTAB || operatorID == SPARTAB || operatorID == PARTAB2 )
     {
       int option = 1;
-
       if ( operatorID == SPARTAB ) option = 4;
       if ( operatorID == PARTAB2 ) option = 2;
       
-      partab(stdout, vlistID, option);
+      partab(stdout, streamID, option);
     }
   else if ( operatorID == FILEDES )
     {
@@ -425,5 +452,5 @@ void *Filedes(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
