@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -680,27 +680,24 @@ void ctl_options(FILE *gdp, int yrev, int zrev, int sequential, int bigendian, i
 static
 void ctl_undef(FILE *gdp, int vlistID)
 {
-  double missval;
-
-  missval = vlistInqVarMissval(vlistID, 0);
+  double missval = vlistInqVarMissval(vlistID, 0);
   fprintf(gdp, "UNDEF  %g\n", missval);
 }
 
 static
 void ctl_vars(FILE *gdp, int filetype, int vlistID, int nvarsout, int *vars)
 {
-  int varID, nvars;
   int ltype, code;
   int zaxisID, nlev;
   int i, j;
   int len;
   char varname[CDI_MAX_NAME], varlongname[CDI_MAX_NAME], varunits[CDI_MAX_NAME];
 
-  nvars   = vlistNvars(vlistID);
+  int nvars = vlistNvars(vlistID);
 
   fprintf(gdp, "VARS  %d\n", nvarsout);
 
-  for ( varID = 0; varID < nvars; varID++ )
+  for ( int varID = 0; varID < nvars; varID++ )
     {
       if ( vars[varID] == TRUE )
         {
@@ -1043,7 +1040,7 @@ void *Gradsdes(void *argument)
        filetype != FILETYPE_GRB )
     {
       if ( filetype == FILETYPE_NC )
-        //        cdoAbort("Unsupported file format: netCDF");
+        //        cdoAbort("Unsupported file format: NetCDF");
         ;
       else if ( filetype == FILETYPE_GRB2 )
         //cdoAbort("Unsupported file format: GRIB2");
@@ -1193,7 +1190,7 @@ void *Gradsdes(void *argument)
     }
   else if ( filetype == FILETYPE_NC )
     {
-      fprintf(gdp, "DTYPE  netCDF\n");
+      fprintf(gdp, "DTYPE  NetCDF\n");
     }
 
   /* XYHEADER */
