@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -58,8 +58,8 @@ void *Harmonic(void *argument)
 
   operatorCheckArgc(2);
 
-  n_out = atoi(operatorArgv()[0]);
-  n     = atoi(operatorArgv()[1]);
+  n_out = parameter2int(operatorArgv()[0]);
+  n     = parameter2int(operatorArgv()[1]);
 
   if ( n_out > 9 ) cdoAbort("Maximum number of wave numbers is 9!");
 
@@ -80,7 +80,7 @@ void *Harmonic(void *argument)
   filesuffix[0] = 0;
   cdoGenFileSuffix(filesuffix, sizeof(filesuffix), streamInqFiletype(streamID1), vlistID1, refname);
 
-  streamIDs = malloc(n_out*sizeof(int));
+  streamIDs = (int*) Malloc(n_out*sizeof(int));
 
   strcpy(filename, cdoStreamName(1)->args);
   nchars = strlen(filename);
@@ -102,35 +102,35 @@ void *Harmonic(void *argument)
 
   nvars = vlistNvars(vlistID1);
 
-  out  = malloc(n_out*sizeof(double **));
-  work = malloc(2*n_out*sizeof(double **));
+  out  = (double ***) Malloc(n_out*sizeof(double **));
+  work = (double ***) Malloc(2*n_out*sizeof(double **));
 
   for ( j = 0; j < n_out; ++j )
     {
-      out[j] = malloc(nvars*sizeof(double *));
+      out[j] = (double **) Malloc(nvars*sizeof(double *));
       for ( varID = 0; varID < nvars; ++varID )
 	{
 	  gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
 	  nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-	  out[j][varID] = malloc(gridsize*nlevel*sizeof(double));
+	  out[j][varID] = (double*) Malloc(gridsize*nlevel*sizeof(double));
 	}
     }
 
   for ( j = 0; j < n_out*2; ++j )
     {
-      work[j] = malloc(nvars*sizeof(double *));
+      work[j] = (double **) Malloc(nvars*sizeof(double *));
       for ( varID = 0; varID < nvars; ++varID )
 	{
 	  gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
 	  nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
-	  work[j][varID] = malloc(gridsize*nlevel*sizeof(double));
+	  work[j][varID] = (double*) Malloc(gridsize*nlevel*sizeof(double));
 	  memset(work[j][varID], 0, gridsize*nlevel*sizeof(double));
 	}
     }
 
 
   gridsize = vlistGridsizeMax(vlistID1);
-  array = malloc(gridsize*sizeof(double));
+  array = (double*) Malloc(gridsize*sizeof(double));
 
   tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
@@ -145,7 +145,6 @@ void *Harmonic(void *argument)
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-	  nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	  offset   = gridsize*levelID;
 
 	  streamReadRecord(streamID1, array, &nmiss);
@@ -169,7 +168,7 @@ void *Harmonic(void *argument)
 
   nts = tsID;
 
-  if ( array ) free(array);
+  if ( array ) Free(array);
 
   streamClose(streamID1);
 
@@ -289,29 +288,29 @@ void *Harmonic(void *argument)
       streamClose(streamID2);
     }
 
-  free(streamIDs);
+  Free(streamIDs);
 
   for ( j = 0; j < n_out; ++j )
     {
       for ( varID = 0; varID < nvars; ++varID )
-	free(out[j][varID]);
+        Free(out[j][varID]);
 
-      free(out[j]);
+      Free(out[j]);
     }
 
-  free(out);
+  Free(out);
 
   for ( j = 0; j < n_out*2; ++j )
     {
       for ( varID = 0; varID < nvars; ++varID )
-	free(work[j][varID]);
+        Free(work[j][varID]);
 
-      free(work[j]);
+      Free(work[j]);
     }
 
-  free(work);
+  Free(work);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
