@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -68,14 +68,12 @@ typedef struct {
 }
 datasets_t;
 
-#define  NINT(x)   ((x) < 0 ? (int)((x)-0.5) : (int)((x)+0.5))
 
 #if defined(HAVE_LIBHDF5)
 static
 void print_filter(hid_t dset_id, char *varname)
 {
   hid_t plist;
-  H5Z_filter_t filter;
   unsigned int flags;
   int idx;
   unsigned int cd_values;
@@ -90,8 +88,7 @@ void print_filter(hid_t dset_id, char *varname)
 
   for ( idx = 0; idx < nfilter; idx++ )
     {
-      filter = H5Pget_filter(plist, idx, &flags, &cd_nelmts, &cd_values, 
-			     pnamelen, pname);
+      H5Pget_filter(plist, idx, &flags, &cd_nelmts, &cd_values, pnamelen, pname);
       cdoPrint("Dataset %s: filter %d =  %s", varname, idx+1, pname);
     }
 
@@ -99,8 +96,7 @@ void print_filter(hid_t dset_id, char *varname)
 }
 
 static
-void get_grid_info(double c0, double re, int *nrxp, int *nryp,
-		   double *r0p, double *s0p, double *cp)
+void get_grid_info(double c0, double re, int *nrxp, int *nryp, double *r0p, double *s0p, double *cp)
 {
   const double pi = M_PI;
   double git, phi, s90;
@@ -109,7 +105,7 @@ void get_grid_info(double c0, double re, int *nrxp, int *nryp,
 
   git=2.*pi*re*cos(pi/6.)/c0;
   /* number of longitude pixels */
-  nrx=2*NINT(0.5*git);
+  nrx=2*(int)lround(0.5*git);
 
   /* central index in longitude */
   r0=nrx/2+0.5;
@@ -177,10 +173,10 @@ int defLonLatGrid(int nx, int ny, double c0, double lts, double re)
       return(-1);
     }
 
-  xvals = malloc(nx*sizeof(double));
-  yvals = malloc(ny*sizeof(double));
-  xbounds = malloc(nx*2*sizeof(double));
-  ybounds = malloc(nx*2*sizeof(double));
+  xvals = (double*) Malloc(nx*sizeof(double));
+  yvals = (double*) Malloc(ny*sizeof(double));
+  xbounds = (double*) Malloc(nx*2*sizeof(double));
+  ybounds = (double*) Malloc(nx*2*sizeof(double));
 
   for ( i = 0; i < nx; ++i )
     {
@@ -215,73 +211,73 @@ int defLonLatGrid(int nx, int ny, double c0, double lts, double re)
   gridDefXbounds(gridID, xbounds);
   gridDefYbounds(gridID, ybounds);
   */
-  free(xvals);
-  free(yvals);
-  free(xbounds);
-  free(ybounds);
+  Free(xvals);
+  Free(yvals);
+  Free(xbounds);
+  Free(ybounds);
 
   return (gridID);
 }
 
 static
-int defSinusoidalGrid(int nx, int ny, double xmin, double xmax, double ymin, double ymax, 
+int defSinusoidalGrid(int nx, int ny, double xmin, double xmax, double ymin, double ymax,
 		      double dx, double dy, double p1, double p2, double p3, double p4)
 {
-  int gridID;
-  int i;
-  double *xvals, *yvals;
+  UNUSED(p1);
+  UNUSED(p2);
+  UNUSED(p3);
+  UNUSED(p4);
+  UNUSED(xmax);
+  UNUSED(ymin);
+  double *xvals = (double*) Malloc(nx*sizeof(double));
+  double *yvals = (double*) Malloc(ny*sizeof(double));
 
-  xvals = malloc(nx*sizeof(double));
-  yvals = malloc(ny*sizeof(double));
-
-  for ( i = 0; i < nx; ++i )
+  for ( int i = 0; i < nx; ++i )
     {
       xvals[i] = xmin + i*dx + dx/2;
       /* printf("x[%d]=%g\n", i, xvals[i]); */
     }
 
-  for ( i = 0; i < ny; ++i )
+  for ( int i = 0; i < ny; ++i )
     {
-      yvals[i] = ymax - i*dx - dx/2;
+      yvals[i] = ymax - i*dy - dy/2;
       /* printf("y[%d]=%g\n", i, yvals[i]); */
     }
 
-  gridID = gridCreate(GRID_SINUSOIDAL, nx*ny);
+  int gridID = gridCreate(GRID_SINUSOIDAL, nx*ny);
   gridDefXsize(gridID, nx);
   gridDefYsize(gridID, ny);
   gridDefXvals(gridID, xvals);
   gridDefYvals(gridID, yvals);
 
-  free(xvals);
-  free(yvals);
+  Free(xvals);
+  Free(yvals);
 
   return (gridID);
 }
 
 static
-int defLaeaGrid(int nx, int ny, double xmin, double xmax, double ymin, double ymax, 
+int defLaeaGrid(int nx, int ny, double xmin, double xmax, double ymin, double ymax,
 		double dx, double dy, double a, double lon0, double lat0)
 {
-  int gridID;
-  int i;
-  double *xvals, *yvals;
+  UNUSED(xmax);
+  UNUSED(ymin);
+  double *xvals = (double*) Malloc(nx*sizeof(double));
+  double *yvals = (double*) Malloc(ny*sizeof(double));
 
-  xvals = malloc(nx*sizeof(double));
-  yvals = malloc(ny*sizeof(double));
-
-  for ( i = 0; i < nx; ++i )
+  for ( int i = 0; i < nx; ++i )
     {
       xvals[i] = xmin + i*dx + dx/2;
       /* printf("x[%d]=%g\n", i, xvals[i]); */
     }
 
-  for ( i = 0; i < ny; ++i )
+  for ( int i = 0; i < ny; ++i )
     {
-      yvals[i] = ymax - i*dx - dx/2;
+      yvals[i] = ymax - i*dy - dy/2;
       /* printf("y[%d]=%g\n", i, yvals[i]); */
     }
 
-  gridID = gridCreate(GRID_LAEA, nx*ny);
+  int gridID = gridCreate(GRID_LAEA, nx*ny);
   gridDefXsize(gridID, nx);
   gridDefYsize(gridID, ny);
   gridDefXvals(gridID, xvals);
@@ -289,8 +285,8 @@ int defLaeaGrid(int nx, int ny, double xmin, double xmax, double ymin, double ym
 
   gridDefLaea(gridID, a, lon0, lat0);
 
-  free(xvals);
-  free(yvals);
+  Free(xvals);
+  Free(yvals);
 
   return (gridID);
 }
@@ -358,7 +354,6 @@ int read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
   hid_t proj_tid, region_tid;
   hid_t str_tid, fltarr_tid;
   hid_t ptype_id;
-  herr_t     status;
   hsize_t dims;
   int xsize, ysize;
   typedef struct proj_t {
@@ -404,7 +399,7 @@ int read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
   H5Tinsert(proj_tid, "Reference ellipsoid", HOFFSET(proj_t, ellipsoid), str_tid);
   H5Tinsert(proj_tid, "Projection parameter", HOFFSET(proj_t, parameter), fltarr_tid);
 
-  if ( projection_name ) free(projection_name);
+  if ( projection_name ) Free(projection_name);
 
   grp_id = H5Gopen(loc_id, "Geolocation");
 
@@ -428,7 +423,7 @@ int read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
   if ( proj_id < 0 )
     memset(&proj, 0, sizeof(proj_t));
   else
-    status = H5Dread(proj_id, proj_tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, &proj);
+    H5Dread(proj_id, proj_tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, &proj);
 
   H5Dclose(proj_id);
   H5Tclose(proj_tid);
@@ -436,9 +431,9 @@ int read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
   H5Tclose(fltarr_tid);
 
   if ( cdoVerbose )
-    cdoPrint("  Projection: name=%s\n\t\t\tellipsoid=%s\n\t\t\tparameter=%g %g %g %g %g %g",  
-	     proj.name, proj.ellipsoid, 
-	     proj.parameter[0], proj.parameter[1], proj.parameter[2], 
+    cdoPrint("  Projection: name=%s\n\t\t\tellipsoid=%s\n\t\t\tparameter=%g %g %g %g %g %g",
+	     proj.name, proj.ellipsoid,
+	     proj.parameter[0], proj.parameter[1], proj.parameter[2],
 	     proj.parameter[3], proj.parameter[4], proj.parameter[5]);
 
   region_tid = H5Tcreate(H5T_COMPOUND, sizeof(region_t));
@@ -452,11 +447,11 @@ int read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
   region_id = H5Dopen(grp_id, "Region");
   if ( region_id < 0 )
     region_id = H5Dopen(grp_id, "region");
-  
+
   if ( region_id < 0 )
     memset(&region, 0, sizeof(region_t));
   else
-    status = H5Dread(region_id, region_tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, &region);
+    H5Dread(region_id, region_tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, &region);
 
   H5Dclose(region_id);
   H5Tclose(region_tid);
@@ -469,15 +464,15 @@ int read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
       if ( cdoVerbose ) cdoPrint("  Swap xmin/xmax");
     }
 
-  if ( cdoVerbose ) 
+  if ( cdoVerbose )
     cdoPrint("  Region: xmin=%g xmax=%g ymin=%g ymax=%g dx=%g dy=%g",
 	     region.xmin, region.xmax, region.ymin, region.ymax, region.dx, region.dy);
 
   H5Gclose(grp_id);
 
   /* check region */
-  xsize = NINT((region.xmax-region.xmin)/region.dx);
-  ysize = NINT((region.ymax-region.ymin)/region.dy);
+  xsize = (int)lround((region.xmax-region.xmin)/region.dx);
+  ysize = (int)lround((region.ymax-region.ymin)/region.dy);
 
   if ( cdoVerbose ) cdoPrint("  Size: xsize=%d  ysize=%d", xsize, ysize);
 
@@ -490,43 +485,43 @@ int read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
        (int)region.ymin == 8887500 && (int)region.ymax == 8887500 &&
        (int)region.dx == 15000 && (int)region.dy  == 15000 )
     {
-       region.xmax = 8887500.0;
+       region.xmax =  8887500.0;
        region.ymin = -8887500.0;
        if ( cdoVerbose )
          cdoPrint("  Corrected region: xmin=%g xmax=%g ymin=%g ymax=%g dx=%g dy=%g",
                 region.xmin, region.xmax, region.ymin, region.ymax, region.dx, region.dy);
 
-       xsize = NINT((region.xmax-region.xmin)/region.dx);
-       ysize = NINT((region.ymax-region.ymin)/region.dy);
+       xsize = (int)lround((region.xmax-region.xmin)/region.dx);
+       ysize = (int)lround((region.ymax-region.ymin)/region.dy);
        if ( cdoVerbose ) cdoPrint("  Corrected size: xsize=%d  ysize=%d", xsize, ysize);
     }
 
   if (nx == 298 && ny == 371 &&
-      (int)region.xmin == -6709222 &&  (int)region.xmax == 6709222 &&
+      (int)region.xmin == -6709222 && (int)region.xmax == 6709222 &&
       (int)region.ymin == -6664078 && (int)region.ymax == 9984898 &&
       (int)region.dx == 45000 && (int)region.dy  == 45000 )
     {
         region.xmin = -6705000;
-        region.xmax = 6705000;
+        region.xmax =  6705000;
         region.ymin = -6705000;
-        region.ymax = 9990000;      
+        region.ymax =  9990000;
         cdoPrint("  Corrected region: xmin=%g xmax=%g ymin=%g ymax=%g dx=%g dy=%g",
                region.xmin, region.xmax, region.ymin, region.ymax, region.dx, region.dy);
 
-        xsize = NINT((region.xmax-region.xmin)/region.dx);
-        ysize = NINT((region.ymax-region.ymin)/region.dy);
+        xsize = (int)lround((region.xmax-region.xmin)/region.dx);
+        ysize = (int)lround((region.ymax-region.ymin)/region.dy);
         if ( cdoVerbose ) cdoPrint("  Corrected size: xsize=%d  ysize=%d", xsize, ysize);
     }
 
-  if ( strcmp(proj.name, "sinusoidal") != 0  && 
+  if ( strcmp(proj.name, "sinusoidal") != 0  &&
        ( (nx == xsize && ny == ysize &&
-	  (int)region.xmin == -8887500 && (int)region.xmax == 8887500 && 
-	  (int)region.ymin == -8887500 && (int)region.ymax == 8887500 && 
+	  (int)region.xmin == -8887500 && (int)region.xmax == 8887500 &&
+	  (int)region.ymin == -8887500 && (int)region.ymax == 8887500 &&
 	  (int)region.dx == 15000 && (int)region.dy  == 15000 )
 	 ||
 	 (nx == xsize && ny == ysize &&
-	  (int)region.xmin == -5827500 && (int)region.xmax == 5827500 && 
-	  (int)region.ymin ==  3307500 && (int)region.ymax == 8887500 && 
+	  (int)region.xmin == -5827500 && (int)region.xmax == 5827500 &&
+	  (int)region.ymin ==  3307500 && (int)region.ymax == 8887500 &&
 	  (int)region.dx == 15000 && (int)region.dy  == 15000 )
 	 ||
 	 (nx == xsize && ny == ysize &&
@@ -542,7 +537,7 @@ int read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
 	 (nx == 298 && ny == 371 &&
 	  (int)region.xmin == -6709222 && (int)region.xmax == 6709222 &&
 	  (int)region.ymin == -6664078 && (int)region.ymax == 9984898 &&
-	  (int)region.dx == 45000 && (int)region.dy  == 45000 ) 
+	  (int)region.dx == 45000 && (int)region.dy  == 45000 )
 	 ||
 	 (nx == xsize && ny == ysize &&
 	  (int)region.xmin == -6705000 && (int)region.xmax == 6705000 &&
@@ -552,25 +547,25 @@ int read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
       if ( cdoVerbose ) cdoPrint("Replacing incorrect projection parameters for sinusoidal products:");
       strcpy(proj.ellipsoid, "WGS-84");
       strcpy(proj.name, "sinusoidal");
-      proj.parameter[0] = 0.0; 
+      proj.parameter[0] = 0.0;
       proj.parameter[1] = 0.0;
-      proj.parameter[2] = 0.0; 
-      proj.parameter[3] = 0.0;  
-      proj.parameter[4] = -99.99;      
-      proj.parameter[5] = -99.99;          
-      if ( cdoVerbose ) 
-	  cdoPrint("proj1 = %g, proj2 = %g, proj3 = %g, proj4 = %g,",  
-		   proj.parameter[0], proj.parameter[1],  
+      proj.parameter[2] = 0.0;
+      proj.parameter[3] = 0.0;
+      proj.parameter[4] = -99.99;
+      proj.parameter[5] = -99.99;
+      if ( cdoVerbose )
+	  cdoPrint("proj1 = %g, proj2 = %g, proj3 = %g, proj4 = %g,",
+		   proj.parameter[0], proj.parameter[1],
 		   proj.parameter[2], proj.parameter[3]);
     }
 
-  if ( nx == xsize && ny == ysize && 
-       strcmp(proj.name, "sinusoidal") == 0 && 
+  if ( nx == xsize && ny == ysize &&
+       strcmp(proj.name, "sinusoidal") == 0 &&
        strcmp(proj.ellipsoid, "WGS-84") == 0  )
     {
-      gridID = defSinusoidalGrid(nx, ny, region.xmin, region.xmax, region.ymin, region.ymax, 
+      gridID = defSinusoidalGrid(nx, ny, region.xmin, region.xmax, region.ymin, region.ymax,
 				 region.dx, region.dy,
-				 proj.parameter[0], proj.parameter[1], 
+				 proj.parameter[0], proj.parameter[1],
 				 proj.parameter[2], proj.parameter[3]);
     }
   /* modification by Frank Kaspar */
@@ -597,17 +592,17 @@ int read_geolocation(hid_t loc_id, int nx, int ny, int lprojtype)
             memcmp(proj.ellipsoid, "Sphere", 6) == 0 )
     {
       double c0  = 0.001*sqrt(proj.parameter[5]);       /* nominal spatial resolution */
-      double lts = proj.parameter[3];      
+      double lts = proj.parameter[3];
       double re  = proj.parameter[4]/1000;  /* Earth radius [km]*/
-      if ( cdoVerbose ) cdoPrint("  c0 = %g, lts = %g, re = %g", c0, lts, re); 
+      if ( cdoVerbose ) cdoPrint("  c0 = %g, lts = %g, re = %g", c0, lts, re);
       gridID = defLonLatGrid(nx, ny, c0, lts, re);
     }
   else if ( nx == 386 && ny == 162 )
     {
       double c0  = 90;       /* nominal spatial resolution */
-      double lts = 30;      
+      double lts = 30;
       double re  = 6371.228;  /* Earth radius [km]*/
-      if ( cdoVerbose ) cdoPrint("  c0 = %g, lts = %g, re = %g", c0, lts, re); 
+      if ( cdoVerbose ) cdoPrint("  c0 = %g, lts = %g, re = %g", c0, lts, re);
       gridID = defLonLatGrid(nx, ny, c0, lts, re);
     }
 
@@ -622,7 +617,7 @@ int read_region(hid_t loc_id, int nx, int ny)
   hid_t region_id;
   hid_t region_tid;
   hid_t str64_tid, str128_tid, fltarr_tid;
-  herr_t     status;
+  herr_t status;
   hsize_t dims;
   typedef struct region_t {
     double area_extent[4];
@@ -689,6 +684,7 @@ int read_region(hid_t loc_id, int nx, int ny)
   }
   */
   status = H5Dread(region_id, region_tid, H5S_ALL, H5S_ALL, H5P_DEFAULT, &region);
+  UNUSED(status);
 
   if ( cdoVerbose )
     {
@@ -737,17 +733,17 @@ int read_region(hid_t loc_id, int nx, int ny)
   dx = (xmax-xmin) / nx;
   dy = (ymax-ymin) / ny;
   /*
-  xsize = NINT((region.xmax-region.xmin)/region.dx);
-  ysize = NINT((region.ymax-region.ymin)/region.dy);
+  xsize = (int)lround((region.xmax-region.xmin)/region.dx);
+  ysize = (int)lround((region.ymax-region.ymin)/region.dy);
 
   if ( cdoVerbose ) cdoPrint("  Size: xsize=%d  ysize=%d", xsize, ysize);
   */
 
   if ( nfound == 4 &&
-       nx == region.xsize && ny == region.ysize && 
+       nx == region.xsize && ny == region.ysize &&
        strcmp(proj, "laea") == 0 )
     {
-      gridID = defLaeaGrid(nx, ny, xmin, xmax, ymin, ymax, 
+      gridID = defLaeaGrid(nx, ny, xmin, xmax, ymin, ymax,
 			   dx, dy, a, lon0, lat0);
     }
 
@@ -758,7 +754,7 @@ static
 void read_dataset(hid_t loc_id, const char *name, void *opdata)
 {
   hid_t dset_id, type_id;
-  hid_t   dataspace;   
+  hid_t   dataspace;
   hsize_t dims_out[9];  /* dataset dimensions           */
   herr_t  status;	/* Generic return value		*/
   hid_t   attr, atype, atype_mem;
@@ -792,7 +788,7 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
   strcpy(varname, name);
 
   dset_id = H5Dopen(loc_id, varname);
-  
+
   type_id = H5Dget_type(dset_id);  /* get datatype*/
 
   type_class = H5Tget_class(type_id);
@@ -862,7 +858,7 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
     if ( isdigit(varname[len-1]) && memcmp(varname, "Data", 4) != 0 )
       {
 	if ( nt > 1 ) cdoAbort("Combination of nlevel > 1 and ntime > 1 not implemented!");
-       
+
 	nz = atoi(&varname[len-1]);
 	varname[len-1] = 0;
       }
@@ -1031,21 +1027,21 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
 	    {
 	      H5Aread(attr, atype_mem, attstring);
 	      if ( ((datasets_t *) opdata)->obj[nset].description )
-		free(((datasets_t *) opdata)->obj[nset].description);
+                Free(((datasets_t *) opdata)->obj[nset].description);
 	      ((datasets_t *) opdata)->obj[nset].description = strdup(attstring);
 	    }
 	  else if ( strcmp(attname, "title") == 0 )
 	    {
 	      H5Aread(attr, atype_mem, attstring);
 	      if ( ((datasets_t *) opdata)->obj[nset].title )
-		free(((datasets_t *) opdata)->obj[nset].title);
+                Free(((datasets_t *) opdata)->obj[nset].title);
 	      ((datasets_t *) opdata)->obj[nset].title = strdup(attstring);
 	    }
 	  else if ( strcmp(attname, "time") == 0 )
 	    {
 	      H5Aread(attr, atype_mem, attstring);
 	      if ( ((datasets_t *) opdata)->obj[nset].time )
-		free(((datasets_t *) opdata)->obj[nset].time);
+                Free(((datasets_t *) opdata)->obj[nset].time);
 	      ((datasets_t *) opdata)->obj[nset].time = strdup(attstring);
 	    }
 	  else if ( strcmp(attname, "unit") == 0 )
@@ -1058,10 +1054,10 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
 	  H5Aclose(attr);
 	  H5Tclose(atype);
 	}
-      
+
       offset = gridsize*(nz-1);
       array  = ((datasets_t *) opdata)->obj[nset].array;
-      array  = realloc(array, gridsize*nz*nt*sizeof(double));
+      array  = (double*) Realloc(array, gridsize*nz*nt*sizeof(double));
       ((datasets_t *) opdata)->obj[nset].array    = array;
       array  = array+offset;
 
@@ -1071,12 +1067,12 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
 	    {
 	      float *farray;
 	      int i;
-	      farray = malloc(gridsize*nt*sizeof(float));
+	      farray = (float*) Malloc(gridsize*nt*sizeof(float));
 	      status = H5Dread(dset_id, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, farray);
 	      if ( status < 0 )
 		cdoAbort("Reading of NATIVE_FLOAT variable %s failed!", varname);
 	      for ( i = 0; i < gridsize*nt; ++i ) array[i] = farray[i];
-	      free(farray);
+	      Free(farray);
 	    }
 	  else
 	    {
@@ -1088,12 +1084,12 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
       else
 	{
 	  int *iarray, i;
-	  iarray = malloc(gridsize*nt*sizeof(int));
+	  iarray = (int*) Malloc(gridsize*nt*sizeof(int));
 	  status = H5Dread(dset_id, H5T_NATIVE_INT, H5S_ALL, H5S_ALL, H5P_DEFAULT, iarray);
 	  if ( status < 0 )
 	    cdoAbort("Reading of NATIVE_INT variable %s failed!", varname);
 	  for ( i = 0; i < gridsize*nt; ++i ) array[i] = iarray[i];
-	  free(iarray);
+	  Free(iarray);
 	}
 
       ((datasets_t *) opdata)->obj[nset].name     = strdup(varname);
@@ -1115,16 +1111,16 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
 	    cdoWarning("Scalefactor changes over levels!");
 
 	  if ( lmissval && !DBL_IS_EQUAL(((datasets_t *) opdata)->obj[nset].missval, missval) )
-	    cdoWarning("Missing value changes over levels!");	       
+	    cdoWarning("Missing value changes over levels!");
 	}
 
       if ( nz == 1 ) ((datasets_t *) opdata)->nsets++;
 
-      mask = malloc(gridsize*nt*sizeof(short));
+      mask = (short*) Malloc(gridsize*nt*sizeof(short));
       memset(mask, 0, gridsize*nt*sizeof(short));
 
       nmiss  = 0;
- 
+
       minval =  1e35;
       maxval = -1e35;
       for ( i = 0; i < gridsize*nt; i++ )
@@ -1154,7 +1150,7 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
       lscalefactor = IS_NOT_EQUAL(scalefactor, 1);
 
       if ( laddoffset || lscalefactor )
-	{	  
+	{
 	  for ( i = 0; i < gridsize*nt; i++ )
 	    if ( !DBL_IS_EQUAL(array[i], missval) )
 	      {
@@ -1169,7 +1165,7 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
 		mask[i] = 1;
 	      }
 	}
-	    
+
       minval =  1e35;
       maxval = -1e35;
       for ( i = 0; i < gridsize*nt; i++ )
@@ -1212,7 +1208,7 @@ void read_dataset(hid_t loc_id, const char *name, void *opdata)
       ((datasets_t *) opdata)->obj[nset].scale    = scalefactor;
       ((datasets_t *) opdata)->obj[nset].missval  = missval;
 
-      free(mask); 
+      Free(mask);
       mask = NULL;
     }
   else
@@ -1244,7 +1240,7 @@ obj_info(hid_t loc_id, const char *name, void *opdata)
     if ( cdoVerbose ) cdoPrint(" Object with name %s is a group", name);
     if ( strcmp(name, "Data") == 0 )
       {
-	((datasets_t *) opdata)->mergelevel = TRUE;	
+	((datasets_t *) opdata)->mergelevel = TRUE;
 	H5Giterate(loc_id, name, NULL, obj_info, opdata);
       }
     else if ( strcmp(name, "Geolocation") == 0 )
@@ -1256,7 +1252,7 @@ obj_info(hid_t loc_id, const char *name, void *opdata)
 	((datasets_t *) opdata)->lmetadata = TRUE;
       }
     break;
-  case H5G_DATASET: 
+  case H5G_DATASET:
     if ( cdoVerbose ) cdoPrint(" Object with name %s is a dataset", name);
     if ( strstr(name, "PALETTE") )
       {
@@ -1273,7 +1269,7 @@ obj_info(hid_t loc_id, const char *name, void *opdata)
 	read_dataset(loc_id, name, opdata);
       }
     break;
-  case H5G_TYPE: 
+  case H5G_TYPE:
     if ( cdoVerbose ) cdoPrint(" Object with name %s is a named datatype", name);
     if ( strcmp(name, "ProjType") == 0 )
       {
@@ -1322,23 +1318,23 @@ void get_global_att(hid_t file_id, const char *obj_path, int vlistID)
 	{
 	  H5Aread(attr, atype_mem, attstring);
 	  vlistDefAttTxt(vlistID, CDI_GLOBAL, attname, (int)strlen(attstring), attstring);
-	} 
+	}
       else if ( type_class == H5T_INTEGER )
 	{
 	  H5Aread(attr, H5T_NATIVE_INT, &attint);
 	  vlistDefAttInt(vlistID, CDI_GLOBAL, attname, DATATYPE_INT32, 1, &attint);
-	} 
+	}
       else if ( type_class == H5T_FLOAT )
 	{
 	  H5Aread(attr, H5T_NATIVE_DOUBLE, &attflt);
 	  vlistDefAttFlt(vlistID, CDI_GLOBAL, attname, DATATYPE_FLT64, 1, &attflt);
-	} 
+	}
       H5Tclose(atype_mem);
       H5Aclose(attr);
       H5Tclose(atype);
     }
 
-  if ( grp_id >= 0 ) H5Gclose(grp_id);	
+  if ( grp_id >= 0 ) H5Gclose(grp_id);
 
 }
 
@@ -1400,7 +1396,7 @@ void dsets_init(datasets_t *dsets)
       dsets->obj[i].loffset     = 0;
       dsets->obj[i].lmissval    = 0;
       dsets->obj[i].missval     = cdiInqMissval();
-      dsets->obj[i].array       = NULL;   
+      dsets->obj[i].array       = NULL;
     }
 }
 #endif
@@ -1457,8 +1453,8 @@ void *Importcmsaf(void *argument)
 
   if ( nt > 1 )
     {
-      vtimes = malloc(nt*sizeof(int));
-      
+      vtimes = (int*) Malloc(nt*sizeof(int));
+
       for ( i = 0; i < nt; ++i ) vtimes[i] = i*10000 + 45*100;
 
       if ( dsets.obj[ivar].time )
@@ -1481,7 +1477,7 @@ void *Importcmsaf(void *argument)
 
   if ( cdoVerbose )
     for ( ivar = 0; ivar < dsets.nsets; ++ivar )
-      cdoPrint(" Var %d %-20s %dx%d nlev = %d nts = %d", 
+      cdoPrint(" Var %d %-20s %dx%d nlev = %d nts = %d",
 	       ivar, dsets.obj[ivar].name, nx, ny, nz, dsets.obj[ivar].nt);
 
   for ( ivar = 1; ivar < dsets.nsets; ++ivar )
@@ -1500,7 +1496,7 @@ void *Importcmsaf(void *argument)
     {
       gridID = read_region(file_id, nx, ny);
     }
-  
+
   if ( gridID == -1 )
     {
       gridID = gridCreate(GRID_GENERIC, gridsize);
@@ -1513,11 +1509,11 @@ void *Importcmsaf(void *argument)
   else
     {
       double *levels;
-      levels = malloc(nz*sizeof(double));
+      levels = (double*) Malloc(nz*sizeof(double));
       for ( i = 0; i < nz; ++i ) levels[i] = i+1;
       zaxisID = zaxisCreate(ZAXIS_GENERIC, nz);
       zaxisDefLevels(zaxisID, levels);
-      free(levels);
+      Free(levels);
     }
 
   vlistID = vlistCreate();
@@ -1544,9 +1540,9 @@ void *Importcmsaf(void *argument)
       if ( dsets.obj[ivar].units )
 	vlistDefVarUnits(vlistID, varID,  dsets.obj[ivar].units);
       if ( dsets.obj[ivar].title )
-	vlistDefAttTxt(vlistID, varID, "title", (int)strlen(dsets.obj[ivar].title), 
+	vlistDefAttTxt(vlistID, varID, "title", (int)strlen(dsets.obj[ivar].title),
 		       dsets.obj[ivar].title);
-	
+
       /*
       vlistDefVarUnits(vlistID, varID, units[i]);
       */
@@ -1582,7 +1578,7 @@ void *Importcmsaf(void *argument)
 	  varID   = ivar;
 
 	  if ( tsID > 0 && dsets.obj[ivar].nt == 1 ) continue;
-	  
+
 	  gridsize = dsets.obj[ivar].gridsize;
 	  missval  = dsets.obj[ivar].missval;
 
@@ -1591,7 +1587,7 @@ void *Importcmsaf(void *argument)
 	      offset = gridsize*levelID;
 	      if ( nz == 1 ) offset = gridsize*tsID;
 	      array  = dsets.obj[ivar].array+offset;
-	      
+
 	      nmiss  = 0;
 	      minval =  1e35;
 	      maxval = -1e35;
@@ -1624,6 +1620,7 @@ void *Importcmsaf(void *argument)
 
   /* Close file */
   status = H5Fclose(file_id);
+  UNUSED(status);
 
   processDefVarNum(vlistNvars(vlistID), streamID);
 
@@ -1635,14 +1632,14 @@ void *Importcmsaf(void *argument)
   taxisDestroy(taxisID);
 
   for ( ivar = 0; ivar < dsets.nsets; ++ivar )
-    free(dsets.obj[ivar].array);
+    Free(dsets.obj[ivar].array);
 
-  if ( vtimes ) free(vtimes);
+  if ( vtimes ) Free(vtimes);
 
   cdoFinish();
 #else
   cdoAbort("HDF5 support not compiled in!");
 #endif
 
-  return (0);
+  return 0;
 }
