@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -53,10 +53,10 @@ int genThinoutGrid(int gridID1, int xinc, int yinc)
 
   if ( gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT )
     {
-      xvals1 = malloc(nlon1*sizeof(double));
-      yvals1 = malloc(nlat1*sizeof(double));
-      xvals2 = malloc(nlon2*sizeof(double));
-      yvals2 = malloc(nlat2*sizeof(double));
+      xvals1 = (double*) Malloc(nlon1*sizeof(double));
+      yvals1 = (double*) Malloc(nlat1*sizeof(double));
+      xvals2 = (double*) Malloc(nlon2*sizeof(double));
+      yvals2 = (double*) Malloc(nlat2*sizeof(double));
       gridInqXvals(gridID1, xvals1);
       gridInqYvals(gridID1, yvals1);
 
@@ -112,19 +112,19 @@ int genBoxavgGrid(int gridID1, int xinc, int yinc)
 
   if ( gridtype == GRID_GAUSSIAN || gridtype == GRID_LONLAT )
     {
-      xvals1 = malloc(nlon1*sizeof(double));
-      yvals1 = malloc(nlat1*sizeof(double));
-      xvals2 = malloc(nlon2*sizeof(double));
-      yvals2 = malloc(nlat2*sizeof(double));
+      xvals1 = (double*) Malloc(nlon1*sizeof(double));
+      yvals1 = (double*) Malloc(nlat1*sizeof(double));
+      xvals2 = (double*) Malloc(nlon2*sizeof(double));
+      yvals2 = (double*) Malloc(nlat2*sizeof(double));
       gridInqXvals(gridID1, xvals1);
       gridInqYvals(gridID1, yvals1);
 
       if ( gridInqYbounds(gridID1, NULL) && gridInqXbounds(gridID1, NULL) )
 	{
-	  grid1_corner_lon = malloc(2*nlon1*sizeof(double));
-	  grid1_corner_lat = malloc(2*nlat1*sizeof(double));
-	  grid2_corner_lon = malloc(2*nlon2*sizeof(double));
-	  grid2_corner_lat = malloc(2*nlat2*sizeof(double));
+	  grid1_corner_lon = (double*) Malloc(2*nlon1*sizeof(double));
+	  grid1_corner_lat = (double*) Malloc(2*nlat1*sizeof(double));
+	  grid2_corner_lon = (double*) Malloc(2*nlon2*sizeof(double));
+	  grid2_corner_lat = (double*) Malloc(2*nlat2*sizeof(double));
 	  gridInqXbounds(gridID1, grid1_corner_lon);
 	  gridInqYbounds(gridID1, grid1_corner_lat);
 	}
@@ -165,8 +165,8 @@ int genBoxavgGrid(int gridID1, int xinc, int yinc)
 	  gridDefXbounds(gridID2, grid2_corner_lon);
 	  gridDefYbounds(gridID2, grid2_corner_lat);
 
-	  free(grid2_corner_lon);
-	  free(grid2_corner_lat);
+	  Free(grid2_corner_lon);
+	  Free(grid2_corner_lat);
 	}
     }
   else
@@ -204,13 +204,13 @@ void boxavg(field_t *field1, field_t *field2, int xinc, int yinc)
   nlon2 = gridInqXsize(gridID2);
   nlat2 = gridInqYsize(gridID2);
 
-  xfield1 = malloc(nlat1*sizeof(double *));
+  xfield1 = (double **) Malloc(nlat1*sizeof(double *));
 
   for ( ilat = 0; ilat < nlat1; ilat++ )
     xfield1[ilat] = array1 + ilat*nlon1;
 
 
-  xfield2 = malloc(nlat2 * sizeof(double *));
+  xfield2 = (double **) Malloc(nlat2 * sizeof(double *));
 
   for ( ilat = 0; ilat < nlat2; ilat++ )
     xfield2[ilat] = array2 + ilat*nlon2;
@@ -242,8 +242,8 @@ void boxavg(field_t *field1, field_t *field2, int xinc, int yinc)
 
   field2->nmiss = nmiss;
 
-  free(xfield2);
-  free(xfield1);
+  Free(xfield2);
+  Free(xfield1);
 }
 
 
@@ -272,12 +272,12 @@ void thinout(field_t *field1, field_t *field2, int xinc, int yinc)
   nlon2 = gridInqXsize(gridID2);
   nlat2 = gridInqYsize(gridID2);
 
-  xfield1 = malloc(nlat1*sizeof(double *));
+  xfield1 = (double **) Malloc(nlat1*sizeof(double *));
 
   for ( ilat = 0; ilat < nlat1; ilat++ )
     xfield1[ilat] = array1 + ilat*nlon1;
 
-  xfield2 = malloc(nlat2*sizeof(double *));
+  xfield2 = (double **) Malloc(nlat2*sizeof(double *));
 
   for ( ilat = 0; ilat < nlat2; ilat++ )
     xfield2[ilat] = array2 + ilat*nlon2;
@@ -300,8 +300,8 @@ void thinout(field_t *field1, field_t *field2, int xinc, int yinc)
   
   field2->nmiss = nmiss;
 
-  free(xfield2);
-  free(xfield1);
+  Free(xfield2);
+  Free(xfield1);
 }
 
 
@@ -345,8 +345,8 @@ void *Intgrid(void *argument)
     {
       operatorInputArg("longitude and latitude");
       operatorCheckArgc(2);
-      slon = atof(operatorArgv()[0]);
-      slat = atof(operatorArgv()[1]);
+      slon = parameter2double(operatorArgv()[0]);
+      slat = parameter2double(operatorArgv()[1]);
       gridID2 = gridCreate(GRID_LONLAT, 1);
       gridDefXsize(gridID2, 1);
       gridDefYsize(gridID2, 1);
@@ -357,8 +357,8 @@ void *Intgrid(void *argument)
     {
       operatorInputArg("xinc, yinc");
       operatorCheckArgc(2);
-      xinc = atoi(operatorArgv()[0]);
-      yinc = atoi(operatorArgv()[1]);
+      xinc = parameter2int(operatorArgv()[0]);
+      yinc = parameter2int(operatorArgv()[1]);
     }
 
   streamID1 = streamOpenRead(cdoStreamName(0));
@@ -408,10 +408,10 @@ void *Intgrid(void *argument)
   streamDefVlist(streamID2, vlistID2);
 
   gridsize = vlistGridsizeMax(vlistID1);
-  array1   = malloc(gridsize*sizeof(double));
+  array1   = (double*) Malloc(gridsize*sizeof(double));
 
   gridsize = gridInqSize(gridID2);
-  array2   = malloc(gridsize*sizeof(double));
+  array2   = (double*) Malloc(gridsize*sizeof(double));
 
   field_init(&field1);
   field_init(&field2);
@@ -461,10 +461,10 @@ void *Intgrid(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( array2 ) free(array2);
-  if ( array1 ) free(array1);
+  if ( array2 ) Free(array2);
+  if ( array1 ) Free(array1);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
