@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -29,168 +29,12 @@
 #include "cdo_int.h"
 #include "pstream.h"
 #include "list.h"
+#include "after_vertint.h"
 
-/*
- * 3d vertical interpolation routine (see interp_lev() in src/Intlevel.c)
- */
-static
-void interp_lev3d(int gridsize, double missval, double *vardata1, double *vardata2,
-	     	  int nlev2, int *lev_idx1, int *lev_idx2, double *lev_wgt1, double *lev_wgt2)
-{
-  int i, ilev;
-  int idx1, idx2;
-  double wgt1, wgt2;
-  double w1, w2;
-  double var1L1, var1L2, *var2;
 
-  for ( ilev = 0; ilev < nlev2; ilev++ )
-    {
-      var2 = vardata2+(ilev*gridsize);
-
-      for ( i = 0; i < gridsize; i++ )
-	{
-          idx1 = lev_idx1[ilev*gridsize+i];
-          idx2 = lev_idx2[ilev*gridsize+i];
-          wgt1 = lev_wgt1[ilev*gridsize+i];
-          wgt2 = lev_wgt2[ilev*gridsize+i];
-
-          /* upper/lower values from input field */
-          var1L1 = *(vardata1+idx1);
-          var1L2 = *(vardata1+idx2);
-
-          /* if (cdoVerbose) printf("i:%d level %d: idx1=%d idx2=%d (ilev*gridsize+i:%d) wgt1=%g wgt2=%g var1L1:%g var1L2:%g ",
-           *                         i,       ilev, idx1,   idx2,    ilev*gridsize+i,    wgt1,   wgt2,   var1L1,   var1L2);
-           */
-
-	  w1 = wgt1;
-	  w2 = wgt2;
-	  if ( DBL_IS_EQUAL(var1L1, missval)  ) w1 = 0;
-	  if ( DBL_IS_EQUAL(var1L2, missval)  ) w2 = 0;
-
-	  if ( IS_EQUAL(w1, 0) && IS_EQUAL(w2, 0) )
-	    {
-	      var2[i] = missval;
-	    }
-	  else if ( IS_EQUAL(w1, 0) )
-	    {
-	      if ( w2 >= 0.5 )
-		var2[i] = var1L2;
-	      else
-		var2[i] = missval;	      
-	    }
-	  else if ( IS_EQUAL(w2, 0) )
-	    {
-	      if ( w1 >= 0.5 )
-		var2[i] = var1L1;
-	      else
-		var2[i] = missval;	      
-	    }
-	  else
-	    {
-	      var2[i] = var1L1*w1 + var1L2*w2;
-	    }
-	}
-    }
-}
-
-/*
- * Create weights for the 3d vertical coordinate
- *
- * The resulting index sets lev_idx1 and lev_idx2 contain absolute numbers,i.e.
- * wrt. the given gridsize. They can directly be used to read values from 3d
- * data fields.
- *
- * 3d version of gen_weights() (src/Intlevel.c)
- */
-static
-void gen_weights3d(int expol, int nlev1, int gridsize, double *lev1, int nlev2, double *lev2,
-                   int *lev_idx1, int *lev_idx2, double *lev_wgt1, double *lev_wgt2)
-{
-  int i,i1, i2;
-  double val1, val2 = 0;
-  int    idx1 = 0, idx2 = 0;
-
-  for ( i = 0; i < gridsize; i++ )
-    {
-      for ( i2 = 0; i2 < nlev2; i2++ )
-        {
-          /* Because 2 levels were added to the source vertical coordinate (one on
-           * top, one at the bottom), its loop starts at 1 */
-          for ( i1 = 1; i1 < nlev1; i1++ )
-            {
-              if ( lev1[(i1-1)*gridsize+i] < lev1[i1*gridsize+i] )
-                {
-                  idx1 = (i1 - 1)*gridsize+i;
-                  idx2 = i1*gridsize+i;
-                }
-              else
-                {
-                  idx1 = i1*gridsize+i;
-                  idx2 = (i1-1)*gridsize+i;
-                }
-              val1 = lev1[idx1];
-              val2 = lev1[idx2];
-
-              if ( lev2[i2*gridsize+i] > val1 && lev2[i2*gridsize+i] <= val2 ) break;
-            }
-
-          if ( i1 == nlev1 ) 
-            {
-              if ( expol )
-                cdoAbort("Level %g at index %d not found! Use extrapolation", lev2[i2*gridsize],i2);
-              else
-                cdoAbort("Level %g at index %d not found!");
-            }
-
-          if ( i1-1 == 0 ) /* destination levels ios not covert by the first two input z levels */
-            {
-              lev_idx1[i2*gridsize+i] = gridsize+i;
-              lev_idx2[i2*gridsize+i] = gridsize+i;
-              lev_wgt1[i2*gridsize+i] = 0;
-              if ( expol || IS_EQUAL(lev2[i2*gridsize+i], val2) )
-                lev_wgt2[i2*gridsize+i] = 1;
-              else
-                lev_wgt2[i2*gridsize+i] = 0;
-            }
-          else if ( i1 == nlev1-1 ) /* destination level is beyond the last value of the input z field */
-            {
-              lev_idx1[i2*gridsize+i] = (nlev1-2)*gridsize+i;
-              lev_idx2[i2*gridsize+i] = (nlev1-2)*gridsize+i;
-              if ( expol || IS_EQUAL(lev2[i2*gridsize+i], val2) )
-                lev_wgt1[i2*gridsize+i] = 1;
-              else
-                lev_wgt1[i2*gridsize+i] = 0;
-              lev_wgt2[i2*gridsize+i] = 0;
-            }
-          else /* target z values has two bounday values in input z field */
-            {
-              lev_idx1[i2*gridsize+i] = idx1;
-              lev_idx2[i2*gridsize+i] = idx2;
-              lev_wgt1[i2*gridsize+i] = (lev1[idx2]        - lev2[i2*gridsize+i]) / (lev1[idx2] - lev1[idx1]);
-              lev_wgt2[i2*gridsize+i] = (lev2[i2*gridsize+i] - lev1[idx1])        / (lev1[idx2] - lev1[idx1]);
-
-            }
-  /*         if (cdoVerbose)
-   *         {
-   *           printf("i:%d i2:%d\ti2*gridsize+i:%d\tlev2[i2*gridsize+i]:%g\tidx1:%d\tidx2:%d\tlev1[idx1]:%g\tlev1[idx2]:%g\t",
-   *                   i, i2, i2*gridsize+i,         lev2[i2*gridsize+i],    idx1,    idx2,    lev1[idx1],    lev1[idx2]);
-   *           printf("\tlev_wgt1:%g\tlev_wgt2:%g\n", lev_wgt1[i2*gridsize+i], lev_wgt2[i2*gridsize+i]);
-   *         }
-   */
-          /* backshift of the indices because if the two additional levels in input vertical coordinate */
-          lev_idx1[i2*gridsize+i] -= gridsize;
-          lev_idx2[i2*gridsize+i] -= gridsize;
-
-        }
-    }
-}
 
 void *Intlevel3d(void *argument)
 {
-  int INTLEVEL3D, INTLEVELX3D;
-  int operatorID;
-  int streamID0, streamID1, streamID2,streamID3;
-  int vlistID0, vlistID1, vlistID2, vlistID3;
   int gridsize,gridSize,gridsizei,gridsizeo;
   int recID, nrecs;
   int i, offset;
@@ -200,10 +44,10 @@ void *Intlevel3d(void *argument)
   int nmiss;
 
   int nlonIn, nlatIn, nlonOut, nlatOut;
-  double *lonIn, *latIn, *lonOut, *latOut;
+  //double *lonIn, *latIn, *lonOut, *latOut;
 
   int zaxisID1 = -1, zaxisID3;
-  int gridID, zaxisID;
+  int gridID3 = -1, gridID, zaxisID;
   int nlevi, nlevo, nlevel = 0, maxlev;
   int lup, ldown;
   int **varnmiss = NULL;
@@ -223,10 +67,10 @@ void *Intlevel3d(void *argument)
 
   cdoInitialize(argument);
 
-  INTLEVEL3D  = cdoOperatorAdd("intlevel3d",  0, 0, NULL);
-  INTLEVELX3D = cdoOperatorAdd("intlevelx3d",  0, 0, NULL);
+  int INTLEVEL3D  = cdoOperatorAdd("intlevel3d",  0, 0, NULL);
+  int INTLEVELX3D = cdoOperatorAdd("intlevelx3d", 0, 0, NULL);
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
   if      ( operatorID == INTLEVEL3D )  expol = FALSE;
   else if ( operatorID == INTLEVELX3D ) expol = TRUE;
@@ -237,16 +81,16 @@ void *Intlevel3d(void *argument)
   operatorInputArg("filename for vertical source coordinates variable");
   operatorCheckArgc(1);
   argument_t *fileargument = file_argument_new(operatorArgv()[0]);
-  streamID0 = streamOpenRead(fileargument);                     /*  3d vertical input coordinate */
+  int streamID0 = streamOpenRead(fileargument);                     /*  3d vertical input coordinate */
   file_argument_free(fileargument);
-  streamID1 = streamOpenRead(cdoStreamName(0));                 /*  input data */
-  streamID2 = streamOpenRead(cdoStreamName(1));                 /*  3d target vertical coordinate */
-  streamID3 = streamOpenWrite(cdoStreamName(2),cdoFiletype());  /*  output stream */
+  int streamID1 = streamOpenRead(cdoStreamName(0));                 /*  input data */
+  int streamID2 = streamOpenRead(cdoStreamName(1));                 /*  3d target vertical coordinate */
+  int streamID3 = streamOpenWrite(cdoStreamName(2),cdoFiletype());  /*  output stream */
 
-  vlistID0 = streamInqVlist(streamID0);
-  vlistID1 = streamInqVlist(streamID1); taxisID1 = vlistInqTaxis(vlistID1);
-  vlistID2 = streamInqVlist(streamID2);
-  vlistID3 = vlistDuplicate(vlistID1);  taxisID3 = taxisDuplicate(taxisID1);
+  int vlistID0 = streamInqVlist(streamID0);
+  int vlistID1 = streamInqVlist(streamID1); taxisID1 = vlistInqTaxis(vlistID1);
+  int vlistID2 = streamInqVlist(streamID2);
+  int vlistID3 = vlistDuplicate(vlistID1);  taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID3, taxisID1);
 
   /*
@@ -264,12 +108,13 @@ void *Intlevel3d(void *argument)
 
     nlonIn  = gridInqXsize(gridID);
     nlatIn  = gridInqYsize(gridID);
-    lonIn   = (double*) malloc(nlonIn*sizeof(double));
-    latIn   = (double*) malloc(nlatIn*sizeof(double));
+    /*
+    lonIn   = (double*) Malloc(nlonIn*sizeof(double));
+    latIn   = (double*) Malloc(nlatIn*sizeof(double));
     gridInqXvals(gridID, lonIn);
     gridInqYvals(gridID, latIn);
-
-    zlevels_in = (double*) malloc(gridsize*(nlevel+2)*sizeof(double));
+    */
+    zlevels_in = (double*) Malloc(gridsize*(nlevel+2)*sizeof(double));
     nlevi      = nlevel;   /* number of input levels for later use */
     gridsizei  = gridsize; /* horizontal gridsize of input z coordinate */
     nrecs      = streamInqTimestep(streamID0, 0);
@@ -292,18 +137,20 @@ void *Intlevel3d(void *argument)
     nvars = vlistNvars(vlistID2);
     if (nvars != 1) cdoAbort("Only one single variable is allowed!");
     gridID      = vlistInqVarGrid(vlistID2, varID);
+    gridID3     = gridID;
     zaxisID     = vlistInqVarZaxis(vlistID2, varID);
     gridsize    = gridInqSize(gridID);
     nlevel      = zaxisInqSize(zaxisID);
 
     nlonOut = gridInqXsize(gridID);
     nlatOut = gridInqYsize(gridID);
-    lonOut  = (double*) malloc(nlonOut*sizeof(double));
-    latOut  = (double*) malloc(nlatOut*sizeof(double));
+    /*
+    lonOut  = (double*) Malloc(nlonOut*sizeof(double));
+    latOut  = (double*) Malloc(nlatOut*sizeof(double));
     gridInqXvals(gridID, lonOut);
     gridInqYvals(gridID, latOut);
-
-    zlevels_out = (double*) malloc(gridsize*nlevel*sizeof(double));
+    */
+    zlevels_out = (double*) Malloc(gridsize*nlevel*sizeof(double));
     nlevo       = nlevel;  /* number of output levels for later use */
     gridsizeo   = gridsize;/* horizontal gridsize of output z coordinate */
     nrecs       = streamInqTimestep(streamID2, 0);
@@ -321,14 +168,14 @@ void *Intlevel3d(void *argument)
 
   /* Missing values are not allowed for coordinate variables */
   if ( 0 != zlevels_in_miss  )
-    cdoAbort("Input vertical coordinate variables are not allowd to contian missing values.");
+    cdoAbort("Input vertical coordinate variables are not allowd to contain missing values.");
   else
     {
       if ( cdoVerbose ) cdoPrint("Input vertical coordinate has no missing values.");
     }
 
   if ( 0 != zlevels_out_miss )
-    cdoAbort("Output vertical coordinate variables are not allowd to contian missing values.");
+    cdoAbort("Output vertical coordinate variables are not allowd to contain missing values.");
   else
     {
       if ( cdoVerbose ) cdoPrint("Output vertical coordinate has no missing values.");
@@ -346,9 +193,9 @@ void *Intlevel3d(void *argument)
 
    /* input and output vertical coordinates must have exactly the same horizontal grid */
    if ( nlonIn != nlonOut || 
-        nlatIn != nlatOut ||
+        nlatIn != nlatOut /*||
         memcmp(lonIn,lonOut,nlonIn*sizeof(double)) ||
-        memcmp(latIn,latOut,nlatIn*sizeof(double)) )
+        memcmp(latIn,latOut,nlatIn*sizeof(double))*/ )
      {
        /* i =0; printf ( "lonIn:%g latIn:%g lonOut:%g latOut:%g\n",lonIn[i],latIn[i],lonOut[i],latOut[i] ); */
        cdoAbort("Input and output vertical coordinates do NOT exactly have the same horizontal grid.");
@@ -423,18 +270,18 @@ void *Intlevel3d(void *argument)
   /*
    * Create weights for later interpolation - assumption: input vertical correct is constant in time
    */
-  lev_idx1 = (int*) malloc(nlevo*gridSize*sizeof(int));
-  lev_idx2 = (int*) malloc(nlevo*gridSize*sizeof(int));
-  lev_wgt1 = (double*) malloc(nlevo*gridSize*sizeof(double));
-  lev_wgt2 = (double*) malloc(nlevo*gridSize*sizeof(double));
+  lev_idx1 = (int*) Malloc(nlevo*gridSize*sizeof(int));
+  lev_idx2 = (int*) Malloc(nlevo*gridSize*sizeof(int));
+  lev_wgt1 = (double*) Malloc(nlevo*gridSize*sizeof(double));
+  lev_wgt2 = (double*) Malloc(nlevo*gridSize*sizeof(double));
 
-  gen_weights3d(expol, nlevi+2, gridSize, zlevels_in, nlevo, zlevels_out, lev_idx1, lev_idx2, lev_wgt1, lev_wgt2);
+  vert_gen_weights3d(expol, nlevi+2, gridSize, zlevels_in, nlevo, zlevels_out, lev_idx1, lev_idx2, lev_wgt1, lev_wgt2);
 
   /*
    * Copy z-axis information to output z-axis
    */
   zaxisID3 = zaxisCreate(zaxisInqType(zaxisID1), nlevo);
-  lev2 = (double*) malloc(nlevo*sizeof(double));
+  lev2 = (double*) Malloc(nlevo*sizeof(double));
   /* fill values with its indices */
   for (i=0;i<nlevo;i++)
     lev2[i] = (double) i;
@@ -448,7 +295,7 @@ void *Intlevel3d(void *argument)
     if ( zaxisID1 == vlistZaxis(vlistID1, i) )
       vlistChangeZaxisIndex(vlistID3, i, zaxisID3);
   /* add the vertical output field to the output stream */
-  int oz3dvarID = vlistDefVar(vlistID3,0,zaxisID3,TSTEP_INSTANT);
+  int oz3dvarID = vlistDefVar(vlistID3, gridID3, zaxisID3, TSTEP_INSTANT);
   {
     char str[256];
     str[0] = 0;
@@ -456,7 +303,7 @@ void *Intlevel3d(void *argument)
     vlistDefVarName(vlistID3,oz3dvarID,str);
     str[0] = 0;
     vlistInqVarLongname(vlistID2,0,str);
-    if ( str[0] ) vlistDefVarLongname(vlistID3,zaxisID1, str);
+    if ( str[0] ) vlistDefVarLongname(vlistID3,oz3dvarID, str);
     str[0] = 0;
     vlistInqVarUnits(vlistID2,0, str);
     if ( str[0] ) vlistDefVarUnits(vlistID3,oz3dvarID, str);
@@ -464,16 +311,16 @@ void *Intlevel3d(void *argument)
 
   streamDefVlist(streamID3, vlistID3);
 
-  maxlev   = nlevi > nlevo ? nlevi : nlevo;
+  maxlev    = nlevi > nlevo ? nlevi : nlevo;
   nvars     = vlistNvars(vlistID1);
-  vars      = (int*) malloc(nvars*sizeof(int));
-  vardata1  = (double**) malloc(nvars*sizeof(double*)); /* input                                         */
-  vardata2  = (double**) malloc(nvars*sizeof(double*)); /* output                                        */
-  varnmiss  = (int**) malloc(nvars*sizeof(int*));    /* can for missing values of arbitrary variables */
-  varinterp = (int*) malloc(nvars*sizeof(int));     /* marker for variables to be interpolated       */
+  vars      = (int*) Malloc(nvars*sizeof(int));
+  vardata1  = (double**) Malloc(nvars*sizeof(double*)); /* input                                         */
+  vardata2  = (double**) Malloc(nvars*sizeof(double*)); /* output                                        */
+  varnmiss  = (int**) Malloc(nvars*sizeof(int*));    /* can for missing values of arbitrary variables */
+  varinterp = (int*) Malloc(nvars*sizeof(int));     /* marker for variables to be interpolated       */
 
   /* by default no variable should be interpolated */
-  for (i = 0; i < nvars;i++)
+  for ( i = 0; i < nvars; i++ )
     varinterp[varID] = FALSE;
 
   for ( varID = 0; varID < nvars; varID++ )
@@ -485,37 +332,38 @@ void *Intlevel3d(void *argument)
 
       vlistInqVarName(vlistID1, varID, varname);
 
-      vardata1[varID] = (double*) malloc(gridsize*nlevel*sizeof(double));
+      vardata1[varID] = (double*) Malloc(gridsize*nlevel*sizeof(double));
 
       /*  variabls for interpolation:
        *  * have the required vertical axis, i.e. the correct number of levels (nlevi)
        *  * have the same number of horizontal grid points (i.e. same gridSize) like the two vertical coordinates
        *  * are NOT the output vertical coordinates itself
        */
-      if ( zaxisID == zaxisID1 && varID != oz3dvarID && gridsize == gridSize)
+      if ( zaxisID == zaxisID1 && varID != oz3dvarID && gridsize == gridSize )
         {
           nlonIn  = gridInqXsize(gridID);
           nlatIn  = gridInqYsize(gridID);
-          lonIn   = (double*) malloc(nlonIn*sizeof(double));
-          latIn   = (double*) malloc(nlatIn*sizeof(double));
+	  /*
+          lonIn   = (double*) Malloc(nlonIn*sizeof(double));
+          latIn   = (double*) Malloc(nlatIn*sizeof(double));
           gridInqXvals(gridID, lonIn);
           gridInqYvals(gridID, latIn);
-
+	  */
           if ( nlonIn != nlonOut || 
-               nlatIn != nlatOut ||
+               nlatIn != nlatOut /*||
                memcmp(lonIn,lonOut,nlonIn*sizeof(double)) ||
-               memcmp(latIn,latOut,nlatIn*sizeof(double)) )
+               memcmp(latIn,latOut,nlatIn*sizeof(double))*/ )
             {
               varinterp[varID] = FALSE;
               vardata2[varID]  = vardata1[varID];
-              varnmiss[varID]  = (int*) malloc(nlevel*sizeof(int));
+              varnmiss[varID]  = (int*) Malloc(nlevel*sizeof(int));
               if ( cdoVerbose ) cdoPrint("Ignore variable %s with %d levels\n",varname,nlevel);
             }
           else
             {
               varinterp[varID] = TRUE;
-              vardata2[varID]  = (double*) malloc(gridsize*nlevo*sizeof(double));
-              varnmiss[varID]  = (int*) malloc(maxlev*sizeof(int));
+              vardata2[varID]  = (double*) Malloc(gridsize*nlevo*sizeof(double));
+              varnmiss[varID]  = (int*) Malloc(maxlev*sizeof(int));
               memset(varnmiss[varID], 0, maxlev*sizeof(int));
             }
         }
@@ -523,7 +371,7 @@ void *Intlevel3d(void *argument)
         {
           varinterp[varID] = FALSE;
           vardata2[varID]  = vardata1[varID];
-          varnmiss[varID]  = (int*) malloc(nlevel*sizeof(int));
+          varnmiss[varID]  = (int*) Malloc(nlevel*sizeof(int));
           if ( cdoVerbose ) cdoPrint("Ignore variable %s with %d levels\n",varname,nlevel);
         }
       }
@@ -561,8 +409,8 @@ void *Intlevel3d(void *argument)
 	      missval  = vlistInqVarMissval(vlistID1, varID);
 	      gridsize = gridInqSize(gridID);
 
-	      interp_lev3d(gridsize, missval, vardata1[varID], vardata2[varID],
-			 nlevo, lev_idx1, lev_idx2, lev_wgt1, lev_wgt2);
+	      vert_interp_lev3d(gridsize, missval, vardata1[varID], vardata2[varID],
+				nlevo, lev_idx1, lev_idx2, lev_wgt1, lev_wgt2);
 
 	      for ( levelID = 0; levelID < nlevo; levelID++ )
 		{
@@ -600,6 +448,7 @@ void *Intlevel3d(void *argument)
 		}
 	    }
 	}
+
       /* copy output z coordinate to output stream */
       nlevel = zaxisInqSize(vlistInqVarZaxis(vlistID3, oz3dvarID));
       for ( levelID = 0; levelID < nlevel; levelID++ )
@@ -617,22 +466,22 @@ void *Intlevel3d(void *argument)
   nvars     = vlistNvars(vlistID1);
   for ( varID = 0; varID < nvars; varID++ )
     {
-      free(varnmiss[varID]);
-      free(vardata1[varID]);
-      if ( varinterp[varID] ) free(vardata2[varID]);
+      Free(varnmiss[varID]);
+      Free(vardata1[varID]);
+      if ( varinterp[varID] ) Free(vardata2[varID]);
     } 
 
 
-  free(varinterp);
-  free(varnmiss);
-  free(vardata2);
-  free(vardata1);
-  free(vars);
+  Free(varinterp);
+  Free(varnmiss);
+  Free(vardata2);
+  Free(vardata1);
+  Free(vars);
 
-  free(lev_idx1);
-  free(lev_idx2);
-  free(lev_wgt1);
-  free(lev_wgt2);
+  Free(lev_idx1);
+  Free(lev_idx2);
+  Free(lev_wgt1);
+  Free(lev_wgt2);
 
   streamClose(streamID0);
   streamClose(streamID1);
@@ -643,5 +492,5 @@ void *Intlevel3d(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
