@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -105,7 +105,7 @@ void *Isosurface(void *argument)
 
   operatorCheckArgc(1);
 
-  isoval = atof(operatorArgv()[0]);
+  isoval = parameter2double(operatorArgv()[0]);
 
   if ( cdoVerbose ) cdoPrint("Isoval: %g\n", isoval);
 
@@ -135,7 +135,7 @@ void *Isosurface(void *argument)
   if ( i == nzaxis ) cdoAbort("No processable variable found!");
 
   nlev1 = nlevel;
-  lev1  = malloc((nlev1)*sizeof(double));
+  lev1  = (double*) Malloc((nlev1)*sizeof(double));
   zaxisInqLevels(zaxisID1, lev1);
 
   zaxisIDsfc = zaxisCreate(ZAXIS_SURFACE, 1);
@@ -150,13 +150,13 @@ void *Isosurface(void *argument)
   gridsize = vlistGridsizeMax(vlistID1);
 
   field_init(&field);
-  field.ptr = malloc(gridsize*sizeof(double));
+  field.ptr = (double*) Malloc(gridsize*sizeof(double));
 
   nvars = vlistNvars(vlistID1);
 
-  liso  =     malloc(nvars*sizeof(int));
-  vars  =     malloc(nvars*sizeof(int));
-  vars1 = malloc(nvars*sizeof(field_t));
+  liso  = (int*)     Malloc(nvars*sizeof(int));
+  vars  = (int*)     Malloc(nvars*sizeof(int));
+  vars1 = (field_t*) Malloc(nvars*sizeof(field_t));
 
   for ( varID = 0; varID < nvars; varID++ )
     {
@@ -176,7 +176,7 @@ void *Isosurface(void *argument)
       vars1[varID].zaxis   = zaxisID;
       vars1[varID].nmiss   = 0;
       vars1[varID].missval = missval;
-      vars1[varID].ptr     = malloc(gridsize*nlevel*sizeof(double));
+      vars1[varID].ptr     = (double*) Malloc(gridsize*nlevel*sizeof(double));
     }
 
   tsID = 0;
@@ -196,7 +196,6 @@ void *Isosurface(void *argument)
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  gridsize = gridInqSize(vlistInqVarGrid(vlistID1, varID));
-	  nlevel   = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	  offset   = gridsize*levelID;
 	  single   = vars1[varID].ptr + offset;
 	  
@@ -241,14 +240,14 @@ void *Isosurface(void *argument)
       tsID++;
     }
 
-  for ( varID = 0; varID < nvars; varID++ ) free(vars1[varID].ptr);
-  free(vars1);
+  for ( varID = 0; varID < nvars; varID++ ) Free(vars1[varID].ptr);
+  Free(vars1);
 
-  free(vars);
-  free(liso);
-  if (lev1) free(lev1);
+  Free(vars);
+  Free(liso);
+  if (lev1) Free(lev1);
 
-  free(field.ptr);
+  Free(field.ptr);
 
   streamClose(streamID2);
   streamClose(streamID1);
@@ -257,5 +256,5 @@ void *Isosurface(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
