@@ -2,8 +2,7 @@
 #  include "config.h" /* HAVE_LIBMAGICS */
 #endif
 
-#include<limits.h>  /* TEMPORARY FIX, UNTIL NEXT MAGICS LIBRARY RELEASE */ 
-
+#include <limits.h>
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
@@ -11,28 +10,16 @@
 #include "pstream.h"
 
 #if defined(HAVE_LIBMAGICS)
+
 #include "magics_api.h"
-#endif
 
-
-#if defined(HAVE_LIBXML2)
-
-#include<libxml/parser.h>
-#include<libxml/tree.h>
-#include "template_parser.h"
 #include "magics_template_parser.h"
 #include "results_template_parser.h"
-#include <ctype.h>
-
-
-extern xmlNode  *magics_node;
-
-#endif
+#include "StringUtilities.h"
 
 #define DBG 0
 
-
-char *line_colours[] = {     "red", "green", "blue", "yellow", "cyan", "magenta",
+const char *line_colours[] = {"red", "green", "blue", "yellow", "cyan", "magenta",
 			     "avocado","beige", "brick", "brown", "burgundy",
 			     "charcoal", "chestnut", "coral", "cream", 
 			     "evergreen", "gold", 
@@ -52,18 +39,14 @@ char *line_colours[] = {     "red", "green", "blue", "yellow", "cyan", "magenta"
 			     "bluishpurple", "purple",
 			};
 
-char  *graph_params[] = {"ymin","ymax","sigma","stat","obsv","device"};
+const char  *graph_params[] = {"ymin","ymax","sigma","stat","obsv","device"};
 
 int graph_param_count = sizeof(graph_params)/sizeof(char*);
 int num_colours = sizeof( line_colours )/sizeof( char* );
 
-void VerifyGraphParameters( int num_param, char **param_names );
-int compareDateOrTimeStr( char *datetimestr1, char *datetimestr2, char *sep_char );
+int compareDateOrTimeStr( char *datetimestr1, char *datetimestr2, const char *sep_char );
 
-extern int checkdevice();
-extern int IsNumeric();
-extern void StrToUpperCase();
-extern int StringSplitWithSeperator();
+extern int checkdevice(char *device_in);
 
 extern char *DEVICE;
 extern char *DEVICE_TABLE;
@@ -73,14 +56,13 @@ extern int DEVICE_COUNT;
 static
 void maggraph(const char *plotfile, const char *varname,const char *varunits, long nfiles, long *nts, int **vdate, int **vtime, double **datatab, int nparam, char **params)
 {
-  
   char *lines[1];
   char *temp_str;
   char **split_str = NULL;
-  char *sep_char = "=";
+  const char *sep_char = "=";
   char **date_time_str[nfiles];
   char min_date_time_str[1024], max_date_time_str[1024];
-  int  min_index, max_index;
+  int  min_index = 0, max_index = 0;
   char vdatestr[32], vtimestr[32], legend_text_data[256];
   char vdatestr1[32], vtimestr1[32];
   char vdatestr2[32], vtimestr2[32];
@@ -91,11 +73,11 @@ void maggraph(const char *plotfile, const char *varname,const char *varunits, lo
   int count ;
   int num_years = 0, num_months = 0, num_days = 0;
   int ret;
-  long tsID, fileID, i, ntime_steps;
-  double *date_time;
+  long tsID, fileID, i, ntime_steps = 0;
+  double *date_time = NULL;
   double min_val = 1.0e+200, max_val = -1.0e+200;
-  double *mean_val, *std_dev_val;
-  double *spread_min, *spread_max;
+  double *mean_val = NULL, *std_dev_val = NULL;
+  double *spread_min = NULL, *spread_max = NULL;
   double y_min_val = 1.0e+200, y_max_val = -1.0e+200;
   
   if( DBG )
@@ -170,7 +152,7 @@ void maggraph(const char *plotfile, const char *varname,const char *varunits, lo
 	  mag_setc ("output_format", DEVICE );
 	}
 	
-      free( split_str );
+      Free( split_str );
     }
 
   if ( DBG )
@@ -256,20 +238,20 @@ void maggraph(const char *plotfile, const char *varname,const char *varunits, lo
   if ( stat == TRUE )
     {
 	/* if all files are of same number of steps, only one date_time_str array is being used */
-        date_time_str[0] = malloc( ntime_steps*sizeof(char *) ); 
+        date_time_str[0] = (char **) Malloc( ntime_steps*sizeof(char *)); 
 	
-	date_time   = malloc( ntime_steps*sizeof(double) );
-	mean_val    = malloc( ntime_steps*sizeof(double) );
-	std_dev_val = malloc( ntime_steps*sizeof(double) );
-	spread_min  = malloc( ntime_steps*sizeof(double) );
-	spread_max  = malloc( ntime_steps*sizeof(double) );
+	date_time   = (double*) Malloc( ntime_steps*sizeof(double));
+	mean_val    = (double*) Malloc( ntime_steps*sizeof(double));
+	std_dev_val = (double*) Malloc( ntime_steps*sizeof(double));
+	spread_min  = (double*) Malloc( ntime_steps*sizeof(double));
+	spread_max  = (double*) Malloc( ntime_steps*sizeof(double));
 	
 	for ( tsID = 0; tsID < ntime_steps; ++tsID )
 	  {
 	    date_time[tsID] = tsID+1;
 	    date2str(vdate[0][tsID], vdatestr, sizeof(vdatestr));
 	    time2str(vtime[0][tsID], vtimestr, sizeof(vtimestr));
-	    date_time_str[0][tsID] = malloc(256);
+	    date_time_str[0][tsID] = (char*) Malloc(256);
 	    sprintf(date_time_str[0][tsID], "%s %s", vdatestr, vtimestr);
 	    mean_val[tsID] = 0.;
 	    std_dev_val[tsID] = 0.;
@@ -358,8 +340,8 @@ void maggraph(const char *plotfile, const char *varname,const char *varunits, lo
 	{
 	  if ( DBG )
 	    fprintf(stderr,"FILE  %ld\n", fileID );
-	  date_time             = malloc( nts[fileID]*sizeof(double) );
-	  date_time_str[fileID] = malloc( nts[fileID]*sizeof(char *) );
+	  date_time             = (double*) Malloc( nts[fileID]*sizeof(double));
+	  date_time_str[fileID] = (char **) Malloc( nts[fileID]*sizeof(char *));
 	  
 	  for ( tsID = 0; tsID <  nts[fileID]; ++tsID )
 	    {
@@ -367,7 +349,7 @@ void maggraph(const char *plotfile, const char *varname,const char *varunits, lo
 	      date2str(vdate[fileID][tsID], vdatestr, sizeof(vdatestr));
 	      time2str(vtime[fileID][tsID], vtimestr, sizeof(vtimestr));
 	      
-	      date_time_str[fileID][tsID] = malloc(256);
+	      date_time_str[fileID][tsID] = (char*) Malloc(256);
 	      sprintf(date_time_str[fileID][tsID], "%s %s", vdatestr, vtimestr);
 	      if ( DBG )
 		fprintf( stderr,"%s %s %s\n", vdatestr, vtimestr, date_time_str[fileID][tsID] );
@@ -377,7 +359,7 @@ void maggraph(const char *plotfile, const char *varname,const char *varunits, lo
 	      if( datatab[fileID][tsID] > max_val )
 	        max_val = datatab[ fileID ][ tsID ];	
 	    }
-	  free( date_time );
+	  Free( date_time );
 	  
 	  if( fileID == 0 )
 	    {
@@ -458,10 +440,11 @@ void maggraph(const char *plotfile, const char *varname,const char *varunits, lo
     split_str_count = 0;
     sep_char = "-";
     split_str_count = StringSplitWithSeperator( max_date_time_str, sep_char, &split_str );
+    (void)split_str_count;
     num_years  = atoi( split_str[0] );
     num_months = atoi( split_str[1] );
     num_days   = atoi( split_str[2] );
-    free( split_str  );
+    Free( split_str  );
     
     split_str_count = StringSplitWithSeperator( min_date_time_str, sep_char, &split_str );
     num_years -= atoi( split_str[0] );
@@ -478,7 +461,7 @@ void maggraph(const char *plotfile, const char *varname,const char *varunits, lo
 	else if( num_months == 1 )
 	  num_days += ( 31- atoi( split_str[2] ) );
       }
-    free( split_str );
+    Free( split_str );
     
     if( DBG )
       fprintf(stderr," %d %d\n", num_years, num_months );
@@ -487,10 +470,7 @@ void maggraph(const char *plotfile, const char *varname,const char *varunits, lo
 	1. Loop over the Files
 	2. Loop over the number of time steps 
 	3. Set the attributes for the magics data and plot
-  */  
-   
-#if defined(HAVE_LIBMAGICS)
-
+  */
 
   /* magics_template_parser( magics_node ); */
 
@@ -668,7 +648,7 @@ void maggraph(const char *plotfile, const char *varname,const char *varunits, lo
     }
   
   
-  lines[0] = malloc(1024);
+  lines[0] = (char*) Malloc(1024);
   /* To be obtained from Meta Data */
   /*sprintf( lines[0],"%s","ExpID : " );*/ 
   /*sprintf( lines[0],"%sxxxx  Variable : %s[%s]",lines[0], varname, varunits );*/
@@ -691,24 +671,20 @@ void maggraph(const char *plotfile, const char *varname,const char *varunits, lo
 
   if ( stat == TRUE )
     {
-      free( date_time );
-      free( mean_val );
-      free( std_dev_val );
-      free( spread_min );
-      free( spread_max );
+      Free( date_time );
+      Free( mean_val );
+      Free( std_dev_val );
+      Free( spread_min );
+      Free( spread_max );
     }
   
   
   if( DBG )
     fprintf(stderr, "%s\n",lines[0]);
-
-#endif
-
 }
 
-int compareDateOrTimeStr( char *datetimestr1, char *datetimestr2, char *sep_char )
+int compareDateOrTimeStr( char *datetimestr1, char *datetimestr2, const char *sep_char )
 {
-  
   int    split_str_count1, split_str_count2;
   int	 i,flag[3]; /*  '3' since, three fields are expected in the input strings */
   char   **split_str1 = NULL;
@@ -722,7 +698,7 @@ int compareDateOrTimeStr( char *datetimestr1, char *datetimestr2, char *sep_char
     split_str_count2 = StringSplitWithSeperator( datetimestr2, sep_char, &split_str2 );
   else
     {
-      free( split_str1 );
+      Free( split_str1 );
       return -999;
     }
   
@@ -734,13 +710,13 @@ int compareDateOrTimeStr( char *datetimestr1, char *datetimestr2, char *sep_char
     }
   else
     {
-      free( split_str1 );
-      free( split_str2 );
+      Free( split_str1 );
+      Free( split_str2 );
       return -999;
     }
   
-  free( split_str1 );
-  free( split_str2 );
+  Free( split_str1 );
+  Free( split_str2 );
   
   for ( i = 0;i < 3 ; i++ )
     {
@@ -754,12 +730,8 @@ int compareDateOrTimeStr( char *datetimestr1, char *datetimestr2, char *sep_char
     return 0;
 }
 
-
-#if defined(HAVE_LIBMAGICS)
-
 static
 void init_MAGICS( )
-
 {
   setenv( "MAGPLUS_QUIET","1",1 ); /* To suppress magics messages */
   mag_open();
@@ -770,197 +742,22 @@ void init_MAGICS( )
 
 }
 
-
 static
 void quit_MAGICS( )
-
 {
-
   mag_close ();
   if( DBG )
     fprintf( stdout,"Exiting From MAGICS\n" );
 
 }
 
-#endif
-
-#define NINC_ALLOC 1024
-
-void *Maggraph(void *argument)
-{
-  const char *ofilename;
-  char varname[CDI_MAX_NAME], units[CDI_MAX_NAME];
-  char **pnames = NULL;
-  int varID, levelID;
-  int gridID;
-  int nrecs;
-  int tsID;
-  int streamID;
-  int vlistID, vlistID0 = -1;
-  int nmiss;
-  int taxisID;
-  int **vdate = NULL, **vtime = NULL;
-  int fileID, nfiles;
-  long *nts, nts_alloc;
-  int nparam = 0;
-  double **datatab = NULL;
-  double val;
-  int i;
-  
-  cdoInitialize(argument);
-
-  nparam = operatorArgc();
-  pnames = operatorArgv();
-  
-  if( nparam )
-    VerifyGraphParameters(nparam,pnames);
-  
-  nfiles = cdoStreamCnt() - 1;
-  ofilename = cdoStreamName(nfiles)->args;
-  
-  if( DBG )
-    {
-       fprintf( stderr," Num of files %d\n",nfiles );
-       fprintf( stderr," files %s\n",ofilename );
-    }
-	
-  datatab = malloc(nfiles*sizeof(double *));
-  vdate   = malloc(nfiles*sizeof(int *));
-  vtime   = malloc(nfiles*sizeof(int *));
-  nts     = malloc(nfiles*sizeof(long));
-  
-  for ( fileID = 0; fileID < nfiles; fileID++ )
-    {
-      datatab[fileID] = NULL;
-      vdate[fileID]   = NULL;
-      vtime[fileID]   = NULL;
-      nts[fileID]     = 0;
-    }
-
-  for ( fileID = 0; fileID < nfiles; fileID++ )
-    {
-      
-     
-      if( DBG )
-        fprintf( stderr," file %d is %s\n", fileID, cdoStreamName(fileID)->args );
-      streamID = streamOpenRead(cdoStreamName(fileID));
-
-      vlistID = streamInqVlist(streamID);
-      taxisID = vlistInqTaxis(vlistID);
-
-      vlistInqVarUnits(vlistID, 0, units);
-      if( DBG )
-	fprintf(stderr," %s\n", units );
-      if ( fileID == 0 )
-	{
-	  vlistInqVarName(vlistID, 0, varname);
-	  
-	  
-	  gridID = vlistInqVarGrid(vlistID, 0);
-
-	  if ( gridInqSize(gridID) != 1 ) cdoAbort("Variable has more than one grid point!");
-
-	  vlistID0 = vlistDuplicate(vlistID);
-	}
-      else
-	{
-	  vlistCompare(vlistID0, vlistID, CMP_ALL);
-	}
-
-      tsID = 0;
-      nts_alloc = 0;
-      while ( (nrecs = streamInqTimestep(streamID, tsID)) )
-	{
-	  if ( nrecs != 1 ) cdoAbort("Input streams have more than one record!\n");
-	  
-	  if ( tsID == 0 )
-	    {
-	      nts_alloc += NINC_ALLOC;
-	      datatab[ fileID ] = malloc( nts_alloc*sizeof(double) );
-	      vdate[ fileID ]   = malloc(  nts_alloc*sizeof(int) );
-	      vtime[ fileID ]   = malloc(  nts_alloc*sizeof(int) );
-	    }
-		
-	  nts[ fileID ]++;
-
-	  if ( nts[ fileID ] > nts_alloc )
-	    {
-	      nts_alloc += NINC_ALLOC;
-	      datatab[ fileID ] = realloc(datatab[fileID], nts_alloc*sizeof(double));
-	      vdate[ fileID ]   = realloc(vdate[fileID], nts_alloc*sizeof(int));
-	      vtime[ fileID ]   = realloc(vtime[fileID], nts_alloc*sizeof(int));
-	    }
-	  
-	  streamInqRecord( streamID, &varID, &levelID );
-	  streamReadRecord( streamID, &val, &nmiss );	
-	  datatab[ fileID ][ tsID ] = val;
-	  vdate[ fileID ][ tsID ] = taxisInqVdate(taxisID);
-	  vtime[ fileID ][ tsID ] = taxisInqVtime(taxisID);
-
-          if( DBG )
-	    fprintf(stderr, "%f %f\n", datatab[ fileID ][ tsID ],val ); 
-	  tsID++;
-	}
-      streamClose(streamID);
-    }
-  
-#if defined(HAVE_LIBXML2)
-  /* HARDCODED THE FILE NAME .. TO BE SENT AS COMMAND LINE ARGUMENT FOR THE MAGICS OPERATOR */
-  /*
-  init_XMLtemplate_parser( Filename );
-  updatemagics_and_results_nodes( );
-  */
-#endif
-
-
-#if defined(HAVE_LIBMAGICS)
-  init_MAGICS( );
-#endif
-
-  cdoPrint(" Creating PLOT for %s", varname);
-  if( DBG )
-    {
-      fprintf(stderr, "Num params %d\n", nparam);
-  
-      for( i = 0; i< nparam; i++ )
-	fprintf(stderr, "Param %s\n", pnames[i]);
-    }
-  maggraph(ofilename, varname, units, nfiles, nts, vdate, vtime, datatab, nparam, pnames);
-
-#if defined(HAVE_LIBXML2)
-  /* quit_XMLtemplate_parser( ); */
-#endif
-
-#if defined(HAVE_LIBMAGICS)
-  quit_MAGICS( );
-#endif
-
-  if ( vlistID0 != -1 ) vlistDestroy(vlistID0);
-
-  for ( fileID = 0; fileID < nfiles; fileID++ )
-    {
-      if ( datatab[fileID] ) free(datatab[fileID]);
-    }
-
-  free(datatab);
-
-
-  if ( vdate ) free(vdate);
-  if ( vtime ) free(vtime);
-
-  cdoFinish();
-
-  return (0);
-}
-
-
+static
 void VerifyGraphParameters( int num_param, char **param_names )
-
 {
   int i, j;
   int  found = FALSE, syntax = TRUE, halt_flag = FALSE, split_str_count;
   char **split_str = NULL;
-  char *sep_char = "=";
+  const char *sep_char = "=";
   char *temp_str;
   
   for ( i = 0; i < num_param; ++i )
@@ -1038,12 +835,10 @@ void VerifyGraphParameters( int num_param, char **param_names )
 			}
 		      else
 			{
-#if defined(HAVE_LIBXML2)
 			  // HARDCODED THE FILE NAME .. TO BE SENT AS COMMAND LINE ARGUMENT FOR THE MAGICS OPERATOR 
 			  fclose(fp);
 			  init_XMLtemplate_parser( split_str[1] );
 			  updatemagics_and_results_nodes( );
-#endif			
 			}
 		    }
 */
@@ -1065,12 +860,169 @@ void VerifyGraphParameters( int num_param, char **param_names )
 	  halt_flag = TRUE;
 	  fprintf( stderr,"Invalid parameter specification  '%s'!\n", param_names[i] );
 	}
-      free( split_str );
+      Free( split_str );
     }
       
     if( halt_flag == TRUE )
     {
       exit(0);
     }
-    
+}
+#endif
+
+#define NINC_ALLOC 1024
+
+void *Maggraph(void *argument)
+{
+  cdoInitialize(argument);
+
+#if defined(HAVE_LIBMAGICS)
+  char varname[CDI_MAX_NAME], units[CDI_MAX_NAME];
+  int varID, levelID;
+  int gridID;
+  int nrecs;
+  int vlistID0 = -1;
+  int nmiss;
+  long nts_alloc;
+  
+  int nparam = operatorArgc();
+  char **pnames = operatorArgv();
+  
+  if ( nparam ) VerifyGraphParameters(nparam,pnames);
+  
+  int nfiles = cdoStreamCnt() - 1;
+  const char *ofilename = cdoStreamName(nfiles)->args;
+  
+  if( DBG )
+    {
+       fprintf( stderr," Num of files %d\n",nfiles );
+       fprintf( stderr," files %s\n",ofilename );
+    }
+	
+  double **datatab = (double **) Malloc(nfiles*sizeof(double *));
+  int **vdate   = (int **) Malloc(nfiles*sizeof(int *));
+  int **vtime   = (int **) Malloc(nfiles*sizeof(int *));
+  long *nts     = (long*) Malloc(nfiles*sizeof(long));
+  
+  for ( int fileID = 0; fileID < nfiles; fileID++ )
+    {
+      datatab[fileID] = NULL;
+      vdate[fileID]   = NULL;
+      vtime[fileID]   = NULL;
+      nts[fileID]     = 0;
+    }
+
+  for ( int fileID = 0; fileID < nfiles; fileID++ )
+    {
+      if( DBG )
+        fprintf( stderr," file %d is %s\n", fileID, cdoStreamName(fileID)->args );
+      
+      int streamID = streamOpenRead(cdoStreamName(fileID));
+
+      int vlistID = streamInqVlist(streamID);
+      int taxisID = vlistInqTaxis(vlistID);
+
+      vlistInqVarUnits(vlistID, 0, units);
+      if( DBG )
+	fprintf(stderr," %s\n", units );
+      if ( fileID == 0 )
+	{
+	  vlistInqVarName(vlistID, 0, varname);
+	  
+	  gridID = vlistInqVarGrid(vlistID, 0);
+          int zaxisID = vlistInqVarZaxis(vlistID, 0);
+          int nvars = vlistNvars(vlistID);
+
+          if ( nvars > 1 ) cdoAbort("Input stream has more than on variable!");
+	  if ( gridInqSize(gridID) != 1 ) cdoAbort("Variable has more than one grid point!");
+	  if ( zaxisInqSize(zaxisID) != 1 ) cdoAbort("Variable has more than one level!");
+
+	  vlistID0 = vlistDuplicate(vlistID);
+	}
+      else
+	{
+	  vlistCompare(vlistID0, vlistID, CMP_ALL);
+	}
+
+      int tsID = 0;
+      nts_alloc = 0;
+      while ( (nrecs = streamInqTimestep(streamID, tsID)) )
+	{
+	  if ( nrecs != 1 ) cdoAbort("Input stream has more than one point in time!");
+	  
+	  if ( tsID == 0 )
+	    {
+	      nts_alloc += NINC_ALLOC;
+	      datatab[ fileID ] = (double*) Malloc( nts_alloc*sizeof(double));
+	      vdate[ fileID ]   = (int*) Malloc(  nts_alloc*sizeof(int));
+	      vtime[ fileID ]   = (int*) Malloc(  nts_alloc*sizeof(int));
+	    }
+		
+	  nts[ fileID ]++;
+
+	  if ( nts[ fileID ] > nts_alloc )
+	    {
+	      nts_alloc += NINC_ALLOC;
+	      datatab[ fileID ] = (double*) Realloc(datatab[fileID], nts_alloc*sizeof(double));
+	      vdate[ fileID ]   = (int*) Realloc(vdate[fileID], nts_alloc*sizeof(int));
+	      vtime[ fileID ]   = (int*) Realloc(vtime[fileID], nts_alloc*sizeof(int));
+	    }
+
+          double val;
+	  streamInqRecord( streamID, &varID, &levelID );
+	  streamReadRecord( streamID, &val, &nmiss );	
+	  datatab[ fileID ][ tsID ] = val;
+	  vdate[ fileID ][ tsID ] = taxisInqVdate(taxisID);
+	  vtime[ fileID ][ tsID ] = taxisInqVtime(taxisID);
+
+          if( DBG )
+	    fprintf(stderr, "%f %f\n", datatab[ fileID ][ tsID ],val ); 
+	  tsID++;
+	}
+      streamClose(streamID);
+    }
+  
+  /* HARDCODED THE FILE NAME .. TO BE SENT AS COMMAND LINE ARGUMENT FOR THE MAGICS OPERATOR */
+  /*
+  init_XMLtemplate_parser( Filename );
+  updatemagics_and_results_nodes( );
+  */
+
+  init_MAGICS( );
+
+  cdoPrint(" Creating PLOT for %s", varname);
+  if( DBG )
+    {
+      fprintf(stderr, "Num params %d\n", nparam);
+  
+      for( int i = 0; i< nparam; i++ )
+	fprintf(stderr, "Param %s\n", pnames[i]);
+    }
+  maggraph(ofilename, varname, units, nfiles, nts, vdate, vtime, datatab, nparam, pnames);
+
+  /* quit_XMLtemplate_parser( ); */
+
+  quit_MAGICS( );
+
+  if ( vlistID0 != -1 ) vlistDestroy(vlistID0);
+
+  for ( int fileID = 0; fileID < nfiles; fileID++ )
+    {
+      if ( datatab[fileID] ) Free(datatab[fileID]);
+    }
+
+  Free(datatab);
+
+  if ( vdate ) Free(vdate);
+  if ( vtime ) Free(vtime);
+
+#else
+  
+  cdoAbort("MAGICS support not compiled in!");
+
+#endif
+
+  cdoFinish();
+
+  return 0;
 }
