@@ -8,33 +8,26 @@
 #include "grid.h"
 #include "pstream.h"
 
+
+#if defined(HAVE_LIBMAGICS)
+
 #include "magics_api.h"
 
-#include<libxml/parser.h>
-#include<libxml/tree.h>
-#include "template_parser.h"
 #include "magics_template_parser.h"
 #include "results_template_parser.h"
-
-extern xmlNode  *magics_node;
+#include "StringUtilities.h"
 
 #define DBG 0
 
 int VECTOR, STREAM;
-char  *vector_params[] = {"thin_fac","unit_vec","device","step_freq"};
+const char  *vector_params[] = {"thin_fac","unit_vec","device","step_freq"};
 int vector_param_count = sizeof(vector_params)/sizeof(char*);
 
-void VerifyVectorParameters( int num_param, char **param_names, int opID );
-
 /* Default Magics Values */
 double THIN_FAC = 2.0, UNIT_VEC = 25.0;
 extern int ANIM_FLAG,STEP_FREQ;
 
-extern int checkdevice();
-extern int IsNumeric();
-extern void StrToUpperCase();
-extern int StringSplitWithSeperator();
-
+extern int checkdevice(char *device_in);
 
 extern char *DEVICE;
 extern char *DEVICE_TABLE;
@@ -48,12 +41,13 @@ void magvector( const char *plotfile, int operatorID, const char *varname, long
         double dlon = 0, dlat = 0;
 	int split_str_count;
 	char plotfilename[4096];
-	char *sep_char= "=";
+	const char *sep_char= "=";
 	char **split_str=NULL;
 	char *temp_str = NULL;
 	char *titlename;
 	
-
+        (void)varname;
+        
 	if( uarray == NULL && varray == NULL )
 	  {
 	    fprintf( stderr," No Velocity Components in input file, cannot creaate Vector PLOT!\n" );
@@ -80,7 +74,8 @@ void magvector( const char *plotfile, int operatorID, const char *varname, long
 	    split_str_count = 0;
 	    sep_char = "=";
 	    split_str_count = StringSplitWithSeperator( params[i], sep_char, &split_str );
-	    
+	    (void)split_str_count;
+            
 	    if( !strcmp( split_str[0],"thin_fac" ) )
 	      {
 		THIN_FAC = atof( split_str[1] );
@@ -113,7 +108,7 @@ void magvector( const char *plotfile, int operatorID, const char *varname, long
 		  fprintf(stderr,"STEP FREQ %d\n",STEP_FREQ );
 	      }
 	      
-	    free( split_str );  
+	    Free( split_str );  
 	  }
 	  
         if( nlon > 1 )
@@ -128,10 +123,6 @@ void magvector( const char *plotfile, int operatorID, const char *varname, long
 		dlat /= (nlat-1);
 	  }
 
-
-/* #if defined(HAVE_LIBMAGICS) */
-
-
         /* magics_template_parser( magics_node ); */
 
         /* results_template_parser(results_node, varname ); */
@@ -176,11 +167,11 @@ void magvector( const char *plotfile, int operatorID, const char *varname, long
 		mag_seti( "wind_arrow_thickness",1 );
 		mag_coast();
 		
-		if( THIN_FAC != 2.0f )
+		if( IS_NOT_EQUAL(THIN_FAC, 2.0f) )
 		  mag_setr("wind_thinning_factor",THIN_FAC);
 		
 		/*wind_arrow_unit_velocity */
-		if( UNIT_VEC != 25.0f )
+		if( IS_NOT_EQUAL(UNIT_VEC, 25.0f) )
 		  mag_setr("wind_arrow_unit_velocity",UNIT_VEC);
                 
 		mag_wind();
@@ -193,7 +184,6 @@ void magvector( const char *plotfile, int operatorID, const char *varname, long
 	  }
 }
 
-
 static
 void init_MAGICS( )
 
@@ -218,225 +208,17 @@ void quit_MAGICS( )
 
 }
 
-void *Magvector(void *argument)
-
-{
-  int operatorID;
-  int varID, recID;
-  int gridsize;
-  int gridID;
-  int nrecs;
-  int levelID;
-  int tsID;
-  int streamID;
-  int vlistID;
-  int nmiss;
-  int nlon, nlat;
-  int nlev;
-  int zaxisID, taxisID;
-  int vdate, vtime;
-  int found;
-  int nparam = 0;
-  int i;
-  char **pnames = NULL;
-  char varname[CDI_MAX_NAME];
-  double missval;
-  double *uarray = NULL;
-  double *varray = NULL;
-  double *grid_center_lat = NULL, *grid_center_lon = NULL;
-  char units[CDI_MAX_NAME];
-  char vdatestr[32],vtimestr[32],datetimestr[64];
-
-
-  cdoInitialize(argument);
-
-  nparam = operatorArgc();
-  pnames = operatorArgv();
-  
-  VECTOR  = cdoOperatorAdd("vector", 0, 0, NULL);
-  STREAM  = cdoOperatorAdd("stream", 0, 0, NULL);
-
-  operatorID = cdoOperatorID();
-  
-  if( nparam )
-    {
-      if( DBG )
-	{
-	  for( i = 0; i < nparam; i++ )
-	    fprintf( stderr,"Param %d is %s!\n",i+1, pnames[i] );
-	}
-      
-      VerifyVectorParameters( nparam, pnames, operatorID );
-    }
-
-  streamID = streamOpenRead(cdoStreamName(0));
-
-  vlistID = streamInqVlist(streamID);
-  taxisID = vlistInqTaxis(vlistID);
-
-  found = 0;
-  varID = 0;
-  gridID  = vlistInqVarGrid(vlistID, varID);
-  zaxisID = vlistInqVarZaxis(vlistID, varID);
-  missval = vlistInqVarMissval(vlistID, varID);
-
-  if ( gridInqType(gridID) == GRID_GME          ) cdoAbort("GME grid unspported!");
-  if ( gridInqType(gridID) == GRID_UNSTRUCTURED ) cdoAbort("Unstructured grid unspported!");
-
-  if ( gridInqType(gridID) != GRID_CURVILINEAR )
-    gridID = gridToCurvilinear(gridID, 1);
-
-  gridsize = gridInqSize(gridID);
-  nlon     = gridInqXsize(gridID);
-  nlat     = gridInqYsize(gridID);
-  nlev     = zaxisInqSize(zaxisID);
-
-  uarray          = malloc(gridsize*sizeof(double));
-  varray          = malloc(gridsize*sizeof(double));
-  grid_center_lat = malloc(gridsize*sizeof(double));
-  grid_center_lon = malloc(gridsize*sizeof(double));
-
-  gridInqYvals(gridID, grid_center_lat);
-  gridInqXvals(gridID, grid_center_lon);
-
-  /* Convert lat/lon units if required */
-  gridInqXunits(gridID, units);
-  grid_to_degree(units, gridsize, grid_center_lon, "grid center lon");
-  gridInqYunits(gridID, units);
-  grid_to_degree(units, gridsize, grid_center_lat, "grid center lat");
-					
-  tsID = 0;
-
-  /* HARDCODED THE FILE NAME .. TO BE SENT AS COMMAND LINE ARGUMENT FOR THE MAGICS OPERATOR */
-  /*
-  init_XMLtemplate_parser( Filename );
-  updatemagics_and_results_nodes( );
-  */
-
-
-  init_MAGICS( );
-
-  while( (nrecs = streamInqTimestep(streamID, tsID)) )
-    {
-      if( ANIM_FLAG )
-        {
-          if( tsID % STEP_FREQ )
-            {
-                tsID++;
-                continue;
-            }
-        }
-      else
-        {
-          if( !STEP_FREQ  && tsID )
-            {
-          	 cdoWarning("File has values at more than one time step! Image created for first time step!!!");
-           	 break;
-            }
-        }
-
-      vdate = taxisInqVdate(taxisID);
-      vtime = taxisInqVtime(taxisID);
-	      
-      date2str(vdate, vdatestr, sizeof(vdatestr));
-      time2str(vtime, vtimestr, sizeof(vtimestr));
-      sprintf(datetimestr, "%s %s", vdatestr,vtimestr);
-
-      for( recID = 0; recID < nrecs; recID++ )
-	{
-	  streamInqRecord(streamID, &varID, &levelID);
-
-	  vlistInqVarName(vlistID, varID, varname);
-
-          if( operatorID == VECTOR )
-	    {
-	       if( !strcmp( varname, "var131" ) || !strcmp( varname, "u" ) ) /* U Velocity as per GRIB is 'var131, as per NC 'u' */
-	  	 {
-                      if( DBG )
-          		fprintf( stderr,"Found U VEL in Varname %s\n",varname );
-		      streamReadRecord(streamID, uarray, &nmiss);
-		      found ++;
-	  	 }
-	       if( !strcmp( varname, "var132" ) || !strcmp( varname, "v" ) ) /* V Velocity as per GRIB  is 'var132, as per NC 'v'*/
-	  	 {
-                      if( DBG )
-          		fprintf( stderr,"Found V VEL in Varname %s\n",varname );
-		      streamReadRecord(streamID, varray, &nmiss);
-		      found ++;
-	  	 }	
-	       if( found == 2 )
-	    	 break;
-	    }
-	  else if ( operatorID == STREAM )
-            fprintf( stderr," Stream Operator Un-Supported!\n" );
-	  else 
-            fprintf( stderr," Operator Un-Supported!\n" );
-        }
-         
-      if( operatorID == VECTOR )
-	{
-	   if( found == 2 )
-	     {
-                if( DBG )
-          	  fprintf( stderr,"Found Both U & V VEL, Creating vector fields! \n" );
-		magvector( cdoStreamName(1)->args, operatorID, varname, nlon, nlat, grid_center_lon, grid_center_lat, uarray, varray, nparam, pnames, datetimestr );
-	     }
-	   else if( found == 1 )
-             {
-                fprintf( stderr,"Found only one Velocity Component in input file, cannot create Vector PLOT!\n" );
-                break;
-             }
-	   else if( found == 0 )
-             {
-                fprintf( stderr,"No Velocity Components in input file, cannot create Vector PLOT!\n" );
-                break;
-             }
-	}
-    
-      tsID++;
-
-      /*
-      if( ANIM_FLAG )
-        tsID++;
-      else
-        {
-           cdoWarning("File has values at more than one time step! Image created for first time step!!!");
-           if( STEP_FREQ > 1 )
-             cdoWarning("Step frequency parameter ignored!!!");
-           break;
-        }
-      */
-    }
-
-  streamClose(streamID);
-
-  if ( uarray  ) free(uarray);
-  if ( varray  ) free(varray);
-  if ( grid_center_lon ) free(grid_center_lon);
-  if ( grid_center_lat ) free(grid_center_lat);
-
-  /*   quit_XMLtemplate_parser( ); */
-
-  quit_MAGICS( );
-
-  cdoFinish();
-
-  return (0);
-
-}
-
-
-
+static
 void VerifyVectorParameters( int num_param, char **param_names, int opID )
 
 {
   
   int i, j;
   int found = FALSE, syntax = TRUE, halt_flag = FALSE, split_str_count;
-  int param_count;
-  char **params;
+  int param_count = 0;
+  const char **params = NULL;
   char **split_str = NULL;
-  char *sep_char = "=";
+  const char *sep_char = "=";
 
   /* char  *vector_params[] = {"min","max","count","interval","list","colour","thickness","style","RGB"}; */
 
@@ -516,12 +298,205 @@ void VerifyVectorParameters( int num_param, char **param_names, int opID )
 	}
 	
       if( split_str ) 	  
-	free( split_str );
+        Free( split_str );
     }
       
     if( halt_flag == TRUE )
     {
       exit(0);
     }
+}
+#endif
+
+
+void *Magvector(void *argument)
+{
+  cdoInitialize(argument);
+
+#if defined(HAVE_LIBMAGICS)
+  int nrecs;
+  int levelID;
+  int nmiss;
+  char varname[CDI_MAX_NAME];
+  char units[CDI_MAX_NAME];
+  char vdatestr[32],vtimestr[32],datetimestr[64];
+
+  int nparam = operatorArgc();
+  char **pnames = operatorArgv();
+  
+  int VECTOR  = cdoOperatorAdd("vector", 0, 0, NULL);
+  int STREAM  = cdoOperatorAdd("stream", 0, 0, NULL);
+
+  int operatorID = cdoOperatorID();
+  
+  if ( nparam )
+    {
+      if( DBG )
+	{
+	  for( int i = 0; i < nparam; i++ )
+	    fprintf( stderr,"Param %d is %s!\n",i+1, pnames[i] );
+	}
+      
+      VerifyVectorParameters( nparam, pnames, operatorID );
+    }
+
+  int streamID = streamOpenRead(cdoStreamName(0));
+
+  int vlistID = streamInqVlist(streamID);
+  int taxisID = vlistInqTaxis(vlistID);
+
+  int found = 0;
+  int varID = 0;
+  int gridID  = vlistInqVarGrid(vlistID, varID);
+  // int zaxisID = vlistInqVarZaxis(vlistID, varID);
+  // double missval = vlistInqVarMissval(vlistID, varID);
+
+  if ( gridInqType(gridID) == GRID_GME          ) cdoAbort("GME grid unspported!");
+  if ( gridInqType(gridID) == GRID_UNSTRUCTURED ) cdoAbort("Unstructured grid unspported!");
+
+  if ( gridInqType(gridID) != GRID_CURVILINEAR )
+    gridID = gridToCurvilinear(gridID, 1);
+
+  int gridsize = gridInqSize(gridID);
+  int nlon     = gridInqXsize(gridID);
+  int nlat     = gridInqYsize(gridID);
+  // int nlev     = zaxisInqSize(zaxisID);
+
+  double *uarray          = (double*) Malloc(gridsize*sizeof(double));
+  double *varray          = (double*) Malloc(gridsize*sizeof(double));
+  double *grid_center_lat = (double*) Malloc(gridsize*sizeof(double));
+  double *grid_center_lon = (double*) Malloc(gridsize*sizeof(double));
+
+  gridInqYvals(gridID, grid_center_lat);
+  gridInqXvals(gridID, grid_center_lon);
+
+  /* Convert lat/lon units if required */
+  gridInqXunits(gridID, units);
+  grid_to_degree(units, gridsize, grid_center_lon, "grid center lon");
+  gridInqYunits(gridID, units);
+  grid_to_degree(units, gridsize, grid_center_lat, "grid center lat");
+					
+  int tsID = 0;
+
+  /* HARDCODED THE FILE NAME .. TO BE SENT AS COMMAND LINE ARGUMENT FOR THE MAGICS OPERATOR */
+  /*
+  init_XMLtemplate_parser( Filename );
+  updatemagics_and_results_nodes( );
+  */
+
+
+  init_MAGICS( );
+
+  while( (nrecs = streamInqTimestep(streamID, tsID)) )
+    {
+      if( ANIM_FLAG )
+        {
+          if( tsID % STEP_FREQ )
+            {
+                tsID++;
+                continue;
+            }
+        }
+      else
+        {
+          if( !STEP_FREQ  && tsID )
+            {
+          	 cdoWarning("File has values at more than one time step! Image created for first time step!!!");
+           	 break;
+            }
+        }
+
+      int vdate = taxisInqVdate(taxisID);
+      int vtime = taxisInqVtime(taxisID);
+	      
+      date2str(vdate, vdatestr, sizeof(vdatestr));
+      time2str(vtime, vtimestr, sizeof(vtimestr));
+      sprintf(datetimestr, "%s %s", vdatestr,vtimestr);
+
+      for( int recID = 0; recID < nrecs; recID++ )
+	{
+	  streamInqRecord(streamID, &varID, &levelID);
+
+	  vlistInqVarName(vlistID, varID, varname);
+
+          if( operatorID == VECTOR )
+	    {
+	       if( !strcmp( varname, "var131" ) || !strcmp( varname, "u" ) ) /* U Velocity as per GRIB is 'var131, as per NC 'u' */
+	  	 {
+                      if( DBG )
+          		fprintf( stderr,"Found U VEL in Varname %s\n",varname );
+		      streamReadRecord(streamID, uarray, &nmiss);
+		      found ++;
+	  	 }
+	       if( !strcmp( varname, "var132" ) || !strcmp( varname, "v" ) ) /* V Velocity as per GRIB  is 'var132, as per NC 'v'*/
+	  	 {
+                      if( DBG )
+          		fprintf( stderr,"Found V VEL in Varname %s\n",varname );
+		      streamReadRecord(streamID, varray, &nmiss);
+		      found ++;
+	  	 }	
+	       if( found == 2 )
+	    	 break;
+	    }
+	  else if ( operatorID == STREAM )
+            fprintf( stderr," Stream Operator Un-Supported!\n" );
+	  else 
+            fprintf( stderr," Operator Un-Supported!\n" );
+        }
+         
+      if( operatorID == VECTOR )
+	{
+	   if( found == 2 )
+	     {
+                if( DBG )
+          	  fprintf( stderr,"Found Both U & V VEL, Creating vector fields! \n" );
+		magvector( cdoStreamName(1)->args, operatorID, varname, nlon, nlat, grid_center_lon, grid_center_lat, uarray, varray, nparam, pnames, datetimestr );
+	     }
+	   else if( found == 1 )
+             {
+                fprintf( stderr,"Found only one Velocity Component in input file, cannot create Vector PLOT!\n" );
+                break;
+             }
+	   else if( found == 0 )
+             {
+                fprintf( stderr,"No Velocity Components in input file, cannot create Vector PLOT!\n" );
+                break;
+             }
+	}
     
+      tsID++;
+
+      /*
+      if( ANIM_FLAG )
+        tsID++;
+      else
+        {
+           cdoWarning("File has values at more than one time step! Image created for first time step!!!");
+           if( STEP_FREQ > 1 )
+             cdoWarning("Step frequency parameter ignored!!!");
+           break;
+        }
+      */
+    }
+
+  streamClose(streamID);
+
+  if ( uarray  ) Free(uarray);
+  if ( varray  ) Free(varray);
+  if ( grid_center_lon ) Free(grid_center_lon);
+  if ( grid_center_lat ) Free(grid_center_lat);
+
+  /*   quit_XMLtemplate_parser( ); */
+
+  quit_MAGICS( );
+
+#else
+  
+  cdoAbort("MAGICS support not compiled in!");
+
+#endif
+
+  cdoFinish();
+
+  return 0;
 }
