@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -46,7 +46,6 @@ int ReadCoords(double *xvals, double *yvals, const char *polyfile, FILE *fp)
   while ( readline(fp, line, MAX_LINE) )
     {
       i = 0;
-      xcoord = 0;
       if ( line[0] == '#' ) 
          { 
            jumpedlines++;
@@ -118,19 +117,122 @@ static
 void maskbox(int *mask, int gridID,
 	     int lat1, int lat2, int lon11, int lon12, int lon21, int lon22)
 {
-  int nlon, nlat;
-  int ilat, ilon;
+  int nlon = gridInqXsize(gridID);
+  int nlat = gridInqYsize(gridID);
 
-  nlon = gridInqXsize(gridID);
-  nlat = gridInqYsize(gridID);
-
-  for ( ilat = 0; ilat < nlat; ilat++ )
-    for ( ilon = 0; ilon < nlon; ilon++ )
+  for ( int ilat = 0; ilat < nlat; ilat++ )
+    for ( int ilon = 0; ilon < nlon; ilon++ )
       if (  (lat1 <= ilat && ilat <= lat2 && 
 	      ((lon11 <= ilon && ilon <= lon12) || (lon21 <= ilon && ilon <= lon22))) )
 	mask[nlon*ilat + ilon] = 0;
 }
 
+void getlonlatparams(int argc_offset, double *xlon1, double *xlon2, double *xlat1, double *xlat2);
+void genlonlatbox_curv(int gridID, double xlon1, double xlon2, double xlat1, double xlat2,
+                       int *lat1, int *lat2, int *lon11, int *lon12, int *lon21, int *lon22);
+
+static
+void maskbox_curv(int *mask, int gridID)
+{
+  double xlon1 = 0, xlon2 = 0, xlat1 = 0, xlat2 = 0;
+  getlonlatparams(0, &xlon1, &xlon2, &xlat1, &xlat2);
+
+  int nlon = gridInqXsize(gridID);
+  int nlat = gridInqYsize(gridID);
+
+  int gridsize = nlon*nlat;
+
+  int grid_is_circular = gridIsCircular(gridID);
+
+  double *xvals = (double *) Malloc(gridsize*sizeof(double));
+  double *yvals = (double *) Malloc(gridsize*sizeof(double));
+
+  gridInqXvals(gridID, xvals);
+  gridInqYvals(gridID, yvals);
+
+  char xunits[CDI_MAX_NAME];
+  char yunits[CDI_MAX_NAME];
+  gridInqXunits(gridID, xunits);
+  gridInqYunits(gridID, yunits);
+
+  double xfact = 1, yfact = 1;
+  if ( strncmp(xunits, "radian", 6) == 0 ) xfact = RAD2DEG;
+  if ( strncmp(yunits, "radian", 6) == 0 ) yfact = RAD2DEG;
+
+  double xval, yval, xfirst, xlast, ylast;
+  int lp2 = FALSE;
+
+  if ( xlon1 > xlon2 ) 
+    cdoAbort("The second longitude have to be greater than the first one!");
+
+  if ( xlat1 > xlat2 )
+    {
+      double xtemp = xlat1;
+      xlat1 = xlat2;
+      xlat2 = xtemp;
+    }
+
+  int ilat, ilon;
+  
+  for ( ilat = 0; ilat < nlat; ilat++ )
+    {
+      xlast = xfact * xvals[ilat*nlon + nlon-1];
+      ylast = yfact * yvals[ilat*nlon + nlon-1];
+      if ( ylast >= xlat1 && ylast <= xlat2 )
+        if ( grid_is_circular && xlon1 <= xlast && xlon2 > xlast && (xlon2-xlon1) < 360 )
+          {
+            lp2 = TRUE;
+          }
+    }
+
+  for ( ilat = 0; ilat < nlat; ilat++ )
+    {
+      for ( ilon = 0; ilon < nlon; ilon++ )
+        {
+          mask[nlon*ilat + ilon] = 1;
+
+          xval = xvals[ilat*nlon + ilon];
+          yval = yvals[ilat*nlon + ilon];
+
+          xval *= xfact;
+          yval *= yfact;
+
+          if ( yval >= xlat1 && yval <= xlat2 )
+            {
+              if ( lp2 )
+                {
+                  xfirst = xfact * xvals[ilat*nlon];
+                  if ( xfirst < xlon1 ) xfirst = xlon1;
+                  
+                  xlast = xfact * xvals[ilat*nlon + nlon-1];
+                  if ( xlast > xlon2 ) xlast = xlon2;
+
+                  if ( xval >= xlon1 && xval <= xlast )
+                    {
+                      mask[nlon*ilat + ilon] = 0;
+                    }
+                  else if ( xval >= xfirst && xval <= xlon2 )
+                    {
+                      mask[nlon*ilat + ilon] = 0;
+                    }
+                }
+              else
+                {
+                  if ( ((xval     >= xlon1 && xval     <= xlon2) ||
+                        (xval-360 >= xlon1 && xval-360 <= xlon2) ||
+                        (xval+360 >= xlon1 && xval+360 <= xlon2)) )
+                    {
+                      mask[nlon*ilat + ilon] = 0;
+                    }
+                }
+            }
+        }
+    }
+
+  Free(xvals);
+  Free(yvals);
+}
+
 static
 void maskregion(int *mask, int gridID, double *xcoords, double *ycoords, int nofcoords)
 {
@@ -145,8 +247,8 @@ void maskregion(int *mask, int gridID, double *xcoords, double *ycoords, int nof
   nlon = gridInqXsize(gridID);
   nlat = gridInqYsize(gridID);
 
-  xvals = (double*) malloc(nlon*sizeof(double));
-  yvals = (double*) malloc(nlat*sizeof(double));
+  xvals = (double*) Malloc(nlon*sizeof(double));
+  yvals = (double*) Malloc(nlat*sizeof(double));
 
   gridInqXvals(gridID, xvals);
   gridInqYvals(gridID, yvals);  
@@ -228,52 +330,42 @@ void maskregion(int *mask, int gridID, double *xcoords, double *ycoords, int nof
 	}
     }
       
-  free(xvals);
-  free(yvals);
+  Free(xvals);
+  Free(yvals);
 }
 
 
 void *Maskbox(void *argument)
 {
-  int MASKLONLATBOX, MASKINDEXBOX, MASKREGION;
-  int operatorID;
-  int streamID1, streamID2;
-  int nrecs, nvars;
-  int tsID, recID, varID, levelID;
-  int gridsize;
-  int vlistID1, vlistID2;
+  int nrecs;
+  int recID, varID, levelID;
   int gridID = -1;
-  int index, ngrids, gridtype;
+  int index, gridtype = -1;
   int nmiss;
-  int *vars;
   int i, i2;
-  int ndiffgrids;
   int lat1, lat2, lon11, lon12, lon21, lon22;
   int number = 0, nfiles;
   double missval;
-  int *mask;
-  double *array;
-  int taxisID1, taxisID2;
   double *xcoords, *ycoords;
   FILE *fp;
   const char *polyfile;
 
   cdoInitialize(argument);
 
-  MASKLONLATBOX = cdoOperatorAdd("masklonlatbox", 0, 0, "western and eastern longitude and southern and northern latitude");
-  MASKINDEXBOX  = cdoOperatorAdd("maskindexbox",  0, 0, "index of first and last longitude and index of first and last latitude");
-  MASKREGION    = cdoOperatorAdd("maskregion",    0, 0, "limiting coordinates of the region");
+  int MASKLONLATBOX = cdoOperatorAdd("masklonlatbox", 0, 0, "western and eastern longitude and southern and northern latitude");
+  int MASKINDEXBOX  = cdoOperatorAdd("maskindexbox",  0, 0, "index of first and last longitude and index of first and last latitude");
+  int MASKREGION    = cdoOperatorAdd("maskregion",    0, 0, "limiting coordinates of the region");
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
+  int vlistID1 = streamInqVlist(streamID1);
 
-  ngrids = vlistNgrids(vlistID1);
-  ndiffgrids = 0;
+  int ngrids = vlistNgrids(vlistID1);
+  int ndiffgrids = 0;
   for ( index = 1; index < ngrids; index++ )
     if ( vlistGrid(vlistID1, 0) != vlistGrid(vlistID1, index))
       ndiffgrids++;
@@ -297,14 +389,14 @@ void *Maskbox(void *argument)
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  nvars = vlistNvars(vlistID1);
-  vars  = (int*) malloc(nvars*sizeof(int));
+  int nvars = vlistNvars(vlistID1);
+  int *vars = (int *) Malloc(nvars*sizeof(int));
   for ( varID = 0; varID < nvars; varID++ )
     {
       if ( gridID == vlistInqVarGrid(vlistID1, varID) )
@@ -313,19 +405,26 @@ void *Maskbox(void *argument)
 	vars[varID] = FALSE;
     }
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  gridsize = gridInqSize(gridID);
-  array = (double*) malloc( gridsize*sizeof(double));
-  mask  = (int*) malloc( gridsize*sizeof(int));
-  for( i=0;  i < gridsize; i++) mask[i] = 1;
+  int gridsize = gridInqSize(gridID);
+  double *array = (double *) Malloc(gridsize*sizeof(double));
+  int *mask  = (int *) Malloc(gridsize*sizeof(int));
+  for ( i = 0;  i < gridsize; ++i ) mask[i] = 1;
  
   if ( operatorID == MASKLONLATBOX )
     {
-      genlonlatbox(0, gridID, &lat1, &lat2, &lon11, &lon12, &lon21, &lon22);
-      maskbox(mask, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
+      if ( gridtype == GRID_CURVILINEAR )
+        {
+          maskbox_curv(mask, gridID);
+        }
+      else
+        {
+          genlonlatbox(0, gridID, &lat1, &lat2, &lon11, &lon12, &lon21, &lon22);
+          maskbox(mask, gridID, lat1, lat2, lon11, lon12, lon21, lon22);
+        }
     }
   if ( operatorID == MASKINDEXBOX )
     {
@@ -334,8 +433,8 @@ void *Maskbox(void *argument)
     }
   if ( operatorID == MASKREGION )
     {
-      xcoords = (double*) malloc( MAX_VALS*sizeof(double));
-      ycoords = (double*) malloc( MAX_VALS*sizeof(double));
+      xcoords = (double*) Malloc( MAX_VALS*sizeof(double));
+      ycoords = (double*) Malloc( MAX_VALS*sizeof(double));
       nfiles = operatorArgc();
      
       for ( i2 = 0; i2 < nfiles; i2++ )
@@ -353,11 +452,11 @@ void *Maskbox(void *argument)
 	    }
 	  fclose(fp); 
 	}
-      if ( xcoords ) free ( xcoords );
-      if ( ycoords ) free ( ycoords );
+      if ( xcoords ) Free( xcoords );
+      if ( ycoords ) Free( ycoords );
     }
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
@@ -393,11 +492,11 @@ void *Maskbox(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( vars  ) free(vars);
-  if ( array ) free(array);
-  if ( mask )  free(mask);
+  if ( vars  ) Free(vars);
+  if ( array ) Free(array);
+  if ( mask )  Free(mask);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
