@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -40,6 +40,7 @@ void *Monarith(void *argument)
   int tsID, tsID2;
   int varID, levelID;
   int offset;
+  int nmiss;
   int vlistID1, vlistID2, vlistID3;
   int taxisID1, taxisID2, taxisID3;
   int vdate;
@@ -72,8 +73,8 @@ void *Monarith(void *argument)
   field_init(&field1);
   field_init(&field2);
 
-  field1.ptr = malloc(gridsize*sizeof(double));
-  field2.ptr = malloc(gridsize*sizeof(double));
+  field1.ptr = (double*) Malloc(gridsize*sizeof(double));
+  field2.ptr = (double*) Malloc(gridsize*sizeof(double));
 
   taxisID1 = vlistInqTaxis(vlistID1);
   taxisID2 = vlistInqTaxis(vlistID2);
@@ -86,15 +87,15 @@ void *Monarith(void *argument)
 
   nvars  = vlistNvars(vlistID2);
 
-  vardata2  = malloc(nvars*sizeof(double *));
-  varnmiss2 = malloc(nvars*sizeof(int *));
+  vardata2  = (double **) Malloc(nvars*sizeof(double *));
+  varnmiss2 = (int **) Malloc(nvars*sizeof(int *));
 
   for ( varID = 0; varID < nvars; varID++ )
     {
       gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
       nlev     = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
-      vardata2[varID]  = malloc(nlev*gridsize*sizeof(double));
-      varnmiss2[varID] = malloc(nlev*sizeof(int));
+      vardata2[varID]  = (double*) Malloc(nlev*gridsize*sizeof(double));
+      varnmiss2[varID] = (int*) Malloc(nlev*sizeof(int));
     }
 
   tsID  = 0;
@@ -140,8 +141,8 @@ void *Monarith(void *argument)
 	      gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
 	      offset   = gridsize*levelID;
 
-	      streamReadRecord(streamID2, vardata2[varID]+offset, &field2.nmiss);
-	      varnmiss2[varID][levelID] = field2.nmiss;
+	      streamReadRecord(streamID2, vardata2[varID]+offset, &nmiss);
+	      varnmiss2[varID][levelID] = nmiss;
 	    }
 
 	  tsID2++;
@@ -154,23 +155,22 @@ void *Monarith(void *argument)
       for ( recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
-	  streamReadRecord(streamID1, field1.ptr, &field1.nmiss);
+	  streamReadRecord(streamID1, field1.ptr, &nmiss);
+          field1.nmiss   = (size_t) nmiss;
+	  field1.grid    = vlistInqVarGrid(vlistID1, varID);
+	  field1.missval = vlistInqVarMissval(vlistID1, varID);
 
 	  gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
 	  offset   = gridsize*levelID;
 	  memcpy(field2.ptr, vardata2[varID]+offset, gridsize*sizeof(double));
-	  field2.nmiss = varnmiss2[varID][levelID];
-
-	  field1.grid    = vlistInqVarGrid(vlistID1, varID);
-	  field1.missval = vlistInqVarMissval(vlistID1, varID);
-
+	  field2.nmiss   = varnmiss2[varID][levelID];
 	  field2.grid    = vlistInqVarGrid(vlistID2, varID);
 	  field2.missval = vlistInqVarMissval(vlistID2, varID);
 
 	  farfun(&field1, field2, operfunc);
 
 	  streamDefRecord(streamID3, varID, levelID);
-	  streamWriteRecord(streamID3, field1.ptr, field1.nmiss);
+	  streamWriteRecord(streamID3, field1.ptr, (int)field1.nmiss);
 	}
 
       tsID++;
@@ -182,17 +182,17 @@ void *Monarith(void *argument)
 
   for ( varID = 0; varID < nvars; varID++ )
     {
-      free(vardata2[varID]);
-      free(varnmiss2[varID]);
+      Free(vardata2[varID]);
+      Free(varnmiss2[varID]);
     }
 
-  free(vardata2);
-  free(varnmiss2);
+  Free(vardata2);
+  Free(varnmiss2);
 
-  if ( field1.ptr ) free(field1.ptr);
-  if ( field2.ptr ) free(field2.ptr);
+  if ( field1.ptr ) Free(field1.ptr);
+  if ( field2.ptr ) Free(field2.ptr);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
