@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -68,9 +68,7 @@ void *Output(void *argument)
   double missval;
   double lon, lat;
   char name[CDI_MAX_NAME];
-  int npar = 0;
   int year, month, day;
-  char **parnames = NULL;
   int *keys = NULL, nkeys = 0, k;
   int nKeys;
   int Keylen[]           = {      0,        8,      11,      4,      8,     6,     6,     6,     6,      4,      4,          6,     10,      8,      5,       2,     2 };
@@ -102,20 +100,20 @@ void *Output(void *argument)
       if ( operatorArgc() < 1 ) cdoAbort("Too few arguments!");
 
       format = operatorArgv()[0];
-      if ( operatorArgc() == 2 ) nelem  = atoi(operatorArgv()[1]);
+      if ( operatorArgc() == 2 ) nelem = parameter2int(operatorArgv()[1]);
     }
   else if ( operatorID == OUTPUTTAB )
     {
       operatorInputArg("keys to print");
  
-      npar     = operatorArgc();
-      parnames = operatorArgv();
+      int npar = operatorArgc();
+      char **parnames = operatorArgv();
 
       if ( cdoVerbose )
 	for ( i = 0; i < npar; i++ )
 	  cdoPrint("key %d = %s", i+1, parnames[i]);
 
-      keys = (int*) malloc(npar*sizeof(int));
+      keys = (int*) Malloc(npar*sizeof(int));
       nkeys = 0;
       nKeys = sizeof(Keynames)/sizeof(char *);
       for ( i = 0; i < npar; i++ )
@@ -175,7 +173,7 @@ void *Output(void *argument)
       gridsize = gridInqSize(gridID);
       gridtype = gridInqType(gridID);
 
-      array = (double*) malloc(gridsize*sizeof(double));
+      array = (double*) Malloc(gridsize*sizeof(double));
 
       if ( operatorID == OUTPUTFLD || operatorID == OUTPUTXYZ || operatorID == OUTPUTTAB )
 	{
@@ -186,8 +184,8 @@ void *Output(void *argument)
 
 	  gridtype = gridInqType(gridID);
 
-	  grid_center_lon = (double*) malloc(gridsize*sizeof(double));
-	  grid_center_lat = (double*) malloc(gridsize*sizeof(double));
+	  grid_center_lon = (double*) Malloc(gridsize*sizeof(double));
+	  grid_center_lat = (double*) Malloc(gridsize*sizeof(double));
 	  gridInqXvals(gridID, grid_center_lon);
 	  gridInqYvals(gridID, grid_center_lat);
 
@@ -335,7 +333,7 @@ void *Output(void *argument)
 		{
 		  if ( tsID == 0 && recID == 0 )
 		    {
-		      char *fname = "frontplane.xyz";
+		      const char *fname = "frontplane.xyz";
 		      FILE *fp;
 		      double fmin = 0;
 		      double dx, x0, y0, z0, x, y, z;
@@ -451,14 +449,14 @@ void *Output(void *argument)
 	}
       streamClose(streamID);
 
-      if ( array ) free(array);
-      if ( grid_center_lon ) free(grid_center_lon);
-      if ( grid_center_lat ) free(grid_center_lat);
+      if ( array ) Free(array);
+      if ( grid_center_lon ) Free(grid_center_lon);
+      if ( grid_center_lat ) Free(grid_center_lat);
     }
 
-  if ( keys ) free(keys);
+  if ( keys ) Free(keys);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
