@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -29,30 +29,27 @@
 #include "cdo.h"
 #include "cdo_int.h"
 #include "pstream.h"
-#include "vinterp.h"
+#include "after_vertint.h"
 #include "list.h"
 #include "stdnametable.h"
 
 
 void *Pressure(void *argument)
 {
-  int PRESSURE_FL, PRESSURE_HL, DELTAP;
-  int operatorID;
   int mode;
   enum {ECHAM_MODE, WMO_MODE};
-  int geop_code = 0, temp_code = 0, ps_code = 0, lsp_code = 0;
-  int streamID1, streamID2;
-  int vlistID1, vlistID2;
-  int gridsize, ngp = 0;
+  int ps_code = 0, lsp_code = 0;
+  int streamID2;
+  int vlistID2;
   int recID, nrecs;
   int i, k, offset;
   int tsID, varID, levelID;
   int nvars;
   int zaxisIDp, zaxisIDh = -1, nzaxis;
-  int ngrids, gridID, zaxisID;
-  int nhlev = 0, nhlevf = 0, nhlevh = 0, nlevel;
+  int gridID, zaxisID;
+  int nhlev = 0, nhlevf = 0, nhlevh = 0, nlevel = 0;
   int nvct;
-  int geopID = -1, tempID = -1, psID = -1, lnpsID = -1, pvarID = -1;
+  int psID = -1, lnpsID = -1, pvarID = -1;
   int code, param;
   char paramstr[32];
   char varname[CDI_MAX_NAME];
@@ -70,37 +67,17 @@ void *Pressure(void *argument)
 
   cdoInitialize(argument);
 
-  PRESSURE_FL = cdoOperatorAdd("pressure_fl", 0, 0, NULL);
-  PRESSURE_HL = cdoOperatorAdd("pressure_hl", 0, 0, NULL);
-  DELTAP      = cdoOperatorAdd("deltap",      0, 0, NULL);
+  int PRESSURE_FL = cdoOperatorAdd("pressure_fl", 0, 0, NULL);
+  int PRESSURE_HL = cdoOperatorAdd("pressure_hl", 0, 0, NULL);
+  int DELTAP      = cdoOperatorAdd("deltap",      0, 0, NULL);
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
+  int vlistID1 = streamInqVlist(streamID1);
 
-  ngrids  = vlistNgrids(vlistID1);
-  for ( i = 0; i < ngrids; i++ )
-    {
-      gridID = vlistGrid(vlistID1, i);
-      if ( gridInqType(gridID) != GRID_SPECTRAL )
-	{
-	  ngp = gridInqSize(gridID);
-	  break;
-	}
-    }
-
-  /* check gridsize */
-  for ( i = 0; i < ngrids; i++ )
-    {
-      gridID = vlistGrid(vlistID1, i);
-      if ( gridInqType(gridID) != GRID_SPECTRAL )
-	{
-	  if ( ngp != gridInqSize(gridID) )
-	    cdoAbort("Grids have different size!");
-	}
-    }
+  int gridsize = vlist_check_gridsize(vlistID1);
 
   nzaxis  = vlistNzaxis(vlistID1);
   lhavevct = FALSE;
@@ -116,14 +93,14 @@ void *Pressure(void *argument)
 	{
 	  double *level;
 	  int l;
-	  level = (double*) malloc(nlevel*sizeof(double));
+	  level = (double*) Malloc(nlevel*sizeof(double));
 	  zaxisInqLevels(zaxisID, level);
 	  for ( l = 0; l < nlevel; l++ )
 	    {
 	      if ( (l+1) != (int) (level[l]+0.5) ) break;
 	    }
 	  if ( l == nlevel ) mono_level = TRUE; 
-	  free(level);
+	  Free(level);
 	}
 
       if ( (zaxisInqType(zaxisID) == ZAXIS_HYBRID || zaxisInqType(zaxisID) == ZAXIS_HYBRID_HALF) &&
@@ -140,7 +117,7 @@ void *Pressure(void *argument)
 		  nhlevf   = nhlev;
 		  nhlevh   = nhlevf + 1;
 	      
-		  vct = (double*) malloc(nvct*sizeof(double));
+		  vct = (double*) Malloc(nvct*sizeof(double));
 		  zaxisInqVct(zaxisID, vct);
 		}
 	    }
@@ -154,7 +131,7 @@ void *Pressure(void *argument)
 		  nhlevf   = nhlev - 1;
 		  nhlevh   = nhlev;
 	      
-		  vct = (double*) malloc(nvct*sizeof(double));
+		  vct = (double*) Malloc(nvct*sizeof(double));
 		  zaxisInqVct(zaxisID, vct);
 		}
 	    }
@@ -166,7 +143,7 @@ void *Pressure(void *argument)
 		  int voff = 4;
 		  double *rvct = NULL;
 
-		  rvct = (double*) malloc(nvct*sizeof(double));
+		  rvct = (double*) Malloc(nvct*sizeof(double));
 		  zaxisInqVct(zaxisID,rvct);
 
 		  if ( (int)(rvct[0]+0.5) == 100000 && rvct[voff] < rvct[voff+1] )
@@ -178,7 +155,7 @@ void *Pressure(void *argument)
 		      nhlevh   = nhlev + 1;
 
 		      vctsize = 2*nhlevh;
-		      vct = (double*) malloc(vctsize*sizeof(double));
+		      vct = (double*) Malloc(vctsize*sizeof(double));
 
 		      /* calculate VCT for LM */
 
@@ -202,7 +179,7 @@ void *Pressure(void *argument)
 			    fprintf(stdout, "%5d %25.17f %25.17f\n", i, vct[i], vct[vctsize/2+i]);
 			}
 		    }
-		  free(rvct);
+		  Free(rvct);
 		}
 	    }
 	}
@@ -211,12 +188,12 @@ void *Pressure(void *argument)
 
   nvars = vlistNvars(vlistID1);
 
-  if ( zaxisIDh != -1 && ngp > 0 )
+  if ( zaxisIDh != -1 && gridsize > 0 )
     {
-      ps_prog    = (double*) malloc(ngp*sizeof(double));
-      deltap     = (double*) malloc(ngp*nhlevf*sizeof(double));
-      full_press = (double*) malloc(ngp*nhlevf*sizeof(double));
-      half_press = (double*) malloc(ngp*nhlevh*sizeof(double));
+      ps_prog    = (double*) Malloc(gridsize*sizeof(double));
+      deltap     = (double*) Malloc(gridsize*nhlevf*sizeof(double));
+      full_press = (double*) Malloc(gridsize*nhlevf*sizeof(double));
+      half_press = (double*) Malloc(gridsize*nhlevh*sizeof(double));
     }
   else
     cdoAbort("No 3D variable with hybrid sigma pressure coordinate found!");
@@ -229,10 +206,10 @@ void *Pressure(void *argument)
   {
     double *level;
     int l;
-    level = (double*) malloc(nhlevh*sizeof(double));
+    level = (double*) Malloc(nhlevh*sizeof(double));
     for ( l = 0; l < nhlevh; l++ ) level[l] = l+1;
     zaxisDefLevels(zaxisIDp, level);
-    free(level);
+    Free(level);
   }
 
   zaxisDefVct(zaxisIDp, 2*nhlevh, vct);
@@ -255,7 +232,6 @@ void *Pressure(void *argument)
     {
       gridID   = vlistInqVarGrid(vlistID1, varID);
       zaxisID  = vlistInqVarZaxis(vlistID1, varID);
-      gridsize = gridInqSize(gridID);
       nlevel   = zaxisInqSize(zaxisID);
       instNum  = institutInqCenter(vlistInqVarInstitut(vlistID1, varID));
       tableNum = tableInqNum(vlistInqVarTable(vlistID1, varID));
@@ -270,15 +246,11 @@ void *Pressure(void *argument)
 	  if ( tableNum == 2 )
 	    {
 	      mode = WMO_MODE;
-	      geop_code  =   6;
-	      temp_code  =  11;
 	      ps_code    =   1;
 	    }
 	  else if ( tableNum == 128 )
 	    {
 	      mode = ECHAM_MODE;
-	      geop_code  = 129;
-	      temp_code  = 130;
 	      ps_code    = 134;
 	      lsp_code   = 152;
 	    }
@@ -288,8 +260,6 @@ void *Pressure(void *argument)
       else
 	{
 	  mode = ECHAM_MODE;
-	  geop_code  = 129;
-	  temp_code  = 130;
 	  ps_code    = 134;
 	  lsp_code   = 152;
 	}
@@ -314,17 +284,13 @@ void *Pressure(void *argument)
 
       if ( mode == ECHAM_MODE )
 	{
-	  if      ( code == geop_code  && nlevel == 1     ) geopID  = varID;
-	  else if ( code == temp_code  && nlevel == nhlev ) tempID  = varID;
-	  else if ( code == ps_code    && nlevel == 1     ) psID    = varID;
+	  if      ( code == ps_code    && nlevel == 1     ) psID    = varID;
 	  else if ( code == lsp_code   && nlevel == 1     ) lnpsID  = varID;
 	  /* else if ( code == 156 ) gheightID = varID; */
 	}
       else if ( mode == WMO_MODE )
 	{
-	  if      ( code == geop_code  && nlevel == 1     ) geopID  = varID;
-	  else if ( code == temp_code  && nlevel == nhlev ) tempID  = varID;
-	  else if ( code == ps_code    && nlevel == 1     ) psID    = varID;
+	  if ( code == ps_code    && nlevel == 1     ) psID    = varID;
 	}
     }
 
@@ -350,13 +316,12 @@ void *Pressure(void *argument)
   if ( gridInqType(gridID) == GRID_SPECTRAL )
     cdoAbort("%s on spectral representation not supported!", var_stdname(surface_air_pressure));
 
-  gridsize = gridInqSize(gridID);
-  pdata = (double*) malloc(gridsize*sizeof(double));
+  pdata = (double*) Malloc(gridsize*sizeof(double));
 
 
   vlistID2 = vlistCreate();
   varID = vlistDefVar(vlistID2, gridID, zaxisIDp, TSTEP_INSTANT);
-  vlistDefVarCode(vlistID2, varID, 1);
+  vlistDefVarParam(vlistID2, varID, cdiEncodeParam(1, 255, 255));
   vlistDefVarName(vlistID2, varID, "pressure");
   vlistDefVarStdname(vlistID2, varID, "air_pressure");
   vlistDefVarUnits(vlistID2, varID, "Pa");
@@ -391,16 +356,16 @@ void *Pressure(void *argument)
       if ( zaxisIDh != -1 )
 	{
 	  if ( lnpsID != -1 )
-	    for ( i = 0; i < ngp; i++ ) ps_prog[i] = exp(pdata[i]);
+	    for ( i = 0; i < gridsize; i++ ) ps_prog[i] = exp(pdata[i]);
 	  else if ( psID != -1 )
-	    memcpy(ps_prog, pdata, ngp*sizeof(double));
+	    memcpy(ps_prog, pdata, gridsize*sizeof(double));
 
 	  /* check range of ps_prog */
-	  minmaxval(ngp, ps_prog, NULL, &minval, &maxval);
+	  minmaxval(gridsize, ps_prog, NULL, &minval, &maxval);
 	  if ( minval < MIN_PS || maxval > MAX_PS )
 	    cdoWarning("Surface pressure out of range (min=%g max=%g)!", minval, maxval);
 	    
-	  presh(full_press, half_press, vct, ps_prog, nhlevf, ngp);
+	  presh(full_press, half_press, vct, ps_prog, nhlevf, gridsize);
 	}
 
       if ( operatorID == PRESSURE_FL )
@@ -412,14 +377,14 @@ void *Pressure(void *argument)
 	{
 	  nlevel = nhlevf;
 	  for ( k = 0; k < nhlevf; ++k )
-	    for ( i = 0; i < ngp; ++i )
+	    for ( i = 0; i < gridsize; ++i )
 	      {
-		deltap[k*ngp+i] = half_press[(k+1)*ngp+i] - half_press[k*ngp+i];
+		deltap[k*gridsize+i] = half_press[(k+1)*gridsize+i] - half_press[k*gridsize+i];
 	      }
 
 	  pout = deltap;
 	}
-      else
+      else if ( operatorID == PRESSURE_HL )
 	{
 	  nlevel = nhlevh;
 	  pout = half_press;
@@ -439,14 +404,14 @@ void *Pressure(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( pdata      ) free(pdata);
-  if ( ps_prog    ) free(ps_prog);
-  if ( deltap     ) free(deltap);
-  if ( full_press ) free(full_press);
-  if ( half_press ) free(half_press);
-  if ( vct        ) free(vct);
+  if ( pdata      ) Free(pdata);
+  if ( ps_prog    ) Free(ps_prog);
+  if ( deltap     ) Free(deltap);
+  if ( full_press ) Free(full_press);
+  if ( half_press ) Free(half_press);
+  if ( vct        ) Free(vct);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
