@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -88,7 +88,7 @@ void get_map_type(int operfunc, int *map_type, int *submap_type, int *num_neighb
     case REMAPDIS:
     case GENDIS:
       *map_type = MAP_TYPE_DISTWGT;
-      *num_neighbors = 4;
+      if ( *num_neighbors == 0 ) *num_neighbors = 4;
       break;
     case REMAPNN:
     case GENNN:
@@ -162,7 +162,7 @@ int maptype2operfunc(int map_type, int submap_type, int num_neighbors, int remap
 } 
 
 static
-void print_remap_info(int operfunc, remapgrid_t *src_grid, remapgrid_t *tgt_grid, int nmiss)
+void print_remap_info(int operfunc, int remap_genweights, remapgrid_t *src_grid, remapgrid_t *tgt_grid, int nmiss)
 {
   char line[256];
   char tmpstr[256];
@@ -171,15 +171,19 @@ void print_remap_info(int operfunc, remapgrid_t *src_grid, remapgrid_t *tgt_grid
 
   if      ( operfunc == REMAPBIL  || operfunc == GENBIL  )  strcpy(line, "SCRIP bilinear");
   else if ( operfunc == REMAPBIC  || operfunc == GENBIC  )  strcpy(line, "SCRIP bicubic");
-  else if ( operfunc == REMAPNN   || operfunc == GENNN   )  strcpy(line, "SCRIP nearest neighbor");
-  else if ( operfunc == REMAPDIS  || operfunc == GENDIS  )  strcpy(line, "SCRIP distance-weighted average");
+  else if ( operfunc == REMAPNN   || operfunc == GENNN   )  strcpy(line, "Nearest neighbor");
+  else if ( operfunc == REMAPDIS  || operfunc == GENDIS  )  strcpy(line, "Distance-weighted average");
   else if ( operfunc == REMAPCON  || operfunc == GENCON  )  strcpy(line, "SCRIP first order conservative");
   else if ( operfunc == REMAPCON2 || operfunc == GENCON2 )  strcpy(line, "SCRIP second order conservative");
   else if ( operfunc == REMAPLAF  || operfunc == GENLAF  )  strcpy(line, "YAC largest area fraction");
   else if ( operfunc == REMAPYCON || operfunc == GENYCON )  strcpy(line, "YAC first order conservative");
   else                                                      strcpy(line, "Unknown");
 
-  strcat(line, " remapping from ");
+  if ( remap_genweights )
+    strcat(line, " weights from ");
+  else
+    strcat(line, " remapping from ");
+
   strcat(line, gridNamePtr(gridInqType(src_grid->gridID)));
   if ( src_grid->rank == 2 )
     sprintf(tmpstr, " (%dx%d)", src_grid->dims[0], src_grid->dims[1]);
@@ -204,21 +208,62 @@ void print_remap_info(int operfunc, remapgrid_t *src_grid, remapgrid_t *tgt_grid
   cdoPrint(line);
 }
 
+static
+void print_remap_warning(const char *remap_file, int operfunc, remapgrid_t *src_grid, int nmiss)
+{
+  char line[256];
+  char tmpstr[256];
+
+  line[0] = 0;
+  /*
+  if      ( operfunc == REMAPBIL  || operfunc == GENBIL  )  strcpy(line, "SCRIP bilinear");
+  else if ( operfunc == REMAPBIC  || operfunc == GENBIC  )  strcpy(line, "SCRIP bicubic");
+  else if ( operfunc == REMAPNN   || operfunc == GENNN   )  strcpy(line, "SCRIP nearest neighbor");
+  else if ( operfunc == REMAPDIS  || operfunc == GENDIS  )  strcpy(line, "SCRIP distance-weighted average");
+  else if ( operfunc == REMAPCON  || operfunc == GENCON  )  strcpy(line, "SCRIP first order conservative");
+  else if ( operfunc == REMAPCON2 || operfunc == GENCON2 )  strcpy(line, "SCRIP second order conservative");
+  else if ( operfunc == REMAPLAF  || operfunc == GENLAF  )  strcpy(line, "YAC largest area fraction");
+  else if ( operfunc == REMAPYCON || operfunc == GENYCON )  strcpy(line, "YAC first order conservative");
+  else                                                      strcpy(line, "Unknown");
+
+  strcat(line, " remap weights from ");
+  */
+  strcat(line, "Remap weights from ");
+  strcat(line, remap_file);
+  strcat(line, " not used, ");
+  strcat(line, gridNamePtr(gridInqType(src_grid->gridID)));
+  if ( src_grid->rank == 2 )
+    sprintf(tmpstr, " (%dx%d)", src_grid->dims[0], src_grid->dims[1]);
+  else
+    sprintf(tmpstr, " (%d)", src_grid->dims[0]);
+  strcat(line, tmpstr);
+  strcat(line, " grid");
+
+  if ( nmiss > 0 )
+    {
+      sprintf(tmpstr, " with mask (%d)", gridInqSize(src_grid->gridID)-nmiss);
+      strcat(line, tmpstr);
+    }
+
+  strcat(line, " not found!");
+
+  cdoWarning(line);
+}
+
 
 double remap_threshhold = 2;
-double remap_search_radius = 180;
+double gridsearch_radius = 180;
 int remap_test = 0;
 int remap_order = 1;
 int remap_non_global = FALSE;
 int remap_num_srch_bins = 180;
 int lremap_num_srch_bins = FALSE;
 int remap_extrapolate = FALSE;
-int remap_genweights = TRUE;
 int lextrapolate = FALSE;
 int max_remaps = -1;
 int sort_mode = HEAP_SORT;
 double remap_frac_min = 0;
-
+int remap_genweights = TRUE;
 
 static
 void get_remap_env(void)
@@ -317,25 +362,42 @@ void get_remap_env(void)
 
   remap_set_threshhold(remap_threshhold);
 
-  envstr = getenv("CDO_REMAP_SEARCH_RADIUS");
+  envstr = getenv("CDO_REMAP_RADIUS");
   if ( envstr )
     {
       double fval;
       fval = atof(envstr);
       if ( fval < 0 || fval > 180 )
 	{
-	  cdoAbort("CDO_REMAP_SEARCH_RADIUS=%g out of bounds (0-180)", fval);
+	  cdoAbort("CDO_REMAP_RADIUS=%g out of bounds (0-180)", fval);
 	}
       else
 	{
-	  remap_search_radius = fval;
+	  gridsearch_radius = fval;
 	  if ( cdoVerbose )
-	    cdoPrint("Set CDO_REMAP_SEARCH_RADIUS to %g", remap_search_radius);
+	    cdoPrint("Set CDO_REMAP_RADIUS to %g", gridsearch_radius);
+	}
+    }
+
+  envstr = getenv("CDO_GRIDSEARCH_RADIUS");
+  if ( envstr )
+    {
+      double fval;
+      fval = atof(envstr);
+      if ( fval < 0 || fval > 180 )
+	{
+	  cdoAbort("CDO_GRIDSEARCH_RADIUS=%g out of bounds (0-180)", fval);
+	}
+      else
+	{
+	  gridsearch_radius = fval;
+	  if ( cdoVerbose )
+	    cdoPrint("Set CDO_GRIDSEARCH_RADIUS to %g", gridsearch_radius);
 	}
     }
   
   if ( cdoVerbose )
-    cdoPrint("remap_search_radius = %g", remap_search_radius);
+    cdoPrint("remap_radius = %g", gridsearch_radius);
 
   envstr = getenv("REMAP_AREA_MIN");
   if ( envstr )
@@ -506,7 +568,7 @@ void scale_gridbox_area(long gridsize, const double *restrict array1, long grids
 }
 
 static
-int set_remapgrids(int vlistID, int ngrids, int *remapgrids)
+int set_remapgrids(int filetype, int vlistID, int ngrids, int *remapgrids)
 {
   int index, gridID, gridtype;
 
@@ -527,7 +589,12 @@ int set_remapgrids(int vlistID, int ngrids, int *remapgrids)
 	   gridtype != GRID_UNSTRUCTURED )
 	{
 	  if ( gridtype == GRID_GAUSSIAN_REDUCED )
-	    cdoAbort("Unsupported grid type: %s, use CDO option -R to convert reduced to regular grid!", gridNamePtr(gridtype));
+	    {
+	      if ( !cdoRegulargrid && filetype == FILETYPE_GRB )
+		cdoAbort("Unsupported grid type: %s, use CDO option -R to convert reduced to regular grid!", gridNamePtr(gridtype));
+	      else
+		cdoAbort("Unsupported grid type: %s, use CDO operator -setgridtype,regular to convert reduced to regular grid!", gridNamePtr(gridtype));
+	    }
 	  else if ( gridtype == GRID_GENERIC && gridInqSize(gridID) == 1 )
 	    remapgrids[index] = FALSE;
 	  else
@@ -572,8 +639,9 @@ static
 int get_norm_opt(void)
 {
   int norm_opt = NORM_OPT_FRACAREA;
-  char *envstr = getenv("CDO_REMAP_NORMALIZE_OPT");
+  char *envstr;
 
+  envstr = getenv("CDO_REMAP_NORMALIZE_OPT"); // obsolate
   if ( envstr && *envstr )
     {
       if      ( memcmp(envstr, "frac", 4) == 0 ) norm_opt = NORM_OPT_FRACAREA;
@@ -582,6 +650,15 @@ int get_norm_opt(void)
       else cdoWarning("CDO_REMAP_NORMALIZE_OPT=%s unsupported!", envstr);
     }
 
+  envstr = getenv("CDO_REMAP_NORM");
+  if ( envstr && *envstr )
+    {
+      if      ( memcmp(envstr, "frac", 4) == 0 ) norm_opt = NORM_OPT_FRACAREA;
+      else if ( memcmp(envstr, "dest", 4) == 0 ) norm_opt = NORM_OPT_DESTAREA;
+      else if ( memcmp(envstr, "none", 4) == 0 ) norm_opt = NORM_OPT_NONE;
+      else cdoWarning("CDO_REMAP_NORM=%s unsupported!", envstr);
+    }
+
   if ( cdoVerbose )
     {
       if      ( norm_opt == NORM_OPT_FRACAREA ) cdoPrint("Normalization option: frac");
@@ -644,7 +721,7 @@ void remap_set_frac_min(int gridsize, double *array, double missval, remapgrid_t
 
 
 int timer_remap, timer_remap_init, timer_remap_sort;
-int timer_remap_bil, timer_remap_nn, timer_remap_con, timer_remap_con_l1, timer_remap_con_l2;
+int timer_remap_bil, timer_remap_bic, timer_remap_dis, timer_remap_con, timer_remap_con_l1, timer_remap_con_l2;
 
 static
 void init_remap_timer(void)
@@ -653,7 +730,8 @@ void init_remap_timer(void)
   timer_remap_init   = timer_new("remap init");
   timer_remap_sort   = timer_new("remap sort");
   timer_remap_bil    = timer_new("remap bil");
-  timer_remap_nn     = timer_new("remap nn");
+  timer_remap_bic    = timer_new("remap bic");
+  timer_remap_dis    = timer_new("remap dis");
   timer_remap_con    = timer_new("remap con");
   timer_remap_con_l1 = timer_new("remap con loop1");
   timer_remap_con_l2 = timer_new("remap con loop2");
@@ -670,13 +748,13 @@ void sort_remap_add(remapvars_t *remapvars)
       ** OpenMP parallelism is supported
       */   
       sort_iter(remapvars->num_links, remapvars->num_wts,
-		remapvars->tgt_grid_add, remapvars->src_grid_add,
+		remapvars->tgt_cell_add, remapvars->src_cell_add,
 		remapvars->wts, ompNumThreads);
     }
   else
     { /* use a pure heap sort without any support of parallelism */
       sort_add(remapvars->num_links, remapvars->num_wts,
-	       remapvars->tgt_grid_add, remapvars->src_grid_add,
+	       remapvars->tgt_cell_add, remapvars->src_cell_add,
 	       remapvars->wts);
     }
   if ( cdoTimer ) timer_stop(timer_remap_sort);
@@ -685,15 +763,11 @@ void sort_remap_add(remapvars_t *remapvars)
 
 void *Remap(void *argument)
 {
-  int operatorID;
-  int operfunc;
-  int streamID1, streamID2 = -1;
-  int nrecs, ngrids;
+  int streamID2 = -1;
+  int nrecs;
   int index;
   int tsID, recID, varID, levelID;
   int gridsize, gridsize2;
-  int vlistID1, vlistID2;
-  int taxisID1, taxisID2;
   int gridID1 = -1, gridID2;
   int nmiss1, nmiss2, i, j, r = -1;
   int *imask = NULL;
@@ -701,17 +775,15 @@ void *Remap(void *argument)
   int norm_opt = NORM_OPT_NONE;
   int map_type = -1;
   int submap_type = SUBMAP_TYPE_NONE;
-  int num_neighbors = 4;
+  int num_neighbors = 0;
   int need_gradiants = FALSE;
   int grid1sizemax;
-  int *remapgrids = NULL;
   char varname[CDI_MAX_NAME];
   double missval;
   double *array1 = NULL, *array2 = NULL;
   double *grad1_lat = NULL, *grad1_lon = NULL, *grad1_latlon = NULL;
   remap_t *remaps = NULL;
   char *remap_file = NULL;
-  int lwrite_remap;
 
   if ( cdoTimer ) init_remap_timer();
 
@@ -736,9 +808,10 @@ void *Remap(void *argument)
   cdoOperatorAdd("remapycon",    REMAPYCON,    0, NULL);
   cdoOperatorAdd("genycon",      GENYCON,      1, NULL);
 
-  operatorID   = cdoOperatorID();
-  operfunc     = cdoOperatorF1(operatorID);
-  lwrite_remap = cdoOperatorF2(operatorID);
+  int operatorID   = cdoOperatorID();
+  int operfunc     = cdoOperatorF1(operatorID);
+  int lwrite_remap = cdoOperatorF2(operatorID);
+  int lremapxxx    = operfunc == REMAPXXX;
 
   remap_set_int(REMAP_WRITE_REMAP, lwrite_remap);
 
@@ -756,9 +829,9 @@ void *Remap(void *argument)
 	cdoPrint("Extrapolation disabled!");
     }
 
-  if ( operfunc == REMAPXXX )
+  if ( lremapxxx )
     {
-      operatorInputArg("grid description file or name, remap weights file (SCRIP netCDF)");
+      operatorInputArg("grid description file or name, remap weights file (SCRIP NetCDF)");
       operatorCheckArgc(2);
       gridID2 = cdoDefineGrid(operatorArgv()[0]);
       remap_file = operatorArgv()[1];
@@ -766,22 +839,35 @@ void *Remap(void *argument)
   else
     {
       operatorInputArg("grid description file or name");
-      operatorCheckArgc(1);
-      gridID2 = cdoDefineGrid(operatorArgv()[0]);
+      if ( operfunc == REMAPDIS && operatorArgc() == 2 )
+        {
+          gridID2 = cdoDefineGrid(operatorArgv()[0]);
+          int inum = parameter2int(operatorArgv()[1]);
+          //  if ( inum < 1 || inum > 9 ) cdoAbort("Number of nearest neighbors out of range (1-9)!", inum);
+          num_neighbors = inum;
+        }
+      else
+        {
+          operatorCheckArgc(1);
+          gridID2 = cdoDefineGrid(operatorArgv()[0]);
+        }
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  if ( gridInqType(gridID2) == GRID_GENERIC ) cdoAbort("Unsupported target grid type (generic)!");
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int streamID1 = streamOpenRead(cdoStreamName(0));
+  int filetype = streamInqFiletype(streamID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
+
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  ngrids = vlistNgrids(vlistID1);
-  remapgrids = (int*) malloc(ngrids*sizeof(int));
-  index = set_remapgrids(vlistID1, ngrids, remapgrids);
+  int ngrids = vlistNgrids(vlistID1);
+  int remapgrids[ngrids];
+  index = set_remapgrids(filetype, vlistID1, ngrids, remapgrids);
   gridID1 = vlistGrid(vlistID1, index);
 
   for ( index = 0; index < ngrids; index++ )
@@ -792,19 +878,20 @@ void *Remap(void *argument)
 
   if ( max_remaps > 0 )
     {
-      remaps = (remap_t*) malloc(max_remaps*sizeof(remap_t));
+      remaps = (remap_t*) Malloc(max_remaps*sizeof(remap_t));
       for ( r = 0; r < max_remaps; r++ )
 	{
+	  remaps[r].nused    = 0;
 	  remaps[r].gridID   = -1;
 	  remaps[r].gridsize = 0;
 	  remaps[r].nmiss    = 0;
 	}
     }
 
-  if ( lwrite_remap || operfunc == REMAPXXX )
+  if ( lwrite_remap || lremapxxx )
     remap_genweights = TRUE;
 
-  if ( operfunc == REMAPXXX )
+  if ( lremapxxx )
     {
       int gridsize2;
 
@@ -812,9 +899,8 @@ void *Remap(void *argument)
 		       &remap_order, &remaps[0].src_grid, &remaps[0].tgt_grid, &remaps[0].vars);
       nremaps = 1;
       gridsize = remaps[0].src_grid.size;
-      remaps[0].gridID = gridID1;
+      remaps[0].gridID   = gridID1;
       remaps[0].gridsize = gridInqSize(gridID1);
-      remaps[0].nmiss = 0;
 
       if ( map_type == MAP_TYPE_DISTWGT && !lextrapolate ) remap_extrapolate = TRUE;
       if ( gridIsCircular(gridID1)      && !lextrapolate ) remap_extrapolate = TRUE;
@@ -842,7 +928,7 @@ void *Remap(void *argument)
 	  int gridID2_gme;
 	  int isize = 0;
 	  remaps[0].tgt_grid.nvgp = gridInqSize(gridID2);
-	  remaps[0].tgt_grid.vgpm = (int*) realloc(remaps[0].tgt_grid.vgpm, gridInqSize(gridID2)*sizeof(int));
+	  remaps[0].tgt_grid.vgpm = (int*) Realloc(remaps[0].tgt_grid.vgpm, gridInqSize(gridID2)*sizeof(int));
 	  gridID2_gme = gridToUnstructured(gridID2, 1);
 	  gridInqMaskGME(gridID2_gme, remaps[0].tgt_grid.vgpm);
 	  for ( i = 0; i < gridsize2; ++i )
@@ -864,9 +950,13 @@ void *Remap(void *argument)
       get_map_type(operfunc, &map_type, &submap_type, &num_neighbors, &remap_order);
     }
 
-  if ( remap_genweights == FALSE && map_type != MAP_TYPE_BILINEAR && map_type != MAP_TYPE_BICUBIC && map_type != MAP_TYPE_CONSERV_YAC )
+  if ( remap_genweights == FALSE &&
+       map_type != MAP_TYPE_BILINEAR && map_type != MAP_TYPE_BICUBIC &&
+       map_type != MAP_TYPE_DISTWGT  && map_type != MAP_TYPE_CONSERV_YAC )
     remap_genweights = TRUE;
 
+  remap_set_int(REMAP_GENWEIGHTS, remap_genweights);
+
   if ( map_type == MAP_TYPE_CONSERV || map_type == MAP_TYPE_CONSERV_YAC ) norm_opt = get_norm_opt();
 
   grid1sizemax = vlistGridsizeMax(vlistID1);
@@ -882,16 +972,16 @@ void *Remap(void *argument)
 
   if ( need_gradiants )
     {
-      grad1_lat    = (double*) malloc(grid1sizemax*sizeof(double));
-      grad1_lon    = (double*) malloc(grid1sizemax*sizeof(double));
-      grad1_latlon = (double*) malloc(grid1sizemax*sizeof(double));
+      grad1_lat    = (double*) Malloc(grid1sizemax*sizeof(double));
+      grad1_lon    = (double*) Malloc(grid1sizemax*sizeof(double));
+      grad1_latlon = (double*) Malloc(grid1sizemax*sizeof(double));
     }
 
-  array1 = (double*) malloc(grid1sizemax*sizeof(double));
-  imask  = (int*) malloc(grid1sizemax*sizeof(int));
+  array1 = (double*) Malloc(grid1sizemax*sizeof(double));
+  imask  = (int*) Malloc(grid1sizemax*sizeof(int));
 
   gridsize = gridInqSize(gridID2);
-  array2   = (double*) malloc(gridsize*sizeof(double));
+  array2   = (double*) Malloc(gridsize*sizeof(double));
 
   if ( ! lwrite_remap )
     {
@@ -946,14 +1036,14 @@ void *Remap(void *argument)
 	      if ( gridsize_new > grid1sizemax )
 		{
 		  grid1sizemax = gridsize_new;
-		  array1 = (double*) realloc(array1, grid1sizemax*sizeof(double));
-		  imask  = (int*) realloc(imask, grid1sizemax*sizeof(int));
+		  array1 = (double*) Realloc(array1, grid1sizemax*sizeof(double));
+		  imask  = (int*) Realloc(imask, grid1sizemax*sizeof(int));
 
 		  if ( need_gradiants )
 		    {
-		      grad1_lat    = (double*) realloc(grad1_lat, grid1sizemax*sizeof(double));
-		      grad1_lon    = (double*) realloc(grad1_lon, grid1sizemax*sizeof(double));
-		      grad1_latlon = (double*) realloc(grad1_latlon, grid1sizemax*sizeof(double));
+		      grad1_lat    = (double*) Realloc(grad1_lat, grid1sizemax*sizeof(double));
+		      grad1_lon    = (double*) Realloc(grad1_lon, grid1sizemax*sizeof(double));
+		      grad1_latlon = (double*) Realloc(grad1_latlon, grid1sizemax*sizeof(double));
 		    }
 		}
 	      
@@ -978,8 +1068,11 @@ void *Remap(void *argument)
 	      if ( gridID1 == remaps[r].gridID && nmiss1 == remaps[r].nmiss )
 		{
 		  if ( memcmp(imask, remaps[r].src_grid.mask, remaps[r].src_grid.size*sizeof(int)) == 0 )
-		    break;
-		}	      
+                    {
+                      remaps[r].nused++;
+                      break;
+                    }
+                }
 	    }
 
 	  if ( cdoVerbose && r >= 0 ) cdoPrint("Using remap %d", r);
@@ -993,7 +1086,23 @@ void *Remap(void *argument)
 		}
 	      else
 		{
+                  /*
 		  r = nremaps - 1;
+                  remapVarsFree(&remaps[r].vars);
+                  remapGridFree(&remaps[r].src_grid);
+                  remapGridFree(&remaps[r].tgt_grid);
+                  */
+                  int n0 = 0;
+                  if ( max_remaps > 1 && remaps[0].nused > remaps[1].nused ) n0 = 1;
+                  remapVarsFree(&remaps[n0].vars);
+                  remapGridFree(&remaps[n0].src_grid);
+                  remapGridFree(&remaps[n0].tgt_grid);
+                  for ( r = n0+1; r < nremaps; r++ ) memcpy(&remaps[r-1], &remaps[r], sizeof(remap_t));
+                  r = nremaps - 1;
+                  remaps[r].nused    = 0;
+                  remaps[r].gridID   = -1;
+                  remaps[r].gridsize = 0;
+                  remaps[r].nmiss    = 0;
 		}
 
 	      if ( remaps[r].gridID != gridID1 )
@@ -1064,17 +1173,17 @@ void *Remap(void *argument)
 	      remap_vars_init(map_type, remaps[r].src_grid.size, remaps[r].tgt_grid.size, &remaps[r].vars);
 	      if ( cdoTimer ) timer_stop(timer_remap_init);
 
+              print_remap_info(operfunc, remap_genweights, &remaps[r].src_grid, &remaps[r].tgt_grid, nmiss1);
+
 	      if ( remap_genweights )
 		{
-		  print_remap_info(operfunc, &remaps[r].src_grid, &remaps[r].tgt_grid, nmiss1);
+		  if      ( map_type == MAP_TYPE_CONSERV     ) scrip_remap_conserv_weights(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
+		  else if ( map_type == MAP_TYPE_BILINEAR    ) scrip_remap_bilinear_weights(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
+		  else if ( map_type == MAP_TYPE_BICUBIC     ) scrip_remap_bicubic_weights(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
+		  else if ( map_type == MAP_TYPE_DISTWGT     ) remap_distwgt_weights(num_neighbors, &remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
+		  else if ( map_type == MAP_TYPE_CONSERV_YAC ) remap_conserv_weights(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
 
-		  if      ( map_type == MAP_TYPE_CONSERV     ) scrip_remap_weights_conserv(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
-		  else if ( map_type == MAP_TYPE_BILINEAR    ) scrip_remap_weights_bilinear(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
-		  else if ( map_type == MAP_TYPE_BICUBIC     ) scrip_remap_weights_bicubic(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
-		  else if ( map_type == MAP_TYPE_DISTWGT     ) scrip_remap_weights_distwgt(num_neighbors, &remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
-		  else if ( map_type == MAP_TYPE_CONSERV_YAC ) remap_weights_conserv(&remaps[r].src_grid, &remaps[r].tgt_grid, &remaps[r].vars);
-
-		  if ( remaps[r].vars.num_links != remaps[r].vars.max_links )
+		  if ( map_type == MAP_TYPE_CONSERV && remaps[r].vars.num_links != remaps[r].vars.max_links )
 		    resize_remap_vars(&remaps[r].vars, remaps[r].vars.num_links-remaps[r].vars.max_links);
 		  
 		  if ( remaps[r].vars.sort_add ) sort_remap_add(&remaps[r].vars);
@@ -1094,6 +1203,8 @@ void *Remap(void *argument)
 	  
 	  if ( remap_genweights )
 	    {
+              remaps[r].nused++;
+
 	      if ( need_gradiants )
 		{
 		  if ( remaps[r].src_grid.rank != 2 && remap_order == 2 )
@@ -1104,19 +1215,20 @@ void *Remap(void *argument)
 
 	      if ( operfunc == REMAPLAF )
 		remap_laf(array2, missval, gridInqSize(gridID2), remaps[r].vars.num_links, remaps[r].vars.wts,
-			  remaps[r].vars.num_wts, remaps[r].vars.tgt_grid_add, remaps[r].vars.src_grid_add, array1);
+			  remaps[r].vars.num_wts, remaps[r].vars.tgt_cell_add, remaps[r].vars.src_cell_add, array1);
 	      else if ( operfunc == REMAPSUM )
 		remap_sum(array2, missval, gridInqSize(gridID2), remaps[r].vars.num_links, remaps[r].vars.wts,
-			  remaps[r].vars.num_wts, remaps[r].vars.tgt_grid_add, remaps[r].vars.src_grid_add, array1);
+			  remaps[r].vars.num_wts, remaps[r].vars.tgt_cell_add, remaps[r].vars.src_cell_add, array1);
 	      else
 		remap(array2, missval, gridInqSize(gridID2), remaps[r].vars.num_links, remaps[r].vars.wts,
-		      remaps[r].vars.num_wts, remaps[r].vars.tgt_grid_add, remaps[r].vars.src_grid_add,
+		      remaps[r].vars.num_wts, remaps[r].vars.tgt_cell_add, remaps[r].vars.src_cell_add,
 		      array1, grad1_lat, grad1_lon, grad1_latlon, remaps[r].vars.links);
 	    }
 	  else
 	    {
 	      if      ( map_type == MAP_TYPE_BILINEAR    ) scrip_remap_bilinear(&remaps[r].src_grid, &remaps[r].tgt_grid, array1, array2, missval);
 	      else if ( map_type == MAP_TYPE_BICUBIC     ) scrip_remap_bicubic(&remaps[r].src_grid, &remaps[r].tgt_grid, array1, array2, missval);
+              else if ( map_type == MAP_TYPE_DISTWGT     ) remap_distwgt(num_neighbors, &remaps[r].src_grid, &remaps[r].tgt_grid, array1, array2, missval);
 	      else if ( map_type == MAP_TYPE_CONSERV_YAC ) remap_conserv(&remaps[r].src_grid, &remaps[r].tgt_grid, array1, array2, missval);
 	    }
 
@@ -1194,17 +1306,19 @@ void *Remap(void *argument)
 
   streamClose(streamID1);
 
-  if ( remapgrids ) free(remapgrids);
-  if ( imask )  free(imask);
-  if ( array2 ) free(array2);
-  if ( array1 ) free(array1);
+  if ( imask )  Free(imask);
+  if ( array2 ) Free(array2);
+  if ( array1 ) Free(array1);
 
-  if ( grad1_latlon ) free(grad1_latlon);
-  if ( grad1_lon ) free(grad1_lon);
-  if ( grad1_lat ) free(grad1_lat);
+  if ( grad1_latlon ) Free(grad1_latlon);
+  if ( grad1_lon ) Free(grad1_lon);
+  if ( grad1_lat ) Free(grad1_lat);
 
   if ( max_remaps > 0 )
     {
+      if ( lremapxxx && remap_genweights && remaps[0].nused == 0 )
+        print_remap_warning(remap_file, operfunc, &remaps[0].src_grid, remaps[0].nmiss);
+      
       for ( r = 0; r < nremaps; r++ )
 	{
 	  remapVarsFree(&remaps[r].vars);
@@ -1212,10 +1326,10 @@ void *Remap(void *argument)
 	  remapGridFree(&remaps[r].tgt_grid);
 	}
       
-      if ( remaps ) free(remaps);
+      if ( remaps ) Free(remaps);
     }
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
