@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,9 +24,9 @@
       Runstat    runmean         Running mean
       Runstat    runavg          Running average
       Runstat    runvar          Running variance
-      Runstat    runvar1         Running variance [Divisor is (n-1)]
+      Runstat    runvar1         Running variance [Normalize by (n-1)]
       Runstat    runstd          Running standard deviation
-      Runstat    runstd1         Running standard deviation [Divisor is (n-1)]
+      Runstat    runstd1         Running standard deviation [Normalize by (n-1)]
 */
 
 #include <cdi.h>
@@ -46,7 +46,7 @@ void *Runstat(void *argument)
   int levelID;
   int tsID;
   int otsID;
-  int inp, its;
+  int its;
   int nmiss;
   int nlevel;
   int runstat_nomiss = 0;
@@ -82,7 +82,7 @@ void *Runstat(void *argument)
   int lmean   = operfunc == func_mean || operfunc == func_avg;
   int lstd    = operfunc == func_std || operfunc == func_std1;
   int lvarstd = operfunc == func_std || operfunc == func_var || operfunc == func_std1 || operfunc == func_var1;
-  double divisor = operfunc == func_std1 || operfunc == func_var1;
+  int divisor = operfunc == func_std1 || operfunc == func_var1;
 
   int streamID1 = streamOpenRead(cdoStreamName(0));
 
@@ -100,18 +100,18 @@ void *Runstat(void *argument)
   int nvars    = vlistNvars(vlistID1);
   int nrecords = vlistNrecs(vlistID1);
 
-  int *recVarID   = (int*) malloc(nrecords*sizeof(int));
-  int *recLevelID = (int*) malloc(nrecords*sizeof(int));
+  int *recVarID   = (int*) Malloc(nrecords*sizeof(int));
+  int *recLevelID = (int*) Malloc(nrecords*sizeof(int));
 
   dtlist_type *dtlist = dtlist_new();
   dtlist_set_stat(dtlist, timestat_date);
   dtlist_set_calendar(dtlist, taxisInqCalendar(taxisID1));
 
-  vars1 = (field_t ***) malloc((ndates+1)*sizeof(field_t **));
+  vars1 = (field_t ***) Malloc((ndates+1)*sizeof(field_t **));
   if ( !runstat_nomiss )
-    samp1 = (field_t ***) malloc((ndates+1)*sizeof(field_t **));
+    samp1 = (field_t ***) Malloc((ndates+1)*sizeof(field_t **));
   if ( lvarstd )
-    vars2 = (field_t ***) malloc((ndates+1)*sizeof(field_t **));
+    vars2 = (field_t ***) Malloc((ndates+1)*sizeof(field_t **));
 
   for ( its = 0; its < ndates; its++ )
     {
@@ -123,7 +123,7 @@ void *Runstat(void *argument)
     }
 
   int gridsizemax = vlistGridsizeMax(vlistID1);
-  int *imask = (int*) malloc(gridsizemax*sizeof(int));
+  int *imask = (int*) Malloc(gridsizemax*sizeof(int));
 
   for ( tsID = 0; tsID < ndates; tsID++ )
     {
@@ -162,12 +162,12 @@ void *Runstat(void *argument)
 		samp1[tsID][varID][levelID].ptr[i] = (double) imask[i];
 
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared) private(i, inp)
+#pragma omp parallel for default(none) shared(tsID,gridsize,imask,samp1,varID,levelID)
 #endif
-	      for ( inp = 0; inp < tsID; inp++ )
+	      for ( int inp = 0; inp < tsID; inp++ )
 		{
 		  double *ptr = samp1[inp][varID][levelID].ptr;
-		  for ( i = 0; i < gridsize; i++ )
+		  for ( int i = 0; i < gridsize; i++ )
 		    if ( imask[i] > 0 ) ptr[i]++;
 		}
 	    }
@@ -176,9 +176,9 @@ void *Runstat(void *argument)
 	    {
 	      farmoq(&vars2[tsID][varID][levelID], vars1[tsID][varID][levelID]);
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared)
+#pragma omp parallel for default(none) shared(tsID,vars1,vars2,varID,levelID)
 #endif
-	      for ( inp = 0; inp < tsID; inp++ )
+	      for ( int inp = 0; inp < tsID; inp++ )
 		{
 		  farsumq(&vars2[inp][varID][levelID], vars1[tsID][varID][levelID]);
 		  farsum(&vars1[inp][varID][levelID], vars1[tsID][varID][levelID]);
@@ -187,9 +187,9 @@ void *Runstat(void *argument)
 	  else
 	    {
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared)
+#pragma omp parallel for default(none) shared(tsID,vars1,operfunc,varID,levelID)
 #endif
-	      for ( inp = 0; inp < tsID; inp++ )
+	      for ( int inp = 0; inp < tsID; inp++ )
 		{
 		  farfun(&vars1[inp][varID][levelID], vars1[tsID][varID][levelID], operfunc);
 		}
@@ -208,7 +208,7 @@ void *Runstat(void *argument)
 	    for ( levelID = 0; levelID < nlevel; levelID++ )
 	      {
 		if ( runstat_nomiss )
-		  farcmul(&vars1[0][varID][levelID], 1.0/ndates);
+		  farcdiv(&vars1[0][varID][levelID], (double)ndates);
 		else
 		  fardiv(&vars1[0][varID][levelID], samp1[0][varID][levelID]);
 	      }
@@ -223,16 +223,16 @@ void *Runstat(void *argument)
 		if ( runstat_nomiss )
 		  {
 		    if ( lstd )
-		      farcstdx(&vars1[0][varID][levelID], vars2[0][varID][levelID], ndates, divisor);
+		      farcstd(&vars1[0][varID][levelID], vars2[0][varID][levelID], ndates, divisor);
 		    else
-		      farcvarx(&vars1[0][varID][levelID], vars2[0][varID][levelID], ndates, divisor);
+		      farcvar(&vars1[0][varID][levelID], vars2[0][varID][levelID], ndates, divisor);
 		  }
 		else
 		  {
 		    if ( lstd )
-		      farstdx(&vars1[0][varID][levelID], vars2[0][varID][levelID], samp1[0][varID][levelID], divisor);
+		      farstd(&vars1[0][varID][levelID], vars2[0][varID][levelID], samp1[0][varID][levelID], divisor);
 		    else
-		      farvarx(&vars1[0][varID][levelID], vars2[0][varID][levelID], samp1[0][varID][levelID], divisor);
+		      farvar(&vars1[0][varID][levelID], vars2[0][varID][levelID], samp1[0][varID][levelID], divisor);
 		  }
 	      }
 	  }
@@ -261,7 +261,7 @@ void *Runstat(void *argument)
       if ( lvarstd )
         vars2[ndates] = vars2[0];
 
-      for ( inp = 0; inp < ndates; inp++ )
+      for ( int inp = 0; inp < ndates; inp++ )
 	{
 	  vars1[inp] = vars1[inp+1];
 	  if ( !runstat_nomiss )
@@ -299,12 +299,12 @@ void *Runstat(void *argument)
 		samp1[ndates-1][varID][levelID].ptr[i] = (double) imask[i];
 
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared) private(i, inp)
+#pragma omp parallel for default(none) shared(ndates,imask,gridsize,samp1,varID,levelID)
 #endif
-	      for ( inp = 0; inp < ndates-1; inp++ )
+	      for ( int inp = 0; inp < ndates-1; inp++ )
 		{
 		  double *ptr = samp1[inp][varID][levelID].ptr;
-		  for ( i = 0; i < gridsize; i++ )
+		  for ( int i = 0; i < gridsize; i++ )
 		    if ( imask[i] > 0 ) ptr[i]++;
 		}
 	    }
@@ -313,9 +313,9 @@ void *Runstat(void *argument)
 	    {
 	      farmoq(&vars2[ndates-1][varID][levelID], vars1[ndates-1][varID][levelID]);
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared)
+#pragma omp parallel for default(none) shared(ndates,vars1,vars2,varID,levelID)
 #endif
-	      for ( inp = 0; inp < ndates-1; inp++ )
+	      for ( int inp = 0; inp < ndates-1; inp++ )
 		{
 		  farsumq(&vars2[inp][varID][levelID], vars1[ndates-1][varID][levelID]);
 		  farsum(&vars1[inp][varID][levelID], vars1[ndates-1][varID][levelID]);
@@ -324,9 +324,9 @@ void *Runstat(void *argument)
 	  else
 	    {
 #if defined(_OPENMP)
-#pragma omp parallel for default(shared)
+#pragma omp parallel for default(none) shared(ndates,vars1,varID,levelID,operfunc)
 #endif
-	      for ( inp = 0; inp < ndates-1; inp++ )
+	      for ( int inp = 0; inp < ndates-1; inp++ )
 		{
 		  farfun(&vars1[inp][varID][levelID], vars1[ndates-1][varID][levelID], operfunc);
 		}
@@ -343,13 +343,13 @@ void *Runstat(void *argument)
       if ( lvarstd ) field_free(vars2[its], vlistID1);
     }
 
-  free(vars1);
-  if ( !runstat_nomiss ) free(samp1);
-  if ( lvarstd ) free(vars2);
+  Free(vars1);
+  if ( !runstat_nomiss ) Free(samp1);
+  if ( lvarstd ) Free(vars2);
 
-  if ( recVarID   ) free(recVarID);
-  if ( recLevelID ) free(recLevelID);
-  if ( imask )      free(imask);
+  if ( recVarID   ) Free(recVarID);
+  if ( recLevelID ) Free(recLevelID);
+  if ( imask )      Free(imask);
 
   dtlist_delete(dtlist);
 
@@ -358,5 +358,5 @@ void *Runstat(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
