@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,9 +22,9 @@
       Seltime    seltime         Select times
       Seltime    selhour         Select hours
       Seltime    selday          Select days
-      Seltime    selmon          Select months
+      Seltime    selmonth        Select months
       Seltime    selyear         Select years
-      Seltime    selseas         Select seasons
+      Seltime    selseason       Select seasons
       Seltime    seldate         Select dates
       Seltime    selsmon         Select single month
 */
@@ -40,98 +40,103 @@
 #include "list.h"
 
 
-#define  NOPERATORS  32
+void season_to_months(const char *season, int *imonths)
+{
+  const char *smons = "JFMAMJJASONDJFMAMJJASOND";
+  const int imons[] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12};
+  assert(strlen(smons)==(sizeof(imons)/sizeof(int)));
+
+  size_t len = strlen(season);
+  if ( len == 3 && strcmp(season, "ANN") == 0 )
+    {
+      for ( size_t k = 0; k < 12; ++k ) imonths[k+1] = 1;
+    }
+  else
+    {
+      if ( len > 12 ) cdoAbort("Too many months %d (limit=12)!", (int)len);
+      //      char *sstr = strcasestr(smons, season); // nonstandard extension
+      const char *sstr = strstr(smons, season);
+      if ( sstr == NULL ) cdoAbort("Season %s not available!", season);
+      size_t ks = (size_t)(sstr-smons);
+      size_t ke = ks + len;
+      for ( size_t k = ks; k < ke; ++k ) imonths[imons[k]]++;
+    }
+}
 
 static
 int seaslist(LIST *ilist)
 {
-  int i;
-  int nsel;
-  char Seas[3];
-  int seas[4] = {FALSE, FALSE, FALSE, FALSE};
-  int imon[17]; /* 1-16 ! */
-  int ival;
-  size_t len;
-  int season_start;
+  int imon[13]; /* 1-12 ! */
+  for ( int i = 0; i < 13; ++i ) imon[i] = 0;
 
-  season_start = get_season_start();
-  nsel = operatorArgc();
+  int nsel = operatorArgc();
   if ( isdigit(*operatorArgv()[0]))
-    for ( i = 0; i < nsel; i++ )
-      {
-	ival = atoi(operatorArgv()[i]);
-	if      ( ival == 1 || ival == 13 ) seas[0] = TRUE;
-	else if ( ival == 2 || ival == 14 ) seas[1] = TRUE;
-	else if ( ival == 3 || ival == 15 ) seas[2] = TRUE;
-	else if ( ival == 4 || ival == 16 ) seas[3] = TRUE;
-	else cdoAbort("Season %d not available!", ival);
-      }
-  else
-    for ( i = 0; i < nsel; i++ )
-      {
-	len = strlen(operatorArgv()[i]);
-	if ( len > 3 ) len = 3;
-	while ( len-- > 0 ) Seas[len] = toupper(operatorArgv()[i][len]);
-	if ( season_start == START_DEC )
-	  {
-	    if      ( memcmp(Seas, "DJF", 3) == 0 ) seas[0] = TRUE;
-	    else if ( memcmp(Seas, "MAM", 3) == 0 ) seas[1] = TRUE;
-	    else if ( memcmp(Seas, "JJA", 3) == 0 ) seas[2] = TRUE;
-	    else if ( memcmp(Seas, "SON", 3) == 0 ) seas[3] = TRUE;
-	    else cdoAbort("Season %s not available!", operatorArgv()[i]);
-	  }
-	else
-	  {
-	    if      ( memcmp(Seas, "JFM", 3) == 0 ) seas[0] = TRUE;
-	    else if ( memcmp(Seas, "AMJ", 3) == 0 ) seas[1] = TRUE;
-	    else if ( memcmp(Seas, "JAS", 3) == 0 ) seas[2] = TRUE;
-	    else if ( memcmp(Seas, "OND", 3) == 0 ) seas[3] = TRUE;
-	    else cdoAbort("Season %s not available!", operatorArgv()[i]);
-	  }
-      }
-  
-  for ( i = 0; i < 17; ++i ) imon[i] = 0;
-  
-  if ( season_start == START_DEC )
     {
-      if ( seas[0] ) { imon[12]++; imon[ 1]++; imon[ 2]++; imon[13]++; }
-      if ( seas[1] ) { imon[ 3]++; imon[ 4]++; imon[ 5]++; imon[14]++; }
-      if ( seas[2] ) { imon[ 6]++; imon[ 7]++; imon[ 8]++; imon[15]++; }
-      if ( seas[3] ) { imon[ 9]++; imon[10]++; imon[11]++; imon[16]++; }
+      for ( int i = 0; i < nsel; i++ )
+        {
+          int ival = parameter2int(operatorArgv()[i]);
+          if      ( ival == 1 ) { imon[12]++; imon[ 1]++; imon[ 2]++; }
+          else if ( ival == 2 ) { imon[ 3]++; imon[ 4]++; imon[ 5]++; }
+          else if ( ival == 3 ) { imon[ 6]++; imon[ 7]++; imon[ 8]++; }
+          else if ( ival == 4 ) { imon[ 9]++; imon[10]++; imon[11]++; }
+          else cdoAbort("Season %d not available!", ival);
+        }
     }
   else
     {
-      if ( seas[0] ) { imon[ 1]++; imon[ 2]++; imon[ 3]++; imon[13]++; }
-      if ( seas[1] ) { imon[ 4]++; imon[ 5]++; imon[ 6]++; imon[14]++; }
-      if ( seas[2] ) { imon[ 7]++; imon[ 8]++; imon[ 9]++; imon[15]++; }
-      if ( seas[3] ) { imon[10]++; imon[11]++; imon[12]++; imon[16]++; }
-    }
-  
-  nsel = 0;
-  for ( i = 1; i < 17; ++i )
-    {
-      if ( imon[i] )
-	listSetInt(ilist, nsel++, i);
+      for ( int i = 0; i < nsel; i++ )
+        season_to_months(operatorArgv()[i], imon);
     }
 
-  return (nsel);
+  nsel = 0;
+  for ( int i = 1; i < 13; ++i ) if ( imon[i] ) listSetInt(ilist, nsel++, i);
+
+  return nsel;
+}
+
+
+double datestr_to_double(const char *datestr, int opt)
+{
+  int year = 1, month = 1, day = 1, hour = 0, minute = 0, second = 0;
+  double fval = 0;
+
+  if ( opt ) {hour = 23; minute = 59; second = 59;}
+
+  if ( strchr(datestr, '-') == NULL )
+    {
+      fval = parameter2double(datestr);
+    }
+  else if ( strchr(datestr, 'T') )
+    {
+      sscanf(datestr, "%d-%d-%dT%d:%d:%d", &year, &month, &day, &hour, &minute, &second);
+      fval = cdiEncodeTime(hour, minute, second);
+      if ( fabs(fval) > 0 ) fval /= 1000000;
+      fval += cdiEncodeDate(year, month, day);
+    }
+  else
+    {
+      sscanf(datestr, "%d-%d-%d", &year, &month, &day);
+      fval = cdiEncodeTime(hour, minute, second);
+      if ( fabs(fval) > 0 ) fval /= 1000000;
+      fval += cdiEncodeDate(year, month, day);
+    }
+
+  return fval;
 }
 
 static
 int datelist(LIST *flist)
 {
-  int nsel;
-  int i;
-  int status;
   int set2 = TRUE;
-  int year = 1, month = 1, day = 1, hour = 0, minute = 0, second = 0;
   double fval = 0;
 
-  nsel = operatorArgc();
+  int nsel = operatorArgc();
   if ( nsel < 1 ) cdoAbort("Too few arguments!");
-  for ( i = 0; i < nsel; i++)
+  if ( nsel > 2 ) cdoAbort("Too many arguments!");
+
+  for ( int i = 0; i < nsel; i++ )
     {
-      if      ( operatorArgv()[i][0] == '-' && operatorArgv()[i][1] == 0 )
+      if ( operatorArgv()[i][0] == '-' && operatorArgv()[i][1] == 0 )
 	{
 	  if ( i == 0 )
 	    fval = -99999999999.;
@@ -140,34 +145,16 @@ int datelist(LIST *flist)
 
 	  listSetFlt(flist, i,  fval);
 	}
-      else if ( strchr(operatorArgv()[i], '-') == NULL )
-	{
-	  fval = atof(operatorArgv()[i]);
-	  listSetFlt(flist, i, fval);
-	}
       else
 	{
-	  year = 1; month = 1; day = 1; hour = 0; minute = 0, second = 0;
+	  fval = datestr_to_double(operatorArgv()[i], 0);
 	  if ( strchr(operatorArgv()[i], 'T') )
-	    {
-	      status = sscanf(operatorArgv()[i], "%d-%d-%dT%d:%d:%d",
-			      &year, &month, &day, &hour, &minute, &second);
-	      fval = cdiEncodeTime(hour, minute, second);
-	      if ( fabs(fval) > 0 ) fval /= 1000000;
-	      fval += cdiEncodeDate(year, month, day);
-	      listSetFlt(flist, i, fval);
-	      set2 = FALSE;
-	    }
-	  else
-	    {
-	      status = sscanf(operatorArgv()[i], "%d-%d-%d", &year, &month, &day);
-	      fval = cdiEncodeDate(year, month, day);
-	      
-	      if ( nsel > 1 && i > 0 ) fval += 0.999;
-
-	      listSetFlt(flist, i, fval);
-	    }
+	    set2 = FALSE;
+	  else if ( nsel > 1 && i > 0 )
+	    fval += 0.999;
 	}
+
+      listSetFlt(flist, i, fval);
     }
 
   if ( nsel == 1 && set2 == TRUE )
@@ -183,36 +170,28 @@ int datelist(LIST *flist)
 
 void *Seltime(void *argument)
 {
-  int SELTIMESTEP, SELDATE, SELTIME, SELHOUR, SELDAY, SELMON, SELYEAR, SELSEAS, SELSMON;
-  int operatorID;
-  int operfunc, intval;
-  int moddat[NOPERATORS];
-  int streamID1, streamID2 = -1;
-  int tsID, tsID2, nrecs;
+  int streamID2 = -1;
+  int nrecs;
   int recID, varID, levelID;
-  int *intarr, nsel = 0, selival;
-  int vlistID1 = -1, vlistID2 = -1;
-  int taxisID1, taxisID2;
+  int nsel = 0, selival;
   int vdate, vtime;
   int copytimestep;
   int copy_nts2 = FALSE;
   int i, isel;
-  int lcopy = FALSE;
   int gridsize;
   int nmiss;
-  int lnts1;
   int ncts = 0, nts, it;
   int *selfound = NULL;
   int hour = 0, minute = 0, second = 0;
   int nts1 = 0, nts2 = 0;
   int its1 = 0, its2 = 0;
-  double selfval = 0, *fltarr;
+  double selfval = 0;
   double *array = NULL;
   LIST *ilist = listNew(INT_LIST);
   LIST *flist = listNew(FLT_LIST);
   int gridID;
-  int nvars, nlevel;
-  int nconst, lconstout = FALSE;
+  int nlevel;
+  int lconstout = FALSE;
   int process_nts1 = FALSE, process_nts2 = FALSE;
   int *vdate_list = NULL, *vtime_list = NULL;
   double *single;
@@ -220,37 +199,26 @@ void *Seltime(void *argument)
 
   cdoInitialize(argument);
 
-  SELTIMESTEP = cdoOperatorAdd("seltimestep", func_step,     1, "timesteps");
-  SELDATE     = cdoOperatorAdd("seldate",     func_datetime, 1, "start date and end date (format YYYY-MM-DDThh:mm:ss)");
-  SELTIME     = cdoOperatorAdd("seltime",     func_time,     1, "times (format hh:mm:ss)");
-  SELHOUR     = cdoOperatorAdd("selhour",     func_time, 10000, "hours");
-  SELDAY      = cdoOperatorAdd("selday",      func_date,     1, "days");
-  SELMON      = cdoOperatorAdd("selmon",      func_date,   100, "months");
-  SELYEAR     = cdoOperatorAdd("selyear",     func_date, 10000, "years");
-  SELSEAS     = cdoOperatorAdd("selseas",     func_date,   100, "seasons");
-  SELSMON     = cdoOperatorAdd("selsmon",     func_date,   100, "month[,nts1[,nts2]]");
+  int SELTIMESTEP = cdoOperatorAdd("seltimestep", func_step,     0, "timesteps");
+  int SELDATE     = cdoOperatorAdd("seldate",     func_datetime, 0, "start date and end date (format YYYY-MM-DDThh:mm:ss)");
+  int SELTIME     = cdoOperatorAdd("seltime",     func_time,     0, "times (format hh:mm:ss)");
+  int SELHOUR     = cdoOperatorAdd("selhour",     func_time,     0, "hours");
+  int SELDAY      = cdoOperatorAdd("selday",      func_date,     0, "days");
+  int SELMONTH    = cdoOperatorAdd("selmonth",    func_date,     0, "months");
+  int SELYEAR     = cdoOperatorAdd("selyear",     func_date,     0, "years");
+  int SELSEASON   = cdoOperatorAdd("selseason",   func_date,     0, "seasons");
+  int SELSMON     = cdoOperatorAdd("selsmon",     func_date,     0, "month[,nts1[,nts2]]");
 
-  moddat[SELTIMESTEP] =          1;
-  /*  moddat[SELDATE]     = 1000000000; */
-  moddat[SELDATE]     =          0;
-  moddat[SELTIME]     =    1000000;
-  moddat[SELHOUR]     =      10000;
-  moddat[SELDAY]      =        100;
-  moddat[SELMON]      =        100;
-  moddat[SELYEAR]     = 1000000000;
-  moddat[SELSEAS]     =        100;
-  moddat[SELSMON]     =        100;
+  int operatorID = cdoOperatorID();
 
-  operatorID = cdoOperatorID();
-
-  operfunc = cdoOperatorF1(operatorID);
-  intval   = cdoOperatorF2(operatorID);
+  int operfunc = cdoOperatorF1(operatorID);
 
+  int lcopy = FALSE;
   if ( UNCHANGED_RECORD ) lcopy = TRUE;
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
-  if ( operatorID == SELSEAS )
+  if ( operatorID == SELSEASON )
     {
       nsel = seaslist(ilist);
     }
@@ -271,7 +239,7 @@ void *Seltime(void *argument)
 	    }
 	  else
 	    {
-	      listSetInt(ilist, i, atoi(operatorArgv()[i]));
+	      listSetInt(ilist, i, parameter2int(operatorArgv()[i]));
 	    }
 	}
     }
@@ -282,8 +250,8 @@ void *Seltime(void *argument)
 
   if ( nsel < 1 ) cdoAbort("No timestep selected!");
 
-  intarr = (int *) listArrayPtr(ilist);
-  fltarr = (double *) listArrayPtr(flist);
+  int *intarr = (int *) listArrayPtr(ilist);
+  double *fltarr = (double *) listArrayPtr(flist);
 
   if ( operatorID == SELSMON )
     {
@@ -301,10 +269,45 @@ void *Seltime(void *argument)
 
   if ( nsel )
     {
-      selfound = malloc(nsel*sizeof(int));
+      selfound = (int*) Malloc(nsel*sizeof(int));
       for ( i = 0; i < nsel; i++ ) selfound[i] = FALSE;
     }
 
+  int streamID1 = streamOpenRead(cdoStreamName(0));
+
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
+
+  if ( nsel == 1 && operfunc == func_step )  vlistDefNtsteps(vlistID2,  1);
+  else                                       vlistDefNtsteps(vlistID2, -1);
+
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
+  vlistDefTaxis(vlistID2, taxisID2);
+
+  if ( ! lcopy )
+    {
+      gridsize = vlistGridsizeMax(vlistID1);
+      if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
+      array = (double*) Malloc(gridsize*sizeof(double));
+    }
+
+  int ntsteps = vlistNtsteps(vlistID1);
+
+  /* add support for negative timestep values */
+  if ( operatorID == SELTIMESTEP && ntsteps > 0 )
+    {
+      for ( i = 0; i < nsel; i++ )
+	{
+	  if ( intarr[i] < 0 )
+	    {
+	      if ( cdoVerbose )
+		cdoPrint("timestep %d changed to %d", intarr[i], ntsteps + 1 + intarr[i]);
+	      intarr[i] = ntsteps + 1 + intarr[i];
+	    }
+	}
+    }
+
   if ( cdoVerbose )
     {
       for ( i = 0; i < nsel; i++ )
@@ -314,47 +317,28 @@ void *Seltime(void *argument)
 	  cdoPrint("intarr entry: %d %d", i+1, intarr[i]);
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
-
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
-
-  if ( nsel == 1 && operfunc == func_step )  vlistDefNtsteps(vlistID2,  1);
-  else                                       vlistDefNtsteps(vlistID2, -1);
-
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
-  vlistDefTaxis(vlistID2, taxisID2);
-
-  if ( ! lcopy )
-    {
-      gridsize = vlistGridsizeMax(vlistID1);
-      if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-      array = malloc(gridsize*sizeof(double));
-    }
-
-  nvars = vlistNvars(vlistID1);
-  nconst = 0;
+  int nvars = vlistNvars(vlistID1);
+  int nconst = 0;
   for ( varID = 0; varID < nvars; varID++ )
     if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) nconst++;
       
-  lnts1 = operatorID == SELSMON && nts1 > 0;
+  int lnts1 = operatorID == SELSMON && nts1 > 0;
 
   if ( lnts1 || nconst )
     {
       if ( lnts1 )
 	{
-	  vdate_list = malloc(nts1*sizeof(int));
-	  vtime_list = malloc(nts1*sizeof(int));
+	  vdate_list = (int*) Malloc(nts1*sizeof(int));
+	  vtime_list = (int*) Malloc(nts1*sizeof(int));
 	}
       else
 	{
 	  nts1 = 1;
 	}
 
-      vars  = malloc(nts1*sizeof(field_t **));
+      vars  = (field_t ***) Malloc(nts1*sizeof(field_t **));
 
-      for ( tsID = 0; tsID < nts1; tsID++ )
+      for ( int tsID = 0; tsID < nts1; tsID++ )
 	{
 	  vars[tsID] = field_malloc(vlistID1, FIELD_NONE);
 
@@ -368,15 +352,15 @@ void *Seltime(void *argument)
 		  
 		  for ( levelID = 0; levelID < nlevel; levelID++ )
 		    {
-		      vars[tsID][varID][levelID].ptr = malloc(gridsize*sizeof(double));
+		      vars[tsID][varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
 		    }
 		}
 	    }
 	}
     }
 
-  tsID  = 0;
-  tsID2 = 0;
+  int tsID  = 0;
+  int tsID2 = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       vdate = taxisInqVdate(taxisID1);
@@ -392,11 +376,18 @@ void *Seltime(void *argument)
 	}
       else if ( operfunc == func_date )
 	{
-	  selival = (vdate/intval)%moddat[operatorID];
+          int year, month, day;
+          cdiDecodeDate(vdate, &year, &month, &day);
+          if      ( operatorID == SELYEAR ) selival = year;
+          else if ( operatorID == SELDAY  ) selival = day;
+          else                              selival = month;
 	}
       else if ( operfunc == func_time )
 	{
-	  selival = (vtime/intval)%moddat[operatorID];
+          int hour, minute, second;
+          cdiDecodeTime(vtime, &hour, &minute, &second);
+          if      ( operatorID == SELHOUR ) selival = hour;
+          else if ( operatorID == SELTIME ) selival = vtime;
 	}
       else if ( operfunc == func_datetime )
 	{
@@ -411,6 +402,10 @@ void *Seltime(void *argument)
 	      selfound[0]      = TRUE;
 	      selfound[nsel-1] = TRUE;
 	    }
+          else if ( selfval > fltarr[nsel-1] )
+            {
+              break;
+            }
 	}
       else
 	{
@@ -593,8 +588,7 @@ void *Seltime(void *argument)
     if ( its2 < nts2 )
       cdoWarning("%d timesteps missing after the last month!", nts2-its2);
 
-  if ( ! lcopy )
-    if ( array ) free(array);
+  if ( array ) Free(array);
 
   for ( isel = 0; isel < nsel; isel++ )
     {
@@ -642,7 +636,7 @@ void *Seltime(void *argument)
 	    {
 	      cdoWarning("Day %d not found!", intarr[isel]);
 	    }
-	  else if ( operatorID == SELMON )
+	  else if ( operatorID == SELMONTH )
 	    {
 	      cdoWarning("Month %d not found!", intarr[isel]);
 	    }
@@ -650,7 +644,7 @@ void *Seltime(void *argument)
 	    {
 	      cdoWarning("Year %d not found!", intarr[isel]);
 	    }
-	  else if ( operatorID == SELSEAS )
+	  else if ( operatorID == SELSEASON )
 	    {
 	      if ( isel < 3 )
 		cdoWarning("Month %d not found!", intarr[isel]);
@@ -658,7 +652,7 @@ void *Seltime(void *argument)
 	}
     }
 
-  if ( selfound ) free(selfound);
+  if ( selfound ) Free(selfound);
 
   listDelete(ilist);
 
@@ -666,14 +660,16 @@ void *Seltime(void *argument)
     {
       for ( tsID = 0; tsID < nts1; tsID++ ) field_free(vars[tsID], vlistID2);
 
-      if ( vars  ) free(vars);
-      if ( vdate_list ) free(vdate_list);
-      if ( vtime_list ) free(vtime_list);
+      if ( vars  ) Free(vars);
+      if ( vdate_list ) Free(vdate_list);
+      if ( vtime_list ) Free(vtime_list);
     }
 
   vlistDestroy(vlistID2);
 
+  if ( tsID2 == 0 ) cdoAbort("No timesteps selected!");
+
   cdoFinish();
 
-  return (NULL);
+  return 0;
 }
