@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -67,6 +67,7 @@ void *Selvar(void *argument)
   int lcopy = FALSE;
   int gridsize;
   int nmiss;
+  int gridnum = 0;
   double *array = NULL;
   LIST *ilist = listNew(INT_LIST);
   LIST *flist = listNew(FLT_LIST);
@@ -78,20 +79,20 @@ void *Selvar(void *argument)
 # define TAKES_INTEGERS(id) (cdoOperatorF2(id) & 4)
 # define TAKES_FLOATS(id) (cdoOperatorF2(id) & 8)
 
-  int SELPARAM     = cdoOperatorAdd("selparam",     0, 2, "parameters");
-  int SELCODE      = cdoOperatorAdd("selcode",      0, 4, "code numbers");
-  int SELNAME      = cdoOperatorAdd("selname",      0, 2, "variable names");
-  int SELSTDNAME   = cdoOperatorAdd("selstdname",   0, 2, "standard names");
-  int SELLEVEL     = cdoOperatorAdd("sellevel",     0, 8, "levels");
-  int SELLEVIDX    = cdoOperatorAdd("sellevidx",    0, 4, "index of levels");
+  int SELPARAM     = cdoOperatorAdd("selparam",     0, 2,   "parameters");
+  int SELCODE      = cdoOperatorAdd("selcode",      0, 4,   "code numbers");
+  int SELNAME      = cdoOperatorAdd("selname",      0, 2,   "variable names");
+  int SELSTDNAME   = cdoOperatorAdd("selstdname",   0, 2,   "standard names");
+  int SELLEVEL     = cdoOperatorAdd("sellevel",     0, 8,   "levels");
+  int SELLEVIDX    = cdoOperatorAdd("sellevidx",    0, 4,   "index of levels");
   int SELGRID      = cdoOperatorAdd("selgrid",      0, 4|2, "list of grid names or numbers");
   int SELZAXIS     = cdoOperatorAdd("selzaxis",     0, 4|2, "list of zaxis types or numbers");
-  int SELZAXISNAME = cdoOperatorAdd("selzaxisname", 0, 2, "list of zaxis names");
-  int SELTABNUM    = cdoOperatorAdd("seltabnum",    0, 4, "table numbers");
+  int SELZAXISNAME = cdoOperatorAdd("selzaxisname", 0, 2,   "list of zaxis names");
+  int SELTABNUM    = cdoOperatorAdd("seltabnum",    0, 4,   "table numbers");
   int DELPARAM     = cdoOperatorAdd("delparam",     0, 2|1, "parameter");
-  int DELCODE      = cdoOperatorAdd("delcode",      0, 1, "code numbers");
+  int DELCODE      = cdoOperatorAdd("delcode",      0, 1,   "code numbers");
   int DELNAME      = cdoOperatorAdd("delname",      0, 2|1, "variable names");
-  int SELLTYPE     = cdoOperatorAdd("selltype",     0, 4, "GRIB level types"); 
+  int SELLTYPE     = cdoOperatorAdd("selltype",     0, 4,   "GRIB level types"); 
 
   if ( UNCHANGED_RECORD ) lcopy = TRUE;
 
@@ -131,7 +132,7 @@ void *Selvar(void *argument)
 
   if ( nsel )
     {
-      selfound = (int*) malloc(nsel*sizeof(int));
+      selfound = (int*) Malloc(nsel*sizeof(int));
       for ( i = 0; i < nsel; i++ ) selfound[i] = FALSE;
     }
 
@@ -142,11 +143,36 @@ void *Selvar(void *argument)
   int streamID1 = streamOpenRead(cdoStreamName(0));
 
   int vlistID1 = streamInqVlist(streamID1);
+  int nvars = vlistNvars(vlistID1);
+  int *vars = (int*) Malloc(nvars*sizeof(int));
+
+  if ( operatorID == SELGRID && !args_are_numeric && nsel == 1 && strncmp(argnames[0], "var=", 4) == 0 )
+    {
+      const char *gridvarname = argnames[0] + 4;
+
+      if ( *gridvarname == 0 ) cdoAbort("Variable name missing!", gridvarname);
+      
+      for ( varID = 0; varID < nvars; varID++ )
+        {
+          vlistInqVarName(vlistID1, varID, varname);
+          if ( strcmp(varname, gridvarname) == 0 )
+            {
+              gridID = vlistInqVarGrid(vlistID1, varID);
+              gridnum = 1 + vlistGridIndex(vlistID1, gridID);
+              args_are_numeric = TRUE;
+              intarr = &gridnum;
+              break;
+            }
+        }
+
+      if ( !gridnum ) cdoAbort("Variable %s not found!", gridvarname);
+    }
 
   vlistClearFlag(vlistID1);
-  int nvars = vlistNvars(vlistID1);
   for ( varID = 0; varID < nvars; varID++ )
     {
+      vars[varID] = FALSE;
+
       vlistInqVarName(vlistID1, varID, varname);
       vlistInqVarStdname(vlistID1, varID, stdname);
       param    = vlistInqVarParam(vlistID1, varID);
@@ -187,7 +213,7 @@ void *Selvar(void *argument)
 		}
 	      else if ( operatorID == SELSTDNAME )
 		{
-		  found = strcmp(argnames[isel], stdname) == 0;
+		  found = wildcardmatch(argnames[isel], stdname) == 0;
 		}
 	      else if ( operatorID == SELLEVEL )
 		{
@@ -242,22 +268,30 @@ void *Selvar(void *argument)
 	        {
 		  vlistDefFlag(vlistID1, varID, levID, !INVERTS_SELECTION(operatorID));
 		  selfound[isel] = TRUE;
+                  vars[varID] = TRUE;
 	        }
-
 	    }
 	}
     }
 
   int npar = 0;
+  for ( varID = 0; varID < nvars; varID++ ) if ( vars[varID] ) npar++;
+
   for ( varID = 0; varID < nvars; varID++ )
     {
-      zaxisID = vlistInqVarZaxis(vlistID1, varID);
-      nlevs   = zaxisInqSize(zaxisID);
-
-      for ( levID = 0; levID < nlevs; levID++ )
-	if ( vlistInqFlag(vlistID1, varID, levID) == TRUE ) break;
-	      
-      if ( levID < nlevs ) npar++;
+      if ( vars[varID] )
+        {
+          int zaxisID = vlistInqVarZaxis(vlistID1, varID);
+          if ( zaxisInqType(zaxisID) == ZAXIS_HYBRID )
+            {
+              int psvarid = vlist_get_psvarid(vlistID1, zaxisID);
+              if ( psvarid != -1 && !vars[psvarid] )
+                {
+                  vars[psvarid] = TRUE;
+                  vlistDefFlag(vlistID1, psvarid, 0, !INVERTS_SELECTION(operatorID));
+                }
+            }
+        }
     }
 
   for ( isel = 0; isel < nsel; isel++ )
@@ -343,7 +377,7 @@ void *Selvar(void *argument)
     {
       gridsize = vlistGridsizeMax(vlistID1);
       if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-      array = (double*) malloc(gridsize*sizeof(double));
+      array = (double*) Malloc(gridsize*sizeof(double));
     }
 
   int tsID = 0;
@@ -383,14 +417,16 @@ void *Selvar(void *argument)
   vlistDestroy(vlistID2);
 
   if ( ! lcopy )
-    if ( array ) free(array);
+    if ( array ) Free(array);
 
-  if ( selfound ) free(selfound);
+  if ( vars ) Free(vars);
+
+  if ( selfound ) Free(selfound);
 
   listDelete(ilist);
   listDelete(flist);
 
   cdoFinish();
 
-  return (NULL);
+  return 0;
 }
