@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -53,6 +53,7 @@ void *Setgrid(void *argument)
   int ligme = 0;
   int number = 0, position = 0;
   int grid2_nvgp;
+  int lbounds = TRUE;
   int *grid2_vgpm = NULL;
   char *gridname = NULL;
   char *griduri = NULL;
@@ -85,13 +86,16 @@ void *Setgrid(void *argument)
       operatorCheckArgc(1);
       gridname = operatorArgv()[0];
 
-      if      ( strcmp(gridname, "curvilinear") == 0 )   gridtype = GRID_CURVILINEAR;
-      else if ( strcmp(gridname, "cell") == 0 )          gridtype = GRID_UNSTRUCTURED;
-      else if ( strcmp(gridname, "unstructured") == 0 )  gridtype = GRID_UNSTRUCTURED;
-      else if ( strcmp(gridname, "dereference") == 0 )   ldereference = 1;
-      else if ( strcmp(gridname, "lonlat") == 0 )        gridtype = GRID_LONLAT;
-      else if ( strcmp(gridname, "gaussian") == 0 )      gridtype = GRID_GAUSSIAN;
-      else if ( strcmp(gridname, "regular") == 0 )      {gridtype = GRID_GAUSSIAN; lregular = 1;}
+      if      ( strcmp(gridname, "curvilinear0") == 0 )  {gridtype = GRID_CURVILINEAR; lbounds = 0;}
+      else if ( strcmp(gridname, "curvilinear") == 0 )   {gridtype = GRID_CURVILINEAR; lbounds = 1;}
+      else if ( strcmp(gridname, "cell") == 0 )           gridtype = GRID_UNSTRUCTURED;
+      else if ( strcmp(gridname, "unstructured0") == 0 ) {gridtype = GRID_UNSTRUCTURED; lbounds = 0;}
+      else if ( strcmp(gridname, "unstructured") == 0 )  {gridtype = GRID_UNSTRUCTURED; lbounds = 1;}
+      else if ( strcmp(gridname, "generic") == 0 )        gridtype = GRID_GENERIC;
+      else if ( strcmp(gridname, "dereference") == 0 )    ldereference = 1;
+      else if ( strcmp(gridname, "lonlat") == 0 )         gridtype = GRID_LONLAT;
+      else if ( strcmp(gridname, "gaussian") == 0 )       gridtype = GRID_GAUSSIAN;
+      else if ( strcmp(gridname, "regular") == 0 )       {gridtype = GRID_GAUSSIAN; lregular = 1;}
       else cdoAbort("Unsupported grid name: %s", gridname);
     }
   else if ( operatorID == SETGRIDAREA )
@@ -113,7 +117,7 @@ void *Setgrid(void *argument)
 
       gridID = vlistInqVarGrid(vlistID, varID);
       areasize = gridInqSize(gridID);
-      areaweight = (double*) malloc(areasize*sizeof(double));
+      areaweight = (double*) Malloc(areasize*sizeof(double));
   
       streamReadRecord(streamID, areaweight, &nmiss);
 
@@ -157,7 +161,7 @@ void *Setgrid(void *argument)
       missval  = vlistInqVarMissval(vlistID, varID);
       gridID   = vlistInqVarGrid(vlistID, varID);
       masksize = gridInqSize(gridID);
-      gridmask = (double*) malloc(masksize*sizeof(double));
+      gridmask = (double*) Malloc(masksize*sizeof(double));
   
       streamReadRecord(streamID, gridmask, &nmiss);
 
@@ -170,8 +174,8 @@ void *Setgrid(void *argument)
     {
       if ( operatorArgc() >= 1 && operatorArgc() <= 2 )
 	{
-	  number = atoi(operatorArgv()[0]);
-	  if ( operatorArgc() == 2 ) position = atoi(operatorArgv()[1]);
+	  number = parameter2int(operatorArgv()[0]);
+	  if ( operatorArgc() == 2 ) position = parameter2int(operatorArgv()[1]);
 	}
       else
 	{
@@ -265,7 +269,7 @@ void *Setgrid(void *argument)
 	    {
 	      if      ( gridtype == GRID_CURVILINEAR  )
 		{
-		  gridID2 = gridToCurvilinear(gridID1, 1);
+		  gridID2 = gridToCurvilinear(gridID1, lbounds);
 		}
 	      else if ( gridtype == GRID_UNSTRUCTURED )
 		{
@@ -275,7 +279,7 @@ void *Setgrid(void *argument)
 		  if ( ligme )
 		    {
 		      grid2_nvgp = gridInqSize(gridID2);
-		      grid2_vgpm = (int*) malloc(grid2_nvgp*sizeof(int));
+		      grid2_vgpm = (int*) Malloc(grid2_nvgp*sizeof(int));
 		      gridInqMaskGME(gridID2, grid2_vgpm);
 		      gridCompress(gridID2);
 		    }
@@ -283,7 +287,17 @@ void *Setgrid(void *argument)
 	      else if ( gridtype == GRID_LONLAT && gridInqType(gridID1) == GRID_CURVILINEAR )
 		{
 		  gridID2 = gridCurvilinearToRegular(gridID1);
-		  if ( gridID2 == -1 ) cdoAbort("No regular grid found!");
+		  if ( gridID2 == -1 ) cdoWarning("Conversion of curvilinear grid to regular grid failed!");
+ 		}
+	      else if ( gridtype == GRID_LONLAT && gridInqType(gridID1) == GRID_UNSTRUCTURED )
+		{
+		  gridID2 = -1;
+		  if ( gridID2 == -1 ) cdoWarning("Conversion of unstructured grid to regular grid failed!");
+ 		}
+	      else if ( gridtype == GRID_LONLAT && gridInqType(gridID1) == GRID_GENERIC )
+		{
+		  gridID2 = -1;
+		  if ( gridID2 == -1 ) cdoWarning("Conversion of generic grid to regular grid failed!");
  		}
 	      else if ( gridtype == GRID_LONLAT && gridInqType(gridID1) == GRID_LONLAT )
 		{
@@ -321,19 +335,18 @@ void *Setgrid(void *argument)
 	  gridsize = gridInqSize(gridID1);
 	  if ( gridsize == masksize )
 	    {
-	      int *mask;
-	      mask = (int*) malloc(masksize*sizeof(int));
+	      int *mask = (int*) Malloc(masksize*sizeof(int));
 	      for ( i = 0; i < masksize; i++ )
 		{
 		  if ( gridmask[i] < 0 || gridmask[i] > 255 )
 		    mask[i] = 0;
 		  else
-		    mask[i] = lround(gridmask[i]);
+		    mask[i] = (int)lround(gridmask[i]);
 		}
 	      gridID2 = gridDuplicate(gridID1);
 	      gridDefMask(gridID2, mask);
 	      vlistChangeGridIndex(vlistID2, index, gridID2);
-	      free(mask);
+	      Free(mask);
 	    }
 	}
     }
@@ -360,7 +373,7 @@ void *Setgrid(void *argument)
     gridsize = vlistGridsizeMax(vlistID1);
 
   if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-  array = (double*) malloc(gridsize*sizeof(double));
+  array = (double*) Malloc(gridsize*sizeof(double));
 
   tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
@@ -402,12 +415,12 @@ void *Setgrid(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( gridmask ) free(gridmask);
-  if ( areaweight ) free(areaweight);
-  if ( array ) free(array);
-  if ( grid2_vgpm ) free(grid2_vgpm);
+  if ( gridmask ) Free(gridmask);
+  if ( areaweight ) Free(areaweight);
+  if ( array ) Free(array);
+  if ( grid2_vgpm ) Free(grid2_vgpm);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
