@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -45,6 +45,7 @@
 #include "util.h"
 #include "namelist.h"
 
+int stringToParam(const char *paramstr);
 
 typedef enum {CODE_NUMBER, PARAMETER_ID, VARIABLE_NAME, STANDARD_NAME} pt_mode_t;
 
@@ -173,7 +174,8 @@ void *get_converter(char *src_unit_str, char *tgt_unit_str, int *rstatus)
 
 typedef struct
 {
-  int delete;
+  int convert;
+  int remove;
   // missing value
   int changemissval;
   double missval_old;
@@ -200,7 +202,6 @@ typedef struct
   void *ut_converter;
 } var_t;
 
-int lwarn_udunits = TRUE;
 
 static
 void defineVarAttText(int vlistID2, int varID, const char *attname, const char *atttext)
@@ -210,14 +211,64 @@ void defineVarAttText(int vlistID2, int varID, const char *attname, const char *
 }
 
 static
-void defineVarUnits(var_t *vars, int vlistID2, int varID, char *units, char *name)
+void convertVarUnits(var_t *vars, int varID, char *name)
+{
+  if ( vars[varID].convert == FALSE ) vars[varID].changeunits = FALSE;
+
+  if ( vars[varID].changeunits == TRUE )
+    {
+      char *units = vars[varID].units;
+      char *units_old = vars[varID].units_old;
+#if defined(HAVE_UDUNITS2)
+      int status;
+      UDUNITS_INIT();
+      UDUNITS_LOCK();
+      vars[varID].ut_converter = get_converter(units_old, units, &status);
+      UDUNITS_UNLOCK();
+      if ( vars[varID].ut_converter == NULL )
+	{
+	  if ( status == -2 )
+	    {
+	      if ( cdoVerbose )
+		cdoPrint("%s - not converted from  [%s] to [%s], units are equal!", name, units_old, units);
+	    }
+	  else if ( status == -3 )
+	    {
+	      cdoWarning("%s - converting units from [%s] to [%s] failed, not convertible!", name, units_old, units);
+	    }
+	  else
+	    cdoWarning("%s - converting units from [%s] to [%s] failed!", name, units_old, units);
+	  vars[varID].changeunits = FALSE;
+	}
+      else
+	{
+	  // if ( cdoVerbose )
+	    {
+	      char buf[64];
+	      cv_get_expression((const cv_converter*)vars[varID].ut_converter, buf, 64, name);
+	      cdoPrint("%s - convert units from [%s] to [%s] (expression: %s).", name, units_old, units, buf);
+	    }
+	}
+#else
+      static int lwarn_udunits = TRUE;
+      if ( lwarn_udunits )
+	{
+	  cdoWarning("%s - converting units from [%s] to [%s] failed, UDUNITS2 support not compiled in!", name,units_old, units);
+	  vars[varID].changeunits = FALSE;
+	  lwarn_udunits = FALSE;
+	}
+#endif
+    }
+}
+
+static
+void defineVarUnits(var_t *vars, int vlistID2, int varID, char *units)
 {
   char units_old[CDI_MAX_NAME];
-  size_t len1, len2;
 
   vlistInqVarUnits(vlistID2, varID, units_old);
-  len1 = strlen(units_old);
-  len2 = strlen(units);
+  size_t len1 = strlen(units_old);
+  size_t len2 = strlen(units);
 
   if ( strcmp(units, units_old) != 0 )
     {
@@ -226,44 +277,6 @@ void defineVarUnits(var_t *vars, int vlistID2, int varID, char *units, char *nam
 	  vars[varID].changeunits = TRUE;
 	  strcpy(vars[varID].units_old, units_old);
 	  strcpy(vars[varID].units, units);
-#if defined(HAVE_UDUNITS2)
-	  int status;
-	  UDUNITS_INIT();
-	  UDUNITS_LOCK();
-	  vars[varID].ut_converter = get_converter(units_old, units, &status);
-	  UDUNITS_UNLOCK();
-	  if ( vars[varID].ut_converter == NULL )
-	    {
-	      if ( status == -2 )
-		{
-		  if ( cdoVerbose )
-		    cdoPrint("%s - not converted from  [%s] to [%s], units are equal!", name, units_old, units);
-		}
-	      else if ( status == -3 )
-		{
-		  cdoWarning("%s - converting units from [%s] to [%s] failed, not convertible!", name, units_old, units);
-		}
-	      else
-		cdoWarning("%s - converting units from [%s] to [%s] failed!", name, units_old, units);
-	      vars[varID].changeunits = FALSE;
-	    }
-	  else
-	    {
-	      if ( cdoVerbose )
-		{
-		  char buf[64];
-		  cv_get_expression(vars[varID].ut_converter, buf, 64, name);
-		  cdoPrint("%s - convert units from [%s] to [%s] (expression: %s).", name, units_old, units, buf);
-		}
-	    }
-#else
-	  if ( lwarn_udunits )
-	    {
-	      cdoWarning("Can't convert units, UDUNITS2 support not compiled in!");
-	      vars[varID].changeunits = FALSE;
-	      lwarn_udunits = FALSE;
-	    }
-#endif
 	}
 
       vlistDefVarUnits(vlistID2, varID, units);
@@ -274,36 +287,38 @@ void defineVarUnits(var_t *vars, int vlistID2, int varID, char *units, char *nam
 static
 void read_partab(pt_mode_t ptmode, int nvars, int vlistID2, var_t *vars)
 {
-  FILE *fp;
+  FILE *fp = NULL;
   namelist_t *nml;
-  int nml_code, nml_out_code, nml_table, nml_param, nml_chunktype, nml_datatype, nml_type, nml_name, nml_out_name, nml_stdname;
-  int nml_longname, nml_units, nml_comment, nml_ltype, nml_delete, nml_missval, nml_factor;
+  int nml_code, nml_out_code, nml_table, nml_param, nml_out_param, nml_chunktype, nml_datatype, nml_type, nml_name, nml_out_name, nml_stdname;
+  int nml_longname, nml_units, nml_comment, nml_ltype, nml_delete, nml_convert, nml_missval, nml_factor;
   int nml_cell_methods, nml_cell_measures;
   int nml_valid_min, nml_valid_max, nml_ok_min_mean_abs, nml_ok_max_mean_abs;
   int locc, i;
-  int code, out_code, table, ltype, delete;
+  int code, out_code, table, ltype, remove, convert;
   int nml_index = 0;
   int codenum, tabnum, levtype, param;
   int varID, tableID;
   int num_pt_files;
-  double param_dp;
   double missval, factor;
   double valid_min, valid_max, ok_min_mean_abs, ok_max_mean_abs;
   char *partab = NULL;
   char *chunktypestr = NULL;
   char *datatypestr = NULL;
   char *typestr = NULL;
+  char *paramstr = NULL;
+  char *out_paramstr = NULL;
   char *name = NULL, *out_name = NULL, *stdname = NULL, longname[CDI_MAX_NAME] = "", units[CDI_MAX_NAME] = "";
   char cell_methods[CDI_MAX_NAME] = "", cell_measures[CDI_MAX_NAME] = "";
   char varname[CDI_MAX_NAME];
   char comment[1024] = "";
 
-  num_pt_files = operatorArgc();
+  //num_pt_files = operatorArgc();
+  num_pt_files = 1;
 
   for ( int fileID = 0; fileID < num_pt_files; ++fileID )
     {
       partab = operatorArgv()[fileID];
-      fp = fopen(partab, "r");
+      if ( fileExists(partab) ) fp = fopen(partab, "r");
       if ( fp == NULL ) cdoAbort("Open failed on parameter table %d file name %s!", fileID+1, partab);
 
       nml_index = 0;
@@ -314,14 +329,16 @@ void read_partab(pt_mode_t ptmode, int nvars, int vlistID2, var_t *vars)
       nml_out_code        = namelistAdd(nml, "out_code",        NML_INT,  0, &out_code, 1);
       nml_table           = namelistAdd(nml, "table",           NML_INT,  0, &table, 1);
       nml_ltype           = namelistAdd(nml, "ltype",           NML_INT,  0, &ltype, 1);
-      nml_delete          = namelistAdd(nml, "delete",          NML_INT,  0, &delete, 1);
+      nml_delete          = namelistAdd(nml, "delete",          NML_INT,  0, &remove, 1);
+      nml_convert         = namelistAdd(nml, "convert",         NML_INT,  0, &convert, 1);
       nml_missval         = namelistAdd(nml, "missing_value",   NML_FLT,  0, &missval, 1);
       nml_factor          = namelistAdd(nml, "factor",          NML_FLT,  0, &factor, 1);
       nml_valid_min       = namelistAdd(nml, "valid_min",       NML_FLT,  0, &valid_min, 1);
       nml_valid_max       = namelistAdd(nml, "valid_max",       NML_FLT,  0, &valid_max, 1);
       nml_ok_min_mean_abs = namelistAdd(nml, "ok_min_mean_abs", NML_FLT,  0, &ok_min_mean_abs, 1);
       nml_ok_max_mean_abs = namelistAdd(nml, "ok_max_mean_abs", NML_FLT,  0, &ok_max_mean_abs, 1);
-      nml_param           = namelistAdd(nml, "param",           NML_FLT,  0, &param_dp, 1);
+      nml_param           = namelistAdd(nml, "param",           NML_WORD, 0, &paramstr, 1);
+      nml_out_param       = namelistAdd(nml, "out_param",       NML_WORD, 0, &out_paramstr, 1);
       nml_chunktype       = namelistAdd(nml, "chunktype",       NML_WORD, 0, &chunktypestr, 1);
       nml_datatype        = namelistAdd(nml, "datatype",        NML_WORD, 0, &datatypestr, 1);
       nml_type            = namelistAdd(nml, "type",            NML_WORD, 0, &typestr, 1);
@@ -333,13 +350,15 @@ void read_partab(pt_mode_t ptmode, int nvars, int vlistID2, var_t *vars)
       nml_comment         = namelistAdd(nml, "comment",         NML_TEXT, 0, comment, sizeof(comment));
       nml_cell_methods    = namelistAdd(nml, "cell_methods",    NML_TEXT, 0, cell_methods, sizeof(cell_methods));
       nml_cell_measures   = namelistAdd(nml, "cell_measures",   NML_TEXT, 0, cell_measures, sizeof(cell_measures));
-	      
+
       while ( ! feof(fp) )
 	{
 	  namelistReset(nml);
 
 	  namelistRead(fp, nml);
 
+	  if ( cdoVerbose ) namelistPrint(nml);
+
 	  locc = FALSE;
 	  for ( i = 0; i < nml->size; i++ )
 	    {
@@ -356,7 +375,7 @@ void read_partab(pt_mode_t ptmode, int nvars, int vlistID2, var_t *vars)
 		{
 		  if ( nml->entry[nml_code]->occ == 0 )
 		    {
-		      cdoPrint("Parameter entry %d (table %d) skipped, code number not found!", nml_index, fileID+1);
+		      cdoPrint("Parameter entry %d (parameter table %d) skipped, code number not found!", nml_index, fileID+1);
 		      continue;
 		    }
 		}
@@ -364,7 +383,7 @@ void read_partab(pt_mode_t ptmode, int nvars, int vlistID2, var_t *vars)
 		{
 		  if ( nml->entry[nml_param]->occ == 0 )
 		    {
-		      cdoWarning("Parameter entry %d (table %d) skipped, parameter ID not found!", nml_index, fileID+1);
+		      cdoWarning("Parameter entry %d (parameter table %d) skipped, parameter ID not found!", nml_index, fileID+1);
 		      continue;
 		    }
 		}
@@ -372,7 +391,7 @@ void read_partab(pt_mode_t ptmode, int nvars, int vlistID2, var_t *vars)
 		{
 		  if ( nml->entry[nml_name]->occ == 0 )
 		    {
-		      cdoWarning("Parameter entry %d (table %d) skipped, variable name not found!", nml_index, fileID+1);
+		      cdoWarning("Parameter entry %d (parameter table %d) skipped, variable name not found!", nml_index, fileID+1);
 		      continue;
 		    }
 		}
@@ -386,7 +405,7 @@ void read_partab(pt_mode_t ptmode, int nvars, int vlistID2, var_t *vars)
 		      tabnum  = tableInqNum(tableID);
 		      levtype = zaxisInqLtype(vlistInqVarZaxis(vlistID2, varID));
 		      
-		      //	printf("code = %d  tabnum = %d  ltype = %d\n", codenum, tabnum, levtype);
+		      // printf("code = %d  tabnum = %d  ltype = %d\n", codenum, tabnum, levtype);
 		      
 		      if ( nml->entry[nml_table]->occ == 0 ) table = tabnum;
 		      if ( nml->entry[nml_ltype]->occ == 0 ) ltype = levtype;
@@ -395,21 +414,14 @@ void read_partab(pt_mode_t ptmode, int nvars, int vlistID2, var_t *vars)
 		    }
 		  else if ( ptmode == PARAMETER_ID )
 		    {
+		      int paramid = stringToParam(paramstr);
+
 		      param   = vlistInqVarParam(vlistID2, varID);
-		      codenum = vlistInqVarCode(vlistID2, varID);
-		      tableID = vlistInqVarTable(vlistID2, varID);
-		      tabnum  = tableInqNum(tableID);
 		      levtype = zaxisInqLtype(vlistInqVarZaxis(vlistID2, varID));
-		      
-		      //	printf("code = %d  tabnum = %d  ltype = %d\n", codenum, tabnum, levtype);
-		      code  = (int) param_dp;
-		      table = (param_dp-code)*1000;
-		      printf("code = %d  tabnum = %d  ltype = %d\n", code, table, levtype);
-		      
-		      if ( nml->entry[nml_table]->occ == 0 ) table = tabnum;
+
 		      if ( nml->entry[nml_ltype]->occ == 0 ) ltype = levtype;
 		  
-		      if ( codenum == code && tabnum == table && levtype == ltype ) break;
+		      if ( param == paramid && levtype == ltype ) break;
 		    }
 		  else if ( ptmode == VARIABLE_NAME )
 		    {
@@ -420,19 +432,24 @@ void read_partab(pt_mode_t ptmode, int nvars, int vlistID2, var_t *vars)
 
 	      if ( varID < nvars )
 		{
-		  if ( nml->entry[nml_code]->occ     ) vlistDefVarCode(vlistID2, varID, code);
-		  if ( nml->entry[nml_out_code]->occ ) vlistDefVarCode(vlistID2, varID, out_code);
-		  if ( nml->entry[nml_name]->occ     ) strcpy(vars[varID].name, name);
-		  if ( nml->entry[nml_name]->occ     ) vlistDefVarName(vlistID2, varID, name);
-		  if ( nml->entry[nml_out_name]->occ ) vlistDefVarName(vlistID2, varID, out_name);
-		  if ( nml->entry[nml_out_name]->occ ) defineVarAttText(vlistID2, varID, "original_name", vars[varID].name);
-		  if ( nml->entry[nml_stdname]->occ  ) vlistDefVarStdname(vlistID2, varID, stdname);
-		  if ( nml->entry[nml_longname]->occ ) vlistDefVarLongname(vlistID2, varID, longname);
-		  if ( nml->entry[nml_units]->occ    ) defineVarUnits(vars, vlistID2, varID, units, name);
-		  if ( nml->entry[nml_comment]->occ  ) defineVarAttText(vlistID2, varID, "comment", comment);
-		  if ( nml->entry[nml_cell_methods]->occ  )  defineVarAttText(vlistID2, varID, "cell_methods", cell_methods);
-		  if ( nml->entry[nml_cell_measures]->occ  ) defineVarAttText(vlistID2, varID, "cell_measures", cell_measures);
-		  if ( nml->entry[nml_delete]->occ && delete == 1 ) vars[varID].delete = TRUE;
+                  int pnum, ptab, pdum;
+                  cdiDecodeParam(vlistInqVarParam(vlistID2, varID), &pnum, &ptab, &pdum);
+		  if ( nml->entry[nml_code]->occ     )  vlistDefVarParam(vlistID2, varID, cdiEncodeParam(code, ptab, 255));
+		  if ( nml->entry[nml_out_code]->occ )  vlistDefVarParam(vlistID2, varID, cdiEncodeParam(out_code, ptab, 255));
+		  if ( nml->entry[nml_name]->occ     )  strcpy(vars[varID].name, name);
+		  if ( nml->entry[nml_name]->occ     )  vlistDefVarName(vlistID2, varID, name);
+		  if ( nml->entry[nml_out_name]->occ )  vlistDefVarName(vlistID2, varID, out_name);
+		  if ( nml->entry[nml_out_name]->occ )  defineVarAttText(vlistID2, varID, "original_name", vars[varID].name);
+		  if ( nml->entry[nml_stdname]->occ  )  vlistDefVarStdname(vlistID2, varID, stdname);
+		  if ( nml->entry[nml_longname]->occ )  vlistDefVarLongname(vlistID2, varID, longname);
+		  if ( nml->entry[nml_units]->occ    )  defineVarUnits(vars, vlistID2, varID, units);
+		  if ( nml->entry[nml_comment]->occ  )  defineVarAttText(vlistID2, varID, "comment", comment);
+		  if ( nml->entry[nml_cell_methods]->occ  ) defineVarAttText(vlistID2, varID, "cell_methods", cell_methods);
+		  if ( nml->entry[nml_cell_measures]->occ ) defineVarAttText(vlistID2, varID, "cell_measures", cell_measures);
+		  if ( nml->entry[nml_delete]->occ && remove == 1 ) vars[varID].remove = TRUE;
+		  if ( nml->entry[nml_convert]->occ )   vars[varID].convert = convert==0 ? FALSE : TRUE;
+		  if ( nml->entry[nml_param]->occ )     vlistDefVarParam(vlistID2, varID, stringToParam(paramstr));
+		  if ( nml->entry[nml_out_param]->occ ) vlistDefVarParam(vlistID2, varID, stringToParam(out_paramstr));
 		  if ( nml->entry[nml_datatype]->occ )
 		    {
 		      int datatype = str2datatype(datatypestr);
@@ -580,67 +597,61 @@ void check_data(int vlistID2, int varID2, int varID, var_t *vars, long gridsize,
 
 void *Setpartab(void *argument)
 {
-  int SETPARTAB, SETPARTABN, SETPARTABC, SETPARTABP;
-  int operatorID;
-  int streamID1, streamID2 = CDI_UNDEFID;
-  int nrecs, nvars;
-  int tsID1, recID, varID, levelID;
-  int varID2, levelID2;
-  int vlistID1, vlistID2;
-  int taxisID1, taxisID2;
+  int nrecs;
+  int varID, levelID;
   int nmiss;
   int delvars = FALSE;
-  long gridsize;
   int tableID = -1;
   int tableformat = 0;
-  char *partab = NULL;
   double missval;
-  double *array = NULL;
-  var_t *vars = NULL;
-  pt_mode_t ptmode = CODE_NUMBER;
-
 
   cdoInitialize(argument);
 
-  SETPARTAB  = cdoOperatorAdd("setpartab",  0, 0, "parameter table name");
-  SETPARTABC = cdoOperatorAdd("setpartabc", 0, 0, "parameter table name");
-  SETPARTABP = cdoOperatorAdd("setpartabp", 0, 0, "parameter table name");
-  SETPARTABN = cdoOperatorAdd("setpartabn", 0, 0, "parameter table name");
+  int SETPARTAB  = cdoOperatorAdd("setpartab",  0, 0, "parameter table name");
+  int SETPARTABC = cdoOperatorAdd("setpartabc", 0, 0, "parameter table name");
+  int SETPARTABP = cdoOperatorAdd("setpartabp", 0, 0, "parameter table name");
+  int SETPARTABN = cdoOperatorAdd("setpartabn", 0, 0, "parameter table name");
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
   operatorInputArg(cdoOperatorEnter(operatorID));
 
   if ( operatorArgc() < 1 ) cdoAbort("Too few arguments!");
 
-  if      ( operatorID == SETPARTAB )  ptmode = CODE_NUMBER;
+  int convert_data = FALSE;
+  if ( operatorArgc() == 2 )
+    {
+      if ( strcmp("convert", operatorArgv()[1]) == 0 ) convert_data = TRUE;
+      else cdoAbort("Unknown parameter: >%s<", operatorArgv()[1]); 
+    }
+
+  if ( operatorArgc() > 2 ) cdoAbort("Too many arguments!");
+
+  pt_mode_t ptmode = CODE_NUMBER;
+  if      ( operatorID == SETPARTAB  ) ptmode = CODE_NUMBER;
   else if ( operatorID == SETPARTABC ) ptmode = CODE_NUMBER;
   else if ( operatorID == SETPARTABP ) ptmode = PARAMETER_ID;
   else if ( operatorID == SETPARTABN ) ptmode = VARIABLE_NAME;
 
   if ( ptmode == CODE_NUMBER )
     {
-      FILE *fp;
-      size_t fsize;
-      char *parbuf = NULL;
-      size_t nbytes;
-
-      partab = operatorArgv()[0];
-      fp = fopen(partab, "r");
+      char *partab = operatorArgv()[0];
+      FILE *fp = NULL;
+      if ( fileExists(partab) ) fp = fopen(partab, "r");
       if ( fp != NULL )
 	{
 	  fseek(fp, 0L, SEEK_END);
-	  fsize = (size_t) ftell(fp);
-	  parbuf = malloc(fsize+1);
+	  size_t fsize = (size_t) ftell(fp);
+	  char *parbuf = (char *) Malloc(fsize+1);
 	  fseek(fp, 0L, SEEK_SET);
-	  nbytes = fread(parbuf, fsize, 1, fp);
+	  fread(parbuf, fsize, 1, fp);
 	  parbuf[fsize] = 0;
 	  fseek(fp, 0L, SEEK_SET);
 
 	  if ( atoi(parbuf) == 0 ) tableformat = 1;
 
 	  fclose(fp);
-	  free(parbuf);
+	  Free(parbuf);
 	}
 
       if ( tableformat == 0 ) tableID = defineTable(partab);
@@ -654,46 +665,73 @@ void *Setpartab(void *argument)
       tableformat = 1;
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
   /* vlistPrint(vlistID2);*/
 
-  nvars = vlistNvars(vlistID2);
-  vars = malloc(nvars*sizeof(var_t));
+  int nvars = vlistNvars(vlistID2);
+  var_t *vars = (var_t *) Malloc(nvars*sizeof(var_t));
   memset(vars, 0, nvars*sizeof(var_t));
 
+  if ( convert_data )
+    for ( varID = 0; varID < nvars; ++varID ) vars[varID].convert = TRUE;
+
   if ( tableformat == 0 )
     {
-      for ( varID = 0; varID < nvars; varID++ )
-	vlistDefVarTable(vlistID2, varID, tableID);
+      /*
+        for ( int varID = 0; varID < nvars; varID++ )
+	  vlistDefVarTable(vlistID2, varID, tableID);
+      */
+      char name[CDI_MAX_NAME], longname[CDI_MAX_NAME], units[CDI_MAX_NAME];
+      for ( int varID = 0; varID < nvars; varID++ )
+        {
+          int param = vlistInqVarParam(vlistID2, varID);
+          int pdis, pcat, pnum;
+          cdiDecodeParam(param, &pnum, &pcat, &pdis);
+          if ( pdis == 255 )
+            {
+              int code = pnum;
+              if ( tableInqParName(tableID, code, name) == 0 )
+                {
+                  vlistDefVarName(vlistID2, varID, name);
+                  longname[0] = 0;
+                  tableInqParLongname(tableID, code, longname);
+                  vlistDefVarLongname(vlistID2, varID, longname);
+                  units[0] = 0;
+                  tableInqParUnits(tableID, code, units);
+                  vlistDefVarUnits(vlistID2, varID, units);
+                }
+            }
+          vlistDefVarTable(vlistID2, varID, tableID);
+        }
     }
   else
     {
       read_partab(ptmode, nvars, vlistID2, vars);
 
-      for ( varID = 0; varID < nvars; ++varID )
-	if ( vars[varID].delete ) break;
-
-      if ( varID < nvars ) delvars = TRUE;
+      for ( int varID = 0; varID < nvars; ++varID )
+	if ( vars[varID].remove )
+          {
+            delvars = TRUE;
+            break;
+          }
 
       if ( delvars )
 	{
-	  int levID, nlevs, zaxisID;
-	  int vlistIDx;
 	  vlistClearFlag(vlistID1);
 	  vlistClearFlag(vlistID2);
 
-	  for ( varID = 0; varID < nvars; varID++ )
+	  for ( int varID = 0; varID < nvars; varID++ )
 	    {
-	      zaxisID  = vlistInqVarZaxis(vlistID2, varID);
-	      nlevs    = zaxisInqSize(zaxisID);
-	      for ( levID = 0; levID < nlevs; levID++ )
+	      int zaxisID = vlistInqVarZaxis(vlistID2, varID);
+	      int nlevs   = zaxisInqSize(zaxisID);
+	      for ( int levID = 0; levID < nlevs; levID++ )
 		{
 		  vlistDefFlag(vlistID1, varID, levID, TRUE);
 		  vlistDefFlag(vlistID2, varID, levID, TRUE);
-		  if ( vars[varID].delete )
+		  if ( vars[varID].remove )
 		    {
 		      vlistDefFlag(vlistID1, varID, levID, FALSE);
 		      vlistDefFlag(vlistID2, varID, levID, FALSE);
@@ -701,45 +739,48 @@ void *Setpartab(void *argument)
 		}
 	    }
 
-	  vlistIDx = vlistCreate();
+	  int vlistIDx = vlistCreate();
 	  vlistCopyFlag(vlistIDx, vlistID2);
 
 	  vlistDestroy(vlistID2);
 
 	  vlistID2 = vlistIDx;
 	}
+
+      for ( int varID = 0; varID < nvars; ++varID )
+	convertVarUnits(vars, varID, vars[varID].name);
     }
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
   /* vlistPrint(vlistID2);*/
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  gridsize = vlistGridsizeMax(vlistID1);
+  long gridsize = vlistGridsizeMax(vlistID1);
   if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-  array = malloc(gridsize*sizeof(double));
+  double *array = (double *) Malloc(gridsize*sizeof(double));
 
-  tsID1 = 0;
+  int tsID1 = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID1)) )
     {
       taxisCopyTimestep(taxisID2, taxisID1);
 
       streamDefTimestep(streamID2, tsID1);
 	       
-      for ( recID = 0; recID < nrecs; recID++ )
+      for ( int recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
 
-	  varID2 = varID;
-	  levelID2 = levelID;
+	  int varID2 = varID;
+	  int levelID2 = levelID;
 
 	  if ( delvars )
 	    {
-	      if ( vars[varID].delete ) continue;
+	      if ( vars[varID].remove ) continue;
 
 	      if ( vlistInqFlag(vlistID1, varID, levelID) == TRUE )
 		{
@@ -780,7 +821,7 @@ void *Setpartab(void *argument)
 		{
 		  if ( !DBL_IS_EQUAL(array[i], missval) )
 		    {
-		      array[i] = cv_convert_double(vars[varID].ut_converter, array[i]);
+		      array[i] = cv_convert_double((const cv_converter*)vars[varID].ut_converter, array[i]);
 		      if ( ut_get_status() != UT_SUCCESS ) nerr++;
 		    }
 		}
@@ -807,8 +848,8 @@ void *Setpartab(void *argument)
 #if defined(HAVE_UDUNITS2)
   UDUNITS_LOCK();
 
-  for ( varID = 0; varID < nvars; varID++ )
-    if ( vars[varID].ut_converter ) cv_free(vars[varID].ut_converter);
+  for ( int varID = 0; varID < nvars; varID++ )
+    if ( vars[varID].ut_converter ) cv_free((cv_converter*)vars[varID].ut_converter);
 
   if ( ut_read )
     { 
@@ -819,10 +860,10 @@ void *Setpartab(void *argument)
   UDUNITS_UNLOCK();
 #endif
 
-  if ( array ) free(array);
-  if ( vars  ) free(vars);
+  if ( array ) Free(array);
+  if ( vars  ) Free(vars);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
