@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -35,6 +35,7 @@
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
+#include "calendar.h"
 #include "pstream.h"
 
 
@@ -62,7 +63,7 @@ int get_tunits(const char *unit, int *incperiod, int *incunit, int *tunit)
       else if ( *incperiod == 12 ) { *incperiod = 1; *incunit = 43200; *tunit = TUNIT_12HOURS; }
     }
 
-  return (0);
+  return 0;
 }
 
 static
@@ -104,46 +105,39 @@ void shifttime(int calendar, int tunit, int ijulinc, int *pdate, int *ptime)
 
 void *Settime(void *argument)
 {
-  int SETYEAR, SETMON, SETDAY, SETDATE, SETTIME, SETTUNITS;
-  int SETTAXIS, SETREFTIME, SETCALENDAR, SHIFTTIME;
-  int operatorID;
-  int streamID1, streamID2 = CDI_UNDEFID;
-  int nrecs, newval = 0, ntsteps, nvars;
+  int nrecs, newval = 0;
   int tsID1, recID, varID, levelID;
-  int vlistID1, vlistID2;
   int vdate, vtime;
   int vdateb[2], vtimeb[2];
   int sdate = 0, stime = 0;
-  int taxisID1, taxisID2 = CDI_UNDEFID;
+  int taxisID2 = CDI_UNDEFID;
   int nmiss;
   int gridsize;
   int tunit = TUNIT_DAY;
-  int ijulinc = 0, incperiod = 0, incunit = 0;
+  int ijulinc = 0, incperiod = 1, incunit = 86400;
   int year = 1, month = 1, day = 1, hour = 0, minute = 0, second = 0;
   int day0;
   int taxis_has_bounds, copy_timestep = FALSE;
-  int calendar;
   int newcalendar = CALENDAR_STANDARD;
   // int nargs;
   const char *datestr, *timestr;
-  char *rstr;
   juldate_t juldate;
   double *array = NULL;
 
   cdoInitialize(argument);
 
-  SETYEAR     = cdoOperatorAdd("setyear",     0,  1, "year");
-  SETMON      = cdoOperatorAdd("setmon",      0,  1, "month");
-  SETDAY      = cdoOperatorAdd("setday",      0,  1, "day");
-  SETDATE     = cdoOperatorAdd("setdate",     0,  1, "date (format: YYYY-MM-DD)");
-  SETTIME     = cdoOperatorAdd("settime",     0,  1, "time (format: hh:mm:ss)");
-  SETTUNITS   = cdoOperatorAdd("settunits",   0,  1, "time units (seconds, minutes, hours, days, months, years)");
-  SETTAXIS    = cdoOperatorAdd("settaxis",    0, -2, "date,time<,increment> (format YYYY-MM-DD,hh:mm:ss)");
-  SETREFTIME  = cdoOperatorAdd("setreftime",  0, -2, "date,time<,units> (format YYYY-MM-DD,hh:mm:ss)");
-  SETCALENDAR = cdoOperatorAdd("setcalendar", 0,  1, "calendar (standard, proleptic_gregorian, 360_day, 365_day, 366_day)");
-  SHIFTTIME   = cdoOperatorAdd("shifttime",   0,  1, "shift value");
+  int SETYEAR     = cdoOperatorAdd("setyear",      0,  1, "year");
+  int SETMON      = cdoOperatorAdd("setmon",       0,  1, "month");
+  int SETDAY      = cdoOperatorAdd("setday",       0,  1, "day");
+  int SETDATE     = cdoOperatorAdd("setdate",      0,  1, "date (format: YYYY-MM-DD)");
+  int SETTIME     = cdoOperatorAdd("settime",      0,  1, "time (format: hh:mm:ss)");
+  int SETTUNITS   = cdoOperatorAdd("settunits",    0,  1, "time units (seconds, minutes, hours, days, months, years)");
+  int SETTAXIS    = cdoOperatorAdd("settaxis",     0, -2, "date,time<,increment> (format YYYY-MM-DD,hh:mm:ss)");
+  int SETREFTIME  = cdoOperatorAdd("setreftime",   0, -2, "date,time<,units> (format YYYY-MM-DD,hh:mm:ss)");
+  int SETCALENDAR = cdoOperatorAdd("setcalendar",  0,  1, "calendar (standard, proleptic_gregorian, 360_day, 365_day, 366_day)");
+  int SHIFTTIME   = cdoOperatorAdd("shifttime",    0,  1, "shift value");
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
   // nargs = cdoOperatorF2(operatorID);
 
   operatorInputArg(cdoOperatorEnter(operatorID));
@@ -153,19 +147,19 @@ void *Settime(void *argument)
   if ( operatorID == SETTAXIS || operatorID == SETREFTIME )
     {
       if ( operatorArgc() < 2 ) cdoAbort("Too few arguments!");
+      if ( operatorArgc() > 3 ) cdoAbort("Too many arguments!");
 
       datestr = operatorArgv()[0];
       timestr = operatorArgv()[1];
 
-      if ( strchr(datestr, '-') )
+      if ( strchr(datestr+1, '-') )
 	{
 	  sscanf(datestr, "%d-%d-%d", &year, &month, &day);
 	  sdate = cdiEncodeDate(year, month, day);
 	}
       else
 	{
-	  sdate = (int)strtol(datestr, &rstr, 10);
-	  if ( *rstr != 0 ) cdoAbort("Parameter string contains invalid characters: %s", datestr);
+	  sdate = parameter2int(datestr);
 	}
 
       if ( strchr(timestr, ':') )
@@ -175,17 +169,19 @@ void *Settime(void *argument)
 	}
       else
 	{
-	  stime = (int)strtol(timestr, &rstr, 10);
-	  if ( *rstr != 0 ) cdoAbort("Parameter string contains invalid characters: %s", timestr);
+	  stime = parameter2int(timestr);
 	}
 
       if ( operatorArgc() == 3 )
 	{
 	  const char *timeunits = operatorArgv()[2];
-	  incperiod = (int)strtol(timeunits, NULL, 10);
-	  if ( timeunits[0] == '-' || timeunits[0] == '+' ) timeunits++;
-	  while ( isdigit((int) *timeunits) ) timeunits++;
-
+          int ich = timeunits[0];
+          if ( ich == '-' || ich == '+' || isdigit(ich) )
+            {
+              incperiod = (int)strtol(timeunits, NULL, 10);
+              if ( ich == '-' || ich == '+' ) timeunits++;
+              while ( isdigit((int) *timeunits) ) timeunits++;
+            }
 	  get_tunits(timeunits, &incperiod, &incunit, &tunit);
 	}
       /* increment in seconds */
@@ -193,7 +189,7 @@ void *Settime(void *argument)
     }
   else if ( operatorID == SETDATE )
     {
-      if ( operatorArgc() < 1 ) cdoAbort("Too few arguments!");
+      operatorCheckArgc(1);
       datestr = operatorArgv()[0];
       if ( strchr(datestr, '-') )
 	{
@@ -202,13 +198,12 @@ void *Settime(void *argument)
 	}
       else
 	{
-	  newval = (int)strtol(datestr, &rstr, 10);
-	  if ( *rstr != 0 ) cdoAbort("Parameter string contains invalid characters: %s", datestr);
+	  newval = parameter2int(datestr);
 	}
     }
   else if ( operatorID == SETTIME )
     {
-      if ( operatorArgc() < 1 ) cdoAbort("Too few arguments!");
+      operatorCheckArgc(1);
       timestr = operatorArgv()[0];
 
       if ( strchr(timestr, ':') )
@@ -218,12 +213,12 @@ void *Settime(void *argument)
 	}
       else
 	{
-	  newval = (int)strtol(timestr, &rstr, 10);
-	  if ( *rstr != 0 ) cdoAbort("Parameter string contains invalid characters: %s", timestr);
+	  newval = parameter2int(timestr);
 	}
     }
   else if ( operatorID == SHIFTTIME )
     {
+      operatorCheckArgc(1);
       const char *timeunits = operatorArgv()[0];
       incperiod = (int)strtol(timeunits, NULL, 10);
       if ( timeunits[0] == '-' || timeunits[0] == '+' ) timeunits++;
@@ -236,6 +231,7 @@ void *Settime(void *argument)
     }
   else if ( operatorID == SETTUNITS )
     {
+      operatorCheckArgc(1);
       int idum;
       const char *timeunits = operatorArgv()[0];
       incperiod = 0;
@@ -243,6 +239,7 @@ void *Settime(void *argument)
     }
   else if ( operatorID == SETCALENDAR )
     {
+      operatorCheckArgc(1);
       char *cname = operatorArgv()[0];
       strtolower(cname);
       size_t len = strlen(cname);
@@ -261,19 +258,19 @@ void *Settime(void *argument)
     }
   else
     {
-      newval = (int)strtol(operatorArgv()[0], &rstr, 10);
-      if ( *rstr != 0 ) cdoAbort("Parameter string contains invalid characters: %s", operatorArgv()[0]);
+      operatorCheckArgc(1);
+      newval = parameter2int(operatorArgv()[0]);
     }
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = vlistDuplicate(vlistID1);
 
-  taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
   taxis_has_bounds = taxisHasBounds(taxisID1);
-  ntsteps  = vlistNtsteps(vlistID1);
-  nvars    = vlistNvars(vlistID1);
+  int ntsteps  = vlistNtsteps(vlistID1);
+  int nvars    = vlistNvars(vlistID1);
 
   if ( ntsteps == 1 )
     {
@@ -289,7 +286,7 @@ void *Settime(void *argument)
 	vlistDefVarTsteptype(vlistID2, varID, TSTEP_INSTANT);
     }
 
-  calendar = taxisInqCalendar(taxisID1);
+  int calendar = taxisInqCalendar(taxisID1);
 
   if ( cdoVerbose ) cdoPrint("calendar = %d", calendar);
 
@@ -370,13 +367,13 @@ void *Settime(void *argument)
 
   vlistDefTaxis(vlistID2, taxisID2);
 
-  streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
+  int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
   gridsize = vlistGridsizeMax(vlistID1);
   if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-  array = (double*) malloc(gridsize*sizeof(double));
+  array = (double*) Malloc(gridsize*sizeof(double));
 
   tsID1 = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID1)) )
@@ -486,9 +483,9 @@ void *Settime(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( array ) free(array);
+  if ( array ) Free(array);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
