@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -135,12 +135,9 @@ static int nlat2nlon(int nlat)
 
 int ngp2ntr(int ngp)
 {
-  int ntr, nlatl, nlonl;
-
-  ntr = NINT(sqrt(0.25+ngp)-1.5);
-  nlatl = ntr2nlat_linear(ntr);
-  nlonl = compnlon(nlatl);
-  nlatl = nlonl/2;
+  int ntr   = (int)lround(sqrt(0.25+ngp)-1.5);
+  int nlonl = compnlon(ntr2nlat_linear(ntr));
+  int nlatl = nlonl/2;
 
   ntr = (2*nlatl-1)/2;
 
@@ -266,7 +263,7 @@ void *Specinfo(void *argument)
 
   len = strlen(operatorArgv()[0]);
 
-  if ( (len+1) >= 128 ) cdoAbort("Parameter string to large!");
+  if ( (len+1) >= 128 ) cdoAbort("Parameter string too large!");
 
   for ( i = 0; i < len; i++ ) arg[i] = toupper(operatorArgv()[0][i]);
   arg[len] = 0;
@@ -489,5 +486,5 @@ void *Specinfo(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
