@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,7 @@
 #include "statistic.h"
 
 
-#define  NALLOC_INC  1000
+#define  NALLOC_INC  1024
 
 
 static
@@ -40,6 +40,7 @@ void spectrum(int nrec, double *data, double *spectrum, double *real, double *im
   int k;
   double sumx, sumkx;
   double a, b;
+  double *work_r = NULL, *work_i = NULL;
   int seg_i, offset;
   int bit;
 
@@ -66,6 +67,12 @@ void spectrum(int nrec, double *data, double *spectrum, double *real, double *im
       for (k = 0; k < nrec; k++)
 	data[k] -= a + b * k;
     }
+
+  if ( bit != 1 )
+    {
+      work_r = (double*) Malloc(seg_l*sizeof(double));
+      work_i = (double*) Malloc(seg_l*sizeof(double));
+    }
 	
   for (seg_i = 0; seg_i < seg_n; seg_i += 2)
     {
@@ -123,9 +130,9 @@ void spectrum(int nrec, double *data, double *spectrum, double *real, double *im
 	  imag[k] = 0;
       
       if (bit == 1)	/* seg_l is a power of 2 */
-	fft (real, imag, seg_l, 1);
+	fft(real, imag, seg_l, 1);
       else
-	ft (real, imag, seg_l, 1);
+	ft_r(real, imag, seg_l, 1, work_r, work_i);
 	
       spectrum[0] += real[0] * real[0] + imag[0] * imag[0];
       
@@ -139,6 +146,12 @@ void spectrum(int nrec, double *data, double *spectrum, double *real, double *im
 	  real[seg_l / 2] * real[seg_l / 2] +
 	  imag[seg_l / 2] * imag[seg_l / 2];
     }
+
+  if ( bit != 1 )
+    {
+      Free(work_r);
+      Free(work_i);
+    }
 	
   for (k = 0; k <= seg_l / 2; k++)
     spectrum[k] *= seg_l / (seg_n * wssum);
@@ -194,9 +207,9 @@ void *Spectrum(void *argument)
       if ( tsID >= nalloc )
 	{
 	  nalloc += NALLOC_INC;
-	  vdate = realloc(vdate, nalloc*sizeof(int));
-	  vtime = realloc(vtime, nalloc*sizeof(int));
-	  vars  = realloc(vars, nalloc*sizeof(field_t **));
+	  vdate = (int*) Realloc(vdate, nalloc*sizeof(int));
+	  vtime = (int*) Realloc(vtime, nalloc*sizeof(int));
+	  vars  = (field_t ***) Realloc(vars, nalloc*sizeof(field_t **));
 	}
 
       vdate[tsID] = taxisInqVdate(taxisID1);
@@ -209,7 +222,7 @@ void *Spectrum(void *argument)
 	  streamInqRecord(streamID1, &varID, &levelID);
 	  gridID   = vlistInqVarGrid(vlistID1, varID);
 	  gridsize = gridInqSize(gridID);
-	  vars[tsID][varID][levelID].ptr = malloc(gridsize*sizeof(double));
+	  vars[tsID][varID][levelID].ptr = (double*) Malloc(gridsize*sizeof(double));
 	  streamReadRecord(streamID1, vars[tsID][varID][levelID].ptr, &nmiss);
 	  vars[tsID][varID][levelID].nmiss = nmiss;
 
@@ -234,10 +247,10 @@ void *Spectrum(void *argument)
 
   operatorCheckArgc(4);
 
-  detrend = atoi(operatorArgv()[0]);
-  seg_l = atoi(operatorArgv()[1]);
-  seg_n = atoi(operatorArgv()[2]);
-  which_window = atoi(operatorArgv()[3]);
+  detrend = parameter2int(operatorArgv()[0]);
+  seg_l = parameter2int(operatorArgv()[1]);
+  seg_n = parameter2int(operatorArgv()[2]);
+  which_window = parameter2int(operatorArgv()[3]);
 
   if ( detrend < 0 || detrend > 3 )
     cdoAbort("Illegal value for detrend (=%d)!",detrend);
@@ -251,15 +264,15 @@ void *Spectrum(void *argument)
 
   nfreq = seg_l/2 + 1;
 
-  vars2 = malloc(nfreq*sizeof(field_t **));
+  vars2 = (field_t ***) Malloc(nfreq*sizeof(field_t **));
   for ( freq = 0; freq < nfreq; freq++ )
     vars2[freq] = field_malloc(vlistID1, FIELD_PTR);
 
-  array1  = malloc(nts   * sizeof(double));
-  array2  = malloc(nfreq * sizeof(double));
-  real    = malloc(seg_l * sizeof(double));
-  imag    = malloc(seg_l * sizeof(double));
-  window  = malloc(seg_l * sizeof(double));
+  array1  = (double*) Malloc(nts   * sizeof(double));
+  array2  = (double*) Malloc(nfreq * sizeof(double));
+  real    = (double*) Malloc(seg_l * sizeof(double));
+  imag    = (double*) Malloc(seg_l * sizeof(double));
+  window  = (double*) Malloc(seg_l * sizeof(double));
   	   
   switch (which_window)
     {
@@ -315,8 +328,8 @@ void *Spectrum(void *argument)
 	}
     }
 
-  if ( array1 ) free(array1);
-  if ( array2 ) free(array2);
+  if ( array1 ) Free(array1);
+  if ( array2 ) Free(array2);
 
   for ( tsID = 0; tsID < nts; tsID++ ) field_free(vars[tsID], vlistID1);
 
@@ -343,15 +356,15 @@ void *Spectrum(void *argument)
       field_free(vars2[tsID], vlistID1);
     }
 
-  if ( vars  ) free(vars);
-  if ( vars2 ) free(vars2);
-  if ( vdate ) free(vdate);
-  if ( vtime ) free(vtime);
+  if ( vars  ) Free(vars);
+  if ( vars2 ) Free(vars2);
+  if ( vdate ) Free(vdate);
+  if ( vtime ) Free(vtime);
 
   streamClose(streamID2);
   streamClose(streamID1);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
