@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -33,19 +33,6 @@
 #include "cdo_int.h"
 #include "pstream.h"
 
-void create_uuid(unsigned char uuid[CDI_UUID_SIZE]);
-void uuid2str(const unsigned char *uuid, char *uuidstr);
-
-
-#define UUIDSTR_SIZE (CDI_UUID_SIZE*2 + 4)
-
-static
-void get_uuid(char uuidstr[UUIDSTR_SIZE])
-{
-  unsigned char uuid[CDI_UUID_SIZE];
-  create_uuid(uuid);
-  uuid2str(uuid, uuidstr);
-}
 
 static
 void gen_filename(char *filename, int swap_obase, const char *obase, const char *suffix)
@@ -109,8 +96,6 @@ void *Split(void *argument)
   int vlistID1 = streamInqVlist(streamID1);
 
   int nvars  = vlistNvars(vlistID1);
-  int nrecs  = vlistNrecs(vlistID1);
-  int nzaxis = vlistNzaxis(vlistID1);
 
   if ( swap_obase == 0 )
     {
@@ -124,7 +109,6 @@ void *Split(void *argument)
   
   if ( operatorID == SPLITCODE )
     {
-      int *codes = NULL;
       nsplit = 0;
       for ( varID = 0; varID < nvars; varID++ )
 	{
@@ -139,9 +123,9 @@ void *Split(void *argument)
 	    }
 	}
 
-      codes     = (int*) malloc(nsplit*sizeof(int));
-      vlistIDs  = (int*) malloc(nsplit*sizeof(int));
-      streamIDs = (int*) malloc(nsplit*sizeof(int));
+      vlistIDs  = (int*) Malloc(nsplit*sizeof(int));
+      streamIDs = (int*) Malloc(nsplit*sizeof(int));
+      int codes[nsplit];
       memcpy(codes, itmp, nsplit*sizeof(int));
 
       for ( index = 0; index < nsplit; index++ )
@@ -186,12 +170,10 @@ void *Split(void *argument)
 	  streamIDs[index] = streamOpenWrite(fileargument, cdoFiletype());
 	  file_argument_free(fileargument);
 	}
-      if ( codes ) free(codes);
     }
   else if ( operatorID == SPLITPARAM )
     {
       char paramstr[32];
-      int *params = NULL;
       nsplit = 0;
       for ( varID = 0; varID < nvars; varID++ )
 	{
@@ -206,9 +188,9 @@ void *Split(void *argument)
 	    }
 	}
 
-      params    = (int*) malloc(nsplit*sizeof(int));
-      vlistIDs  = (int*) malloc(nsplit*sizeof(int));
-      streamIDs = (int*) malloc(nsplit*sizeof(int));
+      vlistIDs  = (int*) Malloc(nsplit*sizeof(int));
+      streamIDs = (int*) Malloc(nsplit*sizeof(int));
+      int params[nsplit];
       memcpy(params, itmp, nsplit*sizeof(int));
 
       for ( index = 0; index < nsplit; index++ )
@@ -243,11 +225,9 @@ void *Split(void *argument)
 	  streamIDs[index] = streamOpenWrite(fileargument, cdoFiletype());
 	  file_argument_free(fileargument);
 	}
-      if ( params ) free(params);
     }
   else if ( operatorID == SPLITTABNUM )
     {
-      int *tabnums = NULL;
       nsplit = 0;
       for ( varID = 0; varID < nvars; varID++ )
 	{
@@ -262,9 +242,9 @@ void *Split(void *argument)
 	    }
 	}
 
-      tabnums   = (int*) malloc(nsplit*sizeof(int));
-      vlistIDs  = (int*) malloc(nsplit*sizeof(int));
-      streamIDs = (int*) malloc(nsplit*sizeof(int));
+      vlistIDs  = (int*) Malloc(nsplit*sizeof(int));
+      streamIDs = (int*) Malloc(nsplit*sizeof(int));
+      int tabnums[nsplit];
       memcpy(tabnums, itmp, nsplit*sizeof(int));
 
       for ( index = 0; index < nsplit; index++ )
@@ -295,15 +275,14 @@ void *Split(void *argument)
 	  streamIDs[index] = streamOpenWrite(fileargument, cdoFiletype());
 	  file_argument_free(fileargument);
 	}
-      if ( tabnums ) free(tabnums);
     }
   else if ( operatorID == SPLITNAME )
     {
       char varname[CDI_MAX_NAME];
       nsplit = nvars;
 
-      vlistIDs  = (int*) malloc(nsplit*sizeof(int));
-      streamIDs = (int*) malloc(nsplit*sizeof(int));
+      vlistIDs  = (int*) Malloc(nsplit*sizeof(int));
+      streamIDs = (int*) Malloc(nsplit*sizeof(int));
 
       for ( index = 0; index < nsplit; index++ )
 	{
@@ -333,8 +312,8 @@ void *Split(void *argument)
     }
   else if ( operatorID == SPLITLEVEL )
     {
-      double level, *levels = NULL;
-      nzaxis = vlistNzaxis(vlistID1);
+      double level;
+      int nzaxis = vlistNzaxis(vlistID1);
       nsplit = 0;
       for ( index = 0; index < nzaxis; index++ )
 	{
@@ -350,9 +329,9 @@ void *Split(void *argument)
 	    }
 	}
 
-      levels    = (double*) malloc(nsplit*sizeof(double));
-      vlistIDs  = (int*) malloc(nsplit*sizeof(int));
-      streamIDs = (int*) malloc(nsplit*sizeof(int));
+      vlistIDs  = (int*) Malloc(nsplit*sizeof(int));
+      streamIDs = (int*) Malloc(nsplit*sizeof(int));
+      double levels[nsplit];
       memcpy(levels, ftmp, nsplit*sizeof(double));
 
       for ( index = 0; index < nsplit; index++ )
@@ -383,18 +362,16 @@ void *Split(void *argument)
 	  streamIDs[index] = streamOpenWrite(fileargument, cdoFiletype());
 	  file_argument_free(fileargument);
 	}
-      if ( levels ) free(levels);
     }
   else if ( operatorID == SPLITGRID )
     {
-      int gridID, *gridIDs = NULL;
+      int gridID;
 
       nsplit = vlistNgrids(vlistID1);
 
-      gridIDs   = (int*) malloc(nsplit*sizeof(int));
-      vlistIDs  = (int*) malloc(nsplit*sizeof(int));
-      streamIDs = (int*) malloc(nsplit*sizeof(int));
-
+      vlistIDs  = (int*) Malloc(nsplit*sizeof(int));
+      streamIDs = (int*) Malloc(nsplit*sizeof(int));
+      int gridIDs[nsplit];
       for ( index = 0; index < nsplit; index++ )
 	gridIDs[index] = vlistGrid(vlistID1, index);
 
@@ -426,18 +403,16 @@ void *Split(void *argument)
 	  streamIDs[index] = streamOpenWrite(fileargument, cdoFiletype());
 	  file_argument_free(fileargument);
 	}
-      if ( gridIDs ) free(gridIDs);
     }
   else if ( operatorID == SPLITZAXIS )
     {
-      int zaxisID, *zaxisIDs = NULL;
+      int zaxisID;
 
       nsplit = vlistNzaxis(vlistID1);
 
-      zaxisIDs  = (int*) malloc(nsplit*sizeof(int));
-      vlistIDs  = (int*) malloc(nsplit*sizeof(int));
-      streamIDs = (int*) malloc(nsplit*sizeof(int));
-
+      vlistIDs  = (int*) Malloc(nsplit*sizeof(int));
+      streamIDs = (int*) Malloc(nsplit*sizeof(int));
+      int zaxisIDs[nsplit];
       for ( index = 0; index < nsplit; index++ )
 	zaxisIDs[index] = vlistZaxis(vlistID1, index);
 
@@ -468,7 +443,6 @@ void *Split(void *argument)
 	  streamIDs[index] = streamOpenWrite(fileargument, cdoFiletype());
 	  file_argument_free(fileargument);
 	}
-      if ( zaxisIDs ) free(zaxisIDs);
     }
   else
     {
@@ -477,13 +451,7 @@ void *Split(void *argument)
 
   for ( index = 0; index < nsplit; index++ )
     {
-      if ( uuid_attribute )
-        {
-          char uuidstr[UUIDSTR_SIZE];
-          get_uuid(uuidstr);
-          vlistDefAttTxt(vlistIDs[index], CDI_GLOBAL, uuid_attribute, 
-                         UUIDSTR_SIZE, uuidstr);
-        }
+      if ( uuid_attribute ) cdo_def_tracking_id(vlistIDs[index], uuid_attribute);
 
       streamDefVlist(streamIDs[index], vlistIDs[index]);
     }
@@ -493,9 +461,10 @@ void *Split(void *argument)
     {
       gridsize = vlistGridsizeMax(vlistID1);
       if ( vlistNumber(vlistID1) != CDI_REAL ) gridsize *= 2;
-      array = (double *) malloc(gridsize*sizeof(double));
+      array = (double *) Malloc(gridsize*sizeof(double));
     }
 
+  int nrecs;
   int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
@@ -537,12 +506,12 @@ void *Split(void *argument)
     }
  
   if ( ! lcopy )
-    if ( array ) free(array);
+    if ( array ) Free(array);
 
-  if ( vlistIDs  ) free(vlistIDs);
-  if ( streamIDs ) free(streamIDs);
+  if ( vlistIDs  ) Free(vlistIDs);
+  if ( streamIDs ) Free(streamIDs);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
