@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -40,20 +40,20 @@
 
 
 #if defined(ENABLE_DATA)
-  static double etopo_scale  = 3;
-  static double etopo_offset = 11000;
+  static const double etopo_scale  = 3;
+  static const double etopo_offset = 11000;
   static const unsigned short etopo[] = {
 #include "etopo.h"
   };
 
-  static double temp_scale  =  500;
-  static double temp_offset = -220;
+  static const double temp_scale  =  500;
+  static const double temp_offset = -220;
   static const unsigned short temp[] = {
 #include "temp.h"
   };
 
-  static double mask_scale  =  1;
-  static double mask_offset =  0;
+  static const double mask_scale  =  1;
+  static const double mask_offset =  0;
   static const unsigned short mask[] = {
 #include "mask.h"
   };
@@ -67,8 +67,8 @@
 #define CC_R             (287.05)  /* specific gas constant for air */
 static double TMP4PRESSURE = (C_EARTH_GRAV*SCALEHEIGHT)/(CC_R*T_ZERO);
 
-static double
-std_atm_temperatur(double height)
+static
+double std_atm_temperatur(double height)
 {
   /*
     Compute the temperatur for the given height (in meters) according to the
@@ -77,8 +77,8 @@ std_atm_temperatur(double height)
    return (T_ZERO + T_DELTA * exp((-1)*(height/SCALEHEIGHT)));
 }
 
-static double
-std_atm_pressure(double height)
+static
+double std_atm_pressure(double height)
 {
   /*
     Compute the pressure for the given height (in meters) according to the
@@ -87,35 +87,130 @@ std_atm_pressure(double height)
   return (P_ZERO * exp((-1)*TMP4PRESSURE*log((exp(height/SCALEHEIGHT)*T_ZERO + T_DELTA)/(T_ZERO + T_DELTA))));
 }
 
+static
+void remap_nn_reg2d_reg2d(int nx, int ny, const double *restrict data, int gridID, double *restrict array)
+{
+  if ( !(gridInqType(gridID) == GRID_LONLAT && !gridIsRotated(gridID)) )
+    cdoAbort("Internal error, wrong grid type!");
+
+  int nxvals = gridInqXsize(gridID);
+  int nyvals = gridInqYsize(gridID);
+  double *xvals = (double*) Malloc(nxvals*sizeof(double));
+  double *yvals = (double*) Malloc(nyvals*sizeof(double));
+
+  gridInqXvals(gridID, xvals);
+  gridInqYvals(gridID, yvals);
+
+  /* Convert lat/lon units if required */
+  char units[CDI_MAX_NAME];
+  gridInqXunits(gridID, units);
+  grid_to_degree(units, nxvals, xvals, "grid center lon");
+  gridInqYunits(gridID, units);
+  grid_to_degree(units, nyvals, yvals, "grid center lat");
+
+  int ii, jj;
+  double xval, yval;
+  for ( int j = 0; j < nyvals; j++ )
+    {
+      yval = yvals[j];
+      for ( int i = 0; i < nxvals; i++ )
+        {
+          xval = xvals[i];
+          if ( xval >=  180 ) xval -= 360;
+          if ( xval <  -180 ) xval += 360;
+          ii = (xval + 180)*2;
+          jj = (yval +  90)*2;
+          if ( ii >= nx ) ii = nx-1;
+          if ( jj >= ny ) jj = ny-1;
+          array[j*nxvals+i] = data[jj*nx+ii];
+        }
+    }
+
+  Free(xvals);
+  Free(yvals);
+}
+
+static
+void remap_nn_reg2d_nonreg2d(int nx, int ny, const double *restrict data, int gridID, double *restrict array)
+{
+  int gridID2 = gridID;
+  int gridsize = gridInqSize(gridID2);
+  double *xvals = (double*) Malloc(gridsize*sizeof(double));
+  double *yvals = (double*) Malloc(gridsize*sizeof(double));
+
+  if ( gridInqType(gridID2) == GRID_GME ) gridID2 = gridToUnstructured(gridID2, 0);
+
+  if ( gridInqType(gridID2) != GRID_UNSTRUCTURED && gridInqType(gridID2) != GRID_CURVILINEAR )
+    gridID2 = gridToCurvilinear(gridID2, 0);
+
+  gridInqXvals(gridID2, xvals);
+  gridInqYvals(gridID2, yvals);
+
+  /* Convert lat/lon units if required */
+  char units[CDI_MAX_NAME];
+  gridInqXunits(gridID2, units);
+  grid_to_degree(units, gridsize, xvals, "grid center lon");
+  gridInqYunits(gridID2, units);
+  grid_to_degree(units, gridsize, yvals, "grid center lat");
+
+  int ii, jj;
+  double xval, yval;
+  for ( int i = 0; i < gridsize; i++ )
+    {
+      xval = xvals[i];
+      yval = yvals[i];
+      if ( xval >=  180 ) xval -= 360;
+      if ( xval <  -180 ) xval += 360;
+      ii = (xval + 180)*2;
+      jj = (yval +  90)*2;
+      if ( ii >= nx ) ii = nx-1;
+      if ( jj >= ny ) jj = ny-1;
+      array[i] = data[jj*nx+ii];
+    }
+
+  Free(xvals);
+  Free(yvals);
+
+  if ( gridID != gridID2 ) gridDestroy(gridID2);
+}
+
+static
+void remap_nn_reg2d(int nx, int ny, const double *restrict data, int gridID, double *restrict array)
+{
+  if ( gridInqType(gridID) == GRID_LONLAT && !gridIsRotated(gridID) )
+    remap_nn_reg2d_reg2d(nx, ny, data, gridID, array);
+  else
+    remap_nn_reg2d_nonreg2d(nx, ny, data, gridID, array);
+}
+
+
 void *Vargen(void *argument)
 {
-  int RANDOM, SINCOS, COSHILL, CONST, FOR, TOPO, TEMP, MASK, STDATM;
-  int operatorID;
-  int streamID;
-  int nvars, ntimesteps, nlevels = 1;
-  int tsID, varID, varID2 = -1, levelID;
-  int gridsize, i;
-  int vlistID;
-  int gridID = -1, zaxisID, taxisID;
-  int vdate, vtime, julday;
-  const char *gridfile;
+  int ntimesteps, nlevels = 1;
+  int varID, varID2 = -1, levelID;
+  int i;
+  int gridID = -1, gridIDdata = -1, zaxisID;
+  int vdate, vtime;
   double rval, rstart = 0, rstop = 0, rinc = 0;
   double rconst = 0;
-  double *array, *levels = NULL;
+  double *levels = NULL;
+  double lon[720], lat[360];
+  int nlon = 720;
+  int nlat = 360;
 
   cdoInitialize(argument);
 
-  RANDOM  = cdoOperatorAdd("random",  0, 0, "grid description file or name, <seed>");
-  SINCOS  = cdoOperatorAdd("sincos",  0, 0, "grid description file or name");
-  COSHILL = cdoOperatorAdd("coshill", 0, 0, "grid description file or name");
-  CONST   = cdoOperatorAdd("const",   0, 0, "constant value, grid description file or name");
-  FOR     = cdoOperatorAdd("for",     0, 0, "start, end, <increment>");
-  TOPO    = cdoOperatorAdd("topo",    0, 0, NULL);
-  TEMP    = cdoOperatorAdd("temp",    0, 0, NULL);
-  MASK    = cdoOperatorAdd("mask",    0, 0, NULL);
-  STDATM  = cdoOperatorAdd("stdatm",  0, 0, "height levels [m]");
+  int RANDOM  = cdoOperatorAdd("random",  0, 0, "grid description file or name, <seed>");
+  int SINCOS  = cdoOperatorAdd("sincos",  0, 0, "grid description file or name");
+  int COSHILL = cdoOperatorAdd("coshill", 0, 0, "grid description file or name");
+  int CONST   = cdoOperatorAdd("const",   0, 0, "constant value, grid description file or name");
+  int FOR     = cdoOperatorAdd("for",     0, 0, "start, end, <increment>");
+  int TOPO    = cdoOperatorAdd("topo",    0, 0, NULL);
+  int TEMP    = cdoOperatorAdd("temp",    0, 0, NULL);
+  int MASK    = cdoOperatorAdd("mask",    0, 0, NULL);
+  int STDATM  = cdoOperatorAdd("stdatm",  0, 0, "height levels [m]");
 
-  operatorID = cdoOperatorID();
+  int operatorID = cdoOperatorID();
 
   if ( operatorID == RANDOM )
     {
@@ -123,13 +218,11 @@ void *Vargen(void *argument)
       operatorInputArg(cdoOperatorEnter(operatorID));
       if ( operatorArgc() < 1 ) cdoAbort("Too few arguments!");
       if ( operatorArgc() > 2 ) cdoAbort("Too many arguments!");
-      gridfile = operatorArgv()[0];
-      gridID   = cdoDefineGrid(gridfile);
+      gridID = cdoDefineGrid(operatorArgv()[0]);
       if ( operatorArgc() == 2 )
         {
           int idum = parameter2int(operatorArgv()[1]);
-          if ( idum >= 0 && idum < 0x7FFFFFFF )
-            seed = idum;
+          if ( idum >= 0 && idum < 0x7FFFFFFF ) seed = idum;
         }
       srand(seed);
     }
@@ -137,32 +230,30 @@ void *Vargen(void *argument)
     {
       operatorInputArg(cdoOperatorEnter(operatorID));
       operatorCheckArgc(1);
-      gridfile = operatorArgv()[0];
-      gridID   = cdoDefineGrid(gridfile);
+      gridID = cdoDefineGrid(operatorArgv()[0]);
     }
   else if ( operatorID == CONST )
     {
       operatorInputArg(cdoOperatorEnter(operatorID));
       operatorCheckArgc(2);
-      rconst   = parameter2double(operatorArgv()[0]);
-      gridfile = operatorArgv()[1];
-      gridID   = cdoDefineGrid(gridfile);
+      rconst = parameter2double(operatorArgv()[0]);
+      gridID = cdoDefineGrid(operatorArgv()[1]);
     }
   else if ( operatorID == TOPO || operatorID == TEMP || operatorID == MASK )
     {
-      int nlon, nlat, i;
-      double lon[720], lat[360];
-      nlon = 720;
-      nlat = 360;
-      gridID = gridCreate(GRID_LONLAT, nlon*nlat);
-      gridDefXsize(gridID, nlon);
-      gridDefYsize(gridID, nlat);
+      gridIDdata = gridCreate(GRID_LONLAT, nlon*nlat);
+      gridDefXsize(gridIDdata, nlon);
+      gridDefYsize(gridIDdata, nlat);
 
-      for ( i = 0; i < nlon; i++ ) lon[i] = -179.75 + i*0.5;
-      for ( i = 0; i < nlat; i++ ) lat[i] = -89.75 + i*0.5;
+      for ( int i = 0; i < nlon; i++ ) lon[i] = -179.75 + i*0.5;
+      for ( int i = 0; i < nlat; i++ ) lat[i] = -89.75 + i*0.5;
 
-      gridDefXvals(gridID, lon);
-      gridDefYvals(gridID, lat);
+      gridDefXvals(gridIDdata, lon);
+      gridDefYvals(gridIDdata, lat);
+
+      gridID = gridIDdata;
+
+      if ( operatorArgc() == 1 ) gridID = cdoDefineGrid(operatorArgv()[0]);
     }
   else if ( operatorID == FOR )
     {
@@ -219,12 +310,12 @@ void *Vargen(void *argument)
       nlevels = 1;
     }
 
-  vlistID = vlistCreate();
+  int vlistID = vlistCreate();
 
-  if ( operatorID == FOR )
-    varID = vlistDefVar(vlistID, gridID, zaxisID, TSTEP_INSTANT);
-  else
-    varID = vlistDefVar(vlistID, gridID, zaxisID, TSTEP_CONSTANT);
+  int tsteptype = TSTEP_CONSTANT;
+  if ( operatorID == FOR ) tsteptype = TSTEP_INSTANT;
+
+  varID = vlistDefVar(vlistID, gridID, zaxisID, tsteptype);
   /*
      For the standard atmosphere two output variables are generated: pressure and
      temperatur. The first (varID) is pressure, second (varID2) is temperatur.
@@ -239,12 +330,13 @@ void *Vargen(void *argument)
   if ( operatorID == STDATM )
     {
       vlistDefVarName(vlistID    , varID , "P");
-      vlistDefVarCode(vlistID    , varID , 1);
+      vlistDefVarParam(vlistID   , varID , cdiEncodeParam(1, 255, 255));
       vlistDefVarStdname(vlistID , varID , "air_pressure");
       vlistDefVarLongname(vlistID, varID , "pressure");
       vlistDefVarUnits(vlistID   , varID , "hPa");
+
       vlistDefVarName(vlistID    , varID2, "T");
-      vlistDefVarCode(vlistID    , varID2, 130);
+      vlistDefVarParam(vlistID   , varID2, cdiEncodeParam(130, 128, 255));
       vlistDefVarStdname(vlistID , varID2, var_stdname(air_temperature));
       vlistDefVarLongname(vlistID, varID2, "temperature");
       vlistDefVarUnits(vlistID   , varID2, "K");
@@ -252,26 +344,31 @@ void *Vargen(void *argument)
   else
     {
       vlistDefVarName(vlistID, varID, cdoOperatorName(operatorID));
-      if ( operatorID == TOPO )
-	vlistDefVarUnits(vlistID, varID , "m");	
-      if ( operatorID == TEMP )
-	vlistDefVarUnits(vlistID, varID , "K");	
+      if ( operatorID == TOPO ) vlistDefVarUnits(vlistID, varID , "m");	
+      if ( operatorID == TEMP ) vlistDefVarUnits(vlistID, varID , "K");	
     }
 
-  taxisID = taxisCreate(TAXIS_RELATIVE);
+  int taxisID = taxisCreate(TAXIS_RELATIVE);
   vlistDefTaxis(vlistID, taxisID);
 
-  if ( operatorID == RANDOM || operatorID == SINCOS || operatorID == COSHILL || operatorID == CONST || operatorID == TOPO ||
-       operatorID == TEMP || operatorID == MASK || operatorID == STDATM )
+  if ( operatorID == RANDOM || operatorID == SINCOS || operatorID == COSHILL || operatorID == CONST ||
+       operatorID == TOPO || operatorID == TEMP || operatorID == MASK || operatorID == STDATM )
     vlistDefNtsteps(vlistID, 1);
 
-  streamID = streamOpenWrite(cdoStreamName(0), cdoFiletype());
+  int streamID = streamOpenWrite(cdoStreamName(0), cdoFiletype());
 
   streamDefVlist(streamID, vlistID);
 
-  gridsize = gridInqSize(gridID);
-  array = (double*) malloc(gridsize*sizeof(double));
-
+  int gridsize = gridInqSize(gridID);
+  int datasize = gridsize;
+  double *array = (double*) Malloc(gridsize*sizeof(double));
+  double *data = array;
+  if ( gridID != gridIDdata && gridIDdata != -1 )
+    {
+      datasize = gridInqSize(gridIDdata);
+      data = (double*) Malloc(datasize*sizeof(double));
+    }
+  
   if ( operatorID == FOR )
     ntimesteps = 1.001 + ((rstop-rstart)/rinc);
   else
@@ -280,11 +377,11 @@ void *Vargen(void *argument)
       ntimesteps = 1;
     }
 
-  julday = date_to_julday(CALENDAR_PROLEPTIC, 10101);
+  int julday = date_to_julday(CALENDAR_PROLEPTIC, 10101);
 
-  nvars = vlistNvars(vlistID);
+  int nvars = vlistNvars(vlistID);
 
-  for ( tsID = 0; tsID < ntimesteps; tsID++ )
+  for ( int tsID = 0; tsID < ntimesteps; tsID++ )
     {
       rval  = rstart + rinc*tsID;
       vdate = julday_to_date(CALENDAR_PROLEPTIC, julday + tsID);
@@ -303,12 +400,12 @@ void *Vargen(void *argument)
               if ( operatorID == RANDOM )
                 {
                   for ( i = 0; i < gridsize; i++ )
-                    array[i] = rand()/(RAND_MAX+1.0);
+                    array[i] = ((double)rand())/((double)RAND_MAX);
                 }
               else if ( operatorID == SINCOS || operatorID == COSHILL )
                 {
-		  double *xvals = (double*) malloc(gridsize*sizeof(double));
-		  double *yvals = (double*) malloc(gridsize*sizeof(double));
+		  double *xvals = (double*) Malloc(gridsize*sizeof(double));
+		  double *yvals = (double*) Malloc(gridsize*sizeof(double));
 
 		  if ( gridInqType(gridID) == GRID_GME ) gridID = gridToUnstructured(gridID, 0);
 
@@ -336,8 +433,8 @@ void *Vargen(void *argument)
 			array[i] = 2 - cos(acos(cos(xvals[i]) * cos(yvals[i]))/1.2);
 		    }
 
-		  free(xvals);
-		  free(yvals);
+		  Free(xvals);
+		  Free(yvals);
 		}
               else if ( operatorID == CONST )
                 {
@@ -347,8 +444,8 @@ void *Vargen(void *argument)
               else if ( operatorID == TOPO )
                 {
 #if defined(ENABLE_DATA)
-                  for ( i = 0; i < gridsize; i++ )
-                    array[i] = etopo[i]/etopo_scale - etopo_offset;
+                  for ( i = 0; i < datasize; i++ )
+                    data[i] = etopo[i]/etopo_scale - etopo_offset;
 #else
                   cdoAbort("Operator support disabled!");
 #endif
@@ -356,8 +453,8 @@ void *Vargen(void *argument)
               else if ( operatorID == TEMP )
                 {
 #if defined(ENABLE_DATA)
-                  for ( i = 0; i < gridsize; i++ )
-                    array[i] = temp[i]/temp_scale - temp_offset;
+                  for ( i = 0; i < datasize; i++ )
+                    data[i] = temp[i]/temp_scale - temp_offset;
 #else
                   cdoAbort("Operator support disabled!");
 #endif
@@ -365,8 +462,8 @@ void *Vargen(void *argument)
               else if ( operatorID == MASK )
                 {
 #if defined(ENABLE_DATA)
-                  for ( i = 0; i < gridsize; i++ )
-                    array[i] = mask[i]/mask_scale - mask_offset;
+                  for ( i = 0; i < datasize; i++ )
+                    data[i] = mask[i]/mask_scale - mask_offset;
 #else
                   cdoAbort("Operator support disabled!");
 #endif
@@ -380,6 +477,11 @@ void *Vargen(void *argument)
                   array[0] = (varID == varID2) ? std_atm_temperatur(levels[levelID]) : std_atm_pressure(levels[levelID]);
                 }
 
+              if ( gridID != gridIDdata && (operatorID == TOPO || operatorID == TEMP || operatorID == MASK) )
+                {
+                  remap_nn_reg2d(nlon, nlat, data, gridID, array);
+                }
+
               streamWriteRecord(streamID, array, 0);
             }
         }
@@ -389,10 +491,11 @@ void *Vargen(void *argument)
 
   vlistDestroy(vlistID);
 
-  if ( array ) free(array);
-  if ( levels ) free(levels); 
+  if ( gridID != gridIDdata && gridIDdata != -1 ) Free(data);
+  if ( array ) Free(array);
+  if ( levels ) Free(levels); 
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
