@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -41,15 +41,14 @@ void *Vertintml(void *argument)
   enum {ECHAM_MODE, WMO_MODE};
   enum {func_pl, func_hl};
   enum {type_lin, type_log};
-  int gridsize;
   int recID, nrecs;
   int i, k, offset;
-  int tsID, varID, levelID;
-  int zaxisIDp, zaxisIDh = -1, nzaxis;
+  int varID, levelID;
+  int zaxisIDp, zaxisIDh = -1;
   int gridID, zaxisID;
   int nhlev = 0, nhlevf = 0, nhlevh = 0, nlevel;
   int *vert_index = NULL;
-  int nvct;
+  int nvct = 0;
   int sgeopot_needed = FALSE;
   int sgeopotID = -1, geopotID = -1, tempID = -1, psID = -1, lnpsID = -1, presID = -1, gheightID = -1;
   int code, param;
@@ -66,7 +65,6 @@ void *Vertintml(void *argument)
   double *sgeopot = NULL, *ps_prog = NULL, *full_press = NULL, *half_press = NULL;
   double *hyb_press = NULL;
   int Extrapolate = 0;
-  int lhavevct;
   int mono_level;
   int instNum, tableNum;
   int useTable;
@@ -90,17 +88,13 @@ void *Vertintml(void *argument)
 
   if ( operatorID == ML2PL || operatorID == ML2HL || operatorID == ML2PL_LP || operatorID == ML2HL_LP )
     {
-      char *envstr;
-      envstr = getenv("EXTRAPOLATE");
+      char *envstr = getenv("EXTRAPOLATE");
 
-      if ( envstr )
+      if ( envstr && isdigit((int) envstr[0]) )
 	{
-	  if ( isdigit((int) envstr[0]) )
-	    {
-	      Extrapolate = atoi(envstr);
-	      if ( Extrapolate == 1 )
-		cdoPrint("Extrapolation of missing values enabled!");
-	    }
+          Extrapolate = atoi(envstr);
+          if ( Extrapolate == 1 )
+            cdoPrint("Extrapolation of missing values enabled!");
 	}
     }
   else if ( operatorID == ML2PLX || operatorID == ML2HLX || operatorID == ML2PLX_LP || operatorID == ML2HLX_LP )
@@ -114,10 +108,14 @@ void *Vertintml(void *argument)
   double *plev = NULL;
   if ( operatorArgc() == 1 && strcmp(operatorArgv()[0], "default") == 0 )
     {
+      /*
       double stdlev[] = {100000, 92500, 85000, 77500, 70000, 60000, 50000, 40000, 30000, 25000, 20000,
-                         15000, 10000, 7000, 5000, 3000, 2000, 1000, 700, 500, 300, 200, 100, 50, 20, 10};
+                          15000, 10000, 7000, 5000, 3000, 2000, 1000, 700, 500, 300, 200, 100, 50, 20, 10};
+      */
+      double stdlev[] = {100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000, 20000, 15000,
+                          10000,  7000,  5000,  3000,  2000, 1000 };
       nplev = sizeof(stdlev)/sizeof(*stdlev);
-      plev  = (double *) malloc(nplev*sizeof(double));
+      plev  = (double *) Malloc(nplev*sizeof(double));
       for ( i = 0; i < nplev; ++i ) plev[i] = stdlev[i];
     }
   else
@@ -135,7 +133,7 @@ void *Vertintml(void *argument)
   int taxisID2 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID2, taxisID2);
 
-  gridsize = vlist_check_gridsize(vlistID1);
+  int gridsize = vlist_check_gridsize(vlistID1);
 
   if ( operfunc == func_hl )
     zaxisIDp = zaxisCreate(ZAXIS_HEIGHT, nplev);
@@ -143,8 +141,8 @@ void *Vertintml(void *argument)
     zaxisIDp = zaxisCreate(ZAXIS_PRESSURE, nplev);
 
   zaxisDefLevels(zaxisIDp, plev);
-  nzaxis  = vlistNzaxis(vlistID1);
-  lhavevct = FALSE;
+  int nzaxis  = vlistNzaxis(vlistID1);
+  int lhavevct = FALSE;
   for ( i = 0; i < nzaxis; i++ )
     {
       /* mono_level = FALSE; */
@@ -155,16 +153,14 @@ void *Vertintml(void *argument)
       if ( (zaxisInqType(zaxisID) == ZAXIS_HYBRID || zaxisInqType(zaxisID) == ZAXIS_HYBRID_HALF) &&
 	   nlevel > 1 )
 	{
-	  double *level;
-	  int l;
-	  level = (double*) malloc(nlevel*sizeof(double));
+	  double level[nlevel];
 	  zaxisInqLevels(zaxisID, level);
+	  int l;
 	  for ( l = 0; l < nlevel; l++ )
 	    {
 	      if ( (l+1) != (int) (level[l]+0.5) ) break;
 	    }
 	  if ( l == nlevel ) mono_level = TRUE; 
-	  free(level);
 	}
 
       if ( (zaxisInqType(zaxisID) == ZAXIS_HYBRID || zaxisInqType(zaxisID) == ZAXIS_HYBRID_HALF) &&
@@ -179,9 +175,9 @@ void *Vertintml(void *argument)
 		  zaxisIDh = zaxisID;
 		  nhlev    = nlevel;
 		  nhlevf   = nhlev;
-		  nhlevh   = nhlevf + 1;
-	      
-		  vct = (double*) malloc(nvct*sizeof(double));
+		  nhlevh   = nhlev + 1;
+
+		  vct = (double*) Malloc(nvct*sizeof(double));
 		  zaxisInqVct(zaxisID, vct);
 
 		  vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
@@ -202,7 +198,7 @@ void *Vertintml(void *argument)
 		  nhlevf   = nhlev - 1;
 		  nhlevh   = nhlev;
 	      
-		  vct = (double*) malloc(nvct*sizeof(double));
+		  vct = (double*) Malloc(nvct*sizeof(double));
 		  zaxisInqVct(zaxisID, vct);
 
 		  vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
@@ -220,7 +216,7 @@ void *Vertintml(void *argument)
 		  int vctsize;
 		  int voff = 4;
 		  
-		  rvct = (double*) malloc(nvct*sizeof(double));
+		  rvct = (double*) Malloc(nvct*sizeof(double));
 		  zaxisInqVct(zaxisID, rvct);
 
 		  if ( (int)(rvct[0]+0.5) == 100000 && rvct[voff] < rvct[voff+1] )
@@ -232,7 +228,7 @@ void *Vertintml(void *argument)
 		      nhlevh   = nhlev + 1;
 
 		      vctsize = 2*nhlevh;
-		      vct = (double*) malloc(vctsize*sizeof(double));
+		      vct = (double*) Malloc(vctsize*sizeof(double));
 
 		      vlistChangeZaxisIndex(vlistID2, i, zaxisIDp);
 
@@ -268,6 +264,31 @@ void *Vertintml(void *argument)
 	}
     }
 
+  int psvarID = -1;
+  int linvertvct = FALSE;
+  if ( lhavevct && nvct && nvct%2 == 0 )
+    {
+      psvarID = vlist_get_psvarid(vlistID1, zaxisIDh);
+
+      for ( i = nvct/2+1; i < nvct; i++ )
+        if ( vct[i] > vct[i-1] ) break;
+
+      if ( i == nvct ) linvertvct = TRUE;
+    }
+
+  if ( cdoVerbose ) cdoPrint("linvertvct = %d", linvertvct);
+
+  if ( linvertvct )
+    {
+      double vctbuf[nvct];
+      memcpy(vctbuf, vct, nvct*sizeof(double));
+      for ( i = 0; i < nvct/2; i++ )
+        {
+          vct[nvct/2-1-i] = vctbuf[i];
+          vct[nvct-1-i] = vctbuf[i+nvct/2];
+        }
+    }
+
   int nvars = vlistNvars(vlistID1);
 
   int vars[nvars];
@@ -279,13 +300,13 @@ void *Vertintml(void *argument)
   int maxlev   = nhlevh > nplev ? nhlevh : nplev;
 
   if ( Extrapolate == 0 )
-    pnmiss = (int*) malloc(nplev*sizeof(int));
+    pnmiss = (int*) Malloc(nplev*sizeof(int));
 
   // check levels
   if ( zaxisIDh != -1 )
     {
       int nlev = zaxisInqSize(zaxisIDh);
-      if ( nlev != nhlev ) cdoAbort("Internal error, wrong numner of hybrid level!");
+      if ( nlev != nhlev ) cdoAbort("Internal error, wrong number of hybrid level!");
       double levels[nlev];
       zaxisInqLevels(zaxisIDh, levels);
 
@@ -301,10 +322,10 @@ void *Vertintml(void *argument)
 
   if ( zaxisIDh != -1 && gridsize > 0 )
     {
-      vert_index = (int*) malloc(gridsize*nplev*sizeof(int));
-      ps_prog    = (double*) malloc(gridsize*sizeof(double));
-      full_press = (double*) malloc(gridsize*nhlevf*sizeof(double));
-      half_press = (double*) malloc(gridsize*nhlevh*sizeof(double));
+      vert_index = (int*) Malloc(gridsize*nplev*sizeof(int));
+      ps_prog    = (double*) Malloc(gridsize*sizeof(double));
+      full_press = (double*) Malloc(gridsize*nhlevf*sizeof(double));
+      half_press = (double*) Malloc(gridsize*nhlevh*sizeof(double));
     }
   else
     cdoWarning("No 3D variable with hybrid sigma pressure coordinate found!");
@@ -423,17 +444,17 @@ void *Vertintml(void *argument)
 	cdoAbort("Spectral data unsupported!");
 
       if ( varID == gheightID )
-	vardata1[varID] = (double*) malloc(gridsize*(nlevel+1)*sizeof(double));
+	vardata1[varID] = (double*) Malloc(gridsize*(nlevel+1)*sizeof(double));
       else
-	vardata1[varID] = (double*) malloc(gridsize*nlevel*sizeof(double));
+	vardata1[varID] = (double*) Malloc(gridsize*nlevel*sizeof(double));
 
       /* if ( zaxisInqType(zaxisID) == ZAXIS_HYBRID && zaxisIDh != -1 && nlevel == nhlev ) */
       if ( zaxisID == zaxisIDh ||
 	   (zaxisInqType(zaxisID) == ZAXIS_HYBRID && zaxisIDh != -1 && (nlevel == nhlevh || nlevel == nhlevf)) )
 	{
 	  varinterp[varID] = TRUE;
-	  vardata2[varID]  = (double*) malloc(gridsize*nplev*sizeof(double));
-	  varnmiss[varID]  = (int*) malloc(maxlev*sizeof(int));
+	  vardata2[varID]  = (double*) Malloc(gridsize*nplev*sizeof(double));
+	  varnmiss[varID]  = (int*) Malloc(maxlev*sizeof(int));
 	  memset(varnmiss[varID], 0, maxlev*sizeof(int));
 	}
       else
@@ -443,7 +464,7 @@ void *Vertintml(void *argument)
 		       varID+1, paramstr, nlevel);
 	  varinterp[varID] = FALSE;
 	  vardata2[varID]  = vardata1[varID];
-	  varnmiss[varID]  = (int*) malloc(nlevel*sizeof(int));
+	  varnmiss[varID]  = (int*) Malloc(nlevel*sizeof(int));
 	}
     }
 
@@ -462,7 +483,7 @@ void *Vertintml(void *argument)
 
   if ( zaxisIDh != -1 && sgeopot_needed )
     {
-      sgeopot = (double*) malloc(gridsize*sizeof(double));
+      sgeopot = (double*) Malloc(gridsize*sizeof(double));
       if ( sgeopotID == -1 )
 	{
 	  if ( geopotID == -1 )
@@ -475,10 +496,12 @@ void *Vertintml(void *argument)
     }
 
   if ( zaxisIDh != -1 && gheightID != -1 && tempID == -1 )
-    cdoAbort("Temperature not found, needed to compute geopotheight!");
+    cdoAbort("Temperature not found, needed for vertical interpolation of geopotheight!");
 
   presID = lnpsID;
-  if ( zaxisIDh != -1 && lnpsID == -1 )
+  if ( psvarID != -1 ) presID = psvarID;
+
+  if ( zaxisIDh != -1 && presID == -1 )
     {
       if ( psID == -1 )
 	cdoAbort("%s not found!", var_stdname(surface_air_pressure));
@@ -488,17 +511,30 @@ void *Vertintml(void *argument)
 
   if ( cdoVerbose )
     {
+      vlistInqVarName(vlistID1, presID, varname);
       if ( presID == lnpsID )
-	cdoPrint("using LOG(%s)", var_stdname(surface_air_pressure));      
+        cdoPrint("using LOG(%s) from %s", var_stdname(surface_air_pressure), varname);
       else
-	cdoPrint("using %s", var_stdname(surface_air_pressure));
+        cdoPrint("using %s from %s", var_stdname(surface_air_pressure), varname);
+    }
+
+  // check VCT
+  if ( zaxisIDh != -1 )
+    {
+      double suma = 0, sumb = 0;
+      for ( i = 0; i < nhlevh; i++ )
+        {
+          suma += vct[i];
+          sumb += vct[i+nhlevh];
+        }
+      if ( !(suma>0&&sumb>0) ) cdoWarning("VCT is empty!");
     }
 
   int streamID2 = streamOpenWrite(cdoStreamName(1), cdoFiletype());
 
   streamDefVlist(streamID2, vlistID2);
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
     {
       for ( varID = 0; varID < nvars; ++varID ) vars[varID] = FALSE;
@@ -520,6 +556,9 @@ void *Vertintml(void *argument)
 	      printf("levelID %d\n", levelID);
 	    }
 	  */
+          if ( linvertvct && zaxisIDh != -1 && zaxisID == zaxisIDh )
+            levelID = nlevel-1-levelID;
+
 	  offset   = gridsize*levelID;
 	  single1  = vardata1[varID] + offset;
 
@@ -541,21 +580,21 @@ void *Vertintml(void *argument)
 		{
 		  minmaxval(gridsize, sgeopot, NULL, &minval, &maxval);
 		  if ( minval < MIN_FIS || maxval > MAX_FIS )
-		    cdoWarning("Surface geopotential out of range (min=%g max=%g)!", minval, maxval);
+		    cdoWarning("Surface geopotential out of range (min=%g max=%g) [timestep:%d]!", minval, maxval, tsID+1);
 		  if ( gridsize > 1 && minval >= 0 && maxval <= 9000 )
-		    cdoWarning("Surface geopotential has an unexpected range (min=%g max=%g)!", minval, maxval);
+		    cdoWarning("Surface geopotential has an unexpected range (min=%g max=%g) [timestep:%d]!", minval, maxval, tsID+1);
 		}
 	    }
 
-	  if ( lnpsID != -1 )
+	  if ( presID == lnpsID )
 	    for ( i = 0; i < gridsize; i++ ) ps_prog[i] = exp(vardata1[lnpsID][i]);
-	  else if ( psID != -1 )
-	    memcpy(ps_prog, vardata1[psID], gridsize*sizeof(double));
+	  else if ( presID != -1 )
+	    memcpy(ps_prog, vardata1[presID], gridsize*sizeof(double));
 
 	  /* check range of ps_prog */
 	  minmaxval(gridsize, ps_prog, NULL, &minval, &maxval);
 	  if ( minval < MIN_PS || maxval > MAX_PS )
-	    cdoWarning("Surface pressure out of range (min=%g max=%g)!", minval, maxval);
+	    cdoWarning("Surface pressure out of range (min=%g max=%g) [timestep:%d]!", minval, maxval, tsID+1);
 
 
 	  presh(full_press, half_press, vct, ps_prog, nhlevf, gridsize);
@@ -685,24 +724,24 @@ void *Vertintml(void *argument)
 
   for ( varID = 0; varID < nvars; varID++ )
     {
-      free(varnmiss[varID]);
-      free(vardata1[varID]);
-      if ( varinterp[varID] ) free(vardata2[varID]);
+      Free(varnmiss[varID]);
+      Free(vardata1[varID]);
+      if ( varinterp[varID] ) Free(vardata2[varID]);
     }
 
-  if ( pnmiss     ) free(pnmiss);
+  if ( pnmiss     ) Free(pnmiss);
 
-  if ( sgeopot    ) free(sgeopot);
-  if ( ps_prog    ) free(ps_prog);
-  if ( vert_index ) free(vert_index);
-  if ( full_press ) free(full_press);
-  if ( half_press ) free(half_press);
-  if ( vct        ) free(vct);
-  if ( rvct       ) free(rvct);
+  if ( sgeopot    ) Free(sgeopot);
+  if ( ps_prog    ) Free(ps_prog);
+  if ( vert_index ) Free(vert_index);
+  if ( full_press ) Free(full_press);
+  if ( half_press ) Free(half_press);
+  if ( vct        ) Free(vct);
+  if ( rvct       ) Free(rvct);
 
   listDelete(flist);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
