@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -84,8 +84,8 @@ void yar_store_link_cnsrv(remapvars_t *rv, long add1, long add2, double weight)
   if ( rv->num_links >= rv->max_links )
     resize_remap_vars(rv, rv->resize_increment);
 
-  rv->src_grid_add[nlink] = add1;
-  rv->tgt_grid_add[nlink] = add2;
+  rv->src_cell_add[nlink] = add1;
+  rv->tgt_cell_add[nlink] = add2;
 
   rv->wts[nlink] = weight;
 }
@@ -139,76 +139,38 @@ void set_source_data(double * source_data, double init_value,
 }
 
 
+/*
+  This routine stores the address and weight for four links associated with one destination
+  point in the appropriate address and weight arrays and resizes those arrays if necessary.
+*/
 static
-long find_ij_weights(double plon, double plat, double *restrict src_lats, double *restrict src_lons, double *ig, double *jg)
+void store_link_bilin(remapvars_t *rv, int dst_add, int src_add[4], double weights[4])
 {
-#define  THREE    3.0
-#define  HALF     0.5
-  long    Max_Iter = 100;
-  double  converge = 1.e-10;            /* Convergence criterion */
-  long iter;                     /*  iteration counters   */
-  double iguess, jguess;         /*  current guess for bilinear coordinate  */
-  double deli, delj;             /*  corrections to i,j                     */
-  double dth1, dth2, dth3;       /*  some latitude  differences             */
-  double dph1, dph2, dph3;       /*  some longitude differences             */
-  double dthp, dphp;             /*  difference between point and sw corner */
-  double mat1, mat2, mat3, mat4; /*  matrix elements                        */
-  double determinant;            /*  matrix determinant                     */
+  /*
+    Input variables:
+    int dst_add       ! address on destination grid
+    int src_add[4]    ! addresses on source grid
+    double weights[4] ! array of remapping weights for these links
+  */
+  /* link index */
+  long nlink = rv->num_links;
+  /*
+     Increment number of links and check to see if remap arrays need
+     to be increased to accomodate the new link. Then store the link.
+  */
+  rv->num_links += 4;
 
-  /* Iterate to find i,j for bilinear approximation  */
+  if ( rv->num_links >= rv->max_links ) 
+    resize_remap_vars(rv, rv->resize_increment);
 
-  dth1 = src_lats[1] - src_lats[0];
-  dth2 = src_lats[3] - src_lats[0];
-  dth3 = src_lats[2] - src_lats[1] - dth2;
-
-  dph1 = src_lons[1] - src_lons[0];
-  dph2 = src_lons[3] - src_lons[0];
-  dph3 = src_lons[2] - src_lons[1];
-
-  if ( dph1 >  THREE*PIH ) dph1 -= PI2;
-  if ( dph2 >  THREE*PIH ) dph2 -= PI2;
-  if ( dph3 >  THREE*PIH ) dph3 -= PI2;
-  if ( dph1 < -THREE*PIH ) dph1 += PI2;
-  if ( dph2 < -THREE*PIH ) dph2 += PI2;
-  if ( dph3 < -THREE*PIH ) dph3 += PI2;
-
-  dph3 = dph3 - dph2;
-
-  iguess = HALF;
-  jguess = HALF;
-
-  for ( iter = 0; iter < Max_Iter; ++iter )
+  for ( long n = 0; n < 4; ++n )
     {
-      dthp = plat - src_lats[0] - dth1*iguess - dth2*jguess - dth3*iguess*jguess;
-      dphp = plon - src_lons[0];
-      
-      if ( dphp >  THREE*PIH ) dphp -= PI2;
-      if ( dphp < -THREE*PIH ) dphp += PI2;
-
-      dphp = dphp - dph1*iguess - dph2*jguess - dph3*iguess*jguess;
-
-      mat1 = dth1 + dth3*jguess;
-      mat2 = dth2 + dth3*iguess;
-      mat3 = dph1 + dph3*jguess;
-      mat4 = dph2 + dph3*iguess;
-
-      determinant = mat1*mat4 - mat2*mat3;
-
-      deli = (dthp*mat4 - dphp*mat2)/determinant;
-      delj = (dphp*mat1 - dthp*mat3)/determinant;
-
-      if ( fabs(deli) < converge && fabs(delj) < converge ) break;
-
-      iguess += deli;
-      jguess += delj;
+      rv->src_cell_add[nlink+n] = src_add[n];
+      rv->tgt_cell_add[nlink+n] = dst_add;
+      rv->wts         [nlink+n] = weights[n];
     }
 
-  *ig = iguess;
-  *jg = jguess;
-
-  return (iter);
-}
-
+} /* store_link_bilin */
 
 void yar_remap_bil(field_t *field1, field_t *field2)
 {
@@ -255,15 +217,15 @@ void yar_remap_bil(field_t *field1, field_t *field2)
   nlonIn = gridInqXsize(gridIDin);
   nlatIn = gridInqYsize(gridIDin);
   gridsize1 = gridInqSize(gridIDin);
-  lonIn = malloc(nlonIn*sizeof(double));
-  latIn = malloc(nlatIn*sizeof(double));
+  lonIn = (double*) Malloc(nlonIn*sizeof(double));
+  latIn = (double*) Malloc(nlatIn*sizeof(double));
   gridInqXvals(gridIDin, lonIn);
   gridInqYvals(gridIDin, latIn);
   for ( int i = 0; i < nlonIn; ++i ) lonIn[i] *= DEG2RAD;
   for ( int i = 0; i < nlatIn; ++i ) latIn[i] *= DEG2RAD;
 
-  xlonIn = malloc((nlonIn+1)*sizeof(double));
-  xlatIn = malloc((nlatIn+1)*sizeof(double));
+  xlonIn = (double*) Malloc((nlonIn+1)*sizeof(double));
+  xlatIn = (double*) Malloc((nlatIn+1)*sizeof(double));
   gridInqXvals(gridIDin, xlonIn);
   gridInqYvals(gridIDin, xlatIn);
   dxIn = xlonIn[1] - xlonIn[0];
@@ -280,15 +242,15 @@ void yar_remap_bil(field_t *field1, field_t *field2)
   nlonOut = gridInqXsize(gridIDout);
   nlatOut = gridInqYsize(gridIDout);
   gridsize2 = gridInqSize(gridIDout);
-  lonOut = malloc(nlonOut*sizeof(double));
-  latOut = malloc(nlatOut*sizeof(double));
+  lonOut = (double*) Malloc(nlonOut*sizeof(double));
+  latOut = (double*) Malloc(nlatOut*sizeof(double));
   gridInqXvals(gridIDout, lonOut);
   gridInqYvals(gridIDout, latOut);
   for ( int i = 0; i < nlonOut; ++i ) lonOut[i] *= DEG2RAD;
   for ( int i = 0; i < nlatOut; ++i ) latOut[i] *= DEG2RAD;
 
-  xlonOut = malloc((nlonOut+1)*sizeof(double));
-  xlatOut = malloc((nlatOut+1)*sizeof(double));
+  xlonOut = (double*) Malloc((nlonOut+1)*sizeof(double));
+  xlatOut = (double*) Malloc((nlatOut+1)*sizeof(double));
   gridInqXvals(gridIDout, xlonOut);
   gridInqYvals(gridIDout, xlatOut);
   dxOut = xlonOut[1] - xlonOut[0];
@@ -352,6 +314,12 @@ void yar_remap_bil(field_t *field1, field_t *field2)
 
   search = bucket_search_new(source_grid);
   /*
+  search = sphere_part_search_new(source_grid);
+  printf("search %p\n", search);
+  printf("search->vtable %p\n", search->vtable);
+  printf("search->vtable->do_point_search_p3 %d\n", search->vtable->do_point_search_p3);
+  */
+  /*
 
   printf("total_num_dependencies: %d\n", get_total_num_dependencies(deps));
 
@@ -416,7 +384,7 @@ void yar_remap_bil(field_t *field1, field_t *field2)
 	    }
 
 	  // try it with do_point_search_p3
-	  if ( find_ij_weights(plon, plat, src_lats, src_lons, &iguess, &jguess) < 100 )
+	  if ( find_ij_weights(plon, plat, src_lats, src_lons, &iguess, &jguess) )
 	    {
 
 	      wgts[0] = (1.-iguess)*(1.-jguess);
@@ -436,7 +404,7 @@ void yar_remap_bil(field_t *field1, field_t *field2)
 
   if ( cdoTimer ) timer_start(timer_yar_remap);
   yar_remap(array2, missval, gridInqSize(gridIDout), remap.vars.num_links, remap.vars.wts,
-	    remap.vars.num_wts, remap.vars.tgt_grid_add, remap.vars.src_grid_add, array1);
+	    remap.vars.num_wts, remap.vars.tgt_cell_add, remap.vars.src_cell_add, array1);
   if ( cdoTimer ) timer_stop(timer_yar_remap);
 
   nmiss = 0;
@@ -445,7 +413,7 @@ void yar_remap_bil(field_t *field1, field_t *field2)
 
   field2->nmiss = nmiss;
 
-  // if (array) free(array);
+  // if (array) Free(array);
   //free(lonIn);
   //free(latIn);
   //free(lonOut);
@@ -498,12 +466,12 @@ void yar_remap_con(field_t *field1, field_t *field2)
   nlonIn = gridInqXsize(gridIDin);
   nlatIn = gridInqYsize(gridIDin);
   gridsize1 = gridInqSize(gridIDin);
-  lonIn = malloc((nlonIn+1)*sizeof(double));
-  latIn = malloc((nlatIn+1)*sizeof(double));
+  lonIn = (double*) Malloc((nlonIn+1)*sizeof(double));
+  latIn = (double*) Malloc((nlatIn+1)*sizeof(double));
   gridInqXvals(gridIDin, lonIn);
   gridInqYvals(gridIDin, latIn);
-  xlonIn = malloc((nlonIn)*sizeof(double));
-  xlatIn = malloc((nlatIn)*sizeof(double));
+  xlonIn = (double*) Malloc((nlonIn)*sizeof(double));
+  xlatIn = (double*) Malloc((nlatIn)*sizeof(double));
   gridInqXvals(gridIDin, xlonIn);
   gridInqYvals(gridIDin, xlatIn);
   dxIn = lonIn[1] - lonIn[0];
@@ -520,12 +488,12 @@ void yar_remap_con(field_t *field1, field_t *field2)
   nlonOut = gridInqXsize(gridIDout);
   nlatOut = gridInqYsize(gridIDout);
   gridsize2 = gridInqSize(gridIDout);
-  lonOut = malloc((nlonOut+1)*sizeof(double));
-  latOut = malloc((nlatOut+1)*sizeof(double));
+  lonOut = (double*) Malloc((nlonOut+1)*sizeof(double));
+  latOut = (double*) Malloc((nlatOut+1)*sizeof(double));
   gridInqXvals(gridIDout, lonOut);
   gridInqYvals(gridIDout, latOut);
-  xlonOut = malloc((nlonOut+1)*sizeof(double));
-  xlatOut = malloc((nlatOut+1)*sizeof(double));
+  xlonOut = (double*) Malloc((nlonOut+1)*sizeof(double));
+  xlatOut = (double*) Malloc((nlatOut+1)*sizeof(double));
   gridInqXvals(gridIDout, xlonOut);
   gridInqYvals(gridIDout, xlatOut);
   dxOut = lonOut[1] - lonOut[0];
@@ -605,20 +573,20 @@ void yar_remap_con(field_t *field1, field_t *field2)
   double const epsilon = 1.0e-10; // relative precision 
 
   double *weight;
-  weight = malloc(gridsize1*sizeof(double));
+  weight = (double*) Malloc(gridsize1*sizeof(double));
 
   double tgt_area;
   double *area;
-  area = malloc(gridsize1*sizeof(double));
+  area = (double*) Malloc(gridsize1*sizeof(double));
 
   struct grid_cell *SourceCell;
-  SourceCell = malloc (gridsize1  * sizeof(*SourceCell) );
+  SourceCell = (struct grid_cell*) Malloc(gridsize1  * sizeof(struct grid_cell));
 
   for ( int n = 0; n <  gridsize1; n++ ) {
     SourceCell[n].num_corners   = 4;
     SourceCell[n].edge_type     = quad_type;
-    SourceCell[n].coordinates_x = malloc ( 4 * sizeof(SourceCell[n].coordinates_x[0]) );
-    SourceCell[n].coordinates_y = malloc ( 4 * sizeof(SourceCell[n].coordinates_y[0]) );
+    SourceCell[n].coordinates_x = (double*) Malloc( 4 * sizeof(double));
+    SourceCell[n].coordinates_y = (double*) Malloc( 4 * sizeof(double));
   }
 
   struct grid_cell  TargetCell;
@@ -626,8 +594,8 @@ void yar_remap_con(field_t *field1, field_t *field2)
   TargetCell.num_corners   = 4;
   TargetCell.edge_type     = quad_type;
 
-  TargetCell.coordinates_x = malloc ( 4 * sizeof(*TargetCell.coordinates_x) );
-  TargetCell.coordinates_y = malloc ( 4 * sizeof(*TargetCell.coordinates_y) );
+  TargetCell.coordinates_x = (double*) Malloc( 4 * sizeof(double));
+  TargetCell.coordinates_y = (double*) Malloc( 4 * sizeof(double));
 
   unsigned const * curr_deps;
   //struct polygons polygons;
@@ -740,7 +708,7 @@ void yar_remap_con(field_t *field1, field_t *field2)
 
   if ( cdoTimer ) timer_start(timer_yar_remap);
   yar_remap(array2, missval, gridInqSize(gridIDout), remap.vars.num_links, remap.vars.wts,
-	    remap.vars.num_wts, remap.vars.tgt_grid_add, remap.vars.src_grid_add, array1);
+	    remap.vars.num_wts, remap.vars.tgt_cell_add, remap.vars.src_cell_add, array1);
   if ( cdoTimer ) timer_stop(timer_yar_remap);
 
   nmiss = 0;
@@ -757,8 +725,8 @@ void yar_remap_con(field_t *field1, field_t *field2)
   delete_grid(source_grid);
   delete_grid(target_grid);
 
-  free(weight);
-  free(area);
+  Free(weight);
+  Free(area);
 
   //free(lonIn);
   //free(latIn);
@@ -838,10 +806,10 @@ void *YAR(void *argument)
   streamDefVlist(streamID2, vlistID2);
 
   gridsize = vlistGridsizeMax(vlistID1);
-  array1   = malloc(gridsize*sizeof(double));
+  array1   = (double*) Malloc(gridsize*sizeof(double));
 
   gridsize = gridInqSize(gridID2);
-  array2   = malloc(gridsize*sizeof(double));
+  array2   = (double*) Malloc(gridsize*sizeof(double));
 
   tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID1, tsID)) )
@@ -884,10 +852,10 @@ void *YAR(void *argument)
   streamClose(streamID2);
   streamClose(streamID1);
 
-  if ( array2 ) free(array2);
-  if ( array1 ) free(array1);
+  if ( array2 ) Free(array2);
+  if ( array1 ) Free(array1);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
