@@ -24,13 +24,13 @@
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
+#include "calendar.h"
 #include "pstream.h"
-#include "percentiles.h"
+#include "percentiles_hist.h"
 
 
 #define NDAY 373
 
-static
 int getmonthday(int date)
 {
   int year, month, day;
@@ -41,14 +41,11 @@ int getmonthday(int date)
 
 void *Ydrunpctl(void *argument)
 {
-  int gridsize;
   int varID;
   int recID;
   int gridID;
   int nrecs;
   int levelID;
-  int tsID;
-  int otsID;
   int inp, its;
   int nmiss;
   int nlevels;
@@ -59,7 +56,6 @@ void *Ydrunpctl(void *argument)
   int vdates2[NDAY] /*, vtimes2[NDAY]*/;
   int nsets[NDAY];
   int year, month, day, dayoy;
-  field_t field;
   HISTOGRAM_SET *hsets[NDAY];
     
   cdoInitialize(argument);
@@ -68,10 +64,8 @@ void *Ydrunpctl(void *argument)
   operatorInputArg("percentile number, number of timesteps");
   operatorCheckArgc(2);
   double pn  = parameter2double(operatorArgv()[0]);
+  percentile_check_number(pn);
   int ndates = parameter2int(operatorArgv()[1]);
-
-  if ( !(pn > 0 && pn < 100) )
-    cdoAbort("Illegal argument: percentile number %g is not in the range 0..100!", pn);
   
   for ( dayoy = 0; dayoy < NDAY; dayoy++ )
     {
@@ -111,23 +105,25 @@ void *Ydrunpctl(void *argument)
   int nvars    = vlistNvars(vlistID1);
   int nrecords = vlistNrecs(vlistID1);
 
-  int *recVarID   = (int*) malloc(nrecords*sizeof(int));
-  int *recLevelID = (int*) malloc(nrecords*sizeof(int));
+  int *recVarID   = (int*) Malloc(nrecords*sizeof(int));
+  int *recLevelID = (int*) Malloc(nrecords*sizeof(int));
 
-  gridsize = vlistGridsizeMax(vlistID1);
+  int gridsize = vlistGridsizeMax(vlistID1);
+
+  field_t field;
   field_init(&field);
-  field.ptr = (double*) malloc(gridsize*sizeof(double));
+  field.ptr = (double*) Malloc(gridsize*sizeof(double));
 
-  datetime = (datetime_t*) malloc((ndates+1)*sizeof(datetime_t));
+  datetime = (datetime_t*) Malloc((ndates+1)*sizeof(datetime_t));
   
-  vars1 = (field_t ***) malloc((ndates+1)*sizeof(field_t **));
+  vars1 = (field_t ***) Malloc((ndates+1)*sizeof(field_t **));
   
   for ( its = 0; its < ndates; its++ )
     {
       vars1[its] = field_malloc(vlistID1, FIELD_PTR);
     }
 
-  tsID = 0;
+  int tsID = 0;
   while ( (nrecs = streamInqTimestep(streamID2, tsID)) )
     {
       if ( nrecs != streamInqTimestep(streamID3, tsID) )
@@ -136,7 +132,7 @@ void *Ydrunpctl(void *argument)
       vdate = taxisInqVdate(taxisID2);
       vtime = taxisInqVtime(taxisID2);
       
-      if ( vdate != taxisInqVdate(taxisID3) || vtime != taxisInqVtime(taxisID3) )
+      if ( vdate != taxisInqVdate(taxisID3) )
         cdoAbort("Verification dates at time step %d of %s and %s differ!", tsID+1, cdoStreamName(1)->args, cdoStreamName(2)->args);
         
       if ( cdoVerbose ) cdoPrint("process timestep: %d %d %d", tsID+1, vdate, vtime);
@@ -289,16 +285,14 @@ void *Ydrunpctl(void *argument)
 	vdates1[dayoy] = cdiEncodeDate(outyear, month, day);
       }
   */
-  otsID = 0;
+  int otsID = 0;
   for ( dayoy = 0; dayoy < NDAY; dayoy++ )
     if ( nsets[dayoy] )
       {
         if ( getmonthday(vdates1[dayoy]) != getmonthday(vdates2[dayoy]) )
-          cdoAbort("Verification dates for day %d of %s, %s and %s are different!", dayoy, cdoStreamName(1)->args, cdoStreamName(2)->args, cdoStreamName(3)->args);
-        /*
-        if ( vtimes1[dayoy] != vtimes2[dayoy] )
-          cdoAbort("Verification times for day %d of %s, %s and %s are different!", dayoy, cdoStreamName(1)->args, cdoStreamName(2)->args, cdoStreamName(3)->args);
-        */
+          cdoAbort("Verification dates for day %d of %s, %s and %s are different!",
+                   dayoy, cdoStreamName(0)->args, cdoStreamName(1)->args);
+
 	for ( varID = 0; varID < nvars; varID++ )
 	  {
 	    if ( vlistInqVarTsteptype(vlistID1, varID) == TSTEP_CONSTANT ) continue;
@@ -333,12 +327,12 @@ void *Ydrunpctl(void *argument)
 	{
 	  nlevels = zaxisInqSize(vlistInqVarZaxis(vlistID1, varID));
 	  for ( levelID = 0; levelID < nlevels; levelID++ )
-	    free(vars1[its][varID][levelID].ptr);
-	  free(vars1[its][varID]);
+	    Free(vars1[its][varID][levelID].ptr);
+	  Free(vars1[its][varID]);
 	}
-      free(vars1[its]);
+      Free(vars1[its]);
     }
-  free(vars1);
+  Free(vars1);
 
   for ( dayoy = 0; dayoy < NDAY; dayoy++ )
     {
@@ -349,10 +343,10 @@ void *Ydrunpctl(void *argument)
 	}
     }
 
-  if ( field.ptr ) free(field.ptr);
+  if ( field.ptr ) Free(field.ptr);
   
-  if ( recVarID   ) free(recVarID);
-  if ( recLevelID ) free(recLevelID);
+  if ( recVarID   ) Free(recVarID);
+  if ( recLevelID ) Free(recLevelID);
 
   streamClose(streamID4);
   streamClose(streamID3);
@@ -361,5 +355,5 @@ void *Ydrunpctl(void *argument)
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
