@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,10 @@
       Ymonarith  ymonsub         Subtract multi-year monthly time series
       Ymonarith  ymonmul         Multiply multi-year monthly time series
       Ymonarith  ymondiv         Divide multi-year monthly time series
+      Ymonarith  yseasadd        Add multi-year seasonal time series
+      Ymonarith  yseassub        Subtract multi-year seasonal time series
+      Ymonarith  yseasmul        Multiply multi-year seasonal time series
+      Ymonarith  yseasdiv        Divide multi-year seasonal time series
 */
 
 #include <cdi.h>
@@ -30,62 +34,67 @@
 #include "pstream.h"
 
 
-#define  MAX_MON    20
+#define  MAX_MON    12
+
 
 void *Ymonarith(void *argument)
 {
-  int operatorID;
-  int operfunc;
-  int streamID1, streamID2, streamID3;
-  int gridsize;
+  enum {MONTHLY, SEASONAL};
   int nrecs, nvars, nlev, recID;
   int tsID;
   int varID, levelID;
   int offset;
-  int vlistID1, vlistID2, vlistID3;
-  int taxisID1, taxisID2, taxisID3;
+  int nmiss;
   int vdate, year, mon, day;
   field_t field1, field2;
   int **varnmiss2[MAX_MON];
   double **vardata2[MAX_MON];
+  const char *seas_name[4];
 
   cdoInitialize(argument);
 
-  cdoOperatorAdd("ymonadd", func_add, 0, NULL);
-  cdoOperatorAdd("ymonsub", func_sub, 0, NULL);
-  cdoOperatorAdd("ymonmul", func_mul, 0, NULL);
-  cdoOperatorAdd("ymondiv", func_div, 0, NULL);
+  cdoOperatorAdd("ymonadd",  func_add, MONTHLY, NULL);
+  cdoOperatorAdd("ymonsub",  func_sub, MONTHLY, NULL);
+  cdoOperatorAdd("ymonmul",  func_mul, MONTHLY, NULL);
+  cdoOperatorAdd("ymondiv",  func_div, MONTHLY, NULL);
+  cdoOperatorAdd("yseasadd", func_add, SEASONAL, NULL);
+  cdoOperatorAdd("yseassub", func_sub, SEASONAL, NULL);
+  cdoOperatorAdd("yseasmul", func_mul, SEASONAL, NULL);
+  cdoOperatorAdd("yseasdiv", func_div, SEASONAL, NULL);
 
-  operatorID = cdoOperatorID();
-  operfunc = cdoOperatorF1(operatorID);
+  int operatorID = cdoOperatorID();
+  int operfunc = cdoOperatorF1(operatorID);
+  int opertype = cdoOperatorF2(operatorID);
 
-  streamID1 = streamOpenRead(cdoStreamName(0));
-  streamID2 = streamOpenRead(cdoStreamName(1));
+  int streamID1 = streamOpenRead(cdoStreamName(0));
+  int streamID2 = streamOpenRead(cdoStreamName(1));
 
-  vlistID1 = streamInqVlist(streamID1);
-  vlistID2 = streamInqVlist(streamID2);
-  vlistID3 = vlistDuplicate(vlistID1);
+  int vlistID1 = streamInqVlist(streamID1);
+  int vlistID2 = streamInqVlist(streamID2);
+  int vlistID3 = vlistDuplicate(vlistID1);
 
   vlistCompare(vlistID1, vlistID2, CMP_ALL);
 
-  gridsize = vlistGridsizeMax(vlistID1);
+  int gridsize = vlistGridsizeMax(vlistID1);
 
   field_init(&field1);
   field_init(&field2);
-  field1.ptr = malloc(gridsize*sizeof(double));
-  field2.ptr = malloc(gridsize*sizeof(double));
+  field1.ptr = (double*) Malloc(gridsize*sizeof(double));
+  field2.ptr = (double*) Malloc(gridsize*sizeof(double));
 
-  taxisID1 = vlistInqTaxis(vlistID1);
-  taxisID2 = vlistInqTaxis(vlistID2);
-  taxisID3 = taxisDuplicate(taxisID1);
+  int taxisID1 = vlistInqTaxis(vlistID1);
+  int taxisID2 = vlistInqTaxis(vlistID2);
+  int taxisID3 = taxisDuplicate(taxisID1);
   vlistDefTaxis(vlistID3, taxisID3);
 
-  streamID3 = streamOpenWrite(cdoStreamName(2), cdoFiletype());
+  int streamID3 = streamOpenWrite(cdoStreamName(2), cdoFiletype());
 
   streamDefVlist(streamID3, vlistID3);
 
   nvars  = vlistNvars(vlistID2);
 
+  if ( opertype == SEASONAL ) get_season_name(seas_name);
+
   for ( mon = 0; mon < MAX_MON ; mon++ ) vardata2[mon] = NULL;
 
   tsID = 0;
@@ -94,19 +103,28 @@ void *Ymonarith(void *argument)
       vdate = taxisInqVdate(taxisID2);
 
       cdiDecodeDate(vdate, &year, &mon, &day);
-      if ( mon < 0 || mon >= MAX_MON ) cdoAbort("Month %d out of range!", mon);
+      if ( mon < 1 || mon > MAX_MON ) cdoAbort("Month %d out of range!", mon);
+      mon--;
 
-      if ( vardata2[mon] != NULL ) cdoAbort("Month %d already allocatd!", mon);
+      if ( opertype == SEASONAL ) mon = month_to_season(mon+1);
 
-      vardata2[mon]  = malloc(nvars*sizeof(double *));
-      varnmiss2[mon] = malloc(nvars*sizeof(int *));
+      if ( vardata2[mon] != NULL )
+	{
+	  if ( opertype == SEASONAL )
+	    cdoAbort("Season %s already allocatd!", seas_name[mon]);
+	  else
+	    cdoAbort("Month %d already allocatd!", mon);
+	}
+
+      vardata2[mon]  = (double **) Malloc(nvars*sizeof(double *));
+      varnmiss2[mon] = (int **) Malloc(nvars*sizeof(int *));
 
       for ( varID = 0; varID < nvars; varID++ )
 	{
 	  gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
 	  nlev     = zaxisInqSize(vlistInqVarZaxis(vlistID2, varID));
-	  vardata2[mon][varID]  = malloc(nlev*gridsize*sizeof(double));
-	  varnmiss2[mon][varID] = malloc(nlev*sizeof(int));
+	  vardata2[mon][varID]  = (double*) Malloc(nlev*gridsize*sizeof(double));
+	  varnmiss2[mon][varID] = (int*) Malloc(nlev*sizeof(int));
 	}
 
       for ( recID = 0; recID < nrecs; recID++ )
@@ -116,8 +134,8 @@ void *Ymonarith(void *argument)
 	  gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
 	  offset   = gridsize*levelID;
 
-	  streamReadRecord(streamID2, vardata2[mon][varID]+offset, &field2.nmiss);
-	  varnmiss2[mon][varID][levelID] = field2.nmiss;
+	  streamReadRecord(streamID2, vardata2[mon][varID]+offset, &nmiss);
+	  varnmiss2[mon][varID][levelID] = nmiss;
 	}
 
       tsID++;
@@ -130,7 +148,18 @@ void *Ymonarith(void *argument)
       vdate = taxisInqVdate(taxisID1);
 
       cdiDecodeDate(vdate, &year, &mon, &day);
-      if ( mon < 0 || mon >= MAX_MON ) cdoAbort("Month %d out of range!", mon);
+      if ( mon < 1 || mon > MAX_MON ) cdoAbort("Month %d out of range!", mon);
+      mon--;
+
+      if ( opertype == SEASONAL ) mon = month_to_season(mon+1);
+
+      if ( vardata2[mon] == NULL )
+	{
+	  if ( opertype == SEASONAL )
+	    cdoAbort("Season %s not found!", seas_name[mon]);
+	  else
+	    cdoAbort("Month %d not found!", mon);
+	}
 
       taxisCopyTimestep(taxisID3, taxisID1);
 
@@ -139,24 +168,23 @@ void *Ymonarith(void *argument)
       for ( recID = 0; recID < nrecs; recID++ )
 	{
 	  streamInqRecord(streamID1, &varID, &levelID);
-	  streamReadRecord(streamID1, field1.ptr, &field1.nmiss);
-
-	  gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
-	  offset   = gridsize*levelID;
-	  if ( vardata2[mon] == NULL ) cdoAbort("Month %d not found!", mon);
-	  memcpy(field2.ptr, vardata2[mon][varID]+offset, gridsize*sizeof(double));
-	  field2.nmiss = varnmiss2[mon][varID][levelID];
-
+	  streamReadRecord(streamID1, field1.ptr, &nmiss);
+          field1.nmiss = (size_t) nmiss;
 	  field1.grid    = vlistInqVarGrid(vlistID1, varID);
 	  field1.missval = vlistInqVarMissval(vlistID1, varID);
+          
+	  gridsize = gridInqSize(vlistInqVarGrid(vlistID2, varID));
+	  offset   = gridsize*levelID;
 
+	  memcpy(field2.ptr, vardata2[mon][varID]+offset, gridsize*sizeof(double));
+	  field2.nmiss = varnmiss2[mon][varID][levelID];
 	  field2.grid    = vlistInqVarGrid(vlistID2, varID);
 	  field2.missval = vlistInqVarMissval(vlistID2, varID);
 
 	  farfun(&field1, field2, operfunc);
 
 	  streamDefRecord(streamID3, varID, levelID);
-	  streamWriteRecord(streamID3, field1.ptr, field1.nmiss);
+	  streamWriteRecord(streamID3, field1.ptr, (int)field1.nmiss);
 	}
       tsID++;
     }
@@ -170,18 +198,18 @@ void *Ymonarith(void *argument)
       {
 	for ( varID = 0; varID < nvars; varID++ )
 	  {
-	    free(vardata2[mon][varID]);
-	    free(varnmiss2[mon][varID]);
+	    Free(vardata2[mon][varID]);
+	    Free(varnmiss2[mon][varID]);
 	  }
 
-	free(vardata2[mon]);
-	free(varnmiss2[mon]);
+        Free(vardata2[mon]);
+        Free(varnmiss2[mon]);
       }
 
-  if ( field1.ptr ) free(field1.ptr);
-  if ( field2.ptr ) free(field2.ptr);
+  if ( field1.ptr ) Free(field1.ptr);
+  if ( field2.ptr ) Free(field2.ptr);
 
   cdoFinish();
 
-  return (0);
+  return 0;
 }
