@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2015 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -24,9 +24,13 @@
 
 #include <assert.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <stdbool.h>
 #include <string.h>
 #include <math.h>
 #include <float.h>
+#include <ctype.h>
+#include <stdarg.h>
 
 #include "compare.h"
 #include "timebase.h"
@@ -54,7 +58,7 @@ char *strdup(const char *s);
 ({					      	  \
    const char *__old = (s);			  \
    size_t __len = strlen(__old) + 1;		  \
-   char *__new = malloc(__len);	  \
+   char *__new = Malloc(__len);	  \
    (char *) memcpy(__new, __old, __len);	  \
 })
 */
@@ -117,13 +121,9 @@ void time2str(int time, char *timestr, int maxlen);
 const char * tunit2str(int tunits);
 const char * calendar2str(int calendar);
 
-int     days_per_month(int calendar, int year, int month);
-int     days_per_year(int calendar, int year);
-int     calendar_dpy(int calendar);
-
 void    defineGrid(const char *gridarg);
-void    defineInstitution(char *instarg);
-int     defineTable(char *tablearg);
+void    defineInstitution(const char *instarg);
+int     defineTable(const char *tablearg);
 
 void    cdolog(const char *prompt, double cputime);
 void    cdologs(int noper);
@@ -138,11 +138,9 @@ int  cdf_openread(const char *filename);
 
 void printFiletype(int streamID, int vlistID);
 
-void job_submit(const char *expname, const char *jobfilename, const char *jobname, const char *tmppath, const char *ftppath);
-
 void minmaxval(long nvals, double *array, int *imiss, double *minval, double *maxval);
 
-off_t filesize(const char *filename);
+off_t fileSize(const char *restrict filename);
 
 char *expand_filename(const char *string);
 
@@ -150,5 +148,24 @@ double parameter2double(const char *string);
 int    parameter2int(const char *string);
 int    parameter2intlist(const char *string);
 
+int referenceToGrid(int gridID1);
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+void cdiDefTableID(int tableID);
+
+void gridGenXvals(int xsize, double xfirst, double xlast, double xinc, double *xvals);
+void gridGenYvals(int gridtype, int ysize, double yfirst, double ylast, double yinc, double *yvals);
+
+void gaussaw(double *restrict pa, double *restrict pw, size_t nlat);
+
+int qu2reg3_double(double *pfield, int *kpoint, int klat, int klon,
+		   double msval, int *kret, int omisng, int operio, int oveggy);
+
+#if defined (__cplusplus)
+}
+#endif
 
 #endif  /* _CDO_INT_H */
