@@ -1,13 +1,16 @@
-#include <stdio.h>
-#include <stdlib.h>
+#if defined(HAVE_CONFIG_H)
+#  include "config.h"
+#endif
+
 #include <stdarg.h>
 #include <errno.h>
 #include <cdi.h>
 #include "cdo.h"
+#include "cdo_int.h"
 #include "process.h"
 #include "error.h"
 
-#include "config.h"
+void pstreamCloseAll(void);
 
 void cdiOpenError(int cdiErrno, const char *fmt, const char *path)
 {	
@@ -30,50 +33,38 @@ void cdiOpenError(int cdiErrno, const char *fmt, const char *path)
       switch (filetype)
 	{
 	case FILETYPE_GRB:
-	  {
-	    break;
-	  }
+          break;
 	case FILETYPE_GRB2:
-	  {
-	    fprintf(stderr, "To create a CDO application with GRIB2 support use: ./configure --with-netcdf=<GRIB_API root directory> ...\n");
-	    break;
-	  }
+          fprintf(stderr, "To create a CDO application with GRIB2 support use: ./configure --with-netcdf=<GRIB_API root directory> ...\n");
+          break;
 	case FILETYPE_SRV:
-	  {
-	    break;
-	  }
+          break;
 	case FILETYPE_EXT:
-	  {
-	    break;
-	  }
+          break;
 	case FILETYPE_IEG:
-	  {
-	    break;
-	  }
+          break;
 	case FILETYPE_NC:
 	case FILETYPE_NC2:
 	case FILETYPE_NC4:
 	case FILETYPE_NC4C:
-	  {
-	    const char *ncv = (filetype == FILETYPE_NC4 || filetype == FILETYPE_NC4C) ? "4" : ((filetype == FILETYPE_NC2) ? "2" : "");
+          {
+            const char *ncv = (filetype == FILETYPE_NC4 || filetype == FILETYPE_NC4C) ? "4" : ((filetype == FILETYPE_NC2) ? "2" : "");
 #if defined HAVE_LIBNETCDF
-	    fprintf(stderr, "CDO was build with a netCDF version which doesn't support netCDF%s data!\n", ncv);
+            fprintf(stderr, "CDO was build with a NetCDF version which doesn't support NetCDF%s data!\n", ncv);
 #else
-	    fprintf(stderr, "To create a CDO application with netCDF%s support use: ./configure --with-netcdf=<netCDF%s root directory> ...\n", ncv, ncv);
+            fprintf(stderr, "To create a CDO application with NetCDF%s support use: ./configure --with-netcdf=<NetCDF%s root directory> ...\n", ncv, ncv);
 #endif
-	    break;
-	  }
+            break;
+          }
 	default:
-	  {
-	    break;
-	  }
+          break;
 	}
     }
   
+  if ( _ExitOnError ) pstreamCloseAll();
   if ( _ExitOnError ) exit(EXIT_FAILURE);
 }
 
-void pstreamCloseAll(void);
 void cdoAbort(const char *fmt, ...)
 {
   va_list args;
