@@ -1,6 +1,10 @@
 #ifndef _GRID_H
 #define _GRID_H
 
+#if defined(HAVE_CONFIG_H)
+#include "config.h"
+#endif
+
 #ifndef  M_PI
 #define  M_PI        3.14159265358979323846  /* pi */
 #endif
@@ -18,18 +22,20 @@
 void grid_to_radian(const char *units, long nvals, double *restrict values, const char *description);
 void grid_to_degree(const char *units, long nvals, double *restrict values, const char *description);
 
-void grid_gen_corners(long n, const double* restrict vals, double* restrict corners);
+void grid_gen_corners(int n, const double* restrict vals, double* restrict corners);
+void grid_gen_bounds(int n, const double *restrict vals, double *restrict bounds);
+void grid_check_lat_borders(int n, double *ybounds);
+
+void grid_gen_xbounds2D(int nx, int ny, const double *restrict xbounds, double *restrict xbounds2D);
+void grid_gen_ybounds2D(int nx, int ny, const double *restrict ybounds, double *restrict ybounds2D);
+
 void grid_cell_center_to_bounds_X2D(const char* xunitstr, long xsize, long ysize,
 				    const double* restrict grid_center_lon, double* restrict grid_corner_lon, double dlon);
 void grid_cell_center_to_bounds_Y2D(const char* yunitstr, long xsize, long ysize,
 				    const double* restrict grid_center_lat, double* restrict grid_corner_lat);
 
-void gridGenXvals(int xsize, double xfirst, double xlast, double xinc, double *xvals);
-void gridGenYvals(int gridtype, int ysize, double yfirst, double ylast, double yinc, double *yvals);
-
 int  gridWeights(int gridID, double *weights);
 int  gridGenArea(int gridID, double *area);
-void gaussaw(double pa[], double pw[], int nlat);
 
 int referenceToGrid(int gridID);
 int gridToZonal(int gridID);
