@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -29,17 +29,15 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
-
+#include <limits.h>
 
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
+#include "cdi_uuid.h"
 #include "grid.h"
 #include "griddes.h"
 #include "error.h"
-#include "dtypes.h"
-
-#define  cmpstr(s1, s2, len)  (memcmp(s1, s2, len = strlen(s2)))
 
 /*
 int  extInqPrec(int fileID);
@@ -204,13 +202,13 @@ int gridDefine(griddes_t grid)
 
 	if ( (grid.def_xfirst || grid.def_xlast || grid.def_xinc) && grid.xvals == NULL )
 	  {
-	    grid.xvals = (double*) malloc(grid.xsize*sizeof(double));
+	    grid.xvals = (double*) Malloc(grid.xsize*sizeof(double));
 	    gridGenXvals(grid.xsize, grid.xfirst, grid.xlast, grid.xinc, grid.xvals);
 
 	    if ( grid.genBounds && grid.xbounds == NULL && grid.xsize > 1 )
 	      {
 		grid.nvertex = 2;
-		grid.xbounds = (double*) malloc(grid.xsize*grid.nvertex*sizeof(double));
+		grid.xbounds = (double*) Malloc(grid.xsize*grid.nvertex*sizeof(double));
 		for ( i = 0; i < (int) grid.xsize-1; i++ )
 		  {
 		    grid.xbounds[2*i+1]   = 0.5*(grid.xvals[i] + grid.xvals[i+1]);
@@ -224,13 +222,13 @@ int gridDefine(griddes_t grid)
 	if ( (grid.def_yfirst || grid.def_ylast || grid.def_yinc) && grid.yvals == NULL )
 	  {
 	    if ( ! grid.def_ylast ) grid.ylast = grid.yfirst;
-	    grid.yvals = (double*) malloc(grid.ysize*sizeof(double));
+	    grid.yvals = (double*) Malloc(grid.ysize*sizeof(double));
 	    gridGenYvals(grid.type, grid.ysize, grid.yfirst, grid.ylast, grid.yinc, grid.yvals);
 
 	    if ( grid.genBounds && grid.ybounds == NULL && grid.ysize > 1 )
 	      {
 		grid.nvertex = 2;
-		grid.ybounds = (double*) malloc(grid.ysize*grid.nvertex*sizeof(double));
+		grid.ybounds = (double*) Malloc(grid.ysize*grid.nvertex*sizeof(double));
 		for ( i = 0; i < (int) grid.ysize-1; i++ )
 		  {
 		    grid.ybounds[2*i+1]   = 0.5*(grid.yvals[i] + grid.yvals[i+1]);
@@ -253,13 +251,13 @@ int gridDefine(griddes_t grid)
 	if ( grid.xvals )
 	  {
 	    gridDefXvals(gridID, grid.xvals);
-	    free(grid.xvals);
+	    Free(grid.xvals);
 	  }
 
 	if ( grid.yvals )
 	  {
 	    gridDefYvals(gridID, grid.yvals);
-	    free(grid.yvals);
+	    Free(grid.yvals);
 	  }
 
 	if ( grid.nvertex )
@@ -268,13 +266,13 @@ int gridDefine(griddes_t grid)
 	if ( grid.xbounds )
 	  {
 	    gridDefXbounds(gridID, grid.xbounds);
-	    free(grid.xbounds);
+	    Free(grid.xbounds);
 	  }
 
 	if ( grid.ybounds )
 	  {
 	    gridDefYbounds(gridID, grid.ybounds);
-	    free(grid.ybounds);
+	    Free(grid.ybounds);
 	  }
 
 	if ( grid.isRotated )
@@ -287,7 +285,7 @@ int gridDefine(griddes_t grid)
 	if ( grid.mask )
 	  {
 	    gridDefMask(gridID, grid.mask);
-	    free(grid.mask);
+	    Free(grid.mask);
 	  }
 
 	if ( grid.type == GRID_LAEA )
@@ -333,37 +331,37 @@ int gridDefine(griddes_t grid)
 	if ( grid.xvals )
 	  {
 	    gridDefXvals(gridID, grid.xvals);
-	    free(grid.xvals);
+	    Free(grid.xvals);
 	  }
 
 	if ( grid.yvals )
 	  {
 	    gridDefYvals(gridID, grid.yvals);
-	    free(grid.yvals);
+	    Free(grid.yvals);
 	  }
 
 	if ( grid.area )
 	  {
 	    gridDefArea(gridID, grid.area);
-	    free(grid.area);
+	    Free(grid.area);
 	  }
 
 	if ( grid.xbounds )
 	  {
 	    gridDefXbounds(gridID, grid.xbounds);
-	    free(grid.xbounds);
+	    Free(grid.xbounds);
 	  }
 
 	if ( grid.ybounds )
 	  {
 	    gridDefYbounds(gridID, grid.ybounds);
-	    free(grid.ybounds);
+	    Free(grid.ybounds);
 	  }
 
 	if ( grid.mask )
 	  {
 	    gridDefMask(gridID, grid.mask);
-	    free(grid.mask);
+	    Free(grid.mask);
 	  }
 
 	break;
@@ -396,7 +394,7 @@ int gridDefine(griddes_t grid)
 	if ( grid.mask )
 	  {
 	    gridDefMask(gridID, grid.mask);
-	    free(grid.mask);
+	    Free(grid.mask);
 	  }
 
 	break;
@@ -417,14 +415,14 @@ int gridDefine(griddes_t grid)
 
 	if ( grid.def_xfirst && grid.def_xinc && grid.xvals == NULL )
 	  {
-	    grid.xvals = (double*) malloc(grid.xsize*sizeof(double));
+	    grid.xvals = (double*) Malloc(grid.xsize*sizeof(double));
 	    for ( i = 0; i < grid.xsize; ++i )
 	      grid.xvals[i] = grid.xfirst + i*grid.xinc;
 	  }
 
 	if ( grid.def_yfirst && grid.def_yinc && grid.yvals == NULL )
 	  {
-	    grid.yvals = (double*) malloc(grid.ysize*sizeof(double));
+	    grid.yvals = (double*) Malloc(grid.ysize*sizeof(double));
 	    for ( i = 0; i < grid.ysize; ++i )
 	      grid.yvals[i] = grid.yfirst + i*grid.yinc;
 	  }
@@ -432,13 +430,13 @@ int gridDefine(griddes_t grid)
 	if ( grid.xvals )
 	  {
 	    gridDefXvals(gridID, grid.xvals);
-	    free(grid.xvals);
+	    Free(grid.xvals);
 	  }
 
 	if ( grid.yvals )
 	  {
 	    gridDefYvals(gridID, grid.yvals);
-	    free(grid.yvals);
+	    Free(grid.yvals);
 	  }	
 
 	if ( grid.def_lon_0     == FALSE ) Error("lon_0 undefined!");
@@ -451,7 +449,7 @@ int gridDefine(griddes_t grid)
 	if ( grid.mask )
 	  {
 	    gridDefMask(gridID, grid.mask);
-	    free(grid.mask);
+	    Free(grid.mask);
 	  }
 
 	break;
@@ -491,7 +489,7 @@ int gridDefine(griddes_t grid)
 	if ( grid.mask )
 	  {
 	    gridDefMask(gridID, grid.mask);
-	    free(grid.mask);
+	    Free(grid.mask);
 	  }
 
 	break;
@@ -596,7 +594,7 @@ double *readfield(griddes_t *grid, int record, char *format, char *filename)
   if ( format == NULL )   Error("format undefined!");
   if ( filename == NULL ) Error("file name undefined!");
 
-  vals = (double*) malloc(grid->size*sizeof(double));
+  vals = (double*) Malloc(grid->size*sizeof(double));
 
   if ( strcmp(format, "extra") == 0 )
     {
@@ -631,7 +629,7 @@ double *readfield4(griddes_t *grid, int record, char *format, char *filename)
   if ( format == NULL )   Error("format undefined!");
   if ( filename == NULL ) Error("file name undefined!");
 
-  vals  = (double*) malloc(4*grid->size*sizeof(double));
+  vals  = (double*) Malloc(4*grid->size*sizeof(double));
 
   if ( strcmp(format, "extra") == 0 )
     {
@@ -670,7 +668,6 @@ double readflt(const char *filename, const char *name, const char *pline)
   return (val);
 }
 
-void str2uuid(const char *uuidstr, unsigned char *uuid);
 
 int gridFromFile(FILE *gfp, const char *dname)
 {
@@ -701,229 +698,229 @@ int gridFromFile(FILE *gfp, const char *dname)
       pline = line;
       while ( isspace((int) *pline) ) pline++;
       if ( pline[0] == '\0' ) continue;
-      if ( cmpstr(pline, "gridtype", len) == 0 )
+      if ( cmpstrlen(pline, "gridtype", len) == 0 )
 	{
 	  pline = skipSeparator(pline + len);
-	  if ( cmpstr(pline, "lonlat", len)  == 0 ||
-	       cmpstr(pline, "latlon", len)  == 0 )
+	  if ( cmpstrlen(pline, "lonlat", len)  == 0 ||
+	       cmpstrlen(pline, "latlon", len)  == 0 )
 	    {
 	      grid.type = GRID_LONLAT;
 	      grid.nvertex = 2;
 	    }
-	  else if ( cmpstr(pline, "gaussian", len)  == 0 )
+	  else if ( cmpstrlen(pline, "gaussian", len)  == 0 )
 	    {
 	      grid.type = GRID_GAUSSIAN;
 	      grid.nvertex = 2;
 	    }
-	  else if ( cmpstr(pline, "curvilinear", len)  == 0 )
+	  else if ( cmpstrlen(pline, "curvilinear", len)  == 0 )
 	    {
 	      grid.type = GRID_CURVILINEAR;
 	      grid.nvertex = 4;
 	    }
-	  else if ( cmpstr(pline, "spectral", len)  == 0 )
+	  else if ( cmpstrlen(pline, "spectral", len)  == 0 )
 	    grid.type = GRID_SPECTRAL;
-	  else if ( cmpstr(pline, "unstructured", len)  == 0 )
+	  else if ( cmpstrlen(pline, "unstructured", len)  == 0 )
 	    grid.type = GRID_UNSTRUCTURED;
-	  else if ( cmpstr(pline, "cell", len)  == 0 )
+	  else if ( cmpstrlen(pline, "cell", len)  == 0 )
 	    grid.type = GRID_UNSTRUCTURED;
-	  else if ( cmpstr(pline, "gme", len)  == 0 )
+	  else if ( cmpstrlen(pline, "gme", len)  == 0 )
 	    grid.type = GRID_GME;
-	  else if ( cmpstr(pline, "lcc2", len)  == 0 )
+	  else if ( cmpstrlen(pline, "lcc2", len)  == 0 )
 	    grid.type = GRID_LCC2;
-	  else if ( cmpstr(pline, "lcc", len)  == 0 )
+	  else if ( cmpstrlen(pline, "lcc", len)  == 0 )
 	    grid.type = GRID_LCC;
-	  else if ( cmpstr(pline, "lambert", len)  == 0 )
+	  else if ( cmpstrlen(pline, "lambert", len)  == 0 )
 	    grid.type = GRID_LCC;
-	  else if ( cmpstr(pline, "sinusoidal", len)  == 0 )
+	  else if ( cmpstrlen(pline, "sinusoidal", len)  == 0 )
 	    grid.type = GRID_SINUSOIDAL;
-	  else if ( cmpstr(pline, "laea", len)  == 0 )
+	  else if ( cmpstrlen(pline, "laea", len)  == 0 )
 	    grid.type = GRID_LAEA;
-	  else if ( cmpstr(pline, "generic", len)  == 0 )
+	  else if ( cmpstrlen(pline, "generic", len)  == 0 )
 	    grid.type = GRID_GENERIC;
 	  else
 	    cdoAbort("Invalid grid name : %s (grid description file: %s)", pline, dname);
 	}
-      else if ( cmpstr(pline, "gridprec", len)  == 0 )
+      else if ( cmpstrlen(pline, "gridprec", len)  == 0 )
 	{
 	  grid.prec = atol(skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "gridsize", len)  == 0 )
+      else if ( cmpstrlen(pline, "gridsize", len)  == 0 )
 	{
 	  grid.size = atol(skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "truncation", len)  == 0 )
+      else if ( cmpstrlen(pline, "truncation", len)  == 0 )
 	{
 	  grid.ntr = atoi(skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "np", len)  == 0 )
+      else if ( cmpstrlen(pline, "np", len)  == 0 )
 	{
 	  grid.np = atoi(skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "complexpacking", len)  == 0 )
+      else if ( cmpstrlen(pline, "complexpacking", len)  == 0 )
 	{
 	  grid.lcomplex = atoi(skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "xname", len)  == 0 )
+      else if ( cmpstrlen(pline, "xname", len)  == 0 )
 	{
 	  strcpy(grid.xname, skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "xlongname", len)  == 0 )
+      else if ( cmpstrlen(pline, "xlongname", len)  == 0 )
 	{
 	  strcpy(grid.xlongname, skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "xunits", len)  == 0 )
+      else if ( cmpstrlen(pline, "xunits", len)  == 0 )
 	{
 	  strcpy(grid.xunits, skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "yname", len)  == 0 )
+      else if ( cmpstrlen(pline, "yname", len)  == 0 )
 	{
 	  strcpy(grid.yname, skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "ylongname", len)  == 0 )
+      else if ( cmpstrlen(pline, "ylongname", len)  == 0 )
 	{
 	  strcpy(grid.ylongname, skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "yunits", len)  == 0 )
+      else if ( cmpstrlen(pline, "yunits", len)  == 0 )
 	{
 	  strcpy(grid.yunits, skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "nvertex", len)  == 0 )
+      else if ( cmpstrlen(pline, "nvertex", len)  == 0 )
 	{
 	  grid.nvertex = atol(skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "ni", len)  == 0 )
+      else if ( cmpstrlen(pline, "ni", len)  == 0 )
 	{
 	  grid.ni = atol(skipSeparator(pline + len));
           grid.nd = 10;
 	}
-      else if ( cmpstr(pline, "position", len)  == 0 )
+      else if ( cmpstrlen(pline, "position", len)  == 0 )
 	{
 	  grid.position = atol(skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "number", len)  == 0 )
+      else if ( cmpstrlen(pline, "number", len)  == 0 )
 	{
 	  grid.number = atol(skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "path", len)  == 0 )
+      else if ( cmpstrlen(pline, "path", len)  == 0 )
 	{
 	  strcpy(grid.path, skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "uuid", len)  == 0 )
+      else if ( cmpstrlen(pline, "uuid", len)  == 0 )
 	{
 	  char uuidOfHGridStr[256];
 	  strcpy(uuidOfHGridStr, skipSeparator(pline + len));
-	  str2uuid(uuidOfHGridStr, grid.uuid);
+	  cdiStr2UUID(uuidOfHGridStr, grid.uuid);
 	}
-      else if ( cmpstr(pline, "xsize", len)  == 0 )
+      else if ( cmpstrlen(pline, "xsize", len)  == 0 )
 	{
 	  grid.xsize = atol(skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "nlon", len)  == 0 )
+      else if ( cmpstrlen(pline, "nlon", len)  == 0 )
 	{
 	  grid.xsize = atol(skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "ysize", len)  == 0 )
+      else if ( cmpstrlen(pline, "ysize", len)  == 0 )
 	{
 	  grid.ysize = atol(skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "nlat", len)  == 0 )
+      else if ( cmpstrlen(pline, "nlat", len)  == 0 )
 	{
 	  grid.ysize = atol(skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "xfirst", len)  == 0 )
+      else if ( cmpstrlen(pline, "xfirst", len)  == 0 )
 	{
 	  grid.xfirst = readflt(dname, "xfirst", skipSeparator(pline + len));
 	  grid.def_xfirst = TRUE;
 	}
-      else if ( cmpstr(pline, "lonfirst", len)  == 0 )
+      else if ( cmpstrlen(pline, "lonfirst", len)  == 0 )
 	{
 	  grid.xfirst = readflt(dname, "lonfirst", skipSeparator(pline + len));
 	  grid.def_xfirst = TRUE;
 	}
-      else if ( cmpstr(pline, "yfirst", len)  == 0 )
+      else if ( cmpstrlen(pline, "yfirst", len)  == 0 )
 	{
 	  grid.yfirst = readflt(dname, "yfirst", skipSeparator(pline + len));
 	  grid.def_yfirst = TRUE;
 	}
-      else if ( cmpstr(pline, "latfirst", len)  == 0 )
+      else if ( cmpstrlen(pline, "latfirst", len)  == 0 )
 	{
 	  grid.yfirst = readflt(dname, "latfirst", skipSeparator(pline + len));
 	  grid.def_yfirst = TRUE;
 	}
-      else if ( cmpstr(pline, "xlast", len)  == 0 )
+      else if ( cmpstrlen(pline, "xlast", len)  == 0 )
 	{
 	  grid.xlast = readflt(dname, "xlast", skipSeparator(pline + len));
 	  grid.def_xlast = TRUE;
 	}
-      else if ( cmpstr(pline, "lonlast", len)  == 0 )
+      else if ( cmpstrlen(pline, "lonlast", len)  == 0 )
 	{
 	  grid.xlast = readflt(dname, "lonlast", skipSeparator(pline + len));
 	  grid.def_xlast = TRUE;
 	}
-      else if ( cmpstr(pline, "ylast", len)  == 0 )
+      else if ( cmpstrlen(pline, "ylast", len)  == 0 )
 	{
 	  grid.ylast = readflt(dname, "ylast", skipSeparator(pline + len));
 	  grid.def_ylast = TRUE;
 	}
-      else if ( cmpstr(pline, "latlast", len)  == 0 )
+      else if ( cmpstrlen(pline, "latlast", len)  == 0 )
 	{
 	  grid.ylast = readflt(dname, "latlast", skipSeparator(pline + len));
 	  grid.def_ylast = TRUE;
 	}
-      else if ( cmpstr(pline, "xinc", len)  == 0 )
+      else if ( cmpstrlen(pline, "xinc", len)  == 0 )
 	{
 	  grid.xinc = readflt(dname, "xinc", skipSeparator(pline + len));
 	  grid.def_xinc = TRUE;
 	}
-      else if ( cmpstr(pline, "loninc", len)  == 0 )
+      else if ( cmpstrlen(pline, "loninc", len)  == 0 )
 	{
 	  grid.xinc = readflt(dname, "loninc", skipSeparator(pline + len));
 	  grid.def_xinc = TRUE;
 	}
-      else if ( cmpstr(pline, "yinc", len)  == 0 )
+      else if ( cmpstrlen(pline, "yinc", len)  == 0 )
 	{
 	  grid.yinc = readflt(dname, "yinc", skipSeparator(pline + len));
 	  grid.def_yinc = TRUE;
 	}
-      else if ( cmpstr(pline, "latinc", len)  == 0 )
+      else if ( cmpstrlen(pline, "latinc", len)  == 0 )
 	{
 	  grid.yinc = readflt(dname, "latinc", skipSeparator(pline + len));
 	  grid.def_yinc = TRUE;
 	}
-      else if ( cmpstr(pline, "originLon", len)  == 0 )
+      else if ( cmpstrlen(pline, "originLon", len)  == 0 )
 	{
 	  grid.originLon = readflt(dname, "originLon", skipSeparator(pline + len));
 	  grid.def_originLon = TRUE;
 	}
-      else if ( cmpstr(pline, "originLat", len)  == 0 )
+      else if ( cmpstrlen(pline, "originLat", len)  == 0 )
 	{
 	  grid.originLat = readflt(dname, "originLat", skipSeparator(pline + len));
 	  grid.def_originLat = TRUE;
 	}
-      else if ( cmpstr(pline, "lonParY", len)  == 0 )
+      else if ( cmpstrlen(pline, "lonParY", len)  == 0 )
 	{
 	  grid.lonParY = readflt(dname, "lonParY", skipSeparator(pline + len));
 	  grid.def_lonParY = TRUE;
 	}
-      else if ( cmpstr(pline, "lat1", len)  == 0 )
+      else if ( cmpstrlen(pline, "lat1", len)  == 0 )
 	{
 	  grid.lat1 = readflt(dname, "lat1", skipSeparator(pline + len));
 	  grid.def_lat1 = TRUE;
 	}
-      else if ( cmpstr(pline, "lat2", len)  == 0 )
+      else if ( cmpstrlen(pline, "lat2", len)  == 0 )
 	{
 	  grid.lat2 = readflt(dname, "lat2", skipSeparator(pline + len));
 	  grid.def_lat2 = TRUE;
 	}
-      else if ( cmpstr(pline, "projection", len)  == 0 )
+      else if ( cmpstrlen(pline, "projection", len)  == 0 )
 	{
 	  pline = skipSeparator(pline + len);
-	  if      ( cmpstr(pline, "north", len) == 0 )
+	  if      ( cmpstrlen(pline, "north", len) == 0 )
 	    {
 	      grid.projflag = 0;
 	      grid.scanflag = 64;
 	    }
-	  else if ( cmpstr(pline, "south", len) == 0 )
+	  else if ( cmpstrlen(pline, "south", len) == 0 )
 	    {
 	      grid.projflag = 128;
 	      grid.scanflag = 64;
@@ -931,63 +928,63 @@ int gridFromFile(FILE *gfp, const char *dname)
 	  else
 	    cdoAbort("Invalid projection : %s (grid description file: %s)", pline, dname);
 	}
-      else if ( cmpstr(pline, "lon_0", len)  == 0 )
+      else if ( cmpstrlen(pline, "lon_0", len)  == 0 )
 	{
 	  grid.lon_0 = readflt(dname, "lon_0", skipSeparator(pline + len));
 	  grid.def_lon_0 = TRUE;
 	}
-      else if ( cmpstr(pline, "lat_0", len)  == 0 )
+      else if ( cmpstrlen(pline, "lat_0", len)  == 0 )
 	{
 	  grid.lat_0 = readflt(dname, "lat_0", skipSeparator(pline + len));
 	  grid.def_lat_0 = TRUE;
 	}
-      else if ( cmpstr(pline, "lat_1", len)  == 0 )
+      else if ( cmpstrlen(pline, "lat_1", len)  == 0 )
 	{
 	  grid.lat_1 = readflt(dname, "lat_1", skipSeparator(pline + len));
 	  grid.def_lat_1 = TRUE;
 	}
-      else if ( cmpstr(pline, "lat_2", len)  == 0 )
+      else if ( cmpstrlen(pline, "lat_2", len)  == 0 )
 	{
 	  grid.lat_2 = readflt(dname, "lat_2", skipSeparator(pline + len));
 	  grid.def_lat_2 = TRUE;
 	}
-      else if ( cmpstr(pline, "xnpole", len)  == 0 )
+      else if ( cmpstrlen(pline, "xnpole", len)  == 0 )
 	{
 	  grid.xpole = readflt(dname, "xnpole", skipSeparator(pline + len));
 	  grid.isRotated = TRUE;
 	}
-      else if ( cmpstr(pline, "lonpole", len)  == 0 )
+      else if ( cmpstrlen(pline, "lonpole", len)  == 0 )
 	{
 	  grid.xpole = readflt(dname, "lonpole", skipSeparator(pline + len));
 	  grid.isRotated = TRUE;
 	}
-      else if ( cmpstr(pline, "ynpole", len)  == 0 )
+      else if ( cmpstrlen(pline, "ynpole", len)  == 0 )
 	{
 	  grid.ypole = readflt(dname, "ynpole", skipSeparator(pline + len));
 	  grid.isRotated = TRUE;
 	}
-      else if ( cmpstr(pline, "latpole", len)  == 0 )
+      else if ( cmpstrlen(pline, "latpole", len)  == 0 )
 	{
 	  grid.ypole = readflt(dname, "latpole", skipSeparator(pline + len));
 	  grid.isRotated = TRUE;
 	}
-      else if ( cmpstr(pline, "angle", len)  == 0 )
+      else if ( cmpstrlen(pline, "angle", len)  == 0 )
 	{
 	  grid.angle = readflt(dname, "angle", skipSeparator(pline + len));
 	  grid.isRotated = TRUE;
 	}
-      else if ( cmpstr(pline, "a", len)  == 0 )
+      else if ( cmpstrlen(pline, "a", len)  == 0 )
 	{
 	  grid.a = readflt(dname, "a", skipSeparator(pline + len));
 	}
-      else if ( cmpstr(pline, "gridlatlon", len)  == 0 )
+      else if ( cmpstrlen(pline, "gridlatlon", len)  == 0 )
 	{
 	  int i;
 	  double flat = 0, flon = 0;
 	  if ( grid.size == 0 ) grid.size = grid.xsize * grid.ysize;
 	  
-	  grid.xvals = (double*) malloc(grid.size*sizeof(double));
-	  grid.yvals = (double*) malloc(grid.size*sizeof(double));
+	  grid.xvals = (double*) Malloc(grid.size*sizeof(double));
+	  grid.yvals = (double*) Malloc(grid.size*sizeof(double));
 	  for ( i = 0; i < (int) grid.size; i++ )
 	    {
 	      if ( ! readline(gfp, line, MAX_LINE_LEN) )
@@ -998,7 +995,7 @@ int gridFromFile(FILE *gfp, const char *dname)
 	      grid.xvals[i] = flon;
 	    }
 	}
-      else if ( cmpstr(pline, "mask", len)  == 0 )
+      else if ( cmpstrlen(pline, "mask", len)  == 0 )
 	{
 	  int i = 0;
 	  long lval;
@@ -1010,7 +1007,7 @@ int gridFromFile(FILE *gfp, const char *dname)
 	    {
 	      long count = 0;
 	      pline = skipSeparator(pline + len);
-	      grid.mask = (int*) malloc(size*sizeof(int));
+	      grid.mask = (int*) Malloc(size*sizeof(int));
 
 	      for ( i = 0; i < size; i++ )
 		{
@@ -1031,14 +1028,14 @@ int gridFromFile(FILE *gfp, const char *dname)
 
 	      if ( count == size )
 		{
-		  free(grid.mask);
+		  Free(grid.mask);
 		  grid.mask = NULL;
 		}
 	    }
 	  else
 	    cdoAbort("gridsize undefined (grid description file: %s)!", dname);
 	}
-      else if ( cmpstr(pline, "xvals", len)  == 0 )
+      else if ( cmpstrlen(pline, "xvals", len)  == 0 )
 	{
 	  int i = 0;
 	  double fval;
@@ -1052,7 +1049,7 @@ int gridFromFile(FILE *gfp, const char *dname)
 	  if ( size > 0 )
 	    {
 	      pline = skipSeparator(pline + len);
-	      grid.xvals = (double*) malloc(size*sizeof(double));
+	      grid.xvals = (double*) Malloc(size*sizeof(double));
 
 	      for ( i = 0; i < size; i++ )
 		{
@@ -1073,7 +1070,7 @@ int gridFromFile(FILE *gfp, const char *dname)
 	  else
 	    cdoAbort("xsize or gridsize undefined (grid description file: %s)!", dname);
 	}
-      else if ( cmpstr(pline, "yvals", len)  == 0 )
+      else if ( cmpstrlen(pline, "yvals", len)  == 0 )
 	{
 	  int i = 0;
 	  double fval;
@@ -1087,7 +1084,7 @@ int gridFromFile(FILE *gfp, const char *dname)
 	  if ( size > 0 )
 	    {
 	      pline = skipSeparator(pline + len);
-	      grid.yvals = (double*) malloc(size*sizeof(double));
+	      grid.yvals = (double*) Malloc(size*sizeof(double));
 
 	      for ( i = 0; i < size; i++ )
 		{
@@ -1108,7 +1105,7 @@ int gridFromFile(FILE *gfp, const char *dname)
 	  else
 	    cdoAbort("ysize or gridsize undefined (grid description file: %s)!", dname);
 	}
-      else if ( cmpstr(pline, "xbounds", len)  == 0 )
+      else if ( cmpstrlen(pline, "xbounds", len)  == 0 )
 	{
 	  int i = 0;
 	  double fval;
@@ -1116,6 +1113,7 @@ int gridFromFile(FILE *gfp, const char *dname)
 
 	  if ( grid.nvertex == 0 )
 	    {
+	      if ( grid.type == GRID_LONLAT      ) grid.nvertex = 2;
 	      if ( grid.type == GRID_CURVILINEAR ) grid.nvertex = 4;
 	    }
 
@@ -1127,7 +1125,7 @@ int gridFromFile(FILE *gfp, const char *dname)
 	  if ( size > 0 && grid.nvertex > 0 )
 	    {	  
 	      pline = skipSeparator(pline + len);
-	      grid.xbounds = (double*) malloc(size*grid.nvertex*sizeof(double));
+	      grid.xbounds = (double*) Malloc(size*grid.nvertex*sizeof(double));
 
 	      for ( i = 0; i < (int) (size*grid.nvertex); i++ )
 		{
@@ -1151,7 +1149,7 @@ int gridFromFile(FILE *gfp, const char *dname)
 	      if ( grid.nvertex == 0 ) cdoAbort("nvertex undefined (grid description file: %s)!", dname);
 	    }
 	}
-      else if ( cmpstr(pline, "ybounds", len)  == 0 )
+      else if ( cmpstrlen(pline, "ybounds", len)  == 0 )
 	{
 	  int i = 0;
 	  double fval;
@@ -1159,6 +1157,7 @@ int gridFromFile(FILE *gfp, const char *dname)
 
 	  if ( grid.nvertex == 0 )
 	    {
+	      if ( grid.type == GRID_LONLAT      ) grid.nvertex = 2;
 	      if ( grid.type == GRID_CURVILINEAR ) grid.nvertex = 4;
 	    }
 
@@ -1170,7 +1169,7 @@ int gridFromFile(FILE *gfp, const char *dname)
 	  if ( size > 0 && grid.nvertex > 0 )
 	    {	  
 	      pline = skipSeparator(pline + len);
-	      grid.ybounds = (double*) malloc(size*grid.nvertex*sizeof(double));
+	      grid.ybounds = (double*) Malloc(size*grid.nvertex*sizeof(double));
 
 	      for ( i = 0; i < (int) (size*grid.nvertex); i++ )
 		{
@@ -1254,7 +1253,7 @@ int input_darray(FILE *gfp, int n_values, double *array)
   int i;
   int read_items;
 
-  if ( n_values <= 0 ) return (0);
+  if ( n_values <= 0 ) return 0;
 
   read_items = 0;
   for ( i = 0; i < n_values; i++ )
@@ -1290,8 +1289,8 @@ int gridFromPingo(FILE *gfp, const char *dname)
       grid.xsize = nlon;
       grid.ysize = nlat;
 
-      grid.xvals = (double*) malloc(grid.xsize*sizeof(double));
-      grid.yvals = (double*) malloc(grid.ysize*sizeof(double));
+      grid.xvals = (double*) Malloc(grid.xsize*sizeof(double));
+      grid.yvals = (double*) Malloc(grid.ysize*sizeof(double));
 
       if ( ! input_ival(gfp, &nlon) ) return (gridID);
       if ( nlon == 2 )
@@ -1359,10 +1358,10 @@ int gridFromPingo(FILE *gfp, const char *dname)
       if ( nlat > 2 ) /* check if gaussian */
 	{
 	  double *yvals, *yw;
-	  yvals = (double*) malloc(grid.ysize*sizeof(double));
-	  yw    = (double*) malloc(grid.ysize*sizeof(double));
+	  yvals = (double*) Malloc(grid.ysize*sizeof(double));
+	  yw    = (double*) Malloc(grid.ysize*sizeof(double));
 	  gaussaw(yvals, yw, grid.ysize);
-	  free(yw);
+	  Free(yw);
 	  for ( i = 0; i < (int) grid.ysize; i++ )
 	    yvals[i] = asin(yvals[i])*RAD2DEG;
 
@@ -1371,7 +1370,7 @@ int gridFromPingo(FILE *gfp, const char *dname)
 		      
 	  if ( i == (int) grid.ysize ) lgauss = TRUE;
 
-	  free(yvals);
+	  Free(yvals);
 	}
 
       if ( lgauss )
@@ -1388,9 +1387,7 @@ int gridFromPingo(FILE *gfp, const char *dname)
 
 int nfc2nlat(int nfc, int ntr)
 {
-  int nlat;
-
-  nlat = nfc / (ntr+1);
+  int nlat = nfc / (ntr+1);
   nlat /= 2;
 
   return (nlat);
@@ -1399,9 +1396,7 @@ int nfc2nlat(int nfc, int ntr)
 
 int nlat2ntr(int nlat)
 {
-  int ntr;
-
-  ntr = (nlat*2 - 1) / 3;
+  int ntr = (nlat*2 - 1) / 3;
 
   return (ntr);
 }
@@ -1409,9 +1404,7 @@ int nlat2ntr(int nlat)
 
 int nlat2ntr_linear(int nlat)
 {
-  int ntr;
-
-  ntr = (nlat*2 - 1) / 2;
+  int ntr = (nlat*2 - 1) / 2;
 
   return (ntr);
 }
@@ -1419,14 +1412,12 @@ int nlat2ntr_linear(int nlat)
 
 int ntr2nlat(int ntr)
 {
-  int nlat, nlat2;
-
-  nlat = lround((ntr*3.+1.)/2.);
+  int nlat = (int)lround((ntr*3.+1.)/2.);
   if ( (nlat % 2) > 0 )
     {
       nlat  = nlat + 1;
-      nlat2 = lround(((ntr+1)*3.+1.)/2.);
       /*
+      int nlat2 = (int)lround(((ntr+1)*3.+1.)/2.);
       if ( nlat == nlat2 )
 	Error("Computation of latitudes failed for truncation %d", ntr);
       */
@@ -1438,14 +1429,12 @@ int ntr2nlat(int ntr)
 
 int ntr2nlat_linear(int ntr)
 {
-  int nlat, nlat2;
-
-  nlat = lround((ntr*2.+1.)/2.);
+  int nlat = (int)lround((ntr*2.+1.)/2.);
   if ( (nlat % 2) > 0 )
     {
       nlat  = nlat + 1;
-      nlat2 = lround(((ntr+1)*2.+1.)/2.);
       /*
+      int nlat2 = (int)lround(((ntr+1)*2.+1.)/2.);
       if ( nlat == nlat2 )
 	Error("Computation of latitudes failed for truncation %d", ntr);
       */
@@ -1457,9 +1446,8 @@ int ntr2nlat_linear(int ntr)
 
 int compNlon(int nlat)
 {
-  int nlon, n;
-
-  nlon = 2 * nlat;
+  int n;
+  int nlon = 2 * nlat;
 
   /* check that FFT works with nlon */
   while ( 1 )
@@ -1490,9 +1478,8 @@ int compNlon(int nlat)
 static
 void gen_grid_lonlat(griddes_t *grid, const char *pline, double inc, double lon1, double lon2, double lat1, double lat2)
 {
-  int nlon, nlat, i;
   int gridtype = GRID_LONLAT;
-  char *endptr;
+  int lbounds = TRUE;
 
   if ( *pline != 0 )
     {
@@ -1503,25 +1490,112 @@ void gen_grid_lonlat(griddes_t *grid, const char *pline, double inc, double lon1
 
       if ( ! isdigit((int) *pline) && !ispunct((int) *pline) ) return;
 
-      endptr = (char *) pline;
+      char *endptr = (char *) pline;
       inc = strtod(pline, &endptr);
-      if ( *endptr != 0 ) return;
+      if ( *endptr != 0 )
+        {
+          pline = endptr;
+          if ( *pline == '_' ) pline++;
+          else return;
+          
+          if ( *pline == 0 ) return;
+          if ( *pline == 'c' )
+            {
+              gridtype = GRID_CURVILINEAR;
+              pline++;
+              if ( *pline == '0' )
+                {
+                  lbounds = FALSE;
+                  pline++;
+                }
+            }
+          else if ( *pline == 'u' )
+            {
+              gridtype = GRID_UNSTRUCTURED;
+              pline++;
+              if ( *pline == '0' )
+                {
+                  lbounds = FALSE;
+                  pline++;
+                }
+            }
+          if ( *pline != 0 ) return;          
+        }
 
       if ( inc < 1e-9 ) inc = 1;
     }
 
   grid->type = gridtype;
 
-  nlon = (int) ((lon2 - lon1)/inc + 0.5);
-  nlat = (int) ((lat2 - lat1)/inc + 0.5);
-  grid->xsize = nlon;
-  grid->ysize = nlat;
+  int nlon = (int) ((lon2 - lon1)/inc + 0.5);
+  int nlat = (int) ((lat2 - lat1)/inc + 0.5);
 
-  grid->xvals = (double*) malloc(grid->xsize*sizeof(double));
-  grid->yvals = (double*) malloc(grid->ysize*sizeof(double));
+  double *xvals = (double*) Malloc(nlon*sizeof(double));
+  double *yvals = (double*) Malloc(nlat*sizeof(double));
 
-  for ( i = 0; i < nlon; ++i ) grid->xvals[i] = lon1 + inc/2 + i*inc;
-  for ( i = 0; i < nlat; ++i ) grid->yvals[i] = lat1 + inc/2 + i*inc;
+  for ( int i = 0; i < nlon; ++i ) xvals[i] = lon1 + inc/2 + i*inc;
+  for ( int i = 0; i < nlat; ++i ) yvals[i] = lat1 + inc/2 + i*inc;
+
+  if ( gridtype == GRID_LONLAT )
+    {
+      grid->xsize = nlon;
+      grid->ysize = nlat;
+      grid->xvals = xvals;
+      grid->yvals = yvals;
+      xvals = NULL;
+      yvals = NULL;
+    }
+  else
+    {
+      double gridsize = nlon*nlat;
+      double *xvals2D = (double*) Malloc(gridsize*sizeof(double));
+      double *yvals2D = (double*) Malloc(gridsize*sizeof(double));
+      for ( int j = 0; j < nlat; j++ )
+        for ( int i = 0; i < nlon; i++ )
+          {
+            xvals2D[j*nlon+i] = xvals[i];
+            yvals2D[j*nlon+i] = yvals[j];
+          }
+
+      if ( gridtype == GRID_CURVILINEAR )
+        {
+          grid->xsize = nlon;
+          grid->ysize = nlat;
+        }
+      else
+        {
+          grid->xsize = gridsize;
+          grid->ysize = gridsize;
+          if ( lbounds ) grid->nvertex = 4;
+        }
+      
+      grid->xvals = xvals2D;
+      grid->yvals = yvals2D;
+      
+      if ( lbounds && nlon > 1 && nlat > 1 )
+        {
+          double *xbounds = (double*) Malloc(2*nlon*sizeof(double));
+          grid_gen_bounds(nlon, xvals, xbounds);
+          
+          double *ybounds = (double*) Malloc(2*nlat*sizeof(double));
+          grid_gen_bounds(nlat, yvals, ybounds);
+          grid_check_lat_borders(2*nlat, ybounds);
+
+          double *xbounds2D = (double*) Malloc(4*gridsize*sizeof(double));
+          double *ybounds2D = (double*) Malloc(4*gridsize*sizeof(double));
+
+          grid_gen_xbounds2D(nlon, nlat, xbounds, xbounds2D);
+          grid_gen_ybounds2D(nlon, nlat, ybounds, ybounds2D);
+
+          Free(xbounds);
+          Free(ybounds);
+          grid->xbounds = xbounds2D;
+          grid->ybounds = ybounds2D;
+        }
+   }
+
+  if ( xvals ) Free(xvals);
+  if ( yvals ) Free(yvals);
 }
 
 
@@ -1542,10 +1616,10 @@ int gridFromName(const char *gridname)
 	{
 	  grid.ntr = atoi(pline);
 	  while ( isdigit((int) *pline) ) pline++;
-	  if      ( cmpstr(pline, "grid", len) == 0 ) grid.type = GRID_GAUSSIAN;
-	  else if ( cmpstr(pline, "zon",  len) == 0 ) grid.type = GRID_GAUSSIAN;
-	  else if ( cmpstr(pline, "spec", len) == 0 ) grid.type = GRID_SPECTRAL;
-	  else if ( cmpstr(pline, "",     len) == 0 ) grid.type = GRID_SPECTRAL;
+	  if      ( cmpstrlen(pline, "grid", len) == 0 ) grid.type = GRID_GAUSSIAN;
+	  else if ( cmpstrlen(pline, "zon",  len) == 0 ) grid.type = GRID_GAUSSIAN;
+	  else if ( cmpstrlen(pline, "spec", len) == 0 ) grid.type = GRID_SPECTRAL;
+	  else if ( cmpstrlen(pline, "",     len) == 0 ) grid.type = GRID_SPECTRAL;
       
 	  if ( pline[len] != 0 ) return (gridID);
 
@@ -1553,7 +1627,7 @@ int gridFromName(const char *gridname)
 	    {
 	      grid.ysize = ntr2nlat_linear(grid.ntr);
 	      grid.np    = grid.ysize/2;
-	      if ( cmpstr(pline, "zon",  len) == 0 )
+	      if ( cmpstrlen(pline, "zon",  len) == 0 )
 		grid.xsize = 1;
 	      else
 		grid.xsize = compNlon(grid.ysize);
@@ -1570,10 +1644,10 @@ int gridFromName(const char *gridname)
 	{
 	  grid.ntr = atoi(pline);
 	  while ( isdigit((int) *pline) ) pline++;
-	  if      ( cmpstr(pline, "grid", len) == 0 ) grid.type = GRID_GAUSSIAN;
-	  else if ( cmpstr(pline, "zon",  len) == 0 ) grid.type = GRID_GAUSSIAN;
-	  else if ( cmpstr(pline, "spec", len) == 0 ) grid.type = GRID_SPECTRAL;
-	  else if ( cmpstr(pline, "",     len) == 0 ) grid.type = GRID_SPECTRAL;
+	  if      ( cmpstrlen(pline, "grid", len) == 0 ) grid.type = GRID_GAUSSIAN;
+	  else if ( cmpstrlen(pline, "zon",  len) == 0 ) grid.type = GRID_GAUSSIAN;
+	  else if ( cmpstrlen(pline, "spec", len) == 0 ) grid.type = GRID_SPECTRAL;
+	  else if ( cmpstrlen(pline, "",     len) == 0 ) grid.type = GRID_SPECTRAL;
      
 	  if ( pline[len] != 0 ) return (gridID);
 
@@ -1581,7 +1655,7 @@ int gridFromName(const char *gridname)
 	    {
 	      grid.ysize = ntr2nlat(grid.ntr);
 	      grid.np    = grid.ysize/2;
-	      if ( cmpstr(pline, "zon",  len) == 0 )
+	      if ( cmpstrlen(pline, "zon",  len) == 0 )
 		grid.xsize = 1;
 	      else
 		grid.xsize = compNlon(grid.ysize);
@@ -1617,8 +1691,8 @@ int gridFromName(const char *gridname)
 	  grid.type = GRID_LONLAT;
 	  grid.xsize = 1;
 	  grid.ysize = 1;
-	  grid.xvals = (double*) malloc(sizeof(double));
-	  grid.yvals = (double*) malloc(sizeof(double));
+	  grid.xvals = (double*) Malloc(sizeof(double));
+	  grid.yvals = (double*) Malloc(sizeof(double));
 	  grid.xvals[0] = atof(pline);
 	  while ( isdigit((int) *pline) || ispunct((int) *pline) || *pline == '-' ) pline++;
 	  if ( *pline == '_' ) pline++;
@@ -1671,7 +1745,7 @@ int gridFromName(const char *gridname)
 	  long np = strtol(pline, &endptr, 10);
 	  pline = endptr;
 
-	  if ( cmpstr(pline, "zon",  len) == 0 )
+	  if ( cmpstrlen(pline, "zon",  len) == 0 )
 	    {
 	      grid.xsize = 1;
 	      pline += 3;
@@ -1802,15 +1876,15 @@ int cdoDefineGrid(const char *gridfile)
 
       close(fileno);
 
-      if ( cmpstr(buffer, "CDF", len) == 0 )
+      if ( cmpstrlen(buffer, "CDF", len) == 0 )
 	{
-	  if ( cdoDebug ) cdoPrint("Grid from netCDF file");
+	  if ( cdoDebug ) cdoPrint("Grid from NetCDF file");
 	  gridID = gridFromNCfile(filename);
 	}
 
       if ( gridID == -1 )
 	{
-	  if ( cmpstr(buffer+1, "HDF", len) == 0 )
+	  if ( cmpstrlen(buffer+1, "HDF", len) == 0 )
 	    {
 	      if ( cdoDebug ) cdoPrint("Grid from HDF5 file");
 	      gridID = gridFromH5file(filename);
@@ -1819,9 +1893,9 @@ int cdoDefineGrid(const char *gridfile)
 
       if ( gridID == -1 )
 	{
-	  if ( cmpstr(buffer+1, "HDF", len) == 0 )
+	  if ( cmpstrlen(buffer+1, "HDF", len) == 0 )
 	    {
-	      if ( cdoDebug ) cdoPrint("Grid from netCDF4 file");
+	      if ( cdoDebug ) cdoPrint("Grid from NetCDF4 file");
 	      gridID = gridFromNCfile(filename);
 	    }
 	}
@@ -1847,7 +1921,7 @@ int cdoDefineGrid(const char *gridfile)
 	  if ( cdoDebug ) cdoPrint("grid from ASCII file");
 	  gfp = fopen(filename, "r");
 	  //size_t buffersize = 20*1024*1024;
-	  //char *buffer = (char*) malloc(buffersize);
+	  //char *buffer = (char*) Malloc(buffersize);
 	  //setvbuf(gfp, buffer, _IOFBF, buffersize);
 	  gridID = gridFromFile(gfp, filename);
 	  fclose(gfp);
@@ -1865,7 +1939,7 @@ int cdoDefineGrid(const char *gridfile)
       if ( gridID == -1 ) cdoAbort("Invalid grid description file %s!", filename);
     }
 
-  if ( lalloc ) free(filename);
+  if ( lalloc ) Free(filename);
 
   return (gridID);
 }
