@@ -1,11 +1,6 @@
 #include <cdi.h>
 #include "cdo_int.h"
-
-
-void encode_caldaysec(int calendar, int year, int month, int day, int hour, int minute, int second,
-		      int *julday, int *secofday);
-void decode_caldaysec(int calendar, int julday, int secofday, 
-		      int *year, int *month, int *day, int *hour, int *minute, int *second);
+#include "calendar.h"
 
 
 juldate_t juldate_encode(int calendar, int date, int time)
