@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -35,7 +35,6 @@ int pclose(FILE *stream);
 #include <cdi.h>
 #include "cdo.h"
 #include "cdo_int.h"
-#include "dtypes.h"
 #include "modules.h"
 #include "pstream_int.h"
 #include "cdo_int.h"
@@ -45,8 +44,6 @@ int pclose(FILE *stream);
 #include "dmemory.h"
 
 
-extern int timer_read, timer_write;
-
 static int PSTREAM_Debug = 0;
 
 #define  MAX_PSTREAMS  4096
@@ -102,19 +99,19 @@ void pstream_list_new(void)
 {
   assert(_pstreamList == NULL);
 
-  _pstreamList = (pstreamPtrToIdx*) malloc(_pstream_max*sizeof(pstreamPtrToIdx));
+  _pstreamList = (pstreamPtrToIdx*) Malloc(_pstream_max*sizeof(pstreamPtrToIdx));
 }
 
 static
 void pstream_list_delete(void)
 {
-  if ( _pstreamList ) free(_pstreamList);
+  if ( _pstreamList ) Free(_pstreamList);
 }
 
 static
 void pstream_init_pointer(void)
 {  
-  for ( int i = 0; i < _pstream_max; i++ )
+  for ( int i = 0; i < _pstream_max; ++i )
     {
       _pstreamList[i].next = _pstreamList + i + 1;
       _pstreamList[i].idx  = i;
@@ -144,15 +141,14 @@ pstream_t *pstream_to_pointer(int idx)
   else
     Error("pstream index %d undefined!", idx);
 
-  return (pstreamptr);
+  return pstreamptr;
 }
 
 /* Create an index from a pointer */
 static
 int pstream_from_pointer(pstream_t *ptr)
 {
-  int      idx = -1;
-  pstreamPtrToIdx *newptr;
+  int idx = -1;
 
   if ( ptr )
     {
@@ -160,7 +156,7 @@ int pstream_from_pointer(pstream_t *ptr)
 
       if ( _pstreamAvail )
 	{
-	  newptr        = _pstreamAvail;
+	  pstreamPtrToIdx *newptr = _pstreamAvail;
 	  _pstreamAvail = _pstreamAvail->next;
 	  newptr->next  = 0;
 	  idx	        = newptr->idx;
@@ -177,7 +173,7 @@ int pstream_from_pointer(pstream_t *ptr)
   else
     Error("Internal problem (pointer %p undefined)", ptr);
 
-  return (idx);
+  return idx;
 }
 
 static
@@ -210,25 +206,21 @@ void pstream_init_entry(pstream_t *pstreamptr)
 static
 pstream_t *pstream_new_entry(void)
 {
-  pstream_t *pstreamptr;
-
-  pstreamptr = (pstream_t*) malloc(sizeof(pstream_t));
+  pstream_t *pstreamptr = (pstream_t*) Malloc(sizeof(pstream_t));
 
   if ( pstreamptr ) pstream_init_entry(pstreamptr);
 
-  return (pstreamptr);
+  return pstreamptr;
 }
 
 static
 void pstream_delete_entry(pstream_t *pstreamptr)
 {
-  int idx;
-
-  idx = pstreamptr->self;
+  int idx = pstreamptr->self;
 
   PSTREAM_LOCK();
 
-  free(pstreamptr);
+  Free(pstreamptr);
 
   _pstreamList[idx].next = _pstreamAvail;
   _pstreamList[idx].ptr  = 0;
@@ -243,14 +235,12 @@ void pstream_delete_entry(pstream_t *pstreamptr)
 static
 void pstream_initialize(void)
 {
-  char *env;
-
 #if defined(HAVE_LIBPTHREAD)
   /* initialize global API mutex lock */
   pthread_mutex_init(&_pstream_mutex, NULL);
 #endif
 
-  env = getenv("PSTREAM_DEBUG");
+  char *env = getenv("PSTREAM_DEBUG");
   if ( env ) PSTREAM_Debug = atoi(env);
 
   env = getenv("PSTREAM_MAX");
@@ -270,10 +260,10 @@ void pstream_initialize(void)
 static
 int pstreamFindID(const char *name)
 {
-  int pstreamID;
   pstream_t *pstreamptr;
+  int pstreamID;
 
-  for ( pstreamID = 0; pstreamID < _pstream_max; pstreamID++ )
+  for ( pstreamID = 0; pstreamID < _pstream_max; ++pstreamID )
     {
       pstreamptr = pstream_to_pointer(pstreamID);
 
@@ -284,35 +274,28 @@ int pstreamFindID(const char *name)
 
   if ( pstreamID == _pstream_max ) pstreamID = -1;
 
-  return (pstreamID);
+  return pstreamID;
 }
 
 
 int pstreamIsPipe(int pstreamID)
 {
-  pstream_t *pstreamptr;
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
 
-  pstreamptr = pstream_to_pointer(pstreamID);
-
-  return (pstreamptr->ispipe);
+  return pstreamptr->ispipe;
 }
 
 
 int pstreamOpenRead(const argument_t *argument)
 {
-  int ispipe = FALSE;
-  int fileID;
-  int pstreamID;
-  pstream_t *pstreamptr;
-
   PSTREAM_INIT();
 
-  pstreamptr = pstream_new_entry();
+  pstream_t *pstreamptr = pstream_new_entry();
   if ( ! pstreamptr ) Error("No memory");
 
-  pstreamID = pstreamptr->self;
+  int pstreamID = pstreamptr->self;
 
-  ispipe = argument->args[0] == '-';
+  int ispipe = argument->args[0] == '-';
   /*
   printf("pstreamOpenRead: args >%s<\n", argument->args);
   for ( int i = 0; i < argument->argc; ++i )
@@ -324,7 +307,7 @@ int pstreamOpenRead(const argument_t *argument)
       char *operatorArg;
       char *operatorName;
       char *newarg;
-      char *pipename = (char*) malloc(16);
+      char *pipename = (char*) Malloc(16);
       int rval;
       pthread_t thrID;
       pthread_attr_t attr;
@@ -332,17 +315,17 @@ int pstreamOpenRead(const argument_t *argument)
       size_t len;
       size_t stacksize;
       int status;
-      argument_t *newargument = (argument_t*) malloc(sizeof(argument_t));
+      argument_t *newargument = (argument_t*) Malloc(sizeof(argument_t));
 
       newargument->argc = argument->argc + 1;
-      newargument->argv = (char **) malloc(newargument->argc*sizeof(char *));
+      newargument->argv = (char **) Malloc(newargument->argc*sizeof(char *));
       memcpy(newargument->argv, argument->argv, argument->argc*sizeof(char *));
 
       operatorArg  = argument->argv[0];
       operatorName = getOperatorName(operatorArg);
 
       len = strlen(argument->args);
-      newarg = (char*) malloc(len+16);
+      newarg = (char*) Malloc(len+16);
       strcpy(newarg, argument->args);
       sprintf(pipename, "(pipe%d.%d)", processSelf() + 1, processInqChildNum() + 1);
       newarg[len] = ' ';
@@ -365,21 +348,21 @@ int pstreamOpenRead(const argument_t *argument)
 	cdoPrint("Started child process \"%s\".", newarg+1);
 
       status = pthread_attr_init(&attr);
-      if ( status ) SysError("pthread_attr_init failed for '%s'\n", newarg+1);
+      if ( status ) SysError("pthread_attr_init failed for '%s'", newarg+1);
       status = pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
-      if ( status ) SysError("pthread_attr_setdetachstate failed for '%s'\n", newarg+1);
+      if ( status ) SysError("pthread_attr_setdetachstate failed for '%s'", newarg+1);
       /*
       param.sched_priority = 0;
       status = pthread_attr_setschedparam(&attr, &param);
-      if ( status ) SysError("pthread_attr_setschedparam failed for '%s'\n", newarg+1);
+      if ( status ) SysError("pthread_attr_setschedparam failed for '%s'", newarg+1);
       */
       /* status = pthread_attr_setinheritsched(&attr, PTHREAD_EXPLICIT_SCHED); */
-      /* if ( status ) SysError("pthread_attr_setinheritsched failed for '%s'\n", newarg+1); */
+      /* if ( status ) SysError("pthread_attr_setinheritsched failed for '%s'", newarg+1); */
 
       pthread_attr_getscope(&attr, &pthreadScope);
 
       /* status = pthread_attr_setscope(&attr, PTHREAD_SCOPE_PROCESS); */
-      /* if ( status ) SysError("pthread_attr_setscope failed for '%s'\n", newarg+1); */
+      /* if ( status ) SysError("pthread_attr_setscope failed for '%s'", newarg+1); */
       /* If system scheduling scope is specified, then the thread is scheduled against all threads in the system */
       /* pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM); */
 
@@ -394,10 +377,10 @@ int pstreamOpenRead(const argument_t *argument)
       if ( rval != 0 )
 	{
 	  errno = rval;
-	  SysError("pthread_create failed for '%s'\n", newarg+1);
+	  SysError("pthread_create failed for '%s'", newarg+1);
 	}
 
-      /* free(operatorName); */
+      /* Free(operatorName); */
       processAddStream(pstreamID);
       /*      pipeInqInfo(pstreamID); */
       if ( PSTREAM_Debug ) Message("pipe %s", pipename);
@@ -407,7 +390,6 @@ int pstreamOpenRead(const argument_t *argument)
     }
   else
     {
-      extern int cdoDefaultFileType/*, cdoDefaultInstID*/;
       size_t len, i;
       int nfiles = 1, j;
       char *filename = NULL;
@@ -422,8 +404,8 @@ int pstreamOpenRead(const argument_t *argument)
 	{
 	  pch = &argument->args[i+1];
 	  len -= (i+1);
-	  if ( len && ( memcmp(argument->args, "filelist:", 9) == 0 || 
-			memcmp(argument->args, "flist:", 6) == 0 ) )
+	  if ( len && ( strncmp(argument->args, "filelist:", 9) == 0 || 
+			strncmp(argument->args, "flist:", 6) == 0 ) )
 	    {
 	      for ( i = 0; i < len; i++ ) if ( pch[i] == ',' ) nfiles++;
 
@@ -454,7 +436,7 @@ int pstreamOpenRead(const argument_t *argument)
 		  if ( nfiles == 0 ) cdoAbort("No imput file found in %s", pch);
 
 		  pstreamptr->mfiles = nfiles;
-		  pstreamptr->mfnames = (char **) malloc(nfiles*sizeof(char *));
+		  pstreamptr->mfnames = (char **) Malloc(nfiles*sizeof(char *));
 		  
 		  rewind(fp);
 
@@ -475,7 +457,7 @@ int pstreamOpenRead(const argument_t *argument)
 		  char line[65536];
 
 		  pstreamptr->mfiles = nfiles;
-		  pstreamptr->mfnames = (char **) malloc(nfiles*sizeof(char *));
+		  pstreamptr->mfnames = (char **) Malloc(nfiles*sizeof(char *));
 		  
 		  strcpy(line, pch);
 		  for ( i = 0; i < len; i++ ) if ( line[i] == ',' ) line[i] = 0;
@@ -488,7 +470,7 @@ int pstreamOpenRead(const argument_t *argument)
 		    }
 		}
 	    }
-	  else if ( len && memcmp(argument->args, "ls:", 3) == 0 )
+	  else if ( len && strncmp(argument->args, "ls:", 3) == 0 )
 	    {
 	      char line[4096];
 	      char command[4096];
@@ -512,7 +494,7 @@ int pstreamOpenRead(const argument_t *argument)
 	      pclose(pfp);
 
 	      pstreamptr->mfiles = nfiles;
-	      pstreamptr->mfnames = (char **) malloc(nfiles*sizeof(char *));
+	      pstreamptr->mfnames = (char **) Malloc(nfiles*sizeof(char *));
 
 	      for ( j = 0; j < nfiles; j++ )
 		pstreamptr->mfnames[j] = fnames[j];
@@ -522,14 +504,14 @@ int pstreamOpenRead(const argument_t *argument)
       if ( pstreamptr->mfiles )
 	{
 	  len = strlen(pstreamptr->mfnames[0]);
-	  filename = (char*) malloc(len+1);
+	  filename = (char*) Malloc(len+1);
 	  strcpy(filename, pstreamptr->mfnames[0]);
 	  pstreamptr->nfiles = 1;
 	}
       else
 	{
 	  len = strlen(argument->args);
-	  filename = (char*) malloc(len+1);
+	  filename = (char*) Malloc(len+1);
 	  strcpy(filename, argument->args);
 	}
 
@@ -541,9 +523,13 @@ int pstreamOpenRead(const argument_t *argument)
       else
 	pthread_mutex_lock(&streamOpenReadMutex);
 #endif
-      fileID = streamOpenRead(filename);
-      if ( fileID < 0 ) cdiError(fileID, "Open failed on >%s<", filename);
-
+      int fileID = streamOpenRead(filename);
+      if ( fileID < 0 )
+        {
+          pstreamptr->isopen = FALSE;
+          cdiOpenError(fileID, "Open failed on >%s<", filename);
+        }
+      
       if ( cdoDefaultFileType == CDI_UNDEFID )
 	cdoDefaultFileType = streamInqFiletype(fileID);
       /*
@@ -563,9 +549,9 @@ int pstreamOpenRead(const argument_t *argument)
       pstreamptr->fileID = fileID;
     }
 
-  if ( pstreamID < 0 ) cdiError(pstreamID, "Open failed on %s", argument->args);
+  if ( pstreamID < 0 ) cdiOpenError(pstreamID, "Open failed on >%s<", argument->args);
   
-  return (pstreamID);
+  return pstreamID;
 }
 
 static
@@ -629,14 +615,12 @@ void query_user_exit(const char *argument)
 
 int pstreamOpenWrite(const argument_t *argument, int filetype)
 {
-  int fileID;
   int pstreamID = -1;
-  int ispipe;
   pstream_t *pstreamptr;
 
   PSTREAM_INIT();
 
-  ispipe = memcmp(argument->args, "(pipe", 5) == 0;
+  int ispipe = strncmp(argument->args, "(pipe", 5) == 0;
 
   if ( ispipe )
     {
@@ -654,8 +638,7 @@ int pstreamOpenWrite(const argument_t *argument, int filetype)
     }
   else
     {
-      /* extern int cdoDefaultInstID; */
-      char *filename = (char*) malloc(strlen(argument->args)+1);
+      char *filename = (char*) Malloc(strlen(argument->args)+1);
 
       pstreamptr = pstream_new_entry();
       if ( ! pstreamptr ) Error("No memory");
@@ -683,7 +666,7 @@ int pstreamOpenWrite(const argument_t *argument, int filetype)
       else
 	pthread_mutex_lock(&streamOpenWriteMutex);
 #endif
-      fileID = streamOpenWrite(argument->args, filetype);
+      int fileID = streamOpenWrite(argument->args, filetype);
 #if defined(HAVE_LIBPTHREAD)
       if ( cdoLockIO )
 	pthread_mutex_unlock(&streamMutex);
@@ -691,7 +674,7 @@ int pstreamOpenWrite(const argument_t *argument, int filetype)
 	pthread_mutex_unlock(&streamOpenWriteMutex);
 #endif
       if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_write);
-      if ( fileID < 0 ) cdiError(fileID, "Open failed on %s", argument->args);
+      if ( fileID < 0 ) cdiOpenError(fileID, "Open failed on >%s<", argument->args);
 
       cdoDefHistory(fileID, commandLine());
 
@@ -719,13 +702,13 @@ int pstreamOpenWrite(const argument_t *argument, int filetype)
 
 	  if ( cdoCompType == COMPRESS_SZIP &&
 	       (filetype != FILETYPE_GRB && filetype != FILETYPE_GRB2 && filetype != FILETYPE_NC4 && filetype != FILETYPE_NC4C) )
-	    cdoWarning("SZIP compression not available for non GRIB/netCDF4 data!");
+	    cdoWarning("SZIP compression not available for non GRIB/NetCDF4 data!");
 
 	  if ( cdoCompType == COMPRESS_JPEG && filetype != FILETYPE_GRB2 )
 	    cdoWarning("JPEG compression not available for non GRIB2 data!");
 
 	  if ( cdoCompType == COMPRESS_ZIP && (filetype != FILETYPE_NC4 && filetype != FILETYPE_NC4C) )
-	    cdoWarning("Deflate compression not available for non netCDF4 data!");
+	    cdoWarning("Deflate compression not available for non NetCDF4 data!");
 	}
       /*
       if ( cdoDefaultInstID != CDI_UNDEFID )
@@ -739,18 +722,15 @@ int pstreamOpenWrite(const argument_t *argument, int filetype)
       pstreamptr->filetype = filetype;
    }
 
-  return (pstreamID);
+  return pstreamID;
 }
 
 
 int pstreamOpenAppend(const argument_t *argument)
 {
-  int fileID;
   int pstreamID = -1;
-  int ispipe;
-  pstream_t *pstreamptr;
 
-  ispipe = memcmp(argument->args, "(pipe", 5) == 0;
+  int ispipe = strncmp(argument->args, "(pipe", 5) == 0;
 
   if ( ispipe )
     {
@@ -759,9 +739,9 @@ int pstreamOpenAppend(const argument_t *argument)
     }
   else
     {
-      char *filename = (char*) malloc(strlen(argument->args)+1);
+      char *filename = (char*) Malloc(strlen(argument->args)+1);
 
-      pstreamptr = pstream_new_entry();
+      pstream_t *pstreamptr = pstream_new_entry();
       if ( ! pstreamptr ) Error("No memory");
 
       pstreamID = pstreamptr->self;
@@ -775,7 +755,7 @@ int pstreamOpenAppend(const argument_t *argument)
       else
 	pthread_mutex_lock(&streamOpenReadMutex);
 #endif
-      fileID = streamOpenAppend(argument->args);
+      int fileID = streamOpenAppend(argument->args);
 #if defined(HAVE_LIBPTHREAD)
       if ( cdoLockIO )
 	pthread_mutex_unlock(&streamMutex);
@@ -783,7 +763,7 @@ int pstreamOpenAppend(const argument_t *argument)
 	pthread_mutex_unlock(&streamOpenReadMutex);
 #endif
       if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_write);
-      if ( fileID < 0 ) cdiError(fileID, "Open failed on %s", argument->args);
+      if ( fileID < 0 ) cdiOpenError(fileID, "Open failed on >%s<", argument->args);
       /*
       cdoInqHistory(fileID);
       cdoDefHistory(fileID, commandLine());
@@ -795,15 +775,13 @@ int pstreamOpenAppend(const argument_t *argument)
       pstreamptr->fileID = fileID;
     }
 
-  return (pstreamID);
+  return pstreamID;
 }
 
 
 void pstreamClose(int pstreamID)
 {
-  pstream_t *pstreamptr;
-
-  pstreamptr = pstream_to_pointer(pstreamID);
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
 
   if ( pstreamptr == NULL )
     Error("Internal problem, stream %d not open!", pstreamID);
@@ -839,13 +817,13 @@ void pstreamClose(int pstreamID)
 	  pthread_join(pstreamptr->wthreadID, NULL);
 
 	  pthread_mutex_lock(pipe->mutex);
-	  if ( pstreamptr->name ) free(pstreamptr->name);
+	  if ( pstreamptr->name ) Free(pstreamptr->name);
 	  if ( pstreamptr->argument )
 	    {
 	      argument_t *argument = (argument_t *) (pstreamptr->argument);
-	      if ( argument->argv ) free(argument->argv);
-	      if ( argument->args ) free(argument->args);
-	      free(argument);
+	      if ( argument->argv ) Free(argument->argv);
+	      if ( argument->args ) Free(argument->args);
+	      Free(argument);
 	    }
 	  vlistDestroy(pstreamptr->vlistID);
 	  pthread_mutex_unlock(pipe->mutex);
@@ -882,7 +860,7 @@ void pstreamClose(int pstreamID)
   else
     {
       if ( PSTREAM_Debug )
-	Message("%s fileID %d\n", pstreamptr->name, pstreamptr->fileID);
+	Message("%s fileID %d", pstreamptr->name, pstreamptr->fileID);
 
       if ( pstreamptr->mode == 'r' )
 	{
@@ -910,13 +888,13 @@ void pstreamClose(int pstreamID)
 
       if ( pstreamptr->name )
 	{
-	  free(pstreamptr->name);
+	  Free(pstreamptr->name);
 	  pstreamptr->name = NULL;
 	}
 
       if ( pstreamptr->varlist )
 	{
-	  free(pstreamptr->varlist);
+	  Free(pstreamptr->varlist);
 	  pstreamptr->varlist = NULL;
 	}
 
@@ -927,10 +905,9 @@ void pstreamClose(int pstreamID)
 
 int pstreamInqVlist(int pstreamID)
 {
-  int vlistID;
-  pstream_t *pstreamptr;
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
 
-  pstreamptr = pstream_to_pointer(pstreamID);
+  int vlistID = -1;
 
 #if defined(HAVE_LIBPTHREAD)
   if ( pstreamptr->ispipe )
@@ -942,8 +919,6 @@ int pstreamInqVlist(int pstreamID)
   else
 #endif
     {
-      extern int cdoDefaultTimeType;
-
       if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_read);
 #if defined(HAVE_LIBPTHREAD)
       if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
@@ -954,6 +929,10 @@ int pstreamInqVlist(int pstreamID)
 #endif
       if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_read);
 
+      int nsubtypes = vlistNsubtypes(vlistID);
+      if ( nsubtypes > 0 )
+        cdoWarning("Subtypes are unsupported, the processing results are possibly wrong!");
+
       if ( cdoDefaultTimeType != CDI_UNDEFID )
 	taxisDefType(vlistInqTaxis(vlistID), cdoDefaultTimeType);
 
@@ -968,39 +947,13 @@ int pstreamInqVlist(int pstreamID)
 
   processDefVarNum(vlistNvars(vlistID), pstreamID);
 
-  return (vlistID);
-}
-
-static
-const char *cdoComment(void)
-{
-  static char comment[256];
-  static int init = 0;
-  int size = 0;
-  extern char CDO_Version[];
-
-  if ( ! init )
-    {
-      init = 1;
-
-      size = strlen(CDO_Version);
-
-      strncat(comment, CDO_Version, size);
-      comment[size] = 0;
-    }
-
-  return (comment);
+  return vlistID;
 }
 
 static
 void pstreamDefVarlist(pstream_t *pstreamptr, int vlistID)
 {
-  int varID, nvars;
-  int laddoffset, lscalefactor;
-  int datatype, filetype;
-  varlist_t *varlist;
-
-  filetype = pstreamptr->filetype;
+  int filetype = pstreamptr->filetype;
 
   if ( pstreamptr->vlistID != -1 )
     cdoAbort("Internal problem, vlist already defined!");
@@ -1008,10 +961,10 @@ void pstreamDefVarlist(pstream_t *pstreamptr, int vlistID)
   if ( pstreamptr->varlist != NULL )
     cdoAbort("Internal problem, varlist already allocated!");
 
-  nvars = vlistNvars(vlistID);
-  varlist = (varlist_t*) malloc(nvars*sizeof(varlist_t));
+  int nvars = vlistNvars(vlistID);
+  varlist_t *varlist = (varlist_t*) Malloc(nvars*sizeof(varlist_t));
 
-  for ( varID = 0; varID < nvars; ++varID )
+  for ( int varID = 0; varID < nvars; ++varID )
     {
       varlist[varID].gridsize    = gridInqSize(vlistInqVarGrid(vlistID, varID));
       varlist[varID].datatype    = vlistInqVarDatatype(vlistID, varID);
@@ -1021,10 +974,10 @@ void pstreamDefVarlist(pstream_t *pstreamptr, int vlistID)
 
       varlist[varID].check_datarange = FALSE;
 
-      laddoffset   = IS_NOT_EQUAL(varlist[varID].addoffset, 0);
-      lscalefactor = IS_NOT_EQUAL(varlist[varID].scalefactor, 1);
+      int laddoffset   = IS_NOT_EQUAL(varlist[varID].addoffset, 0);
+      int lscalefactor = IS_NOT_EQUAL(varlist[varID].scalefactor, 1);
 
-      datatype = varlist[varID].datatype;
+      int datatype = varlist[varID].datatype;
 
       if ( filetype == FILETYPE_NC || filetype == FILETYPE_NC2 || filetype == FILETYPE_NC4 || filetype == FILETYPE_NC4C )
 	{
@@ -1062,9 +1015,7 @@ void pstreamDefVarlist(pstream_t *pstreamptr, int vlistID)
 
 void pstreamDefVlist(int pstreamID, int vlistID)
 {
-  pstream_t *pstreamptr;
-
-  pstreamptr = pstream_to_pointer(pstreamID);
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
 
 #if defined(HAVE_LIBPTHREAD)
   if ( pstreamptr->ispipe )
@@ -1101,7 +1052,14 @@ void pstreamDefVlist(int pstreamID, int vlistID)
 	    vlistDefVarChunkType(vlistID, varID, cdoChunkType);
 	}
 
-      vlistDefAttTxt(vlistID, CDI_GLOBAL, "CDO", (int)strlen(cdoComment())+1, cdoComment());
+      if ( CDO_CMOR_Mode )
+        {
+          cdo_def_tracking_id(vlistID, "tracking_id");
+          cdo_def_creation_date(vlistID);
+        }
+
+      if ( CDO_Version_Info )
+        vlistDefAttTxt(vlistID, CDI_GLOBAL, "CDO", (int)strlen(cdoComment())+1, cdoComment());
 
 #if defined(_OPENMP)
       if ( ompNumThreads > 1 )
@@ -1145,7 +1103,7 @@ int pstreamInqRecord(int pstreamID, int *varID, int *levelID)
       if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_read);
     }
 
-  return (0);
+  return 0;
 }
 
 
@@ -1180,11 +1138,9 @@ void pstreamDefRecord(int pstreamID, int varID, int levelID)
 
 void pstreamReadRecord(int pstreamID, double *data, int *nmiss)
 {
-  pstream_t *pstreamptr;
-
   if ( data == NULL ) cdoAbort("Data pointer not allocated (pstreamReadRecord)!");
 
-  pstreamptr = pstream_to_pointer(pstreamID);
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
 
 #if defined(HAVE_LIBPTHREAD)
   if ( pstreamptr->ispipe )
@@ -1205,23 +1161,46 @@ void pstreamReadRecord(int pstreamID, double *data, int *nmiss)
 }
 
 
+void pstreamReadRecordF(int pstreamID, float *data, int *nmiss)
+{
+  if ( data == NULL ) cdoAbort("Data pointer not allocated (pstreamReadRecord)!");
+
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
+
+#if defined(HAVE_LIBPTHREAD)
+  if ( pstreamptr->ispipe )
+    {
+      cdoAbort("pipeReadRecord not implemented for memtype float!");
+      // pipeReadRecord(pstreamptr, data, nmiss);
+    }
+  else
+#endif
+    {
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_read);
+#if defined(HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_lock(&streamMutex);
+#endif
+      streamReadRecordF(pstreamptr->fileID, data, nmiss);
+#if defined(HAVE_LIBPTHREAD)
+      if ( cdoLockIO ) pthread_mutex_unlock(&streamMutex);
+#endif
+      if ( processNums() == 1 && ompNumThreads == 1 ) timer_stop(timer_read);
+    }
+}
+
+
 void pstreamCheckDatarange(pstream_t *pstreamptr, int varID, double *array, int nmiss)
 {
-  long i, ivals, gridsize;
-  int datatype;
-  double missval, addoffset, scalefactor;
-  double arrmin, arrmax, smin, smax;
-  double vmin = 0, vmax = 0;
+  long i;
+  long   gridsize    = pstreamptr->varlist[varID].gridsize;
+  int    datatype    = pstreamptr->varlist[varID].datatype;
+  double missval     = pstreamptr->varlist[varID].missval;
+  double addoffset   = pstreamptr->varlist[varID].addoffset;
+  double scalefactor = pstreamptr->varlist[varID].scalefactor;
 
-  gridsize    = pstreamptr->varlist[varID].gridsize;
-  datatype    = pstreamptr->varlist[varID].datatype;
-  missval     = pstreamptr->varlist[varID].missval;
-  addoffset   = pstreamptr->varlist[varID].addoffset;
-  scalefactor = pstreamptr->varlist[varID].scalefactor;
-
-  ivals   = 0;
-  arrmin  =  1.e300;
-  arrmax  = -1.e300;
+  long ivals   = 0;
+  double arrmin  =  1.e300;
+  double arrmax  = -1.e300;
   if ( nmiss > 0 )
     {
       for ( i = 0; i < gridsize; ++i )
@@ -1246,8 +1225,8 @@ void pstreamCheckDatarange(pstream_t *pstreamptr, int varID, double *array, int
 
   if ( ivals > 0 )
     {
-      smin = (arrmin - addoffset)/scalefactor;
-      smax = (arrmax - addoffset)/scalefactor;
+      double smin = (arrmin - addoffset)/scalefactor;
+      double smax = (arrmax - addoffset)/scalefactor;
 
       if ( datatype == DATATYPE_INT8  || datatype == DATATYPE_UINT8 ||
 	   datatype == DATATYPE_INT16 || datatype == DATATYPE_UINT16 )
@@ -1256,6 +1235,8 @@ void pstreamCheckDatarange(pstream_t *pstreamptr, int varID, double *array, int
 	  smax = (int) round(smax);
 	}
 
+      double vmin = 0, vmax = 0;
+
       if      ( datatype == DATATYPE_INT8   ) { vmin =        -128.; vmax =        127.; }
       else if ( datatype == DATATYPE_UINT8  ) { vmin =           0.; vmax =        255.; }
       else if ( datatype == DATATYPE_INT16  ) { vmin =      -32768.; vmax =      32767.; }
@@ -1276,11 +1257,9 @@ void pstreamCheckDatarange(pstream_t *pstreamptr, int varID, double *array, int
 
 void pstreamWriteRecord(int pstreamID, double *data, int nmiss)
 {
-  pstream_t *pstreamptr;
+  if ( data == NULL ) cdoAbort("Data pointer not allocated (%s)!", __func__);
 
-  if ( data == NULL ) cdoAbort("Data pointer not allocated (pstreamWriteRecord)!");
-
-  pstreamptr = pstream_to_pointer(pstreamID);
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
 
 #if defined(HAVE_LIBPTHREAD)
   if ( pstreamptr->ispipe )
@@ -1312,11 +1291,9 @@ void pstreamWriteRecord(int pstreamID, double *data, int nmiss)
 
 void pstreamWriteRecordF(int pstreamID, float *data, int nmiss)
 {
-  pstream_t *pstreamptr;
+  if ( data == NULL ) cdoAbort("Data pointer not allocated (%s)!", __func__);
 
-  if ( data == NULL ) cdoAbort("Data pointer not allocated (pstreamWriteRecord)!");
-
-  pstreamptr = pstream_to_pointer(pstreamID);
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
 
 #if defined(HAVE_LIBPTHREAD)
   if ( pstreamptr->ispipe )
@@ -1348,10 +1325,9 @@ void pstreamWriteRecordF(int pstreamID, float *data, int nmiss)
 
 int pstreamInqTimestep(int pstreamID, int tsID)
 {
-  int nrecs = 0;
-  pstream_t *pstreamptr;
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
 
-  pstreamptr = pstream_to_pointer(pstreamID);
+  int nrecs = 0;
 
 #if defined(HAVE_LIBPTHREAD)
   if ( pstreamptr->ispipe )
@@ -1359,8 +1335,6 @@ int pstreamInqTimestep(int pstreamID, int tsID)
   else
 #endif
     {
-      extern int cdoDefaultTimeType;
-
       if ( pstreamptr->mfiles ) tsID -= pstreamptr->tsID0;
 
       if ( processNums() == 1 && ompNumThreads == 1 ) timer_start(timer_read);
@@ -1388,7 +1362,7 @@ int pstreamInqTimestep(int pstreamID, int tsID)
 	  streamClose(pstreamptr->fileID);
 
 	  len = strlen(pstreamptr->mfnames[nfile]);
-	  filename = (char*) malloc(len+1);
+	  filename = (char*) Malloc(len+1);
 	  strcpy(filename, pstreamptr->mfnames[nfile]);
 	  pstreamptr->nfiles++;
 
@@ -1413,9 +1387,9 @@ int pstreamInqTimestep(int pstreamID, int tsID)
 	  else
 	    pthread_mutex_unlock(&streamOpenReadMutex);
 #endif
-	  if ( fileID < 0 ) cdiError(fileID, "Open failed on >%s<", filename);
+	  if ( fileID < 0 ) cdiOpenError(fileID, "Open failed on >%s<", filename);
 
-	  free(pstreamptr->name);
+	  Free(pstreamptr->name);
 
 	  pstreamptr->name   = filename;
 	  pstreamptr->fileID = fileID;
@@ -1441,15 +1415,13 @@ int pstreamInqTimestep(int pstreamID, int tsID)
       pstreamptr->tsID = tsID;
     }
 
-  return (nrecs);
+  return nrecs;
 }
 
 
 void pstreamDefTimestep(int pstreamID, int tsID)
 {
-  pstream_t *pstreamptr;
-
-  pstreamptr = pstream_to_pointer(pstreamID);
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
 
 #if defined(HAVE_LIBPTHREAD)
   if ( pstreamptr->ispipe )
@@ -1457,7 +1429,6 @@ void pstreamDefTimestep(int pstreamID, int tsID)
   else
 #endif
     {
-      extern int cdoDefaultTimeType;
       if ( tsID == 0 && cdoDefaultTimeType != CDI_UNDEFID )
 	{
 	  int taxisID, vlistID;
@@ -1483,13 +1454,15 @@ void pstreamDefTimestep(int pstreamID, int tsID)
 
 void pstreamCopyRecord(int pstreamIDdest, int pstreamIDsrc)
 {
-  pstream_t *pstreamptr_dest, *pstreamptr_src;
-
   if ( PSTREAM_Debug )
     Message("pstreamIDdest = %d  pstreamIDsrc = %d", pstreamIDdest, pstreamIDsrc);
 
-  pstreamptr_dest = pstream_to_pointer(pstreamIDdest);
-  pstreamptr_src  = pstream_to_pointer(pstreamIDsrc);
+  pstream_t *pstreamptr_dest = pstream_to_pointer(pstreamIDdest);
+  pstream_t *pstreamptr_src  = pstream_to_pointer(pstreamIDsrc);
+
+  if ( pstreamptr_dest->ispipe || pstreamptr_src->ispipe )
+    cdoAbort("This operator can't be combined with other operators!");
+
   /*
 #if defined(HAVE_LIBPTHREAD)
   if ( pstreamptr_dest->ispipe || pstreamptr_src->ispipe )
@@ -1573,7 +1546,6 @@ void processClosePipes(void)
 void cdoFinish(void)
 {
   int processID = processSelf();
-  INT64 nvals;
   int nvars, ntimesteps;
   char memstring[32] = {""};
   double s_utime, s_stime;
@@ -1586,7 +1558,7 @@ void cdoFinish(void)
     Message("process %d  thread %ld", processID, pthread_self());
 #endif
 
-  nvals = processInqNvals(processID);
+  int64_t nvals = processInqNvals(processID);
   nvars = processInqVarNum();
   ntimesteps = processInqTimesteps();
 
@@ -1597,13 +1569,13 @@ void cdoFinish(void)
       reset_text_color(stderr);
       if ( nvals > 0 )
 	{
-	  if ( sizeof(INT64) > sizeof(long) )
+	  if ( sizeof(int64_t) > sizeof(long) )
 #if defined(_WIN32)
 	    fprintf(stderr, "Processed %I64d value%s from %d variable%s",
 #else
 	    fprintf(stderr, "Processed %jd value%s from %d variable%s",
 #endif
-		    nvals, ADD_PLURAL(nvals), nvars, ADD_PLURAL(nvars));
+		    (intmax_t) nvals, ADD_PLURAL(nvals), nvars, ADD_PLURAL(nvars));
 	  else
 	    fprintf(stderr, "Processed %ld value%s from %d variable%s",
 		    (long) nvals, ADD_PLURAL(nvals), nvars, ADD_PLURAL(nvars));
@@ -1648,7 +1620,6 @@ void cdoFinish(void)
       int mu[] = {'b', 'k', 'm', 'g', 't'};
       int muindex = 0;
       long memmax;
-      extern int cdoLogOff;
 
       memmax = memTotal();
       while ( memmax > 9999 )
@@ -1695,11 +1666,10 @@ void cdoFinish(void)
 
 int pstreamInqFiletype(int pstreamID)
 {
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
+
   int filetype;
-  pstream_t *pstreamptr;
-
-  pstreamptr = pstream_to_pointer(pstreamID);
-
+  
 #if defined(HAVE_LIBPTHREAD)
   if ( pstreamptr->ispipe )
     filetype = pstreamptr->filetype;
@@ -1707,16 +1677,15 @@ int pstreamInqFiletype(int pstreamID)
 #endif
     filetype = streamInqFiletype(pstreamptr->fileID);
 
-  return (filetype);
+  return filetype;
 }
 
 
 int pstreamInqByteorder(int pstreamID)
 {
-  int byteorder;
-  pstream_t *pstreamptr;
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
 
-  pstreamptr = pstream_to_pointer(pstreamID);
+  int byteorder;
 
 #if defined(HAVE_LIBPTHREAD)
   if ( pstreamptr->ispipe )
@@ -1725,19 +1694,26 @@ int pstreamInqByteorder(int pstreamID)
 #endif
     byteorder = streamInqByteorder(pstreamptr->fileID);
 
-  return (byteorder);
+  return byteorder;
 }
 
+ 
 void pstreamInqGRIBinfo(int pstreamID, int *intnum, float *fltnum, off_t *bignum)
 {
-  pstream_t *pstreamptr;
-
-  pstreamptr = pstream_to_pointer(pstreamID);
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
 
   streamInqGRIBinfo(pstreamptr->fileID, intnum, fltnum, bignum);
 }
 
 
+int pstreamFileID(int pstreamID)
+{
+  pstream_t *pstreamptr = pstream_to_pointer(pstreamID);
+
+  return pstreamptr->fileID;
+}
+
+
 void cdoVlistCopyFlag(int vlistID2, int vlistID1)
 {
 #if defined(HAVE_LIBPTHREAD)
