@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -34,7 +34,7 @@ int readline(FILE *fp, char *line, int len)
     }
   line[ipos] = 0;
 
-  if ( feof(fp) && ipos == 0 ) return (0);
+  if ( feof(fp) && ipos == 0 ) return 0;
 
   return (1);
 }
