@@ -16,17 +16,14 @@
 
 
 void remapgrid_init(remapgrid_t *grid);
-void remapGridRealloc(int map_type, remapgrid_t *grid);
+void remapgrid_alloc(int map_type, remapgrid_t *grid);
 
 
 #if defined(HAVE_LIBNETCDF)
 static
 void nce(int istat)
 {
-  /*
-    This routine provides a simple interface to netCDF error message routine.
-  */
-
+  // This routine provides a simple interface to NetCDF error message routine.
   if ( istat != NC_NOERR ) cdoAbort(nc_strerror(istat));
 }
 #endif
@@ -35,20 +32,17 @@ void nce(int istat)
 void write_remap_scrip(const char *interp_file, int map_type, int submap_type, int num_neighbors,
 		       int remap_order, remapgrid_t src_grid, remapgrid_t tgt_grid, remapvars_t rv)
 {
-  /*
-    Writes remap data to a netCDF file using SCRIP conventions
-  */
+  // Writes remap data to a NetCDF file using SCRIP conventions
   /*
     Input variables:
 
     interp_file  ! filename for remap data
   */
-
 #if defined(HAVE_LIBNETCDF)
 
-  /* Local variables */
+  // Local variables
 
-  int nc_file_id;           /* id for netCDF file                       */
+  int nc_file_id;           /* id for NetCDF file                       */
   int nc_srcgrdsize_id;     /* id for source grid size                  */
   int nc_dstgrdsize_id;     /* id for destination grid size             */
   int nc_srcgrdcorn_id = 0; /* id for number of source grid corners     */
@@ -77,20 +71,16 @@ void write_remap_scrip(const char *interp_file, int map_type, int submap_type, i
   int nc_dstadd_id;         /* id for map destination address           */
   int nc_rmpmatrix_id;      /* id for remapping matrix                  */
 
-  int nc_dims2_id[2];       /* netCDF ids for 2d array dims             */
+  int nc_dims2_id[2];       /* NetCDF ids for 2d array dims             */
 
-  char *map_name = "SCRIP remapping with CDO";
+  const char *map_name = "SCRIP remapping with CDO";
   char normalize_opt[64] = "unknown";
   char map_method[64] = "unknown";
   char tmp_string[64] = "unknown";
-  char history[1024] = "date and time";
   char src_grid_name[64] = "source grid";
   char tgt_grid_name[64] = "dest grid";
-  char *src_grid_units = "radians";
-  char *tgt_grid_units = "radians";
-  time_t date_and_time_in_sec;
-  struct tm *date_and_time;
-  long i;
+  const char *src_grid_units = "radians";
+  const char *tgt_grid_units = "radians";
   int lgridarea = FALSE;
   int writemode = NC_CLOBBER;
 
@@ -149,25 +139,23 @@ void write_remap_scrip(const char *interp_file, int map_type, int submap_type, i
       break;
     }
 
+  /*
   if ( rv.num_links == 0 )
     cdoAbort("Number of remap links is 0, no remap weights found!");
-
+  */
   {
-    size_t filesize;
-    size_t nele1, nele2;
-
-    nele1 = 4*8 + 4;
-    nele2 = 4*8 + 4;
+    size_t nele1 = 4*8 + 4;
+    size_t nele2 = 4*8 + 4;
     if ( src_grid.lneed_cell_corners ) nele1 += src_grid.num_cell_corners*2*8;
     if ( tgt_grid.lneed_cell_corners ) nele2 += tgt_grid.num_cell_corners*2*8;
-    filesize = src_grid.size*(nele1) +
-               tgt_grid.size*(nele2) +
-               rv.num_links*(4 + 4 + rv.num_wts*8);
+    size_t filesize = src_grid.size*(nele1) +
+                      tgt_grid.size*(nele2) +
+                      rv.num_links*(4 + 4 + rv.num_wts*8);
 
     if ( cdoVerbose )
       cdoPrint("Filesize for remap weights: ~%lu", (unsigned long) filesize);
     
-    if ( filesize > 0x7FFFFC00 ) /* 2**31 - 1024 (<2GB) */
+    if ( filesize > 0x7FFFFC00 ) // 2**31 - 1024 (<2GB)
       {
 #if defined(NC_64BIT_OFFSET)
 	writemode = NC_CLOBBER | NC_64BIT_OFFSET;
@@ -177,80 +165,82 @@ void write_remap_scrip(const char *interp_file, int map_type, int submap_type, i
       }
   }
 
-  /* Create netCDF file for mapping and define some global attributes */
+  // Create NetCDF file for mapping and define some global attributes
   nce(nc_create(interp_file, writemode, &nc_file_id));
 
-  /* Map name */
+  // Map name
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "title", strlen(map_name), map_name));
 
-  /* Normalization option */
+  // Normalization option
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "normalization", strlen(normalize_opt), normalize_opt));
 
-  /* Map method */
+  // Map method
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "map_method", strlen(map_method), map_method));
 
-  /* Remap order */
+  // Remap order
   if ( map_type == MAP_TYPE_CONSERV && submap_type == SUBMAP_TYPE_NONE )
     nce(nc_put_att_int(nc_file_id, NC_GLOBAL, "remap_order", NC_INT, 1L, &remap_order));
 
-  /* History */
-  date_and_time_in_sec = time(NULL);
-
-  if ( date_and_time_in_sec != -1 )
-    {
-      date_and_time = localtime(&date_and_time_in_sec);
-      (void) strftime(history, 1024, "%d %b %Y : ", date_and_time);
-      strcat(history, commandLine());
-    }
-
-  nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "history", strlen(history), history));
-
-  /* File convention */
+  // File convention
   strcpy(tmp_string, "SCRIP");
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "conventions", strlen(tmp_string), tmp_string));
 
-  /* Source and destination grid names */
+  // Source and destination grid names
   gridName(gridInqType(src_grid.gridID), src_grid_name);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "source_grid", strlen(src_grid_name), src_grid_name));
 
   gridName(gridInqType(tgt_grid.gridID), tgt_grid_name);
   nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "dest_grid", strlen(tgt_grid_name), tgt_grid_name));
 
-  /* Prepare netCDF dimension info */
+  // History
+  time_t date_and_time_in_sec = time(NULL);
+  if ( date_and_time_in_sec != -1 )
+    {
+      char history[1024] = "date and time";
+      struct tm *date_and_time = localtime(&date_and_time_in_sec);
+      (void) strftime(history, 1024, "%d %b %Y : ", date_and_time);
+      strcat(history, commandLine());
+      nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "history", strlen(history), history));
+    }
 
-  /* Define grid size dimensions */
+  if ( CDO_Version_Info )
+    nce(nc_put_att_text(nc_file_id, NC_GLOBAL, "CDO", (int)strlen(cdoComment())+1, cdoComment()));
+
+  // Prepare NetCDF dimension info
+
+  // Define grid size dimensions
   nce(nc_def_dim(nc_file_id, "src_grid_size", src_grid.size, &nc_srcgrdsize_id));
   nce(nc_def_dim(nc_file_id, "dst_grid_size", tgt_grid.size, &nc_dstgrdsize_id));
 
-  /* Define grid corner dimension */
+  // Define grid corner dimension
   if ( src_grid.lneed_cell_corners )
     nce(nc_def_dim(nc_file_id, "src_grid_corners", src_grid.num_cell_corners, &nc_srcgrdcorn_id));
   if ( tgt_grid.lneed_cell_corners )
     nce(nc_def_dim(nc_file_id, "dst_grid_corners", tgt_grid.num_cell_corners, &nc_dstgrdcorn_id));
 
-  /* Define grid rank dimension */
+  // Define grid rank dimension
   nce(nc_def_dim(nc_file_id, "src_grid_rank", src_grid.rank, &nc_srcgrdrank_id));
   nce(nc_def_dim(nc_file_id, "dst_grid_rank", tgt_grid.rank, &nc_dstgrdrank_id));
 
-  /* Define map size dimensions */
+  // Define map size dimensions
   nce(nc_def_dim(nc_file_id, "num_links", rv.num_links, &nc_numlinks_id));
   nce(nc_def_dim(nc_file_id, "num_wgts", rv.num_wts, &nc_numwgts_id));
        
-  /* Define grid dimensions */
+  // Define grid dimensions
   nce(nc_def_var(nc_file_id, "src_grid_dims", NC_INT, 1, &nc_srcgrdrank_id, &nc_srcgrddims_id));
   nce(nc_def_var(nc_file_id, "dst_grid_dims", NC_INT, 1, &nc_dstgrdrank_id, &nc_dstgrddims_id));
 
-  /* Define all arrays for netCDF descriptors */
+  // Define all arrays for NetCDF descriptors
 
-  /* Define grid center latitude array */
+  // Define grid center latitude array
   nce(nc_def_var(nc_file_id, "src_grid_center_lat", NC_DOUBLE, 1, &nc_srcgrdsize_id, &nc_srcgrdcntrlat_id));
   nce(nc_def_var(nc_file_id, "dst_grid_center_lat", NC_DOUBLE, 1, &nc_dstgrdsize_id, &nc_dstgrdcntrlat_id));
 
-  /* Define grid center longitude array */
+  // Define grid center longitude array
   nce(nc_def_var(nc_file_id, "src_grid_center_lon", NC_DOUBLE, 1, &nc_srcgrdsize_id, &nc_srcgrdcntrlon_id));
   nce(nc_def_var(nc_file_id, "dst_grid_center_lon", NC_DOUBLE, 1, &nc_dstgrdsize_id, &nc_dstgrdcntrlon_id));
 
-  /* Define grid corner lat/lon arrays */
+  // Define grid corner lat/lon arrays
 
   nc_dims2_id[0] = nc_srcgrdsize_id;
   nc_dims2_id[1] = nc_srcgrdcorn_id;
@@ -270,7 +260,8 @@ void write_remap_scrip(const char *interp_file, int map_type, int submap_type, i
       nce(nc_def_var(nc_file_id, "dst_grid_corner_lon", NC_DOUBLE, 2, nc_dims2_id, &nc_dstgrdcrnrlon_id));
     }
 
-  /* Define units for all coordinate arrays */
+  // Define units for all coordinate arrays
+
   nce(nc_put_att_text(nc_file_id, nc_srcgrdcntrlat_id, "units", strlen(src_grid_units), src_grid_units));
   nce(nc_put_att_text(nc_file_id, nc_dstgrdcntrlat_id, "units", strlen(tgt_grid_units), tgt_grid_units));
   nce(nc_put_att_text(nc_file_id, nc_srcgrdcntrlon_id, "units", strlen(src_grid_units), src_grid_units));
@@ -286,7 +277,7 @@ void write_remap_scrip(const char *interp_file, int map_type, int submap_type, i
       nce(nc_put_att_text(nc_file_id, nc_dstgrdcrnrlon_id, "units", strlen(tgt_grid_units), tgt_grid_units));
     }
 
-  /* Define grid mask */
+  // Define grid mask
 
   nce(nc_def_var(nc_file_id, "src_grid_imask", NC_INT, 1, &nc_srcgrdsize_id, &nc_srcgrdimask_id));
   nce(nc_put_att_text(nc_file_id, nc_srcgrdimask_id, "units", 8, "unitless"));
@@ -294,7 +285,7 @@ void write_remap_scrip(const char *interp_file, int map_type, int submap_type, i
   nce(nc_def_var(nc_file_id, "dst_grid_imask", NC_INT, 1, &nc_dstgrdsize_id, &nc_dstgrdimask_id));
   nce(nc_put_att_text(nc_file_id, nc_dstgrdimask_id, "units", 8, "unitless"));
 
-  /* Define grid area arrays */
+  // Define grid area arrays
 
   if ( lgridarea )
     {
@@ -305,7 +296,7 @@ void write_remap_scrip(const char *interp_file, int map_type, int submap_type, i
       nce(nc_put_att_text(nc_file_id, nc_dstgrdarea_id, "units", 14, "square radians"));
     }
 
-  /* Define grid fraction arrays */
+  // Define grid fraction arrays
 
   nce(nc_def_var(nc_file_id, "src_grid_frac", NC_DOUBLE, 1, &nc_srcgrdsize_id, &nc_srcgrdfrac_id));
   nce(nc_put_att_text(nc_file_id, nc_srcgrdfrac_id, "units", 8, "unitless"));
@@ -313,7 +304,7 @@ void write_remap_scrip(const char *interp_file, int map_type, int submap_type, i
   nce(nc_def_var(nc_file_id, "dst_grid_frac", NC_DOUBLE, 1, &nc_dstgrdsize_id, &nc_dstgrdfrac_id));
   nce(nc_put_att_text(nc_file_id, nc_dstgrdfrac_id, "units", 8, "unitless"));
 
-  /* Define mapping arrays */
+  // Define mapping arrays
 
   nce(nc_def_var(nc_file_id, "src_address", NC_INT, 1, &nc_numlinks_id, &nc_srcadd_id));      
   nce(nc_def_var(nc_file_id, "dst_address", NC_INT, 1, &nc_numlinks_id, &nc_dstadd_id));
@@ -323,12 +314,12 @@ void write_remap_scrip(const char *interp_file, int map_type, int submap_type, i
 
   nce(nc_def_var(nc_file_id, "remap_matrix", NC_DOUBLE, 2, nc_dims2_id, &nc_rmpmatrix_id));
 
-  /* End definition stage */
+  // End definition stage
   
   nce(nc_enddef(nc_file_id));
 
 
-  /* Write mapping data */
+  // Write mapping data
 
   nce(nc_put_var_int(nc_file_id, nc_srcgrddims_id, src_grid.dims));
   nce(nc_put_var_int(nc_file_id, nc_dstgrddims_id, tgt_grid.dims));
@@ -369,33 +360,31 @@ void write_remap_scrip(const char *interp_file, int map_type, int submap_type, i
 
   nce(nc_put_var_double(nc_file_id, nc_dstgrdfrac_id, tgt_grid.cell_frac));
 
-  for ( i = 0; i < rv.num_links; i++ )
+  for ( long i = 0; i < rv.num_links; i++ )
     {
-      rv.src_grid_add[i]++;
-      rv.tgt_grid_add[i]++;
+      rv.src_cell_add[i]++;
+      rv.tgt_cell_add[i]++;
     }
 
-  nce(nc_put_var_int(nc_file_id, nc_srcadd_id, rv.src_grid_add));
-  nce(nc_put_var_int(nc_file_id, nc_dstadd_id, rv.tgt_grid_add));
+  nce(nc_put_var_int(nc_file_id, nc_srcadd_id, rv.src_cell_add));
+  nce(nc_put_var_int(nc_file_id, nc_dstadd_id, rv.tgt_cell_add));
 
   nce(nc_put_var_double(nc_file_id, nc_rmpmatrix_id, rv.wts));
 
   nce(nc_close(nc_file_id));
 
 #else
-  cdoAbort("netCDF support not compiled in!");
+  cdoAbort("NetCDF support not compiled in!");
 #endif
 
-}  /* write_remap_scrip */
+}  // write_remap_scrip
 
 /*****************************************************************************/
 
 void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *map_type, int *submap_type, int *num_neighbors,
 		      int *remap_order, remapgrid_t *src_grid, remapgrid_t *tgt_grid, remapvars_t *rv)
 {
-  /*
-    The routine reads a netCDF file to extract remapping info in SCRIP format
-  */
+  // The routine reads a NetCDF file to extract remapping info in SCRIP format
   /*
     Input variables
 
@@ -403,11 +392,11 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
   */
 #if defined(HAVE_LIBNETCDF)
 
-  /* Local variables */
+  // Local variables
 
   int lgridarea = FALSE;
   int status;
-  int nc_file_id;           /* id for netCDF file                       */
+  int nc_file_id;           /* id for NetCDF file                       */
   int nc_srcgrdsize_id;     /* id for source grid size                  */
   int nc_dstgrdsize_id;     /* id for destination grid size             */
   int nc_srcgrdcorn_id;     /* id for number of source grid corners     */
@@ -436,8 +425,6 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
   int nc_dstadd_id;         /* id for map destination address           */
   int nc_rmpmatrix_id;      /* id for remapping matrix                  */
 
-  long i;                   /* dummy index */
-
   char map_name[1024];
   char map_method[64];      /* character string for map_type             */
   char normalize_opt[64];   /* character string for normalization option */
@@ -450,13 +437,12 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
 
   int gridID1_gme_c = -1;
 
+  // Open file and read some global information
 
-  /* Open file and read some global information */
-
-  /* nce(nc_open(interp_file, NC_NOWRITE, &nc_file_id)); */
+  // nce(nc_open(interp_file, NC_NOWRITE, &nc_file_id));
   nc_file_id = cdf_openread(interp_file);
 
-  /* Map name */
+  // Map name
 
   nce(nc_get_att_text(nc_file_id, NC_GLOBAL, "title", map_name));
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "title", &attlen));
@@ -468,8 +454,7 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
       cdoPrint("From file: %s", interp_file);
     }
 
-  /* Normalization option */
-
+  // Normalization option
   nce(nc_get_att_text(nc_file_id, NC_GLOBAL, "normalization", normalize_opt));
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "normalization", &attlen));
   normalize_opt[attlen] = 0;
@@ -491,8 +476,7 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
   if ( cdoVerbose )
     cdoPrint("normalize_opt = %s", normalize_opt);
 
-  /* Map method */
-
+  // Map method
   nce(nc_get_att_text (nc_file_id, NC_GLOBAL, "map_method", map_method));
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "map_method", &attlen));
   map_method[attlen] = 0;
@@ -500,10 +484,10 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
   *submap_type = SUBMAP_TYPE_NONE;
   *remap_order = 1;
 
-  if ( strcompare(map_method, "Conservative") == 0 )
+  if ( cmpstr(map_method, "Conservative") == 0 )
     {
       int iatt;
-      if ( strcompare(map_method, "Conservative remapping using clipping on sphere") == 0 )
+      if ( cmpstr(map_method, "Conservative remapping using clipping on sphere") == 0 )
 	rv->map_type = MAP_TYPE_CONSERV_YAC;
       else
 	rv->map_type = MAP_TYPE_CONSERV;
@@ -511,19 +495,19 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
       status = nc_get_att_int(nc_file_id, NC_GLOBAL, "remap_order", &iatt);
       if ( status == NC_NOERR ) *remap_order = iatt;
     }
-  else if ( strcompare(map_method, "Bilinear") == 0 ) rv->map_type = MAP_TYPE_BILINEAR;
-  else if ( strcompare(map_method, "Bicubic")  == 0 ) rv->map_type = MAP_TYPE_BICUBIC;
-  else if ( strcompare(map_method, "Distance") == 0 )
+  else if ( cmpstr(map_method, "Bilinear") == 0 ) rv->map_type = MAP_TYPE_BILINEAR;
+  else if ( cmpstr(map_method, "Bicubic")  == 0 ) rv->map_type = MAP_TYPE_BICUBIC;
+  else if ( cmpstr(map_method, "Distance") == 0 )
     {
       rv->map_type = MAP_TYPE_DISTWGT;
       *num_neighbors = 4;
     }
-  else if ( strcompare(map_method, "Nearest") == 0 )
+  else if ( cmpstr(map_method, "Nearest") == 0 )
     {
       rv->map_type = MAP_TYPE_DISTWGT;
       *num_neighbors = 1;
     }
-  else if ( strcompare(map_method, "Largest") == 0 )
+  else if ( cmpstr(map_method, "Largest") == 0 )
     {
       rv->map_type = MAP_TYPE_CONSERV;
       *submap_type = SUBMAP_TYPE_LAF;
@@ -541,8 +525,7 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
 
   *map_type = rv->map_type;
 
-  /* File convention */
-
+  // File convention
   nce(nc_get_att_text (nc_file_id, NC_GLOBAL, "conventions", convention));
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "conventions", &attlen));
   convention[attlen] = 0;
@@ -556,9 +539,9 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
         cdoAbort("Unknown file convention!");
     }
 
-  /* Read some additional global attributes */
+  // Read some additional global attributes
 
-  /* Source and destination grid names */
+  // Source and destination grid names
 
   nce(nc_get_att_text (nc_file_id, NC_GLOBAL, "source_grid", src_grid_name));
   nce(nc_inq_attlen(nc_file_id, NC_GLOBAL, "source_grid", &attlen));
@@ -571,11 +554,11 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
   if ( cdoVerbose )
     cdoPrint("Remapping between: %s and %s", src_grid_name, tgt_grid_name);
 
-  /* Initialize remapgrid structure */
+  // Initialize remapgrid structure
   remapgrid_init(src_grid);
   remapgrid_init(tgt_grid);
 
-  /* Read dimension information */
+  // Read dimension information
 
   nce(nc_inq_dimid(nc_file_id, "src_grid_size", &nc_srcgrdsize_id));
   nce(nc_inq_dimlen(nc_file_id, nc_srcgrdsize_id, &dimlen));
@@ -620,10 +603,10 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
   nce(nc_inq_dimid(nc_file_id, "num_links", &nc_numlinks_id));
   nce(nc_inq_dimlen(nc_file_id, nc_numlinks_id, &dimlen));
   rv->num_links = dimlen;
-
+  /*
   if ( rv->num_links == 0 )
     cdoAbort("Number of remap links is 0, no remap weights found!");
-
+  */
   nce(nc_inq_dimid(nc_file_id, "num_wgts", &nc_numwgts_id));
   nce(nc_inq_dimlen(nc_file_id, nc_numwgts_id, &dimlen));
   rv->num_wts = dimlen;
@@ -637,8 +620,8 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
       gridID1_gme_c = gridToUnstructured(gridID1, 1);
     }
 
-  remapGridRealloc(rv->map_type, src_grid);
-  remapGridRealloc(rv->map_type, tgt_grid);
+  remapgrid_alloc(rv->map_type, src_grid);
+  remapgrid_alloc(rv->map_type, tgt_grid);
 
   if ( gridInqType(gridID1) == GRID_GME ) gridInqMaskGME(gridID1_gme_c, src_grid->vgpm);    
 
@@ -649,14 +632,20 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
 
   rv->resize_increment = (int) (0.1 * MAX(src_grid->size, tgt_grid->size));
 
-  /* Allocate address and weight arrays for mapping 1 */
+  // Allocate address and weight arrays for mapping 1
 
-  rv->src_grid_add = (int*) malloc(rv->num_links*sizeof(int));
-  rv->tgt_grid_add = (int*) malloc(rv->num_links*sizeof(int));
+  rv->src_cell_add = NULL;
+  rv->tgt_cell_add = NULL;
+  rv->wts = NULL;
+  if ( rv->num_links > 0 )
+    {
+      rv->src_cell_add = (int*) Malloc(rv->num_links*sizeof(int));
+      rv->tgt_cell_add = (int*) Malloc(rv->num_links*sizeof(int));
 
-  rv->wts = (double*) malloc(rv->num_wts*rv->num_links*sizeof(double));
+      rv->wts = (double*) Malloc(rv->num_wts*rv->num_links*sizeof(double));
+    }
 
-  /* Get variable ids */
+  // Get variable ids
 
   nce(nc_inq_varid(nc_file_id, "src_grid_dims", &nc_srcgrddims_id));
   nce(nc_inq_varid(nc_file_id, "src_grid_imask", &nc_srcgrdimask_id));
@@ -692,7 +681,7 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
   nce(nc_inq_varid(nc_file_id, "dst_address", &nc_dstadd_id));
   nce(nc_inq_varid(nc_file_id, "remap_matrix", &nc_rmpmatrix_id));
 
-  /* Read all variables */
+  // Read all variables
 
   nce(nc_get_var_int(nc_file_id, nc_srcgrddims_id, src_grid->dims));
 
@@ -758,23 +747,26 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
 
   nce(nc_get_var_double(nc_file_id, nc_dstgrdfrac_id, tgt_grid->cell_frac));
 
-  nce(nc_get_var_int(nc_file_id, nc_srcadd_id, rv->src_grid_add));
-  nce(nc_get_var_int(nc_file_id, nc_dstadd_id, rv->tgt_grid_add));
-
-  for ( i = 0; i < rv->num_links; i++ )
+  if ( rv->num_links > 0 )
     {
-      rv->src_grid_add[i]--;
-      rv->tgt_grid_add[i]--;
+      nce(nc_get_var_int(nc_file_id, nc_srcadd_id, rv->src_cell_add));
+      nce(nc_get_var_int(nc_file_id, nc_dstadd_id, rv->tgt_cell_add));
+
+      for ( long i = 0; i < rv->num_links; i++ )
+        {
+          rv->src_cell_add[i]--;
+          rv->tgt_cell_add[i]--;
+        }
+
+      nce(nc_get_var_double(nc_file_id, nc_rmpmatrix_id, rv->wts));
     }
 
-  nce(nc_get_var_double(nc_file_id, nc_rmpmatrix_id, rv->wts));
-
-  /* Close input file */
+  // Close input file
 
   nce(nc_close(nc_file_id));
 
 #else
-  cdoAbort("netCDF support not compiled in!");
+  cdoAbort("NetCDF support not compiled in!");
 #endif
 
   rv->links.option    = FALSE;
@@ -784,4 +776,4 @@ void read_remap_scrip(const char *interp_file, int gridID1, int gridID2, int *ma
   rv->links.src_add   = NULL;
   rv->links.dst_add   = NULL;
   rv->links.w_index   = NULL;
-}  /* read_remap_scrip */
+}  // read_remap_scrip
