@@ -2,7 +2,7 @@
   This file is part of CDO. CDO is a collection of Operators to
   manipulate and analyse Climate model Data.
 
-  Copyright (C) 2003-2014 Uwe Schulzweida, Uwe.Schulzweida@zmaw.de
+  Copyright (C) 2003-2016 Uwe Schulzweida, <uwe.schulzweida AT mpimet.mpg.de>
   See COPYING file for copying and redistribution conditions.
 
   This program is free software; you can redistribute it and/or modify
@@ -110,17 +110,17 @@ int zaxisDefine(zaxis_t zaxis)
   if ( zaxis.vals )
     {
       zaxisDefLevels(zaxisID, zaxis.vals);
-      free(zaxis.vals);
+      Free(zaxis.vals);
     }
   if ( zaxis.lbounds )
     {
       zaxisDefLbounds(zaxisID, zaxis.lbounds);
-      free(zaxis.lbounds);
+      Free(zaxis.lbounds);
     }
   if ( zaxis.ubounds )
     {
       zaxisDefUbounds(zaxisID, zaxis.ubounds);
-      free(zaxis.ubounds);
+      Free(zaxis.ubounds);
     }
 
   if ( zaxis.name[0] )     zaxisDefName(zaxisID, zaxis.name);
@@ -177,67 +177,67 @@ int zaxisFromFile(FILE *gfp, const char *dname)
       pline = line;
       while ( isspace((int) *pline) ) pline++;
       if ( pline[0] == '\0' ) continue;
-      if ( memcmp(pline, "zaxistype", 9) == 0 || 
-	   memcmp(pline, "type", 4) == 0 )
+      if ( cmpstrlen(pline, "zaxistype", len) == 0 || 
+	   cmpstrlen(pline, "type", len) == 0 )
 	{
 	  if ( *pline == 'z' )
 	    pline = skipSeparator(pline + 9);
 	  else
 	    pline = skipSeparator(pline + 4);
 
-	  if ( memcmp(pline, "pressure", 6) == 0 )
+	  if ( cmpstrlen(pline, "pressure", len) == 0 )
 	    zaxis.type = ZAXIS_PRESSURE;
-	  else if ( memcmp(pline, "hybrid_half", 11)  == 0 )
+	  else if ( cmpstrlen(pline, "hybrid_half", len)  == 0 )
 	    zaxis.type = ZAXIS_HYBRID_HALF;
-	  else if ( memcmp(pline, "hybrid", 6)  == 0 )
+	  else if ( cmpstrlen(pline, "hybrid", len)  == 0 )
 	    zaxis.type = ZAXIS_HYBRID;
-	  else if ( memcmp(pline, "height", 6) == 0 )
+	  else if ( cmpstrlen(pline, "height", len) == 0 )
 	    zaxis.type = ZAXIS_HEIGHT;
-	  else if ( memcmp(pline, "depth below sea", 15) == 0 ||
-		    memcmp(pline, "depth_below_sea", 15) == 0 )
+	  else if ( cmpstrlen(pline, "depth below sea", len) == 0 ||
+		    cmpstrlen(pline, "depth_below_sea", len) == 0 )
 	    zaxis.type = ZAXIS_DEPTH_BELOW_SEA;
-	  else if ( memcmp(pline, "depth below land", 16) == 0 ||
-		    memcmp(pline, "depth_below_land", 16) == 0 )
+	  else if ( cmpstrlen(pline, "depth below land", len) == 0 ||
+		    cmpstrlen(pline, "depth_below_land", len) == 0 )
 	    zaxis.type = ZAXIS_DEPTH_BELOW_LAND;
-	  else if ( memcmp(pline, "isentropic", 10)  == 0 )
+	  else if ( cmpstrlen(pline, "isentropic", len)  == 0 )
 	    zaxis.type = ZAXIS_ISENTROPIC;
-	  else if ( memcmp(pline, "surface", 7)  == 0 )
+	  else if ( cmpstrlen(pline, "surface", len)  == 0 )
 	    zaxis.type = ZAXIS_SURFACE;
-	  else if ( memcmp(pline, "generic", 7)  == 0 )
+	  else if ( cmpstrlen(pline, "generic", len)  == 0 )
 	    zaxis.type = ZAXIS_GENERIC;
 	  else
 	    cdoAbort("Invalid zaxisname : %s (zaxis description file: %s)", pline, dname);
 	}
-      else if ( memcmp(pline, "size", 4)  == 0 )
+      else if ( cmpstrlen(pline, "size", len)  == 0 )
 	{
-	  zaxis.size = atol(skipSeparator(pline + 4));
+	  zaxis.size = atol(skipSeparator(pline + len));
 	}
-      else if ( memcmp(pline, "vctsize", 7)  == 0 )
+      else if ( cmpstrlen(pline, "vctsize", len)  == 0 )
 	{
-	  zaxis.vctsize = atol(skipSeparator(pline + 7));
+	  zaxis.vctsize = atol(skipSeparator(pline + len));
 	}
-      else if ( memcmp(pline, "name", 4)  == 0 )
+      else if ( cmpstrlen(pline, "name", len)  == 0 )
 	{
-	  strcpy(zaxis.name, skipSeparator(pline + 4));
+	  strcpy(zaxis.name, skipSeparator(pline + len));
 	}
-      else if ( memcmp(pline, "longname", 8)  == 0 )
+      else if ( cmpstrlen(pline, "longname", len)  == 0 )
 	{
-	  strcpy(zaxis.longname, skipSeparator(pline + 8));
+	  strcpy(zaxis.longname, skipSeparator(pline + len));
 	}
-      else if ( memcmp(pline, "units", 5)  == 0 )
+      else if ( cmpstrlen(pline, "units", len)  == 0 )
 	{
-	  strcpy(zaxis.units, skipSeparator(pline + 5));
+	  strcpy(zaxis.units, skipSeparator(pline + len));
 	}
-      else if ( memcmp(pline, "levels", 6)  == 0 )
+      else if ( cmpstrlen(pline, "levels", len)  == 0 )
 	{
 	  int i;
 	  double flev;
 
 	  if ( zaxis.size > 0 )
 	    {
-	      pline = skipSeparator(pline + 6);
+	      pline = skipSeparator(pline + len);
 	  
-	      zaxis.vals = malloc(zaxis.size*sizeof(double));
+	      zaxis.vals = (double*) Malloc(zaxis.size*sizeof(double));
 	      for ( i = 0; i < zaxis.size; i++ )
 		{
 		  pline = skipSeparator(pline);
@@ -262,16 +262,16 @@ int zaxisFromFile(FILE *gfp, const char *dname)
 	      cdoAbort("size undefined (zaxis description file: %s)!", dname);
 	    }
 	}
-      else if ( memcmp(pline, "vct", 3)  == 0 )
+      else if ( cmpstrlen(pline, "vct", len)  == 0 )
 	{
 	  int i;
 	  double flev;
 
 	  if ( zaxis.vctsize > 0 )
 	    {
-	      pline = skipSeparator(pline + 3);
+	      pline = skipSeparator(pline + len);
 	  
-	      zaxis.vct = malloc(zaxis.vctsize*sizeof(double));
+	      zaxis.vct = (double*) Malloc(zaxis.vctsize*sizeof(double));
 	      for ( i = 0; i < zaxis.vctsize; i++ )
 		{
 		  pline = skipSeparator(pline);
@@ -296,16 +296,16 @@ int zaxisFromFile(FILE *gfp, const char *dname)
 	      cdoAbort("vctsize undefined (zaxis description file: %s)!", dname);
 	    }
 	}
-      else if ( memcmp(pline, "lbounds", 7)  == 0 )
+      else if ( cmpstrlen(pline, "lbounds", len)  == 0 )
 	{
 	  int i;
 	  double flev;
 
 	  if ( zaxis.size > 0 )
 	    {
-	      pline = skipSeparator(pline + 7);
+	      pline = skipSeparator(pline + len);
 	  
-	      zaxis.lbounds = malloc(zaxis.size*sizeof(double));
+	      zaxis.lbounds = (double*) Malloc(zaxis.size*sizeof(double));
 	      for ( i = 0; i < zaxis.size; i++ )
 		{
 		  pline = skipSeparator(pline);
@@ -330,16 +330,16 @@ int zaxisFromFile(FILE *gfp, const char *dname)
 	      cdoAbort("size undefined (zaxis description file: %s)!", dname);
 	    }
 	}
-      else if ( memcmp(pline, "ubounds", 7)  == 0 )
+      else if ( cmpstrlen(pline, "ubounds", len)  == 0 )
 	{
 	  int i;
 	  double flev;
 
 	  if ( zaxis.size > 0 )
 	    {
-	      pline = skipSeparator(pline + 7);
+	      pline = skipSeparator(pline + len);
 	  
-	      zaxis.ubounds = malloc(zaxis.size*sizeof(double));
+	      zaxis.ubounds = (double*) Malloc(zaxis.size*sizeof(double));
 	      for ( i = 0; i < zaxis.size; i++ )
 		{
 		  pline = skipSeparator(pline);
@@ -383,11 +383,11 @@ int zaxisFromName(const char *zaxisname)
   zaxisInit(&zaxis);
 
   pline = zaxisname;
-  if ( memcmp(pline, "surface", 7) == 0 ) /* surface */
+  if ( cmpstr(pline, "surface") == 0 ) /* surface */
     {
       zaxis.type = ZAXIS_SURFACE;
       zaxis.size = 1;
-      zaxis.vals = malloc(zaxis.size*sizeof(double));
+      zaxis.vals = (double*) Malloc(zaxis.size*sizeof(double));
       zaxis.vals[0] = 0;
     }
 
@@ -432,27 +432,11 @@ void defineZaxis(const char *zaxisarg)
     }
 }
 
-
-int zaxis2ltype(int zaxisID)
-{
-  int ltype;
-  int zaxistype;
-
-  zaxistype = zaxisInqType(zaxisID);
-
-  ltype = zaxisInqLtype(zaxisID);
-
-  if ( ltype <= 0 ) ltype = ztype2ltype(zaxistype);
-
-  return (ltype);
-}
-
-
+static
 int ztype2ltype(int zaxistype)
 {
-  int ltype;
+  int ltype = -1;
 
-  ltype = -1;
   if      ( zaxistype == ZAXIS_SURFACE           )  ltype =   1;
   else if ( zaxistype == ZAXIS_PRESSURE          )  ltype = 100;
   else if ( zaxistype == ZAXIS_ALTITUDE          )  ltype = 103;
@@ -468,22 +452,16 @@ int ztype2ltype(int zaxistype)
 }
 
 
-int ltype2ztype(int ltype)
+int zaxis2ltype(int zaxisID)
 {
-  int zaxistype = -1;
+  int ltype;
+  int zaxistype;
 
-  if      ( ltype ==   1 ) zaxistype = ZAXIS_SURFACE;
-  else if ( ltype ==   0 ) zaxistype = ZAXIS_SURFACE;
-  else if ( ltype == 100 ) zaxistype = ZAXIS_PRESSURE;
-  else if ( ltype == 103 ) zaxistype = ZAXIS_ALTITUDE;
-  else if ( ltype == 105 ) zaxistype = ZAXIS_HEIGHT;
-  else if ( ltype == 107 ) zaxistype = ZAXIS_SIGMA;
-  else if ( ltype == 109 ) zaxistype = ZAXIS_HYBRID;
-  else if ( ltype == 110 ) zaxistype = ZAXIS_HYBRID_HALF;
-  else if ( ltype == 111 ) zaxistype = ZAXIS_DEPTH_BELOW_LAND;
-  else if ( ltype == 113 ) zaxistype = ZAXIS_ISENTROPIC;
-  else if ( ltype == 160 ) zaxistype = ZAXIS_DEPTH_BELOW_SEA;
-  else                     zaxistype = ZAXIS_GENERIC;
+  zaxistype = zaxisInqType(zaxisID);
 
-  return (zaxistype);
+  ltype = zaxisInqLtype(zaxisID);
+
+  if ( ltype <= 0 ) ltype = ztype2ltype(zaxistype);
+
+  return (ltype);
 }
